var currentCard = null;

var socket = io.connect('http://' + document.location.hostname + ':1337');
socket.on('disconnect', function()
{
	alert('Disconnected! Click OK to reconnect.');
	location.reload();
});

// These are inital cards to spawn (should be retrieved from server)
var initialCards = [
	{ x: 50, y: 10, rotation: 20, type: 'card-heart-one', id: 1, z: 0, touch: { offsetX:0, offsetY:0 } },
	{ x: 100, y: 50, rotation: 30, type: 'card-heart-two', id: 2, z: 1, touch: { offsetX:0, offsetY:0 } },
	{ x: 150, y: 200, rotation: 10, type: 'card-heart-three', id: 3, z: 2, touch: { offsetX:0, offsetY:0 } },
	{ x: 300, y: 150, rotation: 0, type: 'card-heart-four', id: 4, z: 3, touch: { offsetX:0, offsetY:0 } },
	{ x: 200, y: 100, rotation: 40, type: 'card-heart-five', id: 5, z: 4, touch: { offsetX:0, offsetY:0 } },
];

// What if the server adds more cards than the initial amount..?
var topIndex = initialCards.length + 1;

// These are the cards currently on the table (will be mapped @ init)
var cards = {};

document.addEventListener('touchmove', function(event) {
  event.preventDefault();
}, false); 

window.onload = function()
{
	console.log('Starting card application');
	initCards(initialCards);	
	initSockets();
}
function initCards(initialCards)
{
	var cardsDeck = document.getElementById('cards');

	// Using forEach so 'card' remains in the scope of the event listeners
	initialCards.forEach(function(card, index, array)
	{
		// Add to current cards
		cards[card.id] = card;

		// Create DOM card
		var domCard = document.createElement('li');
		domCard.className = 'card ' + card.type;
		domCard.id = card.id;
		
		cardsDeck.appendChild(domCard);
		
		domCard.addEventListener('touchstart', function(event) 
		{
			var firstFinger = event.targetTouches[0];
			card.touch.offsetX = firstFinger.pageX - card.x;
			card.touch.offsetY = firstFinger.pageY - card.y;
			
			topIndex++;

			// Update Z
			card.move(card.x, card.y, topIndex, card.rotation);
			
			currentCard = card;
		}, false);

		domCard.addEventListener('touchmove', function(event) 
		{
			var firstFinger = event.targetTouches[0];

			// Update X/Y
			var x = firstFinger.pageX - card.touch.offsetX;
			var y = firstFinger.pageY - card.touch.offsetY;
			card.move(x, y, card.z, card.rotation);

		}, false);
		
		domCard.addEventListener('touchend', function(event)
		{
			currentCard = null;
		}, false);

		// Maintain reference to DOM card here (avoid GEBI)
		card.domCard = domCard;

		// Refresh function for model/DOM sync (TODO: prototype etc)
		card.refresh = function()
		{
			var style = this.domCard.style;
			style.top = this.y + 'px';
			style.left = this.x + 'px';
			style.zIndex = this.z;
		}

		// Update function
		card.prev = {};
		card.move = function(x, y, z, rotation)
		{
			// Position changed?
			var prev = card.prev;
			if(x != card.prev.x || y != card.prev.y || z != card.prev.z || rotation != card.prev.rotation)
			{
				// Set new position
				card.x = prev.x = x;
				card.y = prev.y = y;
				card.z = prev.z = z;
				card.rotation = prev.rotation = rotation;

				// Sync with UI
				card.refresh();

				// Notify server (TODO: limit this, 30x per second is fine)
				socket.emit('move',
				{
					id: card.id,
					x: card.x,
					y: card.y,
					z: card.z,
					rotation: card.rotation
				});
			}
		}

		// Refresh UI!
		card.refresh();
	});
}

function initSockets()
{	
	// Determine table
	var tableId = document.location.pathname.substr(1);

	// Select table, as a 'player' client
	socket.emit('session', { type: 'player' });
	socket.emit('device',
	{
		'userAgent': navigator.userAgent,
		'width': window.outerWidth,
		'height': window.outerHeight,
		'screenWidth': screen.width,
		'screenHeight': screen.height
	});
	socket.emit('selectTable', { 'id': tableId });
	socket.emit('identify', { 'nick': 'nils', 'color': 'red', 'avatarId': 'd8482y'});
	
	// Handle card updatesß
	socket.on('move', function(data)
	{
		var card = cards[data.id];
		if(card)
		{
			// Update model
			card.x = data.x;
			card.y = data.y;
			card.z = data.z;
			card.rotation = data.rotation;

			// Sync with DOM
			card.refresh();
		}
	});
}