// Import SessionManager
var SessionManager = require('./sessions');

// Constructor
var Session = function(sessionId, socket)
{
	// Set members
	this.id = sessionId;
	this._socket = socket;
	this.remoteAddress = socket.handshake.address.address;
	this.table = null;
	this.player = null;

	// Listen for disconnection
	this._socket.on('disconnect', this._onDisconnect.bind(this));

	// Register handshake events
	this.registerEvents('handshake');
}

// Force disconnection
Session.prototype.end = function()
{
	this._socket.disconnect();
}

// Handle disconnections
Session.prototype._onDisconnect = function()
{
	// Selected a table?
	if(this.table != null)
	{
		this.table.removeSession(this);
	}

	// Notify manager
	SessionManager.removeSession(this);
}

// Sending of message
Session.prototype.send = function(event, data)
{
	this._socket.emit(event, data);
}

// Register events from a given handlers module
Session.prototype.registerEvents = function(moduleName)
{
	// Find the module (cached)
	var module = require('../event/' + moduleName);

	// Module exists?
	if(module)
	{
		// For referencing
		var self = this;
		
		// Reg
		function reg(event, handler)
		{
			self._socket.on(event, function(msg)
			{
				handler(self, msg);
			});
		}

		// Create socket.io event listeners for all the handler functions
		for(var event in module)
		{
			var handler = module[event];
			reg(event, handler);
		}
	}
}

// String representation
Session.prototype.toString = function()
{
	return 'Session #' + this.id;
}

// Expose class
module.exports = Session;