// Import crypto module for random functions
var crypto = require('crypto');

// Import Table
var Table = require('./table');

// Constructor
var TableMaster = function()
{
	this._tables = {};
	this._maxTables = 0;
}

// Set maximum tables
TableMaster.prototype.setTableLimit = function(limit)
{
	if(limit > 0)
	{
		this._maxTables = limit;
		console.log('TableMaster: max ' + limit + ' concurrent game tables');
	}
}

// Callback. Generates a random, unused table ID
TableMaster.prototype.generateTableId = function(callback)
{
	// Generate random bytes...
	crypto.randomBytes(2, (function(err, buf)
	{
		// Bytes to hex string, ID in use? Repeat...
		var tableId = buf.toString('hex');
		if(tableId in this._tables)
		{
			this._generateTableId(callback);
		}
		else
		{
			callback(tableId);
		}
	}).bind(this));
}

// Factory method for Table
TableMaster.prototype.createTable = function(tableId, host)
{
	// Valid table ID?
	if(tableId)
	{
		// Table ID is unused?
		if(!(tableId in this._tables))
		{
			// Limit reached?
			if(this._maxTables > 0 && this._tables.length >= this._maxTables)
			{
				console.log('Server is full! Request to create table #' + tableId + ' was denied...');
			}
			else
			{
				// Create table, add to map
				var table = new Table(tableId, host);
				this._tables[tableId] = table;
				console.log('TableMaster: created #' + table.id);

				// Spawn cards
				table.reset();

				return table;
			}
		}
	}

	// Table not created!
	return null;
}

// Retrieves a table with a given ID
TableMaster.prototype.getTable = function(tableId)
{
	return this._tables[tableId];
}

// Expose a singleton
module.exports = new TableMaster();