// For filesystem-related functions
var fs = require('fs');

// The Express webserver framework
var Express = require('express');

// Constructor
var WebServer = function()
{
	// Create new Express server
	this._server = new Express();
	this._directory = null;
	this._avatarDirectory = null;

	// Serve welcome/instruction page
	this._server.get('/', this._getLandingPage.bind(this));

	// Serve table page
	this._server.get('/:tableId', this._getTablePage.bind(this));

	// Serve static resource files
	this._server.get('/*', this._getStaticResource.bind(this));

	// Return a random, unused table ID
	this._server.get('/get_random_table', this._getRandomTable.bind(this));
}

// Changes the webroot
WebServer.prototype.setDirectory = function(path)
{
	this._directory = path;
	console.log('Serving resources from "' + path + '"');
}

// Changes the avatar directory
WebServer.prototype.setAvatarDirectory = function(path)
{
	this._avatarDirectory = path;
	console.log('Serving avatars from "' + path + '"');
}

// Clears all avatars in the avatar directory
WebServer.prototype.clearAvatars = function()
{
	if(this._avatarDirectory)
	{
		console.log('Deleted all uploaded avatars for x tables');
	}
}

// Serves the landing/root/index page
WebServer.prototype._getLandingPage = function(req, res)
{
	res.sendfile(this._directory + 'welcome.html');
}

// Evaluates the user agent string and serves the appropriate table HTML
WebServer.prototype._getTablePage = function(req, res)
{
	var userAgent = req.header('user-agent');
   	if(/mobile/i.test(userAgent))
   	{
   		res.sendfile(this._directory + 'player/index.html');
   	}
   	else
   	{
   		res.sendfile(this._directory + 'table/index.html');
   	}
   	console.log('Served HTML for #' + req.params.tableId);
}

// Serves any static file from the /client/ directory
WebServer.prototype._getStaticResource = function(req, res)
{
	// Determine path
	var path = req.params[0];
	var fullPath = this._directory + path;

	// File exists?
	fs.stat(fullPath, function(err, stats)
	{
		if(stats != null)
		{
			res.sendfile(fullPath);
		}
		else 
		{
			res.send('<html><head><title>404 Not Found</title></head><body><h1>Not Found</h1><p>The requested resource was not found.</p></body></html>', 404);
		}
	});
}

// Generates a random, unused table ID
WebServer.prototype._getRandomTable = function(req, res)
{

}

// Start server
WebServer.prototype.listen = function(port)
{
	if(this._directory)
	{
		this._server.listen(port);
		console.log('WebServer running on TCP/' + port + ' (http://localhost:' +  port + ')');
	}
}

// Expose a singleton
module.exports = new WebServer();