package org.openhabbo.access;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.openhabbo.OpenHABBO;
import org.openhabbo.core.Logger;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;

public final class AccessControl
{
	private Map<Integer, UserRole> userRoles = new HashMap<Integer, UserRole>();

	private AccessControl()
	{
		
	}

	public void reloadUserRoles()
	{
		synchronized(this.userRoles)
		{
			this.userRoles.clear();

			// Reload all the roles
			DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT id,name,level FROM roles ORDER BY id ASC");
			while(result.row())
			{
				UserRole role = new UserRole();
				role.id = result.getInt("id");
				role.name = result.getString("name");
				role.level = result.getInt("level");
				this.userRoles.put(role.id, role);
			}

			// Load all the rights for every role
			for(UserRole role : this.userRoles.values())
			{
				ArrayList<String> tmp = new ArrayList<String>();
				tmp.add("default");
				result.query("SELECT `right` FROM role_rights WHERE id=" + role.id);
				while(result.row())
				{
					tmp.add(result.getString("right"));
				}
				role.rights = tmp.toArray(new String[0]);
			}
			result.release();
			Logger.getInstance().info("AccessControl", String.format("Reloaded %d user roles.", this.roles()));
		}
	}

	public int logUserLogin(int userId, String ip)
	{
		return OpenHABBO.getInstance().getDb().executeReturnKey("INSERT INTO user_accesslog(user_id,ip,login) VALUES (" + userId + ",'" + SQLHelper.escape(ip) + "','" + SQLHelper.getDateTimeStr() + "')");
	}

	public void logUserLogout(int accessId)
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE user_accesslog SET logout='" + SQLHelper.getDateTimeStr() + "' WHERE id=" + accessId);
	}

	public UserRole getUserRole(int roleId)
	{
		return this.userRoles.get(roleId);
	}

	public int roles()
	{
		return this.userRoles.size();
	}

	/* Begin singleton */
	private static AccessControl instance;

	public final static AccessControl getInstance()
	{
		return instance;
	}

	public static void createInstance()
	{
		instance = new AccessControl();
	}
	/* End singleton */
}
