package org.openhabbo.catalog;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.sessions.messages.SerializableObject;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.storage.DatabaseResult;

public class CatalogPage implements SerializableObject
{
	// Core info
	public int id;
	public int parentId;
	public String name;
	public int minAccessLevel;

	// Flags
	public boolean isVisible;
	public boolean isComingSoon;
	public boolean isClubOnly;

	// Layout data + content
	public int iconColor;
	public int iconImage;
	public String layout;
	public String headlineImage;
	public String teaserImage;
	public String specialImage;
	public String text1;
	public String text2;
	public String details;
	public String teaserText;

	// Inner pages and products
	public List<CatalogPage> pages = new ArrayList<CatalogPage>();
	public List<CatalogProduct> products = new ArrayList<CatalogProduct>();

	@Override
	public void serialize(ServerMessage msg)
	{
		msg.appendInt(this.id);
		msg.appendStr(this.layout);

		// Serialize extra msg for the layout
		if(this.layout.equals("frontpage"))
		{
			msg.appendInt(3);
			msg.appendStr(this.headlineImage);
			msg.appendStr(this.teaserImage);
			msg.appendStr("");
			msg.appendInt(11);
			msg.appendStr(this.text1);
			msg.appendStr("");
			msg.appendStr(this.text2);
			msg.appendStr(this.details);
			msg.appendStr("");
			msg.appendStr("#FAF8CC");
			msg.appendStr("#FAF8CC");
			msg.appendStr("Other ways to get more credits >");
			msg.appendStr("magic.credits");
		}
		else if(this.layout.equals("recycler_info"))
		{
			msg.appendInt(2);
			msg.appendStr(this.headlineImage);
			msg.appendStr(this.teaserImage);
			msg.appendInt(3);
			msg.appendStr(this.text1);
			msg.appendStr(this.text2);
			msg.appendStr(this.details);
		}
		else if(this.layout.equals("recycler_prizes"))
		{
			msg.appendInt(1);
			msg.appendStr("catalog_recycler_headline3");
			msg.appendInt(1);
			msg.appendStr(this.text1);
		}
		else if(this.layout.equals("spaces"))
		{
			msg.appendInt(1);
			msg.appendStr(this.headlineImage);
			msg.appendInt(1);
			msg.appendStr(this.text1);
		}
		else if(this.layout.equals("recycler"))
		{
			msg.appendInt(2);
			msg.appendStr(this.headlineImage);
			msg.appendStr(this.teaserImage);
			msg.appendInt(1);
			msg.appendStr(this.text1, (byte) 10);
			msg.appendStr(this.text2);
			msg.appendStr(this.details);
		}
		else if(this.layout.equals("trophies"))
		{
			msg.appendInt(1);
			msg.appendStr(this.headlineImage);
			msg.appendInt(2);
			msg.appendStr(this.text1);
			msg.appendStr(this.details);
		}
		else if(this.layout.equals("pets"))
		{
			msg.appendInt(2);
			msg.appendStr(this.headlineImage);
			msg.appendStr(this.teaserImage);
			msg.appendInt(4);
			msg.appendStr(this.text1);
			msg.appendStr("Give a name:");
			msg.appendStr("Pick a color:");
			msg.appendStr("Pick a race:");
		}
		else
		{
			msg.appendInt(3);
			msg.appendStr(this.headlineImage);
			msg.appendStr(this.teaserImage);
			msg.appendStr(this.specialImage);
			msg.appendInt(3);
			msg.appendStr(this.text1);
			msg.appendStr(this.details);
			msg.appendStr(this.teaserText);
		}

		// Serialize products
		msg.appendInt(this.products.size());
		for(CatalogProduct product : this.products)
		{
			msg.appendObject(product);
		}
	}

	public static CatalogPage parse(DatabaseResult row)
	{
		CatalogPage page = new CatalogPage();
		page.id = row.getInt("id");
		page.parentId = row.getInt("parent_id");
		page.name = row.getString("name");
		page.minAccessLevel = row.getInt("min_accesslevel");
		page.isVisible = row.getBoolean("visible");
		page.isComingSoon = row.getBoolean("coming_soon");
		page.isClubOnly = row.getBoolean("club_only");
		page.iconColor = row.getInt("icon_color");
		page.iconImage = row.getInt("icon_image");
		page.layout = row.getString("layout");
		page.headlineImage = row.getString("headline_image");
		page.teaserImage = row.getString("teaser_image");
		page.specialImage = row.getString("special_image");
		page.text1 = row.getString("text1");
		page.text2 = row.getString("text2");
		page.details = row.getString("details");
		page.teaserText = row.getString("teaser_text");
		return page;
	}
}
