package org.openhabbo.events;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import org.openhabbo.core.Logger;

public final class Event
{
	private final String id;
	private final List<EventListener> listeners;

	public Event(String id)
	{
		this.id = id;
		this.listeners = new CopyOnWriteArrayList<EventListener>();
	}

	public void addListener(EventListener listener)
	{
		this.listeners.add(listener);
	}

	public void removeListener(EventListener listener)
	{
		this.listeners.remove(listener);
	}

	public final void fire(Object subject)
	{
		this.fire(subject, null);
	}

	public final boolean fire(Object subject, Object args)
	{
		boolean result = true;
		for(EventListener listener : this.listeners)
		{
			try
			{
				result = (result && listener.handle(subject, args));
			}
			catch(Exception ex)
			{
				Logger.getInstance().error(this.id, String.format("Error executing listener %s.", listener.getClass().getName()), ex);
			}
		}
		
		// 'result' will be false if one or more handlers returned false
		return result;
	}

	public final String getId()
	{
		return this.id;
	}

	public final boolean hasListeners()
	{
		return(this.listeners.size() > 0);
	}

	@Override
	public final String toString()
	{
		return this.id;
	}
}
