package org.openhabbo.modules;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;

public final class Event
{
	private final String code;
	private final List<EventHandler> listeners;

	public Event(String code)
	{
		this.code = code;
		this.listeners = new ArrayList<EventHandler>();
	}

	public void addListener(EventHandler listener)
	{
		this.listeners.add(listener);
	}

	public void removeListener(EventHandler listener)
	{
		this.listeners.remove(listener);
	}

	public final void fire()
	{
		this.fire(null);
	}

	public final void fire(Object args)
	{
		for(EventHandler listener : this.listeners)
		{
			try
			{
				listener.handle(args);
			}
			catch(Exception ex)
			{
				Logger.getInstance().error("Event #" + this.code, "Error executing a listener!", ex);
			}
		}
	}

	public final String getCode()
	{
		return this.code;
	}

	public final boolean hasListeners()
	{
		return(this.listeners.size() > 0);
	}
}
