package org.openhabbo.modules;

import java.util.HashMap;
import java.util.Map;

public final class EventArgs
{
	private final int poolIndex;
	private final Map<String, Object> args;

	public EventArgs(int poolIndex)
	{
		this.poolIndex = poolIndex;
		this.args = new HashMap<String, Object>();
	}

	/*
	 * public EventArgs(String key1, Object value1)
	 * {
	 * this();
	 * this.set(key1, value1);
	 * }
	 * public EventArgs(String key1, Object value1, String key2, Object value2)
	 * {
	 * this(key1, value1);
	 * this.set(key2, value2);
	 * }
	 * public EventArgs(String key1, Object value1, String key2, Object value2, String key3, Object value3)
	 * {
	 * this(key1, value1, key2, value2);
	 * this.set(key3, value3);
	 * }
	 * public EventArgs(String key1, Object value1, String key2, Object value2, String key3, Object value3, String key4, Object value4)
	 * {
	 * this(key1, value1, key2, value2, key3, value3);
	 * this.set(key4, value4);
	 * }
	 * public EventArgs(String key1, Object value1, String key2, Object value2, String key3, Object value3, String key4, Object value4, String key5, Object value5)
	 * {
	 * this(key1, value1, key2, value2, key3, value3, key4, value4);
	 * this.set(key5, value5);
	 * }
	 */

	public final <T> EventArgs set(String key, T value)
	{
		this.args.put(key, value);
		return this;
	}

	public final void unset(String key)
	{
		this.args.remove(key);
	}

	public final boolean has(String key)
	{
		return this.args.containsKey(key);
	}

	@SuppressWarnings("unchecked")
	public final <T> T get(String key)
	{
		Object value = this.args.get(key);
		if(value != null)
		{
			return (T) value;
		}
		else
		{
			return null;
		}
	}

	public final void clear()
	{
		this.args.clear();
	}

	public final int getPoolIndex()
	{
		return this.poolIndex;
	}

	private final static int POOL_SIZE = 1024;
	private volatile static EventArgs[] pool;
	private volatile static boolean[] inUse;

	static
	{
		pool = new EventArgs[POOL_SIZE];
		for(int x = 0; x < POOL_SIZE; x++)
		{
			pool[x] = new EventArgs(x);
		}
	}

	public static EventArgs create()
	{
		for(int x = 0; x < POOL_SIZE; x++)
		{
			if(!inUse[x])
			{
				inUse[x] = true;
				return pool[x];
			}
		}
		return new EventArgs(POOL_SIZE - 1);
	}

	public static void release(EventArgs args)
	{
		args.clear();
		inUse[args.getPoolIndex()] = false;
	}
}
