package org.openhabbo.modules;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.openhabbo.core.Logger;

public abstract class ModuleResource
{
	private final static int TOTAL_EVENTS = 500;
	private final static int LISTENERS_PER_EVENT = 10;
	
	private final Map<String, Object> attachments = new HashMap<String, Object>();
	private final static Map<Integer, ArrayList<ModuleEventListener>> eventListeners = new HashMap<Integer, ArrayList<ModuleEventListener>>(TOTAL_EVENTS);
	
	public final <T> void set(String key, Object attachment)
	{
		synchronized(this.attachments)
		{
			this.attachments.put(key, attachment);
		}
	}

	public final void unset(String key)
	{
		synchronized(this.attachments)
		{
			this.attachments.remove(key);
		}
	}

	public final boolean has(String key)
	{
		synchronized(this.attachments)
		{
			return this.attachments.containsKey(key);
		}
	}

	@SuppressWarnings("unchecked")
	public final <T> T get(String key)
	{
		synchronized(this.attachments)
		{
			Object attachment = this.attachments.get(key);
			if(attachment != null)
			{
				return (T) attachment;
			}
			else
			{
				return null;
			}
		}
	}

	@Override
	public abstract String toString();
	
	public final static void addEventListener(int code, ModuleEventListener listener)
	{
		synchronized(eventListeners)
		{
			ArrayList<ModuleEventListener> listeners = eventListeners.get(code);
			if(listeners == null)
			{
				listeners = new ArrayList<ModuleEventListener>(LISTENERS_PER_EVENT);
				eventListeners.put(code, listeners);
			}
			listeners.add(listener);
		}
	}
	
	public final static void removeEventListener(int code, ModuleEventListener listener)
	{
		synchronized(eventListeners)
		{
			ArrayList<ModuleEventListener> listeners = eventListeners.get(code);
			if(listeners != null)
			{
				if(listeners.remove(listener))
				{
					if(listeners.isEmpty())
					{
						eventListeners.remove(code);
					}
				}
			}
		}
	}
	
	protected final void fireEvent(int code)
	{
		this.fireEvent(code, null);
	}
	
	protected final void fireEvent(int code, Object args)
	{
		synchronized(eventListeners)
		{
			ArrayList<ModuleEventListener> listeners = eventListeners.get(code);
			if(listeners != null)
			{
				for(ModuleEventListener listener : listeners)
				{
					try
					{
						Logger.getInstance().debug(this, "firing " + listener.getClass().getName() + " for event #" + code);
						listener.handle(this, args);
					}
					catch(Exception ex)
					{
						Logger.getInstance().error(this, String.format("Error in %s for event #%d.", listener.getClass().getName(), code), ex);
					}
				}
			}
		}
	}
}
