package org.openhabbo.modules;

public final class Version
{
	public final static Version DEFAULT_VERSION = new Version(1, 0, 0);

	public final int major;
	public final int minor;
	public final int build;
	public final String suffix;

	public Version(int major, int minor, int build)
	{
		this(major, minor, build, null);
	}

	public Version(int major, int minor, int build, String suffix)
	{
		this.major = major;
		this.minor = minor;
		this.build = build;
		this.suffix = (suffix != null && suffix.length() > 0) ? suffix : null;
	}

	public boolean isBeta()
	{
		return(this.suffix != null && this.suffix.equals("beta"));
	}

	public boolean isOlder(Version other)
	{
		// Compare majors
		if(this.major < other.major)
		{
			return true;
		}
		else
		{
			// Compare minors
			if(this.minor < other.minor)
			{
				return true;
			}
			else
			{
				// Compare builds
				if(this.build < other.build)
				{
					return true;
				}
				else
				{
					// Compare suffixes?
					if(other.suffix != null)
					{
						// Suffix different?
						if(this.suffix == null || !this.suffix.equals(other.suffix))
						{
							return true;
						}
					}
				}
			}
		}

		// Version is NOT older
		return false;
	}

	public boolean isAtleast(Version other)
	{
		return !this.isOlder(other);
	}

	public boolean equals(Version other)
	{
		return false;
	}

	@Override
	public String toString()
	{
		StringBuilder sb = new StringBuilder(8);
		// sb.append('v');
		sb.append(this.major);
		sb.append('.');
		sb.append(this.minor);
		sb.append('.');
		sb.append(this.build);
		if(this.suffix != null)
		{
			sb.append('-');
			sb.append(this.suffix);
		}
		return sb.toString();
	}

	public static Version parse(String str)
	{
		try
		{
			String[] segments = str.split("\\.");
			int major = Integer.parseInt(segments[0]);
			int minor = Integer.parseInt(segments[1]);
			int build;
			String suffix;
			if(segments[2].indexOf('-') == -1)
			{
				build = Integer.parseInt(segments[2]);
				suffix = null;
			}
			else
			{
				segments = segments[2].split("-", 2);
				build = Integer.parseInt(segments[0]);
				suffix = segments[1];
			}

			return new Version(major, minor, build, suffix);
		}
		catch(Exception ex)
		{
			return new Version(1, 0, 0, null);
		}
	}

	public static void main(String[] args)
	{
		Version v1 = new Version(1, 1, 1);
		Version v2 = new Version(1, 1, 0);
		System.out.println(v1.isOlder(v2));
	}
}
