package org.openhabbo.net;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import org.openhabbo.core.Logger;

public final class NetListener implements Runnable
{
	private int port;
	private NetManager mgr;
	private boolean listening;
	private Thread thread;
	private ServerSocketChannel listener;
	private Selector selector;

	public NetListener(int port, NetManager mgr)
	{
		this.port = port;
		this.mgr = mgr;
	}

	public boolean start()
	{
		if(!this.listening)
		{
			try
			{
				this.selector = Selector.open();
				this.listener = ServerSocketChannel.open();
				this.listener.configureBlocking(false);
				this.listener.socket().bind(new InetSocketAddress(this.port));
				this.listener.register(this.selector, SelectionKey.OP_ACCEPT);
			}
			catch(Exception ex)
			{
				Logger.getInstance().error("NetListener", String.format("Could not start listening on TCP port %d!", this.port), ex);
				return false;
			}
			this.thread = new Thread(this, "NetListener (TCP/" + this.port);
			this.listening = true;
			this.thread.start();
			Logger.getInstance().info("NetListener", String.format("Listening for game connections on TCP port %d.", this.port));
			return true;
		}

		return false;
	}

	public void stop()
	{
		this.listening = false;
		this.thread.interrupt();
		try
		{
			this.listener.close();
			Logger.getInstance().info("NetListener", "Closed listener.");
		}
		catch(IOException ex)
		{
			Logger.getInstance().error("NetListener", String.format("Error closing listener on port %d!", this.port), ex);
		}
	}

	@Override
	public void run()
	{
		while(this.listening)
		{
			try
			{
				this.selector.select();
			}
			catch(IOException ex)
			{
				Logger.getInstance().error("NetListener", "Failed grabbing updated selector keys, retrying...", ex);
				continue;
			}
			for(SelectionKey key : this.selector.selectedKeys())
			{
				if(key.isValid() && key.readyOps() == SelectionKey.OP_ACCEPT)
				{
					try
					{
						SocketChannel chan = this.listener.accept();
						NetConnection conn = new NetConnection(chan);
						this.mgr.onAcceptedByListener(conn);
					}
					catch(ClosedChannelException ex)
					{

					}
					catch(Exception ex)
					{
						Logger.getInstance().error("NetListener", "Failed accepting a connection request.", ex);
					}
				}
			}
		}
	}

	public boolean isListening()
	{
		return this.listening;
	}

	public int getPort()
	{
		return this.port;
	}

	public NetManager getMgr()
	{
		return this.mgr;
	}
}
