package org.openhabbo.net;

import java.io.IOException;
import java.nio.channels.ClosedSelectorException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.util.ArrayList;
import java.util.List;

import org.openhabbo.core.Logger;

public final class NetWorker implements Runnable
{
	private int index;
	private NetManager mgr;
	private boolean running;
	private Thread thread;
	private Selector selector;
	private int load;
	private List<NetConnection> pending;

	public NetWorker(int index, NetManager mgr)
	{
		this.index = index;
		this.mgr = mgr;
		this.pending = new ArrayList<NetConnection>();
		try
		{
			this.selector = Selector.open();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}

	public void start()
	{
		if(!this.running)
		{
			this.running = true;
			this.thread = new Thread(this, this.toString());
			this.thread.start();
		}
	}

	public void stop()
	{
		if(this.running)
		{
			this.running = false;
			try
			{
				this.selector.close();
			}
			catch(IOException ex)
			{
				Logger.getInstance().error(this, "Error while closing selector.", ex);
			}
		}
	}

	public void enqueueNewConnection(NetConnection conn)
	{
		synchronized(this.pending)
		{
			this.pending.add(conn);
		}
		this.selector.wakeup();
	}

	public void unregister(NetConnection conn)
	{
		this.load--;
	}

	@Override
	public void run()
	{
		while(this.running)
		{
			// Any pending connections? We need to register them on this thread
			synchronized(this.pending)
			{
				for(NetConnection conn : this.pending)
				{
					if(conn.assignWorker(this, this.selector))
					{
						this.load++;
						this.mgr.onAcceptedByWorker(conn);
					}
				}
				this.pending.clear();
			}

			// Process any enabled keys
			try
			{
				this.selector.select();
				for(SelectionKey key : this.selector.selectedKeys())
				{
					// Valid key?
					if(!key.isValid())
					{
						continue;
					}

					// Determine the associated connection
					NetConnection conn = (NetConnection) key.attachment();

					// Reading?
					if(key.isReadable())
					{
						try
						{
							if(!conn.onRead())
							{
								key.cancel();
								conn.close("disconnected by user");
								continue;
							}
						}
						catch(Exception ex)
						{
							key.cancel();
							conn.close("reading error");
							continue;
						}
					}

					// Writing?
					if(key.isWritable())
					{
						try
						{
							conn.onWrite();
						}
						catch(Exception ex)
						{
							key.cancel();
							conn.close("sending error");
							continue;
						}
					}
				}
				this.selector.selectedKeys().clear();
			}
			catch(ClosedSelectorException ex)
			{

			}
			catch(Exception ex)
			{
				Logger.getInstance().error(this, "Failed grabbing updated selector keys, retrying...", ex);
				continue;
			}
		}
	}

	public int index()
	{
		return this.index;
	}

	public NetManager getMgr()
	{
		return this.mgr;
	}

	public int load()
	{
		return this.load;
	}

	@Override
	public String toString()
	{
		return "NetWorker #" + this.index;
	}
}
