package org.openhabbo.net.messages;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.openhabbo.util.Base64Encoding;
import org.openhabbo.util.WireEncoding;

public final class ServerMessage
{
	public final static byte MSG_EOF = (byte) 1;

	public int msgId;
	private final ByteArrayOutputStream buff;

	public ServerMessage()
	{
		this.buff = new ByteArrayOutputStream();
	}

	public ServerMessage(int msgId)
	{
		this();
		this.init(msgId);
	}

	public final void reset()
	{
		this.buff.reset();
	}
	public final void init(int msgId)
	{
		this.msgId = msgId;
		this.reset();
		this.append(Base64Encoding.encodeInt(msgId, 2));
	}
	
	public final void append(byte b)
	{
		this.buff.write(b);
	}

	public final void append(char c)
	{
		this.append((byte) c);
	}

	public final void append(byte[] bytes)
	{
		try
		{
			this.buff.write(bytes);
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}

	public final void append(String s)
	{
		if(s.length() > 0)
		{
			this.append(s.getBytes());
		}
	}

	public final void appendBool(boolean b)
	{
		if(b)
		{
			this.append(WireEncoding.POSITIVE);
		}
		else
		{
			this.append(WireEncoding.NEGATIVE);
		}
	}

	public final void appendInt(int i)
	{
		this.append(WireEncoding.encodeInt(i));
	}

	public final void appendStr(String s)
	{
		this.appendStr(s, (byte) 2);
	}

	public final void appendStr(String s, byte tail)
	{
		this.append(s != null ? s : "");
		this.append(tail);
	}

	public final void appendStrLine(String s)
	{
		this.appendStr(s, (byte) 13);
	}

	public final void appendObject(SerializableObject obj)
	{
		obj.serialize(this);
	}

	public final byte[] getResult()
	{
		this.buff.write(ServerMessage.MSG_EOF);
		return this.buff.toByteArray();
	}

	public final String header()
	{
		return new String(Base64Encoding.encodeInt(this.msgId, 2));
	}

	public final int length()
	{
		if(this.buff.size() > 0)
		{
			return this.buff.size() - 2;
		}
		else
		{
			return 0;
		}
	}

	@Override
	public final String toString()
	{
		return new String(this.buff.toByteArray());
	}
}
