package org.openhabbo.rooms;

import org.openhabbo.OpenHABBO;
import org.openhabbo.sessions.messages.SerializableObject;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;

public final class Room implements SerializableObject, Comparable<Room>
{
	public final static byte STATE_OPEN = 0;
	public final static byte STATE_DOORBELL = 1;
	public final static byte STATE_PASSWORD = 2;

	public int id;
	public int ownerId;
	public String ownerName;
	public String name;
	public String description;
	public String modelName;
	public String files;
	public int categoryId;
	public byte state;
	public String password;
	public int usersNow;
	public int usersMax;
	public String wallpaper;
	public String floor;
	public String landscape;
	public boolean hideWalls;
	public int rating;
	public String icon;
	public String[] tags;
	public boolean allowPets;
	public boolean allowPetsEat;
	public boolean allowWalkthrough;

	public boolean isPublicRoom()
	{
		return(this.ownerId == 0);
	}

	public boolean isGuestRoom()
	{
		return(this.ownerId > 0);
	}

	public void saveSettings()
	{
		String tagStr = "";
		for(int x = 0; x < this.tags.length; x++)
		{
			if(x > 0)
			{
				tagStr += " ";
			}
			tagStr += this.tags[x];
		}

		OpenHABBO.getInstance().getDb().execute("UPDATE rooms SET name='" + SQLHelper.escape(this.name) + "',description='" + SQLHelper.escape(this.description) + "',category_id=" + this.categoryId + ",state=" + this.state + ",password='" + SQLHelper.escape(this.password) + "',users_max=" + this.usersMax + ",icon='" + SQLHelper.escape(this.icon) + "',tags='" + SQLHelper.escape(tagStr) + "',allow_pets=" + (this.allowPets ? '1' : '0') + ",allow_pets_eat=" + (this.allowPetsEat ? '1' : '0') + ",allow_walkthrough=" + (this.allowWalkthrough ? '1' : '0') + ",hide_walls=" + (this.hideWalls ? '1' : '0') + " WHERE id=" + this.id);
	}

	public void saveDecoration()
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE rooms SET wallpaper='" + SQLHelper.escape(this.wallpaper) + "',floor='" + SQLHelper.escape(this.floor) + "',landscape='" + SQLHelper.escape(this.landscape) + "' WHERE id=" + this.id);
	}

	public void saveUserCount()
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE rooms SET users_now=" + this.usersNow + " WHERE id=" + this.id);
	}

	public void saveRating()
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE rooms SET rating=" + this.rating + " WHERE id=" + this.id);
	}

	@Override
	public void serialize(ServerMessage msg)
	{
		msg.appendInt(this.id);
		msg.appendBool(false); // Has event?
		msg.appendStr(this.name);
		msg.appendStr(this.ownerName);
		msg.appendInt(this.state);
		msg.appendInt(this.usersNow);
		msg.appendInt(this.usersMax);
		msg.appendStr(this.description);
		msg.appendBool(true); // ??
		msg.appendBool(true); // Trading allowed
		msg.appendInt(this.rating);
		msg.appendInt(this.categoryId);
		msg.appendStr("");
		msg.appendStrList(this.tags);
		msg.append(this.icon);
		msg.appendBool(true);
		msg.appendBool(true);
	}

	public static Room parse(DatabaseResult result)
	{
		Room room = new Room();
		room.id = result.getInt("id");
		room.ownerId = result.getInt("owner_id");
		if(room.ownerId > 0)
		{
			room.ownerName = result.getString("owner");
		}
		else
		{
			room.files = result.getString("files");
		}
		room.name = result.getString("name");
		room.description = result.getString("description");
		room.modelName = result.getString("model_name");
		room.categoryId = result.getInt("category_id");
		room.state = result.getByte("state");
		room.password = result.getString("password");
		room.usersNow = result.getInt("users_now");
		room.usersMax = result.getInt("users_max");
		room.wallpaper = result.getString("wallpaper");
		room.floor = result.getString("floor");
		room.landscape = result.getString("landscape");
		room.hideWalls = result.getBoolean("hide_walls");
		room.rating = result.getInt("rating");
		room.icon = result.getString("icon");
		room.tags = result.getString("tags").split(" ");
		room.allowPets = result.getBoolean("allow_pets");
		room.allowPetsEat = result.getBoolean("allow_pets_eat");
		room.allowWalkthrough = result.getBoolean("allow_walkthrough");
		return room;
	}

	@Override
	public int compareTo(Room other)
	{
		if(this.usersNow > other.usersNow)
		{
			return 1;
		}
		else if(this.usersNow < other.usersNow)
		{
			return -1;
		}
		else
		{
			return 0;
		}
	}
}
