package org.openhabbo.sessions.handlers;

import org.openhabbo.items.EItemBehaviour;
import org.openhabbo.items.Item;
import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;

public class ApplyRoomDecoration implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to apply decoration?
		if(session.getRoomUser() == null || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Verify decoration item
		Item item = session.getItemInventory().getItem(request.readInt());
		if(item == null || !item.is(EItemBehaviour.DECORATION))
		{
			return;
		}

		// Remove from inventory and delete
		session.getItemInventory().removeItem(item.id);
		item.delete();

		// Apply new props
		RoomInstance room = session.getRoomInstance();
		if(item.definition.sprite.equals("wallpaper"))
		{
			room.getInfo().wallpaper = item.customData;
		}
		else if(item.definition.sprite.equals("floor"))
		{
			room.getInfo().floor = item.customData;
		}
		else if(item.definition.sprite.equals("landscape"))
		{
			room.getInfo().landscape = item.customData;
		}
		room.getInfo().saveDecoration();

		// Notify room
		response.init(EServerMessage.RoomProperty);
		response.appendStr(item.definition.sprite);
		response.appendStr(item.customData);
		room.broadcast(response);
	}
}
