package org.openhabbo.sessions.handlers;

import java.util.Collection;
import java.util.List;

import org.openhabbo.events.Event;
import org.openhabbo.items.EItemBehaviour;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;

public class GetRoomData3Handler implements SessionMessageHandler
{
	public final static Event onSendUserInfo = new Event("OnSendUserInfo");

	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Shorthand variable
		RoomInstance instance = session.getRoomInstance();

		// Passive objects
		List<Item> items = instance.getItems(EItemBehaviour.WALLITEM);
		response.init(30);
		response.appendInt(items.size());
		for(Item item : items)
		{
			response.appendObject(item);
		}
		session.send(response);

		// Public rooms have a different 'interface'
		if(instance.getInfo().isPublicRoom())
		{
			response.init(471);
			response.appendBool(false);
			response.appendStr(instance.getInfo().modelName);
			response.appendBool(false);
			session.send(response);
		}
		else
		{
			// Hide walls
			response.init(472);
			response.appendBool(instance.getInfo().hideWalls);
			session.send(response);

			// Send floor item list
			items = instance.getItems(EItemBehaviour.FLOORITEM);
			response.init(EServerMessage.FloorItems);
			response.appendInt(items.size());
			for(Item item : items)
			{
				response.appendObject(item);
			}
			session.send(response);

			// Send wall item list
			items = instance.getItems(EItemBehaviour.WALLITEM);
			response.init(EServerMessage.WallItems);
			response.appendInt(items.size());
			for(Item item : items)
			{
				response.appendObject(item);
			}
			session.send(response);

			// Interface info (edit room etc)
			response.init(471);
			response.appendBool(true);
			response.appendInt(instance.getInfo().id);
			response.appendBool(instance.getInfo().ownerId == session.getUser().id);
			session.send(response);
		}

		// Send room users, bots, pets
		Collection<RoomUser> users = instance.getAllUsers();
		response.init(EServerMessage.Users);
		response.appendInt(users.size());
		for(RoomUser user : users)
		{
			response.appendObject(user);
		}
		session.send(response);

		// Try spawning room user
		if(session.enterRoom())
		{
			session.sendRoomPrivileges();
		}
		else
		{
			return;
		}

		// Send statuses of all room users
		response.init(EServerMessage.UserUpdate);
		response.appendInt(users.size());
		for(RoomUser user : users)
		{
			user.appendStatusString(response);
		}
		session.send(response);

		// Send other special user statuses
		for(RoomUser user : users)
		{
			if(user.danceId >= 1)
			{
				response.init(EServerMessage.Dance);
				response.appendInt(user.virtualId);
				response.appendInt(user.danceId);
				session.send(response);
			}
			if(user.carryItemId != 0 && user.carryItemTimer != 0)
			{
				response.init(EServerMessage.CarryObject);
				response.appendInt(user.virtualId);
				response.appendInt(user.carryItemTimer);
				session.send(response);
			}

			GetRoomData3Handler.onSendUserInfo.fire(session, user);
		}
	}
}