package org.openhabbo.sessions.handlers;

import java.util.Map.Entry;

import org.openhabbo.rooms.Room;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;

public class GetRoomEditDataHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Shorthand variable for the room info
		Room roomInfo = session.getRoomInstance().getInfo();

		// Start the response
		response.init(EServerMessage.RoomSettingsData);

		// Append the core data
		response.appendInt(roomInfo.id);
		response.appendStr(roomInfo.name);
		response.appendStr(roomInfo.description);
		response.appendInt(roomInfo.state);
		response.appendInt(roomInfo.categoryId);
		response.appendInt(roomInfo.usersMax);
		response.appendInt(25); // default max

		// Append the tags
		response.appendStrList(roomInfo.tags);
		
		// Append the rightslist
		response.appendInt(session.getRoomInstance().getUsersWithRights().size());
		for(Entry<Integer, String> user : session.getRoomInstance().getUsersWithRights())
		{
			response.appendInt(user.getKey());
			response.appendStr(user.getValue());
		}
		response.appendInt(session.getRoomInstance().getUsersWithRights().size());

		// Append some switches
		response.appendBool(roomInfo.allowPets);
		response.appendBool(roomInfo.allowPetsEat);
		response.appendBool(roomInfo.allowWalkthrough);
		response.appendBool(roomInfo.hideWalls);

		// Enqueue the response
		session.send(response);
	}
}
