package org.openhabbo.sessions.handlers;

import org.openhabbo.items.EItemBehavior;
import org.openhabbo.items.Item;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.util.SecurityUtil;

public class PlaceItemHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to place items in this room?
		if(!session.getRoomUser().hasRoomRights)
		{
			return;
		}

		// All data is in one string... weird
		String[] data = request.readStr().split(" ");
		int itemId = Integer.parseInt(data[0]);

		// Validate item from inventory
		Item item = session.getItemInventory().getItem(itemId);
		if(item == null)
		{
			return;
		}

		// What kind of item is being placed?
		boolean placementOK = false;
		if(item.is(EItemBehavior.FLOORITEM))
		{
			int tileX = Integer.parseInt(data[1]);
			int tileY = Integer.parseInt(data[2]);
			byte rotation = Byte.parseByte(data[3]);
			placementOK = session.getRoomInstance().placeFloorItem(item, tileX, tileY, rotation, session);
		}
		else if(item.is(EItemBehavior.WALLITEM))
		{
			String position = SecurityUtil.checkWallPosition(data[1] + " " + data[2] + " " + data[3]);
			if(position != null)
			{
				placementOK = session.getRoomInstance().placeWallItem(item, position, session);
			}
		}

		// Placement okay?
		if(placementOK)
		{
			// Remove from inventory!
			session.getItemInventory().removeItem(item.id);
			response.init(EServerMessage.InventoryRemove);
			response.appendInt(item.id);
			session.send(response);
		}
	}
}
