package org.openhabbo.sessions.handlers;

import java.util.List;

import org.openhabbo.catalog.Catalog;
import org.openhabbo.catalog.CatalogProduct;
import org.openhabbo.core.Logger;
import org.openhabbo.items.Item;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.util.SecurityUtil;

public class PurchaseProductHandler implements SessionMessageHandler
{
	@Override
	@SuppressWarnings("unused")
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Parse purchase info
		int pageId = request.readInt();
		int productId = request.readInt();
		String purchaseData = SecurityUtil.filterStr(request.readStr(), false); // Preserve linebreaks

		// Valid product?
		CatalogProduct product = Catalog.getInstance().getProduct(productId);
		if(product == null || !product.isPurchasableBy(session.getUser()))
		{
			Logger.getInstance().warning(this, String.format("User #%d tried to purchase product #%d from the Catalog, which is not existent/available/purchasable to this user.", session.getUser().id, productId));
			return;
		}

		// Attempt item purchase
		List<Item> shipping = Catalog.getInstance().purchaseProduct(product, session.getUser(), purchaseData, true);
		if(shipping == null)
		{
			session.sendNotif(String.format("Purchase of catalog product #%d failed, you will not be billed. Please try again in a few minutes. If this keeps happening, please contact Support.", product.id));
			return;
		}

		// Add items to inventory
		if(!shipping.isEmpty())
		{
			for(Item item : shipping)
			{
				session.getItemInventory().addItem(item, false);
			}
			session.getItemInventory().refresh();
		}
		
		// Bill user for product
		Catalog.getInstance().billUser(session, product);

		// Purchase okay!
		response.init(EServerMessage.PurchaseOK);
		response.appendInt(product.itemDefinition.id);
		response.appendStr(product.name);
		response.appendInt(product.costCredits);
		response.appendInt(product.costPixels);
		response.appendInt(product.costQuestPoints);
		response.appendInt(1);
		response.appendStr(product.getType());
		response.appendInt(product.itemDefinition.spriteId);
		response.appendStr("");
		response.appendInt(1);
		response.appendInt(-1);
		response.appendStr("");
		session.send(response);
	}
}
