package org.openhabbo.sessions.handlers;

import org.openhabbo.core.Logger;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.util.FigureValidator;

public class UpdateLookHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Parse new data
		char newGender = request.readStr().equals("F") ? 'F' : 'M';
		String newFigure = request.readStr();

		// Valid new figure?
		if(FigureValidator.figureIsValid(newFigure))
		{
			session.getUser().figure = newFigure;
			session.getUser().gender = newGender;
			session.getUser().saveAppearance();
		}
		else
		{
			Logger.getInstance().warning(session, String.format("User #%d (%s) tried to use a 'mutant figure': '%s'. Figure not updated.", session.getUser().id, session.getUser().username, newFigure));
		}

		// Send update to client
		response.init(EServerMessage.UserChange);
		response.appendInt(-1);
		response.appendStr(session.getUser().figure);
		response.appendStr(session.getUser().gender);
		response.appendStr(session.getUser().motto);
		session.send(response);

		// If in room, then the room users need an update too!
		if(session.inRoom())
		{
			response.init(EServerMessage.UserChange);
			response.appendInt(session.getRoomUser().avatarId);
			response.appendStr(session.getUser().figure);
			response.appendStr(Character.toString(session.getUser().gender));
			response.appendStr(session.getUser().motto);
			session.getRoomInstance().broadcast(response);
		}
	}
}
