package org.openhabbo.sessions.handlers;

import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.util.SecurityUtil;

public class WhisperHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Parse receiver + text
		final String[] data = request.readStr().split(" ", 2);
		if(data.length != 2)
		{
			return;
		}
		final String receiverName = data[0];
		final String text = SecurityUtil.filterStr(data[1]);

		// Actually in room?
		if(session.inRoom())
		{
			// Receiver in room too?
			RoomUser receiver = session.getRoomInstance().getUserByUsername(receiverName);
			if(receiver != null && receiver != session.getRoomUser())
			{
				// Fire events
				Session.onChat.fire(session, text);
				
				// Send to this user & receiver
				response.init(EServerMessage.Whisper);
				response.appendInt(session.getRoomUser().virtualId);
				response.appendStr(text);
				response.appendBool(false);
				session.send(response);
				receiver.getSession().send(response);
			}
		}
	}
}
