package org.openhabbo.sessions.handlers;

import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.support.ChatlogEntry;
import org.openhabbo.support.SupportCenter;

public class WhisperHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Parse receiver + text
		String[] data = request.readStr().split(" ", 2);
		if(data.length != 2)
		{
			return;
		}

		// Actually in room?
		if(session.getRoomUser() != null)
		{
			// Receiver in room too?
			RoomUser receiver = session.getRoomInstance().getRoomUserByUsername(data[0]);
			if(receiver != null && receiver != session.getRoomUser())
			{
				// Send to this user & receiver
				response.init(EServerMessage.Whisper);
				response.appendInt(session.getRoomUser().virtualId);
				response.appendStr(data[1]);
				response.appendBool(false);
				session.send(response);
				receiver.getSession().send(response);

				// Write chatlog
				ChatlogEntry chatlog = new ChatlogEntry(session.getUser().id, session.getRoomInstance().getInfo().id, ChatlogEntry.Type.whisper, data[1], receiver.getUser().id);
				SupportCenter.getInstance().addChatlog(chatlog);
			}
		}
	}
}
