package org.openhabbo.support;

import java.sql.Types;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;
import org.openhabbo.core.Logger;
import org.openhabbo.storage.DatabaseConnection;
import org.openhabbo.storage.SQLHelper;

public final class SupportCenter implements Runnable
{
	private Thread thread;
	private boolean alive;
	private List<ChatlogEntry> chatlogStack;

	public SupportCenter()
	{
		this.thread = new Thread(this, "SupportCenter worker");
		this.chatlogStack = new ArrayList<ChatlogEntry>();
	}

	public void start()
	{
		this.alive = true;
		this.thread.start();
		Logger.getInstance().info("SupportCenter", "Started worker.");
	}

	public void stop()
	{
		this.alive = false;
		this.thread.interrupt();
		Logger.getInstance().info("SupportCenter", "Stopped worker.");
	}

	public void shutdown()
	{
		this.stop();
	}

	public void writeSupportLog(int issuerId, int targetId, String action, String message, String extraInfo)
	{
		OpenHABBO.getInstance().getDb().execute("INSERT INTO support_log(timestamp,issuer_id,action,target_id,message,extra_info) VALUES('" + SQLHelper.getDateTimeStr() + "'," + issuerId + ",'" + SQLHelper.escape(action) + "'," + targetId + ",'" + SQLHelper.escape(message) + "','" + SQLHelper.escape(extraInfo) + "')");
	}

	public void addChatlog(ChatlogEntry entry)
	{
		synchronized(this.chatlogStack)
		{
			this.chatlogStack.add(entry);
		}
	}

	@Override
	public void run()
	{
		while(this.alive)
		{
			try
			{
				Thread.sleep(15 * 1000);
			}
			catch(InterruptedException e)
			{
				return;
			}

			// Any chatlogs to bulk insert?
			synchronized(this.chatlogStack)
			{
				if(this.chatlogStack.size() > 0)
				{
					// Create connection
					DatabaseConnection conn = OpenHABBO.getInstance().getDb().getConnMgr().getConnection();
					if(conn != null)
					{
						try
						{
							// Create PreparedStatement and batch all the INSERT queries
							PreparedStatement query = conn.getInnerConnection().prepareStatement("INSERT INTO support_chatlog(user_id,room_id,timestamp,type,text,receiver_id) VALUES (?,?,?,?,?,?)");
							for(ChatlogEntry entry : this.chatlogStack)
							{
								query.setInt(1, entry.userId);
								query.setInt(2, entry.roomId);
								query.setTimestamp(3, entry.timestamp);
								query.setString(4, entry.type.toString());
								query.setString(5, entry.text);
								if(entry.receiverId == 0)
								{
									query.setNull(6, Types.INTEGER);
								}
								else
								{
									query.setInt(6, entry.receiverId);
								}
								query.addBatch();
							}

							// Execute the batch and close the statement
							query.executeBatch();
							query.close();
						}
						catch(SQLException ex)
						{
							Logger.getInstance().error("SupportCenter", "Error bulk inserting chatlog stack!", ex);
						}
						finally
						{
							OpenHABBO.getInstance().getDb().getConnMgr().releaseConnection(conn);
						}

						// Done!
						this.chatlogStack.clear();
					}
				}
			}
		}
	}

	/* Begin singleton */
	private static SupportCenter instance;

	public final static SupportCenter getInstance()
	{
		return instance;
	}

	public static void createInstance()
	{
		instance = new SupportCenter();
	}
	/* End singleton */

	public void issueUserAlert(int issuerId, int userId, String message, String string)
	{
		// TODO Auto-generated method stub
		
	}

}
