package org.openhabbo.util;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.openhabbo.access.UserRole;
import org.openhabbo.chatcommands.ChatCmdRegistry;
import org.openhabbo.chatcommands.ChatCommandHandler;
import org.openhabbo.core.Logger;
import org.openhabbo.sessions.Session;

public final class ChatUtil
{
	private final static int GESTURE_NONE = 0;
	private final static int GESTURE_SMILE = 1;
	private final static int GESTURE_ANGRY = 2;
	private final static int GESTURE_SURPRISED = 3;
	private final static int GESTURE_SAD = 4;

	public static int getChatGesture(String text)
	{
		// Find the separate words in the text
		text = text.toLowerCase();
		String[] words = text.toLowerCase().split(" ");

		// Check every word for emotes, first one counts
		for(String word : words)
		{
			if(word.equals(":)") || word.equals(":d") || word.equals("=]") || word.equals("=d") || word.equals(":>"))
			{
				return GESTURE_SMILE;
			}
			else if(word.equals(">:(") || word.equals(":@"))
			{
				return GESTURE_ANGRY;
			}
			else if(word.equals(":o") || word.equals("=o"))
			{
				return GESTURE_SURPRISED;
			}
			else if(word.equals(":(") || word.equals(";(") || word.equals("=[") || word.equals(":'("))
			{
				return GESTURE_SAD;
			}
		}
		return GESTURE_NONE;
	}

	public static Map<String, ChatCommandHandler> getAvailableChatCommands(UserRole role)
	{
		Map<String, ChatCommandHandler> available = new HashMap<String, ChatCommandHandler>();
		for(Entry<String, ChatCommandHandler> cmd : ChatCmdRegistry.get().getEntries())
		{
			String right = cmd.getValue().getRequiredRight();
			if(right == null || role.hasRight(right))
			{
				available.put(cmd.getKey(), cmd.getValue());
			}
		}
		return available;
	}

	public static boolean parseAndHandleCommand(Session session, String text)
	{
		// Actually a chat command?
		if(text.indexOf(ChatCommandHandler.COMMAND_HEAD) != 0)
		{
			// Doesn't start with the COMMAND_HEAD character
			return false;
		}

		// Attempt to separate command and body
		String[] segments = text.substring(1).split(" ", 2);

		// Handler registered?
		ChatCommandHandler handler = ChatCmdRegistry.get().lookup(segments[0]);
		if(handler == null)
		{
			// Doesn't exist...
			return false;
		}

		// User has the required right?
		String right = handler.getRequiredRight();
		if(right != null && !session.getUser().role.hasRight(right))
		{
			return false;
		}

		// Invoke handler and catch exceptions
		String body = (segments.length == 2 ? segments[1] : null);
		try
		{
			handler.handle(session, body);
		}
		catch(Exception ex)
		{
			Logger.getInstance().error("ChatUtil", String.format("Error processing chatcommand '%s'. (body=%s)", segments[0], body), ex);
		}

		// Handled
		return true;
	}
}
