package org.openhabbo.util;

import java.security.SecureRandom;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Random;

import org.openhabbo.rooms.instances.RoomInstance;

public final class GameUtil
{
	private final static Random random = new SecureRandom();
	private final static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("dd-MM-yyy");
	private final static SimpleDateFormat DATETIME_FORMATTER = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");

	public static <T> T getRandomElement(T[] elements)
	{
		return elements[random.nextInt(elements.length)];
	}

	public static int getRandomInt(int min, int max)
	{
		return(min + random.nextInt(max + 1));
	}

	public final static String formatDate(long millis)
	{
		return GameUtil.formatDate(new Date(millis));
	}

	public final static String formatDate(Date date)
	{
		return DATE_FORMAT.format(date);
	}

	public final static String getCurrentDate()
	{
		return GameUtil.formatDate(new Date());
	}

	public final static String formatDateTime(long millis)
	{
		return GameUtil.formatDateTime(new Date(millis));
	}

	public final static String getCurrentDateTime()
	{
		return GameUtil.formatDateTime(new Date());
	}

	public final static String formatDateTime(Date date)
	{
		return DATETIME_FORMATTER.format(date);
	}

	public final static int secondsToRoomPulses(final int seconds)
	{
		return((seconds * 1000) / RoomInstance.PULSE_INTERVAL);
	}

	public final static int secondsToDays(final int seconds)
	{
		return (int) Math.ceil(seconds / (60 * 60 * 24));
	}

	public final static int daysBetween(final long millis1, long millis2)
	{
		long diff = millis2 - millis1;
		return GameUtil.secondsToDays((int) (diff / 1000));
	}

	public final static int daysSince(final long millis)
	{
		return GameUtil.daysBetween(millis, System.currentTimeMillis());
	}

	public final static int parseInt(String customData)
	{
		try
		{
			return Integer.parseInt(customData);
		}
		catch(Exception ex)
		{

		}
		return 0;
	}
}
