package mod_gift;

import java.util.List;

import org.openhabbo.catalog.Catalog;
import org.openhabbo.catalog.CatalogProduct;
import org.openhabbo.core.Logger;
import org.openhabbo.items.Item;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;

public class OpenPresentHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to open presentboxes in this room?
		if(!session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Find the item
		Item item = session.getRoomInstance().getItem(request.readInt());
		if(item == null || !item.definition.sprite.startsWith("present_"))
		{
			return;
		}

		// Pickup + delete the item
		session.getRoomInstance().pickupAndDeleteItem(item.id);

		// Get + delete giftinfo for the item
		GiftInfo giftInfo = mod_gift.getGiftInfoForItem(item.id, true);
		if(giftInfo == null)
		{
			Logger.getInstance().warning(session, String.format("Opening presentbox #%d failed: gift info missing!", item.id));
			return;
		}

		// 'Present OK'
		response.init(219);
		response.appendInt(item.id);
		session.send(response);

		// Redeem contents
		CatalogProduct product = Catalog.getInstance().getProduct(giftInfo.productId);
		List<Item> contents = Catalog.getInstance().purchaseItem(product, session.getUser(), giftInfo.customData);
		if(product == null || contents == null)
		{
			session.sendNotif("Invalid presentbox contents, contents have possibly expired.");
			return;
		}
		
		// Present contents
		response.init(129);
		response.appendStr(Character.toLowerCase(product.getType()));
		response.appendInt(product.itemDefinition.spriteId);
		response.appendStr(product.name);
		session.send(response);
		
		// Add to inventory
		for(Item tmpItem : contents)
		{
			session.getItemInventory().addItem(tmpItem, false);
		}
		session.getItemInventory().refresh();
	}
}
