package mod_gift;

import org.openhabbo.catalog.Catalog;
import org.openhabbo.catalog.CatalogProduct;
import org.openhabbo.core.Logger;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.users.UserManager;
import org.openhabbo.util.SecurityUtil;

public class PurchaseGiftHandler implements SessionMessageHandler
{
	@Override
	@SuppressWarnings("unused")
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Parse purchase info
		int pageId = request.readInt();
		int productId = request.readInt();
		String customData = SecurityUtil.filterStr(request.readStr(), false); // Preserve linebreaks
		String giftUsername = SecurityUtil.filterStr(request.readStr());
		String giftNote = SecurityUtil.filterStr(request.readStr(), false); // Preserve linebreaks

		// Valid product?
		CatalogProduct product = Catalog.getInstance().getProduct(productId);
		if(product == null || !product.isPurchasableBy(session.getUser()) || !product.isPhysicalItem() || product.itemDefinition.is(mod_gift.NOT_GIFTABLE))
		{
			Logger.getInstance().warning(this, String.format("User #%d tried to gift-purchase product #%d from the Catalog, which is not existent/available/giftable by this user.", session.getUser().id, productId));
			return;
		}
		
		// Receiver exists?
		int receiverId = UserManager.getInstance().getUserId(giftUsername);
		if(receiverId == 0)
		{
			response.init(EServerMessage.PurchaseGiftError);
			response.appendInt(1); // "User does not exist"
			response.appendStr(giftUsername);
			session.send(response);
			return;
		}

		// Issue purchase
		if(!mod_gift.purchaseAsGift(product, customData, session.getUser(), receiverId, giftNote))
		{
			session.sendNotif(String.format("Purchase of catalog product #%d failed, you will not be billed. Please try again in a few minutes. If this keeps happening, please contact Support.", product.id));
			return;
		}
		
		// Bill the purchasing user for gifted product
		Catalog.getInstance().billUser(session, product);

		// Purchase okay!
		session.sendNotif("Gift purchased and sent!");
	}
}
