package mod_gift;

import org.openhabbo.OpenHABBO;
import org.openhabbo.catalog.CatalogProduct;
import org.openhabbo.core.Logger;
import org.openhabbo.items.Item;
import org.openhabbo.items.ItemDefinition;
import org.openhabbo.items.ItemManager;
import org.openhabbo.modules.Module;
import org.openhabbo.modules.ModuleInfo;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionManager;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;
import org.openhabbo.users.User;

@ModuleInfo(name = "mod_gift", description = "Enables users to purchase catalog products as gifts, wrapping them in gift presentbox item containing the bought items. Presentboxes can be placed in a room and opened to redeem the content.", author = "Nils 'Nillus'", url = "http://modules.openhabbo.org/mod_gift/", version = "1.0.0", dependencies = {})
public final class mod_gift extends Module
{
	public final static char NOT_GIFTABLE = 'O';

	public mod_gift()
	{
		this.getAutoLoader().msgHandler(78, new OpenPresentHandler());
		this.getAutoLoader().msgHandler(472, new PurchaseGiftHandler());
		this.getAutoLoader().msgHandler(3030, new CanGiftHandler());
	}

	public static boolean purchaseAsGift(CatalogProduct product, String customData, User user, int receiverId, String giftNote)
	{
		// Run pre-purchase logic, filter & apply customdata
		try
		{
			customData = product.itemDefinition.newInstance().generatePurchaseCustomData(user, customData);
		}
		catch(Exception ex)
		{
			customData = Item.ERROR_CUSTOMDATA;
		}
		if(customData != null && customData.equals(Item.ERROR_CUSTOMDATA))
		{
			Logger.getInstance().warning("Catalog", String.format("Gift-purchase of product #%d by user #%d failed, bad customdata.", product.id, user.id));
			return false;
		}

		// Attempt to get a presentbox definition
		ItemDefinition presentBoxDef = ItemManager.getInstance().getRandomPresentBoxDefinition();
		if(presentBoxDef == null)
		{
			return false;
		}

		// Apply giftnote
		Item item = presentBoxDef.newInstance();
		item.customData = "!" + giftNote;

		// Create giftinfo, add to shippings
		ItemManager.getInstance().storeItem(item);
		mod_gift.createGiftInfo(item.id, product.id, customData);

		// Is the receiver online?
		Session receiver = SessionManager.getInstance().getSessionByUserId(receiverId);
		if(receiver != null)
		{
			// Add the item to the inventory component
			receiver.getItemInventory().addItem(item, false);
			receiver.getItemInventory().refresh();

			// Send out a notification (TODO: localize this?)
			receiver.sendNotif("You have received a gift!");
		}

		// Purchase OK!
		return true;
	}

	public static void createGiftInfo(int itemId, int productId, String customData)
	{
		OpenHABBO.getInstance().getDb().execute("INSERT INTO item_giftinfo(item_id,product_id,customdata) VALUES (" + itemId + "," + productId + ",'" + SQLHelper.escape(customData) + "')");
	}
	
	public static void deleteGiftInfo(int itemId)
	{
		OpenHABBO.getInstance().getDb().execute("DELETE FROM item_giftinfo WHERE item_id=" + itemId);
	}

	public static GiftInfo getGiftInfoForItem(int itemId, boolean delete)
	{
		// Get gift info
		GiftInfo info; 
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM item_giftinfo WHERE item_id=" + itemId);
		if(result.row())
		{
			info = new GiftInfo();
			info.itemId = result.getInt("item_id");
			info.productId = result.getInt("product_id");
			info.customData = result.getString("customdata");
		}
		else
		{
			info = null;
		}
		result.release();
		
		// Delete info?
		if(info != null && delete)
		{
			mod_gift.deleteGiftInfo(itemId);
		}

		return info;
	}
}
