package mod_messenger;

import java.util.Collection;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;

public class InstantMessenger
{
	private Session session;
	private boolean appearOffline;

	private Map<Integer, MessengerBuddy> buddies;
	private Map<Integer, String> requests;

	public InstantMessenger(Session session)
	{
		this.session = session;
	}

	public void initialize()
	{
		this.buddies = InstantMessengerService.getInstance().getBuddyListForUser(this.session.getUser().id);
		this.requests = InstantMessengerService.getInstance().getBuddyRequestsForUser(this.session.getUser().id);
		InstantMessengerService.getInstance().getBuddy(this.session.getUser().id, true).messenger = this;

		// Update buddies
		for(MessengerBuddy buddy : this.buddies.values())
		{
			if(buddy.online())
			{
				buddy.messenger.getSession().sendNotif(this.session.getUser().username + " came online.");
			}
		}
	}

	public void offline()
	{
		// Update buddies
		for(MessengerBuddy buddy : this.buddies.values())
		{
			if(buddy.online())
			{
				buddy.messenger.getSession().sendNotif(this.session.getUser().username + " went offline.");
			}
		}

		this.buddies.clear();
		this.requests.clear();
		InstantMessengerService.getInstance().getBuddy(this.session.getUser().id, true).messenger = null;
	}

	public Session getSession()
	{
		return this.session;
	}

	public boolean appearOffline()
	{
		return this.appearOffline;
	}

	public void appearOffline(boolean status)
	{
		this.appearOffline = status;
	}

	public boolean hasBuddy(int userId)
	{
		return this.buddies.containsKey(userId);
	}

	public MessengerBuddy getBuddy(int userId)
	{
		return this.buddies.get(userId);
	}

	public void addBuddy(MessengerBuddy buddy)
	{
		this.buddies.put(buddy.userId, buddy);

		ServerMessage msg = new ServerMessage(EServerMessage.FriendListUpdate);
		msg.appendInt(0);
		msg.appendInt(1);
		msg.appendInt(0);
		msg.appendObject(buddy);
		msg.appendBool(false);
		this.session.send(msg);
	}

	public void removeBuddy(int userId)
	{
		this.buddies.remove(userId);

		ServerMessage msg = new ServerMessage(EServerMessage.FriendListUpdate);
		msg.appendInt(0);
		msg.appendInt(1);
		msg.appendInt(-1);
		msg.appendInt(userId);
		this.session.send(msg);
	}

	public Collection<MessengerBuddy> getBuddies()
	{
		return this.buddies.values();
	}

	public boolean hasRequest(int fromUserId)
	{
		return this.requests.containsKey(fromUserId);
	}

	public void addRequest(int fromUserId, String fromUsername)
	{
		this.requests.put(fromUserId, fromUsername);

		ServerMessage msg = new ServerMessage(EMessengerMessage.NewBuddyRequest);
		msg.appendInt(fromUserId);
		msg.appendStr(fromUsername);
		msg.appendStr(Integer.toString(fromUserId));
		this.session.send(msg);
	}

	public void removeRequest(int fromUserId)
	{
		this.requests.remove(fromUserId);
	}

	public Set<Entry<Integer, String>> getRequests()
	{
		return this.requests.entrySet();
	}

	public void deliverInstantMessage(int fromUserId, String text)
	{
		ServerMessage msg = new ServerMessage(EMessengerMessage.NewInstantMessage);
		msg.appendInt(fromUserId);
		msg.appendStr(text);
		this.session.send(msg);
	}
}
