package mod_messenger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.messages.ServerMessage;

public final class InstantMessenger
{
	private final Session session;
	private boolean appearOffline;

	private Map<Integer, MessengerBuddy> buddies;
	private Map<Integer, String> requests;
	private final List<MessengerBuddy> buddiesToUpdate;
	
	public InstantMessenger(Session session)
	{
		this.session = session;
		this.buddiesToUpdate = new ArrayList<MessengerBuddy>();
	}

	public void initialize()
	{
		// Load buddies + requests
		this.buddiesToUpdate.clear();
		this.buddies = InstantMessengerService.getInstance().getBuddyListForUser(this.session.getUser().id);
		this.requests = InstantMessengerService.getInstance().getBuddyRequestsForUser(this.session.getUser().id);
		
		// Update own buddy info
		MessengerBuddy thisUser = this.getOwnBuddyInfo();
		thisUser.messenger = this;
		
		// Broadcast instant update to all buddies ('online!')
		this.updateStatus(true);
	}

	public void offline()
	{
		// Update own buddy info
		MessengerBuddy thisUser = this.getOwnBuddyInfo();
		thisUser.messenger = null;
		
		// Broadcast instant update to all buddies ('offline!')
		this.updateStatus(true);

		// Clear lists
		this.buddies.clear();
		this.requests.clear();
	}
	
	public void addBuddy(MessengerBuddy buddy)
	{
		// Update through the update system
		this.setUpdateNeeded(buddy);
		this.sendUpdates();
	}

	public void removeBuddy(int userId)
	{
		// Mutate list
		this.buddies.remove(userId);

		// Update manually
		ServerMessage msg = new ServerMessage(EMessengerMessage.FriendListUpdate);
		msg.appendInt(0);
		msg.appendInt(1);
		msg.appendInt(-1);
		msg.appendInt(userId);
		this.session.send(msg);
	}
	
	public void addRequest(int fromUserId, String fromUsername)
	{
		// Mutate list
		this.requests.put(fromUserId, fromUsername);

		// Notify session
		ServerMessage msg = new ServerMessage(EMessengerMessage.NewBuddyRequest);
		msg.appendInt(fromUserId);
		msg.appendStr(fromUsername);
		msg.appendStr(Integer.toString(fromUserId));
		this.session.send(msg);
	}

	public void removeRequest(int fromUserId)
	{
		this.requests.remove(fromUserId);
	}

	public void deliverInstantMessage(int fromUserId, String text)
	{
		ServerMessage msg = new ServerMessage(EMessengerMessage.NewInstantMessage);
		msg.appendInt(fromUserId);
		msg.appendStr(text);
		this.session.send(msg);
	}
	
	public void updateStatus(boolean instantUpdate)
	{
		MessengerBuddy thisBuddy = this.getOwnBuddyInfo();
		for(MessengerBuddy buddy : this.buddies.values())
		{
			if(buddy.online())
			{
				buddy.messenger.setUpdateNeeded(thisBuddy);
				if(instantUpdate)
				{
					buddy.messenger.sendUpdates();
				}
			}
		}
	}
	
	private void setUpdateNeeded(MessengerBuddy buddy)
	{
		if(buddy != null)
		{
			// Not in buddylist yet?
			if(!buddies.containsKey(buddy.userId))
			{
				this.buddies.put(buddy.userId, buddy);
			}
			
			// This list will be processed next time the client asks for it, or after a forced update
			this.buddiesToUpdate.add(buddy);
		}
	}
	
	public void sendUpdates()
	{
		// Send updates
		ServerMessage msg = new ServerMessage(EMessengerMessage.FriendListUpdate);
		msg.appendInt(0);
		msg.appendInt(this.buddiesToUpdate.size());
		msg.appendInt(0);
		for(MessengerBuddy buddy : this.buddiesToUpdate)
		{
			msg.appendObject(buddy);
			msg.appendBool(false);
		}
		this.session.send(msg);
		
		// Updated!
		this.buddiesToUpdate.clear();
	}

	public Session getSession()
	{
		return this.session;
	}
	
	public MessengerBuddy getOwnBuddyInfo()
	{
		return InstantMessengerService.getInstance().getBuddy(this.session.getUser().id, true);
	}
	
	public boolean appearOffline()
	{
		return this.appearOffline;
	}

	public void appearOffline(boolean status)
	{
		this.appearOffline = status;
	}

	public boolean hasBuddy(int userId)
	{
		return this.buddies.containsKey(userId);
	}

	public MessengerBuddy getBuddy(int userId)
	{
		return this.buddies.get(userId);
	}

	public Collection<MessengerBuddy> getBuddies()
	{
		return this.buddies.values();
	}

	public boolean hasRequest(int fromUserId)
	{
		return this.requests.containsKey(fromUserId);
	}
	
	public Set<Entry<Integer, String>> getRequests()
	{
		return this.requests.entrySet();
	}
}
