package mod_moderation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.openhabbo.OpenHABBO;
import org.openhabbo.core.Logger;
import org.openhabbo.storage.DatabaseResult;

public class ModerationPresetManager
{
	private List<String> userMessages;
	private List<String> roomMessages;
	
	public ModerationPresetManager()
	{
		this.userMessages = Collections.emptyList();
		this.roomMessages = Collections.emptyList();
	}
	
	public void reload()
	{
		List<String> tmpUserMessages = new ArrayList<String>();
		List<String> tmpRoomMessages = new ArrayList<String>();
		
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT type,text FROM moderation_presets WHERE enabled=1");
		while(result.row())
		{
			String type = result.getString("type");
			String text = result.getString("text");
			if(type.equals("user"))
			{
				tmpUserMessages.add(text);
			}
			else if(type.equals("room"))
			{
				tmpRoomMessages.add(text);
			}
		}
		result.release();
		
		this.userMessages = Collections.unmodifiableList(tmpUserMessages);
		this.roomMessages = Collections.unmodifiableList(tmpRoomMessages);
		
		Logger.getInstance().info("ModerationPresetManager", String.format("Loaded %d user presets and %d room presets.", tmpUserMessages.size(), tmpRoomMessages.size()));
	}
	
	public List<String> getUserMessages()
	{
		return this.userMessages;
	}
	
	public List<String> getRoomMessages()
	{
		return this.roomMessages;
	}
}
