package mod_moderation.msgs;

import mod_roomevent.RoomEvent;

import org.openhabbo.rooms.Room;
import org.openhabbo.rooms.RoomManager;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;

public class ModGetRoomToolHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Has mod?
		if(!session.getUser().rights.contains("fuse_mod"))
		{
			return;
		}

		// Get room (instance is not required, info is)
		int roomId = request.readInt();
		RoomInstance instance = RoomManager.getInstance().getRoomInstance(roomId, false);
		Room room = (instance != null) ? instance.getInfo() : RoomManager.getInstance().getRoomInfo(roomId);
		if(room == null)
		{
			session.sendNotif("Room does not exist.");
			return;
		}

		// Get event
		RoomEvent event;
		if(instance != null)
		{
			event = instance.get("event");
		}
		else
		{
			event = null;
		}

		// Send room data
		response.init(538);
		response.appendInt(room.id);
		response.appendInt(room.usersNow);
		response.appendBool((instance != null) && (instance.getUserByUserId(room.ownerId) != null)); // Owner inside?
		response.appendInt(room.ownerId);
		response.appendStr(room.ownerName);
		response.appendInt(room.id);
		response.appendStr(room.name);
		response.appendStr(room.description);
		response.appendStrList(room.tags);
		response.appendBool(event != null);
		if(event != null)
		{
			response.appendStr(event.name);
			response.appendStr(event.description);
			response.appendStrList(event.tags);
		}
		session.send(response);
	}
}
