package mod_moderation.msgs;

import mod_moderation.ModerationCenter;
import mod_moderation.UserModerationHistory;

import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionManager;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.users.User;
import org.openhabbo.users.UserManager;

public class ModGetUserInfoHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Has mod?
		if(!session.getUser().rights.contains("fuse_mod"))
		{
			return;
		}
		
		// Get user info + moderation history + other info
		User user = UserManager.getInstance().getUserById(request.readInt(), true);
		if(user == null)
		{
			return;
		}
		UserModerationHistory history = ModerationCenter.getInstance().getUserModerationHistory(user.id);
		if(history == null)
		{
			history = new UserModerationHistory();
		}
		boolean onlineNow = (SessionManager.getInstance().getSessionByUserId(user.id) != null);
		long nowTime = System.currentTimeMillis();
		
		// Send the user info
		response.init(533);
		response.appendInt(user.id);
		response.appendStr(user.username);
		response.appendInt((int) Math.ceil(((nowTime - user.registeredTime) / 1000) / 60)); // Minutes since registration
		response.appendInt((int) Math.ceil(((nowTime - user.lastOnlineTime) / 1000) / 60)); // Minutes since last login time
		response.appendBool(onlineNow);
		response.appendInt(history.cfhs);
		response.appendInt(history.abusiveCfhs);
		response.appendInt(history.cautions);
		response.appendInt(history.bans);
		session.send(response);
	}
}
