package mod_onewaydoor;

import org.openhabbo.items.InteractableItem;
import org.openhabbo.rooms.instances.RoomTile;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.Session;

public final class OneWayDoorItem extends InteractableItem
{
	@Override
	public void onTrigger(Session session, int newState)
	{
		// Attempt to move infront of the gate
		RoomUser user = session.getRoomUser();
		RoomTile frontTile = this.getTileInFront();
		if(!user.at(frontTile) && user.canMove())
		{
			user.requestNewTrip(frontTile.x, frontTile.y);
			return;
		}

		// Is the other end of the gate blocked, not by an interactive item?
		if(this.room.getMap().walkable(this.getTileBehind(), true))
		{
			// Don't bother entering the gate
			return;
		}

		// Not in use yet?
		if(!this.inUse())
		{
			// Tadaa
			this.setUser(user);

			// Lock own movement, move into the gate
			user.lockMovement();
			user.overrideNextTile = true;
			user.requestNewTrip(this.x, this.y);

			// Wait...
			this.setTimeout(1);
		}
	}

	@Override
	protected void onTimeout()
	{
		// In use?
		if(this.inUse())
		{
			// User in the gate?
			RoomUser user = this.getUser();
			if(user.at(this.getTile()))
			{
				// Move to tile behind
				user.requestNewTrip(this.getTileBehind());

				// Set a new timeout
				this.setTimeout(1);
			}

			// User exited the gate?
			else if(user.at(this.getTileBehind()))
			{
				user.unlockMovement();
				this.clearUser();
			}
		}
	}
}
