package mod_roomdimmer;

import java.util.List;

import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.sessions.Session;
import org.openhabbo.util.GameUtil;

public final class RoomDimmerItem extends Item
{
	private final static int PRESET_AMOUNT = 3;

	public boolean enabled;
	public int currentPreset;
	private RoomDimmerPreset[] presets;

	@Override
	public void onPurchase(List<Item> shipping)
	{
		this.reset();
		super.onPurchase(shipping);
	}

	@Override
	protected void onParse()
	{
		this.reloadData();
	}

	@Override
	protected void onRoomAdd(RoomInstance room)
	{
		room.set("dimmer", this);
	}

	@Override
	protected void onRoomRemove(RoomInstance room)
	{
		room.unset("dimmer");
	}

	@Override
	public boolean onPlace(Session session, RoomInstance room)
	{
		if(room.has("dimmer"))
		{
			session.sendNotif("You can only place one roomdimmer in a room!");
			return false;
		}
		else
		{
			return true;
		}
	}

	public void saveSettings()
	{
		this.data = this.generateData();
		this.setState(this.generateState());
	}

	public void reset()
	{
		this.enabled = false;
		this.currentPreset = 1;
		this.presets = new RoomDimmerPreset[RoomDimmerItem.PRESET_AMOUNT];
		for(int x = 0; x < this.presets.length; x++)
		{
			RoomDimmerPreset preset = new RoomDimmerPreset();
			preset.id = (x + 1);
			preset.color = RoomDimmerPreset.DEFAULT_COLOR;
			preset.intensity = 255;
			preset.backgroundOnly = false;
			this.presets[x] = preset;
		}
		this.saveSettings();
	}

	public RoomDimmerPreset getPreset(int presetId)
	{
		for(RoomDimmerPreset preset : this.presets)
		{
			if(preset.id == presetId)
			{
				return preset;
			}
		}
		return null;
	}

	public RoomDimmerPreset[] getPresets()
	{
		return this.presets;
	}

	private String generateState()
	{
		StringBuilder sb = new StringBuilder();
		RoomDimmerPreset preset = this.getPreset(this.currentPreset);
		sb.append(this.enabled ? '2' : '1');
		sb.append(',');
		sb.append(this.currentPreset);
		sb.append(',');
		sb.append(preset.backgroundOnly ? '2' : '1');
		sb.append(',');
		sb.append(preset.color);
		sb.append(',');
		sb.append(preset.intensity);
		return sb.toString();
	}

	private String generateData()
	{
		StringBuilder sb = new StringBuilder();
		sb.append(this.enabled ? '1' : '0');
		sb.append(GameUtil.DATA_DELIMITER);
		sb.append(this.currentPreset);
		sb.append(GameUtil.DATA_DELIMITER);
		sb.append(this.presets.length);
		for(RoomDimmerPreset preset : this.presets)
		{
			sb.append(GameUtil.DATA_DELIMITER);
			sb.append(preset.id);
			sb.append(GameUtil.DATA_DELIMITER);
			sb.append(preset.color);
			sb.append(GameUtil.DATA_DELIMITER);
			sb.append(preset.intensity);
			sb.append(GameUtil.DATA_DELIMITER);
			sb.append(preset.backgroundOnly ? '1' : '0');
		}
		return sb.toString();
	}

	private void reloadData()
	{
		String[] bits = this.data.split(String.valueOf(GameUtil.DATA_DELIMITER));
		int n = 0;

		// Construct presets and settings from data
		try
		{
			this.enabled = (bits[n++].equals("1"));
			this.currentPreset = Integer.parseInt(bits[n++]);
			this.presets = new RoomDimmerPreset[Integer.parseInt(bits[n++])];
			for(int x = 0; x < this.presets.length; x++)
			{
				RoomDimmerPreset preset = new RoomDimmerPreset();
				preset.id = Integer.parseInt(bits[n++]);
				preset.color = bits[n++];
				preset.intensity = Integer.parseInt(bits[n++]);
				preset.backgroundOnly = (bits[n++].equals("1"));
				this.presets[x] = preset;
			}

			// Generate the state from the new presets
			this.state = this.generateState();
		}
		catch(Exception ex)
		{
			// M'kay this is bad
			this.reset();
		}
	}
}
