package mod_roomdimmer;

import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;

public class UpdateRoomDimmerHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Only room owners can deal with the room dimmer
		RoomDimmerItem dimmer = session.getRoomInstance().get("dimmer");
		if(dimmer == null || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}
		
		// Read settings for preset X
		int presetId = request.readInt();
		boolean backgroundOnly = (request.readInt() == 2);
		String color = request.readStr();
		int intensity = request.readInt();
		
		// Validate settings
		RoomDimmerPreset preset = dimmer.getPreset(presetId);
		if(preset == null || !RoomDimmerPreset.isValidColor(color) || !RoomDimmerPreset.isValidIntensity(intensity))
		{
			return;
		}
		
		// Apply & save settings
		dimmer.enabled = true;
		dimmer.currentPreset = presetId;
		preset.backgroundOnly = backgroundOnly;
		preset.color = color;
		preset.intensity = intensity;
		dimmer.saveSettings();
	}
}
