package mod_roomevent;

import org.openhabbo.rooms.Room;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.messages.SerializableObject;
import org.openhabbo.sessions.messages.ServerMessage;

public final class RoomEvent implements SerializableObject
{
	// Error codes
	public final static int ERROR_NONE = 0;
	public final static int ERROR_NOT_IN_ROOM = 1;
	public final static int ERROR_NOT_ROOM_OWNER = 2;
	public final static int ERROR_INCOMPATIBLE_ROOMSTATE = 3;
	public final static int ERROR_EVENTS_DISABLED = 4;
	public final static int ERROR_ALREADY_EVENT_HERE = 5;
	public final static int ERROR_ALREADY_EVENT_USER = 6;

	// Event info
	public String name;
	public String description;
	public int categoryId;
	public String[] tags;

	// Starter
	public Room room;
	public String startTime;

	@Override
	public void serialize(ServerMessage msg)
	{
		msg.appendStr(this.room.ownerId);
		msg.appendStr(this.room.name);
		msg.appendStr(this.room.id);
		msg.appendInt(this.categoryId);
		msg.appendStr(this.name);
		msg.appendStr(this.description);
		msg.appendStr(this.startTime);
		msg.appendStrList(this.tags);
	}

	public static int canCreateRoomEvent(Session session)
	{
		if(!session.inRoom())
		{
			return ERROR_NOT_IN_ROOM;
		}
		else if(session.getUser().id != session.getRoomInstance().getInfo().ownerId)
		{
			return ERROR_NOT_ROOM_OWNER;
		}
		else if(session.getRoomInstance().getInfo().state != Room.STATE_OPEN)
		{
			return ERROR_INCOMPATIBLE_ROOMSTATE;
		}
		else if(session.getRoomInstance().has("event"))
		{
			return ERROR_ALREADY_EVENT_HERE;
		}
		else if(session.has("event"))
		{
			return ERROR_ALREADY_EVENT_USER;
		}
		else
		{
			return ERROR_NONE;
		}
	}
	
	public static ServerMessage createMsg(RoomEvent event)
	{
		ServerMessage msg = new ServerMessage(EServerMessage.RoomEvent);
		if(event == null)
		{
			msg.appendStr("-1");
		}
		else
		{
			msg.appendStr(event.room.ownerId);
			msg.appendStr(event.room.name);
			msg.appendStr(event.room.id);
			msg.appendInt(event.categoryId);
			msg.appendStr(event.name);
			msg.appendStr(event.description);
			msg.appendStr(event.startTime);
			msg.appendStrList(event.tags);
		}
		return msg;
	}
}
