package mod_trading;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.items.EItemBehavior;
import org.openhabbo.items.Item;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.messages.SerializableObject;
import org.openhabbo.sessions.messages.ServerMessage;

public final class TradePartner implements SerializableObject
{
	private Trade trade;
	private Session session;
	private boolean accepted;
	private boolean confirmed;
	private List<Item> offer;

	public TradePartner(Trade trade, Session session)
	{
		this.trade = trade;
		this.session = session;
		this.accepted = false;
		this.confirmed = false;
		this.offer = new ArrayList<Item>(3);

		this.session.set("trade", this.trade);
		this.session.getRoomUser().addStatus("trd");
	}

	public void clear()
	{
		this.offer.clear();

		this.session.unset("trade");
		this.session.getRoomUser().removeStatus("trd");

		this.session.getItemInventory().refresh();
	}

	public void accept(boolean accept)
	{
		// Update accept/confirm status
		this.accepted = accept;
		if(!accept)
		{
			this.confirmed = false;
		}
		
		// Notify users
		ServerMessage msg = new ServerMessage(ETradeMessage.TradingAccept);
		msg.appendInt(this.getSession().getUser().id);
		msg.appendBool(accept);
		this.trade.send(msg);

		// Move into confirm state?
		if(this.trade.bothAccepted())
		{
			msg.init(ETradeMessage.TradingConfirmation);
			this.trade.send(msg);
		}
	}

	public void confirm()
	{
		// this.accepted = false;
		this.confirmed = true;
		this.trade.complete();
	}

	public void addItem(Item item)
	{
		if(this.offer.contains(item))
		{
			return;
		}

		if(!item.is(EItemBehavior.ALLOW_TRADE))
		{
			return;
		}

		// Item added to trade!
		this.offer.add(item);
		this.trade.update();
	}

	public void removeItem(Item item)
	{
		// Item removed from trade!
		this.offer.remove(item);
		this.trade.update();
	}

	public boolean verifyItems()
	{
		// Verify the items are still here
		for(Item item : this.offer)
		{
			if(!this.session.getItemInventory().containsItem(item.id))
			{
				return false;
			}
		}
		return true;
	}

	public void transferItems(TradePartner to)
	{
		for(Item item : this.offer)
		{
			this.session.getItemInventory().removeItem(item.id);
			to.getSession().getItemInventory().addItem(item, true);
		}
	}

	public boolean accepted()
	{
		return this.accepted;
	}
	
	public boolean confirmed()
	{
		return this.confirmed;
	}

	public Trade getTrade()
	{
		return this.trade;
	}

	public Session getSession()
	{
		return this.session;
	}

	public List<Item> getOffer()
	{
		return this.offer;
	}

	@Override
	public void serialize(ServerMessage msg)
	{
		msg.appendInt(this.getSession().getUser().id);
		msg.appendInt(this.offer.size());
		for(Item item : this.offer)
		{
			msg.appendInt(item.id);
			msg.appendStr(item.definition.getType());
			msg.appendInt(item.id);
			msg.appendInt(item.definition.spriteId);
			msg.appendBool(true);
			msg.appendBool(true);
			msg.appendStr("");
			msg.appendBool(false); // xmas data
			msg.appendBool(false); // xmas data
			msg.appendBool(false); // xmas data
			if(item.is(EItemBehavior.FLOORITEM))
			{
				msg.appendInt(-1); // ??
			}
		}
	}
}