package mod_voucher;

import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.users.UserManager;
import org.openhabbo.util.GameUtil;

public class RedeemVoucherHandler implements SessionMessageHandler
{
	public final static int VoucherRedeemOk = 212;
	public final static int VoucherRedeemError = 213;
	
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Voucher exists?
		ContentVoucher voucher = mod_voucher.getVoucher(request.readStr());
		if(voucher == null)
		{
			response.init(RedeemVoucherHandler.VoucherRedeemError);
			response.appendInt(1);
		}
		else
		{
			// Delete voucher
			mod_voucher.deleteVoucher(voucher.code);
			
			// Process payload
			if(voucher.type.equals("credits"))
			{
				// Update credits
				int credits = GameUtil.parseInt(voucher.data);
				session.getUser().credits += credits;
				session.getUser().saveCreditsAndPixels();
				session.sendCreditBalance();
				
				// Log transaction
				UserManager.getInstance().logCreditTransaction(session.getUser().id, credits, "Credit voucher (" + voucher.code + ")");
			}
			
			// Okay!
			response.init(RedeemVoucherHandler.VoucherRedeemOk);
		}
		
		// Send result
		session.send(response);
	}
}
