package mod_corechatcmds;

import org.openhabbo.OpenHABBO;
import org.openhabbo.chatcommands.ChatCommandHandler;
import org.openhabbo.sessions.Session;

public class StatsCommandHandler implements ChatCommandHandler
{
	@Override
	public String getDescription()
	{
		return "Shows some server stats";
	}

	@Override
	public String getRequiredRight()
	{
		return null;
	}

	@Override
	public boolean handle(Session session, String body)
	{
		// Gather the required information
		Runtime runtime = Runtime.getRuntime();
		OpenHABBO instance = OpenHABBO.getInstance();

		// Calculate uptime
		long uptime = (System.currentTimeMillis() - instance.getStartTime());
		long days = (uptime / (1000 * 60 * 60 * 24));
		long hours = (uptime - days * (1000 * 60 * 60 * 24)) / (1000 * 60 * 60);
		long minutes = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60)) / (1000 * 60);
		long seconds = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60) - minutes * (1000 * 60)) / (1000);

		// Build notification
		StringBuilder notif = new StringBuilder();
		
		// Uptime
		notif.append(instance.versionString());
		notif.append("\r");
		notif.append(String.format("Server uptime is %d day(s), %d hour(s), %d minute(s) and %d second(s).\r", days, hours, minutes, seconds));
		notif.append(String.format("There are %d user(s) online, and your session ID is #%d.\r", instance.getSessionMgr().authedSessions(), session.id));
		notif.append(String.format("There are %d room instance(s) loaded.\r\r", instance.getRoomMgr().instances()));

		// Environment
		notif.append(String.format("OS: %s\r", System.getProperty("os.name")));
		notif.append(String.format("CPU cores: %d\r", runtime.availableProcessors()));
		notif.append(String.format("JVM memory usage: %d KB\r\r", (runtime.totalMemory() / 1024)));
		
		// Footer
		notif.append("Type :about for more server information.");
		
		session.sendMotd(notif.toString());

		return true;
	}
}
