package org.openhabbo.chatcommands.std;

import org.openhabbo.OpenHABBO;
import org.openhabbo.chatcommands.ChatCommandHandler;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;

public class StatsCommandHandler implements ChatCommandHandler
{
	@Override
	public String getCommand()
	{
		return "stats";
	}

	@Override
	public String getDescription()
	{
		return "Shows some server stats";
	}

	@Override
	public String getRequiredRight()
	{
		return null;
	}

	@Override
	public boolean handle(Session session, String body)
	{
		// Gather the required information
		Runtime runtime = Runtime.getRuntime();
		OpenHABBO instance = OpenHABBO.getInstance();

		// Calculate uptime
		long uptime = (System.currentTimeMillis() - instance.getStartTime());
		long days = (uptime / (1000 * 60 * 60 * 24));
		long hours = (uptime - days * (1000 * 60 * 60 * 24)) / (1000 * 60 * 60);
		long minutes = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60)) / (1000 * 60);
		long seconds = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60) - minutes * (1000 * 60)) / (1000);

		ServerMessage msg = new ServerMessage(EServerMessage.Notif);
		msg.appendStrLine("SERVER");
		msg.appendStrLine(instance.versionString());
		msg.appendStrLine(String.format("Server uptime is %d day(s), %d hour(s), %d minute(s) and %d second(s).", days, hours, minutes, seconds));
		msg.appendStrLine(String.format("There are %d user(s) online, and your session ID is #%d.", instance.getSessionMgr().authedSessions(), session.id));
		msg.appendStrLine(String.format("There are %d room instance(s) loaded.", instance.getRoomMgr().instances()));
		msg.appendStrLine("");
		msg.appendStrLine("SYSTEM");
		msg.appendStrLine("CPU cores: " + runtime.availableProcessors());
		msg.appendStrLine("JVM memory usage: " + (runtime.totalMemory() / 1024) + " KB");
		session.send(msg);

		return true;
	}
}
