package org.openhabbo.events;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;

public final class Event
{
	private final int code;
	private List<EventListener> listeners;

	public Event(int code)
	{
		this.code = code;
		this.listeners = new ArrayList<EventListener>();
	}

	public void addListener(EventListener listener)
	{
		this.listeners.add(listener);
	}

	public void removeListener(EventListener listener)
	{
		this.listeners.remove(listener);
	}

	public final void fire()
	{
		this.fire(null);
	}

	public final void fire(EventArgs args)
	{
		this.fire(args);
	}

	public final void fire(Object args)
	{
		for(EventListener listener : this.listeners)
		{
			try
			{
				listener.invoke(args);
			}
			catch(Exception ex)
			{
				OpenHABBO.getInstance().getLogger().error("Event #" + this.code, "Error executing a listener!", ex);
			}
		}
	}

	public final int getCode()
	{
		return this.code;
	}

	public final boolean hasListeners()
	{
		return(this.listeners.size() > 0);
	}
}
