package org.openhabbo.events;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;

public final class Event
{
	private final int m_code;
	private List<EventListener> m_listeners;

	public Event(int code)
	{
		m_code = code;
		m_listeners = new ArrayList<EventListener>();
	}

	public void addListener(EventListener listener)
	{
		m_listeners.add(listener);
	}

	public void removeListener(EventListener listener)
	{
		m_listeners.remove(listener);
	}

	public final void fire()
	{
		this.fire(null);
	}

	public final void fire(EventArgs args)
	{
		this.fire(args);
	}

	public final void fire(Object args)
	{
		for(EventListener listener : m_listeners)
		{
			try
			{
				listener.invoke(args);
			}
			catch(Exception ex)
			{
				OpenHABBO.getInstance().getLogger().error("Event #" + m_code, "Error executing a listener!", ex);
			}
		}
	}

	public final int getCode()
	{
		return m_code;
	}

	public final boolean hasListeners()
	{
		return(m_listeners.size() > 0);
	}
}
