package org.openhabbo.items;

import org.openhabbo.OpenHABBO;
import org.openhabbo.items.impl.GenericItem;
import org.openhabbo.storage.DatabaseResult;

public final class ItemDefinition
{
	public int id;
	public String sprite;
	public String color;
	public byte length;
	public byte width;
	public float height;
	private char[] behaviourFlags;
	private Class<Item> implClass;
	public String name;
	public String description;

	private ItemDefinition()
	{

	}

	public boolean is(char flag)
	{
		for(int i = 0; i < this.behaviourFlags.length; i++)
		{
			if(this.behaviourFlags[i] == flag)
			{
				return true;
			}
		}
		return false;
	}

	public static ItemDefinition parse(DatabaseResult result)
	{
		ItemDefinition def = new ItemDefinition();
		def.id = result.getInt("id");
		def.sprite = result.getString("sprite");
		def.color = result.getString("color");
		def.length = result.getByte("length");
		def.width = result.getByte("width");
		def.height = result.getFloat("height");
		def.behaviourFlags = result.getString("behaviour").toCharArray();
		def.name = result.getString("name");
		def.description = result.getString("description");
		def.loadImplClass(result.getString("class"));
		return def;
	}

	@SuppressWarnings("unchecked")
	private void loadImplClass(String name)
	{
		this.implClass = null;
		if(name != null && name.length() > 0)
		{
			String className = "org.openhabbo.items.impl." + name + "Item";
			try
			{
				this.implClass = (Class<Item>) Class.forName(className);
			}
			catch(Exception e)
			{
				OpenHABBO.getInstance().getLogger().warning("ItemDefinition", String.format("Implementation '%s' is invalid, class '%s' does not exist, or is not a valid subclass of org.openhabbo.items.Item. Falling back to GenericItem.", name, className));
			}
		}
	}

	public Item newInstance()
	{
		// Attempt to create an instance of the impl class
		Item item = null;
		if(this.implClass != null)
		{
			try
			{
				item = this.implClass.newInstance();
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
				item = new GenericItem();
			}
		}

		// No implementation constructed? Fall back to GenericItem
		if(item == null)
		{
			item = new GenericItem();
		}

		// Wire it together with the definition and return!
		item.definition = this;
		return item;
	}
}
