package org.openhabbo.items;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import org.openhabbo.OpenHABBO;
import org.openhabbo.items.pets.Pet;
import org.openhabbo.storage.DatabaseResult;

public class ItemInventory
{
	private int m_userId;
	private LinkedHashMap<Integer, Item> m_items;
	private Map<Integer, Pet> m_pets;

	public ItemInventory(int userId)
	{
		m_userId = userId;
		m_items = new LinkedHashMap<Integer, Item>();
		m_pets = new HashMap<Integer, Pet>();
	}

	public void clear()
	{
		m_items.clear();
		m_pets.clear();
	}

	public void reload()
	{
		// Clear old items and pets
		this.clear();

		// Reload items
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM items WHERE owner_id=" + m_userId + " AND room_id=0");
		while(result.row())
		{
			Item item = Item.parse(result);
			if(item != null)
			{
				m_items.put(item.id, item);
			}
		}
		result.release();

		// TODO: Reload pets
	}

	public Item getItem(int itemId)
	{
		return m_items.get(itemId);
	}

	public void addItem(Item item, boolean save)
	{
		item.ownerId = m_userId;
		m_items.put(item.id, item);

		if(save)
		{
			item.update();
		}
	}

	public void removeItem(int itemId)
	{
		m_items.remove(itemId);
	}

	public Collection<Item> getItems()
	{
		return m_items.values();
	}

	public Collection<Pet> getPets()
	{
		return m_pets.values();
	}
}
