package org.openhabbo.items;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.openhabbo.OpenHABBO;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;

public final class ItemManager
{
	private Map<Integer, ItemDefinition> definitions;

	public ItemManager()
	{
		this.definitions = new HashMap<Integer, ItemDefinition>();
	}

	public void reloadDefinitions()
	{
		synchronized(this.definitions)
		{
			this.definitions.clear();

			DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM itemdefinitions");
			while(result.row())
			{
				ItemDefinition definition = ItemDefinition.parse(result);
				this.definitions.put(definition.id, definition);
			}
			result.release();

			OpenHABBO.getInstance().getLogger().info("ItemMgr", String.format("Reloaded %d item definitions.", this.definitions()));
		}
	}

	public ItemDefinition getDefinition(int definitionId)
	{
		return this.definitions.get(definitionId);
	}

	public ItemDefinition getRandomPresentBoxDefinition()
	{
		ItemDefinition definition = this.getDefinition(188);

		// This might help!
		if(definition == null)
		{
			OpenHABBO.getInstance().getLogger().warning("ItemMgr", "No presentbox definition available! Gifts cannot be purchased.");
		}

		return definition;
	}

	public int definitions()
	{
		return this.definitions.size();
	}

	public boolean storeItem(Item item)
	{
		item.id = OpenHABBO.getInstance().getDb().executeReturnKey("INSERT INTO items(definition_id,owner_id,customdata) VALUES (" + item.definition.id + "," + item.ownerId + ",'" + SQLHelper.escape(item.customData) + "')");
		return(item.id > 0);
	}

	public Item getItem(int itemId)
	{
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM items WHERE id=" + itemId);
		Item item = result.row() ? Item.parse(result) : null;
		result.release();

		return item;
	}

	public void createGiftInfo(int itemId, int productId, String customData)
	{
		OpenHABBO.getInstance().getDb().execute("INSERT INTO item_giftinfo(item_id,product_id,customdata) VALUES (" + itemId + "," + productId + ",'" + SQLHelper.escape(customData) + "')");
	}

	public GiftInfo getGiftInfoForItem(int itemId)
	{
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM item_giftinfo WHERE item_id=" + itemId);
		GiftInfo info = result.row() ? GiftInfo.parse(result) : null;
		result.release();

		return info;
	}

	public List<Item> getItemsForRoom(int roomId)
	{
		List<Item> items = new ArrayList<Item>();

		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM items WHERE room_id=" + roomId);
		while(result.row())
		{
			Item item = Item.parse(result);
			if(item != null)
			{
				items.add(item);
			}
		}
		result.release();

		return items;
	}
}
