package org.openhabbo.messenger;

import java.util.Collection;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.openhabbo.OpenHABBO;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;

public class InstantMessenger
{
	private Session m_session;
	private boolean m_appearOffline;

	private boolean m_initialized;
	private Map<Integer, MessengerBuddy> m_buddies;
	private Map<Integer, String> m_requests;

	public InstantMessenger(Session session)
	{
		m_session = session;
		m_initialized = false;
	}

	public void initialize()
	{
		InstantMessengerService svc = OpenHABBO.getInstance().getImSvc();
		m_buddies = svc.getBuddyListForUser(m_session.getUser().id);
		m_requests = svc.getBuddyRequestsForUser(m_session.getUser().id);
		svc.getBuddy(m_session.getUser().id, true).messenger = this;

		m_initialized = true;

		// Update buddies
		for(MessengerBuddy buddy : m_buddies.values())
		{
			if(buddy.online())
			{
				buddy.messenger.getSession().sendNotif(m_session.getUser().username + " came online.");
			}
		}
	}

	public void offline()
	{
		// Update buddies
		for(MessengerBuddy buddy : m_buddies.values())
		{
			if(buddy.online())
			{
				buddy.messenger.getSession().sendNotif(m_session.getUser().username + " went offline.");
			}
		}

		m_buddies.clear();
		m_requests.clear();
		OpenHABBO.getInstance().getImSvc().getBuddy(m_session.getUser().id, true).messenger = null;
	}

	public Session getSession()
	{
		return m_session;
	}

	public boolean initialized()
	{
		return m_initialized;
	}

	public boolean appearOffline()
	{
		return m_appearOffline;
	}

	public void appearOffline(boolean status)
	{
		m_appearOffline = status;
	}

	public boolean hasBuddy(int userId)
	{
		return m_buddies.containsKey(userId);
	}

	public MessengerBuddy getBuddy(int userId)
	{
		return m_buddies.get(userId);
	}

	public void addBuddy(MessengerBuddy buddy)
	{
		m_buddies.put(buddy.userId, buddy);

		ServerMessage msg = new ServerMessage(EServerMessage.FriendListUpdate);
		msg.appendInt(0);
		msg.appendInt(1);
		msg.appendInt(0);
		msg.appendObject(buddy);
		msg.appendBool(false);
		m_session.send(msg);
	}

	public void removeBuddy(int userId)
	{
		m_buddies.remove(userId);

		ServerMessage msg = new ServerMessage(EServerMessage.FriendListUpdate);
		msg.appendInt(0);
		msg.appendInt(1);
		msg.appendInt(-1);
		msg.appendInt(userId);
		m_session.send(msg);
	}

	public Collection<MessengerBuddy> getBuddies()
	{
		return m_buddies.values();
	}

	public boolean hasRequest(int fromUserId)
	{
		return m_requests.containsKey(fromUserId);
	}

	public void addRequest(int fromUserId, String fromUsername)
	{
		m_requests.put(fromUserId, fromUsername);

		ServerMessage msg = new ServerMessage(EServerMessage.NewBuddyRequest);
		msg.appendInt(fromUserId);
		msg.appendStr(fromUsername);
		msg.appendStr(Integer.toString(fromUserId));
		m_session.send(msg);
	}

	public void removeRequest(int fromUserId)
	{
		m_requests.remove(fromUserId);
	}

	public Set<Entry<Integer, String>> getRequests()
	{
		return m_requests.entrySet();
	}

	public void deliverInstantMessage(int fromUserId, String text)
	{
		ServerMessage msg = new ServerMessage(EServerMessage.NewInstantMessage);
		msg.appendInt(fromUserId);
		msg.appendStr(text);
		m_session.send(msg);
	}
}
