package org.openhabbo.modules;

import java.util.HashMap;
import java.util.Map;

public final class ModuleEventArgs
{
	private final int m_poolIndex;
	private final Map<String, Object> m_args;

	public ModuleEventArgs(int poolIndex)
	{
		m_poolIndex = poolIndex;
		m_args = new HashMap<String, Object>();
	}

	/*
	 * public ModuleEventArgs(String key1, Object value1)
	 * {
	 * this();
	 * this.set(key1, value1);
	 * }
	 * public ModuleEventArgs(String key1, Object value1, String key2, Object value2)
	 * {
	 * this(key1, value1);
	 * this.set(key2, value2);
	 * }
	 * public ModuleEventArgs(String key1, Object value1, String key2, Object value2, String key3, Object value3)
	 * {
	 * this(key1, value1, key2, value2);
	 * this.set(key3, value3);
	 * }
	 * public ModuleEventArgs(String key1, Object value1, String key2, Object value2, String key3, Object value3, String key4, Object value4)
	 * {
	 * this(key1, value1, key2, value2, key3, value3);
	 * this.set(key4, value4);
	 * }
	 * public ModuleEventArgs(String key1, Object value1, String key2, Object value2, String key3, Object value3, String key4, Object value4, String key5, Object value5)
	 * {
	 * this(key1, value1, key2, value2, key3, value3, key4, value4);
	 * this.set(key5, value5);
	 * }
	 */

	public final <T> ModuleEventArgs set(String key, T value)
	{
		m_args.put(key, value);
		return this;
	}

	public final void unset(String key)
	{
		m_args.remove(key);
	}

	public final boolean has(String key)
	{
		return m_args.containsKey(key);
	}

	@SuppressWarnings("unchecked")
	public final <T> T get(String key)
	{
		Object value = m_args.get(key);
		if(value != null)
		{
			return (T) value;
		}
		else
		{
			return null;
		}
	}

	public final void clear()
	{
		m_args.clear();
	}

	public final int getPoolIndex()
	{
		return m_poolIndex;
	}

	private final static int POOL_SIZE = 1024;
	private volatile static ModuleEventArgs[] pool;
	private volatile static boolean[] inUse;

	static
	{
		pool = new ModuleEventArgs[POOL_SIZE];
		for(int x = 0; x < POOL_SIZE; x++)
		{
			pool[x] = new ModuleEventArgs(x);
		}
	}

	public static ModuleEventArgs create()
	{
		for(int x = 0; x < POOL_SIZE; x++)
		{
			if(!inUse[x])
			{
				inUse[x] = true;
				return pool[x];
			}
		}
		return new ModuleEventArgs(POOL_SIZE - 1);
	}

	public static void release(ModuleEventArgs args)
	{
		args.clear();
		inUse[args.getPoolIndex()] = false;
	}
}
