package org.openhabbo.modules;

import java.util.HashMap;
import java.util.Map;

import org.openhabbo.chatcommands.ChatCommandHandler;
import org.openhabbo.core.Logger;
import org.openhabbo.items.ItemManager;
import org.openhabbo.rooms.RoomManager;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionManager;
import org.openhabbo.sessions.SessionMessageHandlerRegistry;
import org.openhabbo.storage.SQLDatabase;
import org.openhabbo.users.achievements.AchievementManager;

public final class ModuleHost
{
	private final Logger m_logger;
	private final SQLDatabase m_db;
	private final SessionManager m_sessionMgr;
	private final AchievementManager m_achievementMgr;
	private final ItemManager m_itemMgr;
	private final RoomManager m_roomMgr;

	private GrandCentralDispatch m_gcd;
	private SessionMessageHandlerRegistry m_msgHandlerRegistry = new SessionMessageHandlerRegistry();
	private Map<String, ChatCommandHandler> m_chatCmds = new HashMap<String, ChatCommandHandler>();

	public ModuleHost(Logger logger, SQLDatabase db, SessionManager sessionMgr, AchievementManager achievementMgr, ItemManager itemMgr, RoomManager roomMgr)
	{
		m_logger = logger;
		m_db = db;
		m_sessionMgr = sessionMgr;
		m_achievementMgr = achievementMgr;
		m_itemMgr = itemMgr;
		m_roomMgr = roomMgr;
	}

	public void registerChatCommand(String cmd, ChatCommandHandler handler)
	{
		m_chatCmds.put(cmd, handler);
	}

	public void unregisterChatCommand(String cmd)
	{
		m_chatCmds.remove(cmd);
	}

	public boolean onChatCommand(String cmd, String body, Session session)
	{
		ChatCommandHandler handler = m_chatCmds.get(cmd);
		if(handler == null)
		{
			return false;
		}
		else
		{
			return handler.handle(session, body);
		}
	}

	public Logger getLogger()
	{
		return m_logger;
	}

	public SQLDatabase getDb()
	{
		return m_db;
	}

	public SessionManager getSessionMgr()
	{
		return m_sessionMgr;
	}

	public AchievementManager getAchievementMgr()
	{
		return m_achievementMgr;
	}

	public ItemManager getItemMgr()
	{
		return m_itemMgr;
	}

	public RoomManager getRoomMgr()
	{
		return m_roomMgr;
	}

	public GrandCentralDispatch getGcd()
	{
		return m_gcd;
	}
}
