package org.openhabbo.net;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import org.openhabbo.OpenHABBO;

public class NetListener implements Runnable
{
	private int m_port;
	private NetManager m_mgr;
	private boolean m_listening;
	private Thread m_thread;
	private ServerSocketChannel m_listener;
	private Selector m_selector;

	public NetListener(int port, NetManager mgr)
	{
		m_port = port;
		m_mgr = mgr;
	}

	public boolean start()
	{
		if(!m_listening)
		{
			try
			{
				m_selector = Selector.open();
				m_listener = ServerSocketChannel.open();
				m_listener.configureBlocking(false);
				m_listener.socket().bind(new InetSocketAddress(m_port));
				m_listener.register(m_selector, SelectionKey.OP_ACCEPT);
			}
			catch(Exception ex)
			{
				OpenHABBO.getInstance().getLogger().error("NetListener", String.format("Could not start listening on TCP port %d!", m_port), ex);
				return false;
			}
			m_thread = new Thread(this, "NetListener (TCP/" + m_port);
			m_listening = true;
			m_thread.start();
			OpenHABBO.getInstance().getLogger().info("NetListener", String.format("Listening for connections on TCP port %d.", m_port));
			return true;
		}

		return false;
	}

	public void stop()
	{
		m_listening = false;
		m_thread.interrupt();
		try
		{
			m_listener.close();
			OpenHABBO.getInstance().getLogger().info("NetListener", "Closed listener.");
		}
		catch(IOException ex)
		{
			OpenHABBO.getInstance().getLogger().error("NetListener", String.format("Error closing listener on port %d!", m_port), ex);
		}
	}

	@Override
	public void run()
	{
		while(m_listening)
		{
			try
			{
				m_selector.select();
			}
			catch(IOException ex)
			{
				OpenHABBO.getInstance().getLogger().error("NetListener", "Failed grabbing updated selector keys, retrying...", ex);
				continue;
			}
			for(SelectionKey key : m_selector.selectedKeys())
			{
				if(key.isValid() && key.readyOps() == SelectionKey.OP_ACCEPT)
				{
					try
					{
						SocketChannel chan = m_listener.accept();
						NetConnection conn = new NetConnection(chan);
						m_mgr.onAcceptedByListener(conn);
					}
					catch(ClosedChannelException ex)
					{

					}
					catch(Exception ex)
					{
						OpenHABBO.getInstance().getLogger().error("NetListener", "Failed accepting a connection request.", ex);
					}
				}
			}
		}
	}

	public boolean isListening()
	{
		return m_listening;
	}

	public int getPort()
	{
		return m_port;
	}

	public NetManager getMgr()
	{
		return m_mgr;
	}
}
