package org.openhabbo.net;

import java.io.IOException;
import java.nio.channels.ClosedSelectorException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;

public class NetWorker implements Runnable
{
	private int m_index;
	private NetManager m_mgr;
	private boolean m_running;
	private Thread m_thread;
	private Selector m_selector;
	private int m_load;
	private List<NetConnection> m_pending;

	public NetWorker(int index, NetManager mgr)
	{
		m_index = index;
		m_mgr = mgr;
		m_pending = new ArrayList<NetConnection>();
		try
		{
			m_selector = Selector.open();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}

	public void start()
	{
		if(!m_running)
		{
			m_running = true;
			m_thread = new Thread(this, this.toString());
			m_thread.start();
		}
	}

	public void stop()
	{
		if(m_running)
		{
			m_running = false;
			try
			{
				m_selector.close();
			}
			catch(IOException ex)
			{
				OpenHABBO.getInstance().getLogger().error(this, "Error while closing selector.", ex);
			}
		}
	}

	public void enqueueNewConnection(NetConnection conn)
	{
		synchronized(m_pending)
		{
			m_pending.add(conn);
		}
		m_selector.wakeup();
	}

	public void unregister(NetConnection conn)
	{
		m_load--;
	}

	@Override
	public void run()
	{
		while(m_running)
		{
			// Any pending connections? We need to register them on this thread
			synchronized(m_pending)
			{
				for(NetConnection conn : m_pending)
				{
					if(conn.assignWorker(this, m_selector))
					{
						m_load++;
						m_mgr.onAcceptedByWorker(conn);
					}
				}
				m_pending.clear();
			}

			// Process any enabled keys
			try
			{
				m_selector.select();
				for(SelectionKey key : m_selector.selectedKeys())
				{
					// Valid key?
					if(!key.isValid())
					{
						continue;
					}

					// Determine the associated connection
					NetConnection conn = (NetConnection) key.attachment();

					// Reading?
					if(key.isReadable())
					{
						try
						{
							if(!conn.onRead())
							{
								key.cancel();
								conn.close("disconnected by user");
								continue;
							}
						}
						catch(Exception ex)
						{
							key.cancel();
							conn.close("reading error");
							continue;
						}
					}

					// Writing?
					if(key.isWritable())
					{
						try
						{
							conn.onWrite();
						}
						catch(Exception ex)
						{
							key.cancel();
							conn.close("sending error");
							continue;
						}
					}
				}
				m_selector.selectedKeys().clear();
			}
			catch(ClosedSelectorException ex)
			{

			}
			catch(Exception ex)
			{
				OpenHABBO.getInstance().getLogger().error(this, "Failed grabbing updated selector keys, retrying...", ex);
				continue;
			}
		}
	}

	public int index()
	{
		return m_index;
	}

	public NetManager getMgr()
	{
		return m_mgr;
	}

	public int load()
	{
		return m_load;
	}

	@Override
	public String toString()
	{
		return "NetWorker #" + m_index;
	}
}
