package org.openhabbo.registries;

import java.util.HashMap;
import java.util.Map;

import mod_advertisement.GetAdvertisementHandler;

import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.handlers.*;

public final class SessionMsgRegistry
{
	private final static Map<Integer, SessionMessageHandler> entries = new HashMap<Integer, SessionMessageHandler>();

	private SessionMsgRegistry()
	{
		// Hide constructor
	}

	public final static void register(int msgId, SessionMessageHandler handler)
	{
		entries.put(msgId, handler);
	}

	public final static void unregister(int msgId)
	{
		entries.remove(msgId);
	}

	public final static boolean isKnown(int msgId)
	{
		return entries.containsKey(msgId);
	}

	public final static SessionMessageHandler lookup(int msgId)
	{
		return entries.get(msgId);
	}

	public final static int count()
	{
		return entries.size();
	}

	static
	{
		register(206, new SendSessionParamsHandler());
		register(415, new SsoLoginHandler());
		register(7, new GetUserInfoHandler());
		register(8, new GetBalanceHandler());
		register(26, new GetSubscriptionDataHandler());
		register(228, new GetAvatarEffectInventory());
		register(44, new UpdateLookHandler());
		register(484, new UpdateMottoHandler());
		register(404, new GetInventoryHandler());
		register(391, new OpenFlatHandler());
		register(182, new GetAdvertisementHandler());
		register(388, new GetPubHandler());
		register(2, new OpenPubHandler());
		register(230, new GetGroupBadgesHandler());
		register(53, new GoToHotelViewHandler());
		register(151, new GetFlatCatsHandler());
		register(233, new GetHomeRoomHandler());
		register(380, new GetNavigatorFrontpage());
		register(385, new GetRoomInfoHandler());
		register(430, new GetPopularRoomsHandler());
		register(431, new GetHighRatedRoomsHandler());
		register(432, new GetFriendsRoomsHandler());
		register(433, new GetRoomsWithFriendsHandler());
		register(434, new GetOwnRoomsHandler());
		register(436, new GetRecentRoomsHandler());
		register(439, new GetEventsHandler());
		register(382, new GetPopularTagsHandler());
		register(437, new PerformSearchHandler());
		register(438, new PerformSearch2Handler());
		register(215, new GetSpriteIndexHandler());
		register(390, new GetHeightMapHandler());
		register(126, new GetRoomData3Handler());
		register(52, new TalkHandler());
		register(55, new ShoutHandler());
		register(56, new WhisperHandler());
		register(75, new MoveHandler());
		register(387, new CanCreateRoomHandler());
		register(29, new CreateRoomHandler());
		register(400, new GetRoomEditDataHandler());
		register(386, new SaveRoomIconHandler());
		register(401, new SaveRoomDataHandler());
		register(96, new GiveRightsHandler());
		register(97, new TakeRightsHandler());
		register(155, new TakeAllRightsHandler());
		register(95, new KickUserHandler());
		register(320, new BanUserHandler());
		register(384, new SetHomeRoomHandler());
		register(23, new DeleteRoomHandler());
		register(79, new LookToHandler());
		register(317, new StartTypingHandler());
		register(318, new StopTypingHandler());
		register(319, new IgnoreUserHandler());
		register(322, new UnignoreUserHandler());
		register(345, new CanCreateRoomEventHandler());
		register(346, new StartEventHandler());
		register(347, new StopEventHandler());
		register(348, new EditEventHandler());
		register(94, new WaveHandler());
		register(261, new RateRoomHandler());
		register(93, new DanceHandler());
		register(98, new AnswerDoorbellHandler());
		register(59, new ReqLoadRoomForUserHandler());
		register(66, new ApplyRoomDecoration());
		register(90, new PlaceItemHandler());
		register(67, new TakeItemHandler());
		register(73, new MoveItemHandler());
		register(392, new TriggerItemHandler()); // Generic trigger item
		register(393, new TriggerItemHandler()); // Generic trigger item
		register(78, new OpenPresentHandler());
		register(371, new GiveRespectHandler());
		register(372, new ApplyEffectHandler());
		register(373, new EnableEffectHandler());
		register(232, new TriggerItemHandler()); // One way gates
		register(314, new TriggerItemHandler()); // Love Shuffler
		register(247, new TriggerItemHandler()); // Habbo Wheel
		register(414, new RecycleItemsHandler());
		register(113, new EnterInfobusHandler());
		register(440, new SummonGuideHandler());
		register(441, new KickBotHandler());
		register(480, new SetLookTransferHandler());
		register(315, new HeartbeatHandler());
		register(421, new RegisterClientStatsHandler());
		register(101, new GetCatalogIndexHandler());
		register(102, new GetCatalogPageHandler());
		register(100, new PurchaseProductHandler());
		register(472, new PurchaseGiftHandler());
		register(412, new GetRecyclerRewardsHandler());
		register(3030, new CanGiftHandler());
		register(3011, new GetCataData1Handler());
		register(473, new GetCataData2Handler());
	}
}
