package org.openhabbo.sessions.handlers;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;
import org.openhabbo.catalog.CatalogPage;
import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;

public class GetCatalogIndexHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Request a list of all index pages ('trees') visible to the user's access level
		List<CatalogPage> indexPages = OpenHABBO.getInstance().getCatalog().getVisibleIndexPagesForAccessLevel(session.getUser().role.level);

		// Build the response
		response.init(EServerMessage.CatalogIndex);
		response.appendBool(false);
		response.appendInt(0);
		response.appendInt(0);
		response.appendInt(-1);
		response.appendStr("");
		response.appendBool(false);
		response.appendInt(indexPages.size());
		for(CatalogPage page : indexPages)
		{
			this.serializePageMetaDataForAccessLevel(page, session.getUser().role.level, response);
		}
		session.send(response);
	}

	private void serializePageMetaDataForAccessLevel(CatalogPage page, int accessLevel, ServerMessage msg)
	{
		// Determine the visible children
		List<CatalogPage> children = new ArrayList<CatalogPage>();
		for(CatalogPage child : page.pages)
		{
			if(child.minAccessLevel <= accessLevel)
			{
				children.add(child);
			}
		}

		// Serialize metadata of this page + visible children
		msg.appendBool(page.isVisible);
		msg.appendInt(page.iconColor);
		msg.appendInt(page.iconImage);
		msg.appendInt(page.id);
		msg.appendStr(page.name);
		msg.appendBool(page.isComingSoon);
		msg.appendInt(children.size());
		for(CatalogPage child : children)
		{
			this.serializePageMetaDataForAccessLevel(child, accessLevel, msg);
		}
	}
}
