package org.openhabbo.sessions.handlers;

import org.openhabbo.OpenHABBO;
import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.rooms.Room;
import org.openhabbo.rooms.RoomCategory;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;

public class SaveRoomDataHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Grab affected room
		int roomId = request.readInt();
		if(!session.inRoom() || roomId != session.getRoomInstance().getInfo().id || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}
		Room room = session.getRoomInstance().getInfo();

		// Validate new name
		String name = request.readStr();
		if(name.length() == 0)
		{
			return;
		}

		// Validate new description
		String description = request.readStr();

		// Validate new state + password
		byte state = (byte) request.readInt();
		if(state < Room.STATE_OPEN || state > Room.STATE_PASSWORD)
		{
			state = 0;
		}
		String password = request.readStr();

		// Validate max users
		int maxUsers = request.readInt();
		if(maxUsers != 10 && maxUsers != 15 && maxUsers != 20 && maxUsers != 25)
		{
			return;
		}

		// Validate category
		int categoryId = request.readInt();
		RoomCategory category = OpenHABBO.getInstance().getNavigator().getCategory(categoryId);
		if(category == null || category.minAccessLevel > session.getUser().role.level)
		{
			categoryId = 0;
			session.sendNotif("You are not allowed to use this room category. Your room has been moved to 'No category' instead.");
		}

		// Parse tags
		int tagAmount = request.readInt();
		if(tagAmount < 0 || tagAmount > 2)
		{
			return;
		}

		room.name = name;
		room.description = description;
		room.state = state;
		room.password = password;
		room.usersMax = maxUsers;
		room.categoryId = categoryId;
		room.tags = new String[tagAmount];
		for(int x = 0; x < tagAmount; x++)
		{
			room.tags[x] = request.readStr();
		}
		room.allowPets = request.readFixedBool();
		room.allowPetsEat = request.readFixedBool();
		room.allowWalkthrough = request.readFixedBool();
		room.hideWalls = request.readFixedBool();
		room.saveSettings();

		response.init(EServerMessage.RoomSettingsSaved);
		response.appendInt(room.id);
		session.send(response);

		response.init(EServerMessage.RoomInfoUpdated);
		response.appendInt(room.id);
		session.getRoomInstance().broadcast(response);
	}
}
