package org.openhabbo.sessions.handlers;

import java.util.Map.Entry;

import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.users.User;

public class UpdateBadgesHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Shorthand variable
		User user = session.getUser();

		// Clear current slots
		for(Entry<String, Byte> badge : user.badges.entrySet())
		{
			badge.setValue((byte) 0);
		}
		user.equippedBadges.clear();

		// Parse new slot content from the request
		while(request.remaining() > 0)
		{
			// Parse this slot
			byte slot = (byte) request.readInt();
			String badge = request.readStr();

			// Process the changes, but only if user actually has this badge
			if(user.badges.containsKey(badge))
			{
				user.badges.put(badge, slot);
				user.equippedBadges.put(slot, badge);
			}
		}

		// Force update on logout
		user.requiresBadgeUpdate = true;

		// Build update msg
		response.init(EServerMessage.UserBadges);
		response.appendInt(user.id);
		response.appendInt(user.equippedBadges.size());
		for(Entry<Byte, String> badge : user.equippedBadges.entrySet())
		{
			response.appendInt(badge.getKey());
			response.appendStr(badge.getValue());
		}

		// Send update msg to client(s)
		if(!session.inRoom())
		{
			session.send(response);
		}
		else
		{
			session.getRoomInstance().broadcast(response);
		}
	}
}
