package org.openhabbo.storage;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

public final class DatabaseResult
{
	public final static DatabaseResult INVALID_RESULT = new DatabaseResult();

	private final boolean valid;
	private DatabaseConnection conn;
	private ResultSet rs;

	public DatabaseResult()
	{
		this.valid = false;
	}

	public DatabaseResult(DatabaseConnection conn, ResultSet rs)
	{
		this.conn = conn;
		this.rs = rs;
		this.valid = true;
	}

	public void release()
	{
		if(this.valid)
		{
			try
			{
				this.rs.getStatement().close();
				this.conn.getMgr().releaseConnection(this.conn);
			}
			catch(SQLException ex)
			{
				this.conn.getMgr().getLogger().error("DatabaseResult", "Error closing statement and releasing connection.", ex);
			}
		}
	}

	public boolean isValid()
	{
		return this.valid;
	}

	public boolean row()
	{
		if(this.valid)
		{
			try
			{
				return this.rs.next();
			}
			catch(SQLException ex)
			{
				this.onError("going to next row", ex);
			}
		}
		return false;
	}

	public boolean getBoolean(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.rs.getBoolean(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting boolean value", ex);
			}
		}
		return false;
	}

	public byte getByte(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.rs.getByte(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting byte value", ex);
			}
		}
		return 0;
	}

	public int getInt(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.rs.getInt(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting int value", ex);
			}
		}
		return 0;
	}

	public float getFloat(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.rs.getFloat(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting float value", ex);
			}
		}
		return 0;
	}

	public long getLong(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.rs.getLong(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting long value", ex);
			}
		}
		return 0;
	}

	public String getString(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.rs.getString(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting string value", ex);
			}
		}
		return null;
	}

	public Timestamp getTimestamp(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.rs.getTimestamp(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting timestamp value", ex);
			}
		}
		return null;
	}

	public void query(String sql)
	{
		if(this.valid)
		{
			try
			{
				Statement st = this.rs.getStatement();
				this.rs.close();
				this.rs = st.executeQuery(sql);
			}
			catch(SQLException ex)
			{
				this.onError("executing additional query", ex);
			}
		}
	}

	private void onError(String operation, Exception ex)
	{
		this.conn.getMgr().getLogger().error("DatabaseResult", String.format("Error in operation '%s'.", operation), ex);
	}
}
