package org.openhabbo.storage;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

public final class DatabaseResult
{
	public final static DatabaseResult INVALID_RESULT = new DatabaseResult();

	private final boolean m_valid;
	private DatabaseConnection m_conn;
	private ResultSet m_rs;

	public DatabaseResult()
	{
		m_valid = false;
	}

	public DatabaseResult(DatabaseConnection conn, ResultSet rs)
	{
		m_conn = conn;
		m_rs = rs;
		m_valid = true;
	}

	public void release()
	{
		if(m_valid)
		{
			try
			{
				m_rs.getStatement().close();
				m_conn.getMgr().releaseConnection(m_conn);
			}
			catch(SQLException ex)
			{
				m_conn.getMgr().getLogger().error("DatabaseResult", "Error closing statement and releasing connection.", ex);
			}
		}
	}

	public boolean isValid()
	{
		return m_valid;
	}

	public boolean row()
	{
		if(m_valid)
		{
			try
			{
				return m_rs.next();
			}
			catch(SQLException ex)
			{
				this.onError("going to next row", ex);
			}
		}
		return false;
	}

	public boolean getBoolean(String c)
	{
		if(m_valid)
		{
			try
			{
				return m_rs.getBoolean(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting boolean value", ex);
			}
		}
		return false;
	}

	public byte getByte(String c)
	{
		if(m_valid)
		{
			try
			{
				return m_rs.getByte(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting byte value", ex);
			}
		}
		return 0;
	}

	public int getInt(String c)
	{
		if(m_valid)
		{
			try
			{
				return m_rs.getInt(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting int value", ex);
			}
		}
		return 0;
	}

	public float getFloat(String c)
	{
		if(m_valid)
		{
			try
			{
				return m_rs.getFloat(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting float value", ex);
			}
		}
		return 0;
	}

	public long getLong(String c)
	{
		if(m_valid)
		{
			try
			{
				return m_rs.getLong(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting long value", ex);
			}
		}
		return 0;
	}

	public String getString(String c)
	{
		if(m_valid)
		{
			try
			{
				return m_rs.getString(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting string value", ex);
			}
		}
		return null;
	}

	public Timestamp getTimestamp(String c)
	{
		if(m_valid)
		{
			try
			{
				return m_rs.getTimestamp(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting timestamp value", ex);
			}
		}
		return null;
	}

	public void query(String sql)
	{
		if(m_valid)
		{
			try
			{
				Statement st = m_rs.getStatement();
				m_rs.close();
				m_rs = st.executeQuery(sql);
			}
			catch(SQLException ex)
			{
				this.onError("executing additional query", ex);
			}
		}
	}

	private void onError(String operation, Exception ex)
	{
		m_conn.getMgr().getLogger().error("DatabaseResult", String.format("Error in operation '%s'.", operation), ex);
	}
}
