package org.openhabbo.storage;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.openhabbo.OpenHABBO;
import org.openhabbo.core.Logger;
import org.openhabbo.core.PropertiesBox;

public class SQLDatabase
{
	private Logger m_logger;
	private DatabaseEndpoint m_endpoint;
	private DatabaseConnectionManager m_connMgr;
	private DatabaseQueryQueue m_queryQueue;

	public SQLDatabase(PropertiesBox props, Logger logger)
	{
		m_logger = logger;
		m_endpoint = new DatabaseEndpoint(props.get("db.uid"), props.get("db.pwd"), props.get("db.url"));
		m_connMgr = new DatabaseConnectionManager(m_logger, m_endpoint, props.get("db.driver"), props.getInt("db.maxconnections", 0));
		m_queryQueue = new DatabaseQueryQueue(this);
	}

	public boolean test()
	{
		return m_connMgr.test();
	}

	public void onError(SQLException ex)
	{
		OpenHABBO.getInstance().getLogger().error("SQLDatabase", "Incoming error!", ex);
	}

	public void execute(String sql)
	{
		m_queryQueue.enqueue(sql);
	}

	public int executeReturnKey(String sql)
	{
		DatabaseConnection conn = m_connMgr.getConnection();
		if(conn != null)
		{
			try
			{
				Statement st = conn.getInnerConnection().createStatement();
				st.execute(sql, Statement.RETURN_GENERATED_KEYS);
				ResultSet keys = st.getGeneratedKeys();
				if(keys.next())
				{
					return keys.getInt(1);
				}
				else
				{
					return 0;
				}
			}
			catch(SQLException ex)
			{
				this.onError(ex);
			}
			finally
			{
				m_connMgr.releaseConnection(conn);
			}
		}
		return -1;
	}

	public DatabaseResult query(String sql)
	{
		DatabaseConnection conn = m_connMgr.getConnection();
		if(conn != null)
		{
			try
			{
				ResultSet rs = conn.getInnerConnection().createStatement().executeQuery(sql);
				return new DatabaseResult(conn, rs);
			}
			catch(SQLException ex)
			{
				this.onError(ex);
				m_connMgr.releaseConnection(conn);
			}
		}
		return DatabaseResult.INVALID_RESULT;
	}

	public Logger getLogger()
	{
		return m_logger;
	}

	public DatabaseEndpoint getEndpoint()
	{
		return m_endpoint;
	}

	public DatabaseConnectionManager getConnMgr()
	{
		return m_connMgr;
	}

	public DatabaseQueryQueue getQueryQueue()
	{
		return m_queryQueue;
	}
}
