package org.openhabbo.users.effects;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;
import org.openhabbo.storage.DatabaseResult;

public class AvatarEffectInventory
{
	private int m_userId;
	private int m_currentEffect;
	private List<AvatarEffect> m_effects;

	public AvatarEffectInventory(int userId)
	{
		m_userId = userId;
		m_currentEffect = -1;
		m_effects = new ArrayList<AvatarEffect>();
	}

	public void clear()
	{
		m_effects.clear();
	}

	public void reload()
	{
		// Clear old effects
		this.clear();

		// Load new ones from database
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM user_effects WHERE user_id=" + m_userId);
		while(result.row())
		{
			AvatarEffect effect = AvatarEffect.parse(result);
			if(effect.hasExpired())
			{
				effect.delete();
			}
			else
			{
				m_effects.add(effect);
			}
		}
		result.release();
	}

	public void addEffect(int effectId, int totalDuration)
	{
		// Create effect
		AvatarEffect effect = new AvatarEffect();
		effect.userId = m_userId;
		effect.effectId = effectId;
		effect.totalDuration = totalDuration;
		effect.insert();

		// Add to list
		m_effects.add(effect);
	}

	public int getCurrentEffect()
	{
		return m_currentEffect;
	}

	public void setCurrentEffect(int effectId)
	{
		m_currentEffect = effectId;
	}

	public AvatarEffect getEffect(int effectId)
	{
		for(AvatarEffect effect : m_effects)
		{
			if(effect.effectId == effectId && !effect.hasExpired())
			{
				return effect;
			}
		}
		return null;
	}

	public List<AvatarEffect> getEffects()
	{
		return m_effects;
	}
}
