package org.openhabbo.communication.requests.catalog;

import java.util.List;

import org.openhabbo.catalog.Catalog;
import org.openhabbo.catalog.CatalogProduct;
import org.openhabbo.communication.ClientMessage;
import org.openhabbo.communication.RequestHandler;
import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.core.Logger;
import org.openhabbo.items.EItemFlags;
import org.openhabbo.items.Item;
import org.openhabbo.sessions.Session;
import org.openhabbo.util.SecurityUtil;

public class PurchaseProductHandler implements RequestHandler
{
	@Override
	@SuppressWarnings("unused")
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Parse purchase info
		int pageId = request.readInt();
		int productId = request.readInt();
		String purchaseData = SecurityUtil.filterStr(request.readStr(), false); // Preserve linebreaks

		// Valid product?
		CatalogProduct product = Catalog.getInstance().getProduct(productId);
		if(product == null || !product.isPurchasableBy(session.getUser()))
		{
			Logger.getInstance().warning(this, String.format("User #%d tried to purchase product #%d from the Catalog, which is not existent/available/purchasable to this user.", session.getUser().id, productId));
			return;
		}

		// Attempt item purchase
		List<Item> shipping = Catalog.getInstance().purchaseProduct(product, session.getUser(), purchaseData, true);
		if(shipping == null)
		{
			session.sendNotif(String.format("Purchase of catalog product #%d failed, you will not be billed. Please try again in a few minutes. If this keeps happening, please contact Support.", product.id));
			return;
		}

		// Any items?
		if(!shipping.isEmpty())
		{
			// Add them to inventory
			int tab = 0;
			for(Item item : shipping)
			{
				tab = item.is(EItemFlags.FLOORITEM) ? 1 : 2;
				session.getItemInventory().addItem(item, false);
			}
			session.getItemInventory().refresh();
			
			// Notify of changes
			response.init(EServerMessage.INVENTORY_NEW_ITEM);
			response.appendBool(true);
			response.appendInt(tab);
			response.appendInt(shipping.size());
			for(Item item : shipping)
			{
				response.appendInt(item.id);
			}
			session.send(response);
		}
		
		// Bill user for product
		Catalog.getInstance().billUser(session, product);

		// Purchase okay!
		response.init(EServerMessage.CATALOG_PURCHASE_RESULT);
		response.appendInt(product.itemDefinition.id);
		response.appendStr(product.code);
		response.appendInt(product.costCredits);
		response.appendInt(product.costPixels);
		response.appendInt(product.costQuestPoints);
		response.appendInt(1);
		response.appendStr(product.getType());
		response.appendInt(product.itemDefinition.displayId);
		response.appendStr("");
		response.appendInt(1);
		response.appendInt(-1);
		response.appendStr("");
		session.send(response);
	}
}
