package org.openhabbo.communication.requests.navigator;

import org.openhabbo.communication.ClientMessage;
import org.openhabbo.communication.RequestHandler;
import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.rooms.Room;
import org.openhabbo.rooms.RoomCategory;
import org.openhabbo.rooms.RoomNavigator;
import org.openhabbo.sessions.Session;
import org.openhabbo.util.SecurityUtil;

public class SaveRoomDataHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Grab affected room
		int roomId = request.readInt();
		if(!session.inRoom() || roomId != session.getRoomInstance().getInfo().id || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}
		Room room = session.getRoomInstance().getInfo();

		// Validate new name
		String name = SecurityUtil.filterStr(request.readStr());
		if(name.length() == 0)
		{
			return;
		}

		// Validate new description
		String description = SecurityUtil.filterStr(request.readStr());

		// Validate new state + password
		byte state = (byte) request.readInt();
		if(state < Room.STATE_OPEN || state > Room.STATE_PASSWORD)
		{
			state = 0;
		}
		String password = SecurityUtil.filterStr(request.readStr());

		// Validate max users
		int maxUsers = request.readInt();
		if(maxUsers != 10 && maxUsers != 15 && maxUsers != 20 && maxUsers != 25)
		{
			return;
		}

		// Validate category
		int categoryId = request.readInt();
		RoomCategory category = RoomNavigator.getInstance().getCategory(categoryId);
		if(category == null || !category.isAccessibleTo(session.getUser()))
		{
			categoryId = room.categoryId;
			session.sendNotif("You are not allowed to use this room category. Category has not been changed.");
		}

		// Validate tags
		String[] tags = SecurityUtil.filterStrArray(request.readStrArray());
		if(tags.length > 2)
		{
			return;
		}

		// Save data
		room.name = name;
		room.description = description;
		room.state = state;
		room.password = password;
		room.usersMax = maxUsers;
		room.categoryId = categoryId;
		room.tags = tags;
		room.allowPets = request.readFixedBool();
		room.allowPetsEat = request.readFixedBool();
		room.allowWalkthrough = request.readFixedBool();
		room.hideWalls = request.readFixedBool();
		room.saveSettings();

		// Notify user
		response.init(EServerMessage.RoomSettingsSaved);
		response.appendInt(room.id);
		session.send(response);

		// Inform all room users to request new room info
		response.init(EServerMessage.RoomInfoUpdated);
		response.appendInt(room.id);
		session.getRoomInstance().broadcast(response);
	}
}
