package org.openhabbo.items;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.openhabbo.OpenHABBO;
import org.openhabbo.core.Logger;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;

public final class ItemManager
{
	private Map<Integer, ItemDefinition> definitions;

	private ItemManager()
	{
		this.definitions = Collections.emptyMap();
	}

	public void reloadDefinitions()
	{
		// Create new collection
		Map<Integer, ItemDefinition> tmpDefinitions = new HashMap<Integer, ItemDefinition>();

		// Load definitions from database
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM itemdefinitions");
		while(result.row())
		{
			ItemDefinition definition = ItemDefinition.parse(result);
			tmpDefinitions.put(definition.id, definition);
		}
		result.release();

		// Set the instance field
		this.definitions = Collections.unmodifiableMap(tmpDefinitions);

		// Done!
		Logger.getInstance().info("ItemMgr", String.format("Reloaded %d item definitions.", this.definitions.size()));
	}

	public ItemDefinition getDefinition(int definitionId)
	{
		return this.definitions.get(definitionId);
	}
	
	public List<ItemDefinition> getDefinitionsByClass(Class<? extends Item> cls)
	{
		List<ItemDefinition> definitions = new ArrayList<ItemDefinition>();
		for(ItemDefinition definition : this.definitions.values())
		{
			if(definition.getImplClass() == cls)
			{
				definitions.add(definition);
			}
		}
		
		return definitions;
	}

	public Collection<ItemDefinition> getAllDefinitions()
	{
		return this.definitions.values();
	}

	public void storeItem(Item item)
	{
		if(item.id <= 0)
		{
			item.id = OpenHABBO.getInstance().getDb().executeReturnKey("INSERT INTO items(definition_id,owner_id,state,data) VALUES (" + item.definition.id + "," + item.ownerId + ",'" + SQLHelper.escape(item.state) + "','" + SQLHelper.escape(item.data) + "')");
		}
	}

	public Item getItem(int itemId)
	{
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM items WHERE id=" + itemId);
		Item item = result.row() ? Item.parse(result) : null;
		result.release();

		return item;
	}

	public List<Item> getItemsForRoom(int roomId)
	{
		List<Item> items = new ArrayList<Item>();

		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM items WHERE room_id=" + roomId);
		while(result.row())
		{
			Item item = Item.parse(result);
			if(item != null)
			{
				items.add(item);
			}
		}
		result.release();

		return items;
	}

	/* Begin singleton */
	private final static ItemManager instance = new ItemManager();

	public final static ItemManager getInstance()
	{
		return instance;
	}
	/* End singleton */
}
