package org.openhabbo.modules;

import org.openhabbo.chatcommands.ChatCmdRegistry;
import org.openhabbo.chatcommands.ChatCommandHandler;
import org.openhabbo.communication.RequestHandler;
import org.openhabbo.communication.ClientRequestRegistry;
import org.openhabbo.core.RegistryAutoManageProxy;
import org.openhabbo.events.Event;
import org.openhabbo.events.EventAutoManageProxy;
import org.openhabbo.events.EventListener;
import org.openhabbo.items.Item;
import org.openhabbo.items.ItemImplRegistry;
import org.openhabbo.remote.RemoteCmdRegistry;
import org.openhabbo.remote.RemoteCommandHandler;

public final class ModulePayload
{
	private final EventAutoManageProxy events = new EventAutoManageProxy();
	private final RegistryAutoManageProxy<String, Class<? extends Item>> itemImpls = ItemImplRegistry.get().createProxy();
	private final RegistryAutoManageProxy<Integer, RequestHandler> msgHandlers = ClientRequestRegistry.get().createProxy();
	private final RegistryAutoManageProxy<String, ChatCommandHandler> chatCommands = ChatCmdRegistry.get().createProxy();
	private final RegistryAutoManageProxy<String, RemoteCommandHandler> remoteCommands = RemoteCmdRegistry.get().createProxy();

	public void eventListener(Event event, EventListener listener)
	{
		this.events.add(listener, event);
	}

	public void msgHandler(int type, RequestHandler handler)
	{
		this.msgHandlers.add(type, handler);
	}

	public void itemImpl(String name, Class<? extends Item> impl)
	{
		this.itemImpls.add(name, impl);
	}

	public void chatCmd(String cmd, ChatCommandHandler handler)
	{
		this.chatCommands.add(cmd, handler);
	}

	public void remoteCmd(String cmd, RemoteCommandHandler handler)
	{
		this.remoteCommands.add(cmd, handler);
	}

	public void inject()
	{
		this.events.load();
		this.msgHandlers.load();
		this.itemImpls.load();
		this.chatCommands.load();
		this.remoteCommands.load();
	}

	public void eject()
	{
		this.events.unload();
		this.msgHandlers.unload();
		this.itemImpls.unload();
		this.chatCommands.unload();
		this.remoteCommands.unload();
	}
}
