package org.openhabbo.net;

import org.jboss.netty.buffer.ChannelBuffers;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelFutureListener;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.channel.ChannelStateEvent;
import org.jboss.netty.channel.ExceptionEvent;
import org.jboss.netty.channel.MessageEvent;
import org.jboss.netty.channel.SimpleChannelHandler;

import org.openhabbo.communication.ClientMessage;
import org.openhabbo.communication.ServerMessage;

public final class NetConnectionHandler extends SimpleChannelHandler
{
	public interface CommunicationHandler
	{
		public void onNetNewMessage(ClientMessage msg);
		public void onNetClose(String reason);
	}
	
	private final Channel channel;
	private NetConnectionHandler.CommunicationHandler handler;
	
	public NetConnectionHandler(Channel channel)
	{
		this.channel = channel;
	}

	public void close()
	{
		if(this.channel.isOpen())
		{
			this.channel.write(ChannelBuffers.EMPTY_BUFFER).addListener(ChannelFutureListener.CLOSE);
		}
	}
	
	private void requestClose(String reason)
	{
		this.close();
		if(this.handler != null)
		{
			this.handler.onNetClose(reason);
		}
	}
	
	@Override
	public void disconnectRequested(ChannelHandlerContext ctx, ChannelStateEvent e)
	{
		this.requestClose("disconnected by user");
	}
	
	public void send(ServerMessage msg)
	{
		if(this.channel.isWritable())
		{
			this.channel.write(msg);
		}
	}
	
	@Override
	public void messageReceived(ChannelHandlerContext ctx, MessageEvent e)
	{
		if(e.getMessage() instanceof ClientMessage)
		{
			handler.onNetNewMessage((ClientMessage) e.getMessage());
		}
	}
	
	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, ExceptionEvent e)
	{
		this.requestClose("disconnected by user");
	}
	
	public NetConnectionHandler.CommunicationHandler getHandler()
	{
		return this.handler;
	}

	public void setHandler(NetConnectionHandler.CommunicationHandler handler)
	{
		this.handler = handler;
	}
	
	public String getIpAddress()
	{
		return this.channel.getRemoteAddress().toString();
	}
}
