package org.openhabbo.rooms.instances;

import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.users.User;

public final class RoomUser extends RoomAvatar
{
	private Session session;
	public boolean hasRoomOwner;
	public boolean hasRoomRights;
	
	public int danceId;
	private int roomEffectId;
	private int ownEffectId;
	public int carryItemId;
	public int carryItemTimer;
	
	public RoomUser(Session session)
	{
		this.session = session;
		this.pathfindingAllowedOn = (RoomMap.OPEN | RoomMap.ITEM | RoomMap.SPECIAL);
	}
	
	public Session getSession()
	{
		return this.session;
	}
	
	public User getInfo()
	{
		return this.session.getUser();
	}
	
	@Override
	protected void onChatFromAvatar(RoomAvatar from, String text, boolean isShout)
	{
		// Not on ignore list?
		if(true)
		{
			this.lookTo(from.x, from.y, true);
		}
	}
	
	public void pulse()
	{
		// Call RoomAvatar pulse() logic
		super.pulse();
		
		// Deal with carry item
		if(this.carryItemId > 0)
		{
			this.carryItemTimer--;
			if(this.carryItemTimer <= 0)
			{
				this.stopCarryItem();
			}
		}
	}
	
	public void carryItem(int itemId)
	{
		// Set the ID + timer
		this.carryItemId = itemId;
		if(itemId > 0)
		{
			this.carryItemTimer = 240;
		}
		else
		{
			this.carryItemTimer = 0;
		}

		// Notify the roominstance
		ServerMessage msg = new ServerMessage(EServerMessage.CarryObject);
		msg.appendInt(this.avatarId);
		msg.appendInt(this.carryItemId);
		this.getRoom().broadcast(msg);
	}
	
	public void stopCarryItem()
	{
		this.carryItem(0);
	}
	
	public void setEffect(int effectId, boolean isOwn)
	{
		// Set the effect
		if(isOwn)
		{
			this.ownEffectId = effectId;
		}
		else
		{
			this.roomEffectId = effectId;
		}
		
		// Notify the room
		ServerMessage msg = new ServerMessage(485);
		msg.appendInt(this.avatarId);
		msg.appendInt(this.getCurrentEffect());
		msg.appendBool(false);
		this.getRoom().broadcast(msg);
	}
	
	public int getCurrentEffect()
	{
		if(this.roomEffectId > 0)
		{
			return this.roomEffectId;
		}
		else
		{
			return this.ownEffectId;
		}
	}
	
	@Override
	public void serialize(ServerMessage msg)
	{
		User user = this.getInfo();
		msg.appendInt(user.id);
		msg.appendStr(user.username);
		msg.appendStr(user.motto);
		msg.appendStr(user.figure);
		msg.appendInt(this.avatarId);
		msg.appendInt(this.x);
		msg.appendInt(this.y);
		msg.appendStr(this.z);
		msg.appendInt(2);
		msg.appendInt(1);
		msg.appendStr(user.gender);
		msg.appendInt(-1);
		msg.appendInt(-1);
		msg.appendInt(-1);
		msg.appendStr("");
		msg.appendInt(0);
	}

	@Override
	public String getAvatarKey()
	{
		return "u" + this.session.getUser().id;
	}
}
