package mod_decoration;

import org.openhabbo.communication.ClientMessage;
import org.openhabbo.communication.RequestHandler;
import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.sessions.Session;

public class ApplyRoomDecoration implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to apply decoration?
		if(session.getRoomUser() == null || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Verify decoration item
		Item item = session.getItemInventory().getItem(request.readInt());
		if(item == null || !(item instanceof DecorationItem))
		{
			return;
		}

		// Remove from inventory and delete
		session.getItemInventory().removeItem(item.id);
		item.delete();
		
		// Apply new props
		RoomInstance room = session.getRoomInstance();
		if(item.definition.data.equals("wallpaper"))
		{
			room.getInfo().wallpaper = item.state;
		}
		else if(item.definition.data.equals("floor"))
		{
			room.getInfo().floor = item.state;
		}
		else if(item.definition.data.equals("landscape"))
		{
			room.getInfo().landscape = item.state;
		}
		room.getInfo().saveDecoration();

		// Notify room
		response.init(EServerMessage.RoomProperty);
		response.appendStr(item.definition.sprite);
		response.appendStr(item.state);
		room.broadcast(response);
	}
}
