package mod_autokick;

import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.events.EventListener;
import org.openhabbo.rooms.instances.RoomAvatar;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.util.GameUtil;

public class RoomUserPulseHook implements EventListener
{
	private final static int maxIdleAwakeTime = GameUtil.secondsToRoomPulses(5 * 60); // 5 minutes of idling before sleeping
	private final static int maxIdleSleepTime = (maxIdleAwakeTime + GameUtil.secondsToRoomPulses(10 * 60)); // 5 minutes idling -> sleep, 10 more minutes sleeping -> kick

	@Override
	public boolean handle(Object subject, Object args)
	{
		RoomAvatar avatar = (RoomAvatar) subject;

		// Not sleeping?
		if(!avatar.has("sleep"))
		{
			// Needs to go to sleep?
			if(avatar.idleTime > maxIdleAwakeTime)
			{
				// Set the sleep flag so new incoming users see it too
				avatar.set("sleep");

				// Notify the room that this avatar went asleep!
				ServerMessage msg = new ServerMessage(EServerMessage.Sleep);
				msg.appendInt(avatar.avatarId);
				msg.appendBool(true);
				avatar.getRoom().broadcast(msg);
			}
		}
		else
		{
			// Sleeping long enough for kicking?
			if(avatar.idleTime > maxIdleSleepTime)
			{
				// User is a player?
				if(avatar instanceof RoomUser)
				{
					((RoomUser)avatar).getSession().kickFromRoom("Kicked by mod_autokick, you have been idle for too long.");
				}
			}
		}
		
		return true;
	}

}
