package mod_autopixels;

import org.openhabbo.modules.Module;
import org.openhabbo.modules.ModuleInfo;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionManager;
import org.openhabbo.util.GameUtil;

@ModuleInfo
(
		name = "mod_autopixels",
		description = "Gives online users new pixels at a set interval",
		author = "Nils 'Nillus'",
		url = "http://habbo.nillus.net/modules/mod_autopixels/",
		version = "1.0.0",
		dependencies = { }
)
public final class mod_autopixels extends Module
{
	@Override
	protected boolean onLoad()
	{
		int minutes = GameUtil.parseInt(this.getConfigValue("minutes"));
		if(minutes > 0)
		{
			this.enablePulseTimer(minutes * 60);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	@Override
	protected void onPulse()
	{
		// Get increments + limits
		int minutes = GameUtil.parseInt(this.getConfigValue("minutes"));
		int add = GameUtil.parseInt(this.getConfigValue("add"));
		int max = GameUtil.parseInt(this.getConfigValue("max"));
		String addMsg = this.getConfigValue("add_msg");
		String addMsgNoMax = this.getConfigValue("add_msg_nomax");
		String maxReachedMsg = this.getConfigValue("max_reached_msg");
		boolean showMsg = (this.getConfigValue("show_msg").equals("1"));
		
		for(Session session : SessionManager.getInstance().getAllSessions())
		{
			if(session.authenticated())
			{
				int toAdd = add;
				
				// Max configured?
				if(max > 0)
				{
					// Impose limit?
					if((session.getUser().pixels + toAdd) > max)
					{
						// Top up no ruther than the maxxxxxxxx
						toAdd = (max - session.getUser().pixels);
					}
				}
				
				// Changes?
				if(toAdd > 0)
				{
					// Update user
					session.getUser().pixels += toAdd;
					session.getUser().saveCreditsAndPixels();
					
					// Notify user
					session.sendPixelsAddedNotif(toAdd);
					if(showMsg)
					{
						if(max > 0)
						{
							session.sendNotif(String.format(addMsg, toAdd, minutes, max));
						}
						else
						{
							session.sendNotif(String.format(addMsgNoMax, toAdd, minutes));
						}
					}
				}
				else
				{
					if(showMsg)
					{
						session.sendNotif(String.format(maxReachedMsg, max, minutes));
					}
				}
			}
		}
	}
}
