package mod_pets;

import org.openhabbo.communication.ServerMessage;
import org.openhabbo.items.EItemFlags;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomAvatar;
import org.openhabbo.rooms.instances.RoomTile;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.util.GameUtil;

public final class RoomPet extends RoomAvatar
{
	private Pet pet;
	
	public RoomPet(Pet pet)
	{
		this.pet = pet;
		this.pathfindingAllowedOn = (RoomTile.OPEN | RoomTile.ITEM);
		
		// Can swim?
		if(this.pet.type == 9 || this.pet.type == 14)
		{
			this.pathfindingAllowedOn |= RoomTile.WATER;
		}
	}

	public Pet getInfo()
	{
		return this.pet;
	}

	public void syncAndSaveLocationInfo()
	{
		if(this.getRoom() != null)
		{
			this.pet.roomId = this.getRoom().getInfo().id;
			this.pet.x = this.x;
			this.pet.y = this.y;
			this.pet.z = this.z;
		}
		else
		{
			this.pet.roomId = 0;
			this.pet.x = 0;
			this.pet.y = 0;
			this.pet.z = 0;
		}
		this.pet.save();
	}

	@Override
	protected void onChatFromAvatar(RoomAvatar from, String text, boolean isShout)
	{
		// Only respond to users
		if(!(from instanceof RoomUser))
		{
			return;
		}

		// Only respond to owner
		RoomUser user = (RoomUser) from;
		if(user.getInfo().id != this.getInfo().ownerId)
		{
			return;
		}

		// Find out if text starts with pet name (case insensitive)
		text = text.toLowerCase();
		boolean isCommand = text.startsWith(this.getInfo().name.toLowerCase());

		// Pay attention...
		if(!isCommand)
		{
			// Rotate head
			this.lookTo(from.x, from.y, true);
		}
		else
		{
			// Rotate body too (if needed)
			this.lookTo(from.x, from.y, false);

			// Try for matches
			PetUtil.handlePetCommand(this, user, text);
			
			// La la la testing
			this.statuses.clear();
			if(text.contains("geen"))
			{
				int x = GameUtil.getRandomInt(0, this.getRoom().getModel().getMaxX());
				int y = GameUtil.getRandomInt(0, this.getRoom().getModel().getMaxY());
				this.startMovingTo(x, y);
			}
			else if(text.contains("ga"))
			{
				int x = GameUtil.getRandomInt(0, this.getRoom().getModel().getMaxX());
				int y = GameUtil.getRandomInt(0, this.getRoom().getModel().getMaxY());
				this.startMovingTo(x, y);
			}
			else if(text.contains("zit"))
			{
				this.addStatus("sit", Float.toString(0));
			}
			else if(text.contains("lig"))
			{
				this.addStatus("lay", Float.toString(0));
			}
			else if(text.contains("spring"))
			{
				this.addStatus("jmp");
			}
			else if(text.contains("rol"))
			{
				this.addStatus("rll");
			}
			else if(text.contains("zwaai"))
			{
				this.addStatus("wav");
			}
			else if(text.contains("staan"))
			{
				this.addStatus("stn");
			}
			else if(text.contains("hier"))
			{
				int x = from.x;
				int y = from.y;
				if(from.bodyRotation == 0)
				{
					y--;
				}
				else if(from.bodyRotation == 2)
				{
					x++;
				}
				else if(from.bodyRotation == 4)
				{
					y++;
				}
				else if(from.bodyRotation == 6)
				{
					x--;
				}
				this.startMovingTo(x, y);
			}
			else if(text.contains("smeek"))
			{
				
			}
			else if(text.contains("dood"))
			{
				this.addStatus("ded", Float.toString(pet.z));
			}
			else if(text.contains("blijf"))
			{
				
			}
			else if(text.contains("volg"))
			{
				if(text.contains("stop"))
				{
					this.unset("follow");
				}
				else
				{
					this.set("follow", from);
				}
			}
			else if(text.contains("tv"))
			{
				// Find TV
				Item tv = null;
				for(Item item : this.getRoom().getItems(EItemFlags.FLOORITEM))
				{
					if(item.definition.sprite.contains("tv"))
					{
						tv = item;
						break;
					}
				}
				
				// TV found?
				if(tv == null)
				{
					this.chat("Er is geen tv...", false);
				}
				else
				{
					this.set("tv", tv);
					this.startMovingTo(tv.getTileInFront());
				}
			}
			else if(text.contains("dans"))
			{
				this.addStatus("dnc");
			}
		}
	}
	
	@Override
	public void pulse()
	{
		if(this.path.isEmpty())
		{
			if(this.has("tv"))
			{
				Item dest = this.get("tv");
				if(this.at(dest.getTileInFront()))
				{
					this.lookTo(dest.x, dest.y, false);
					this.chat("*kijkt porno*", false);
					this.unset("tv");
				}
			}
			
			if(!this.has("follow"))
			{
				if(GameUtil.getRandomInt(0, 10) == 0)
				{
					this.startMovingTo(GameUtil.getRandomInt(this.x - 3, this.x + 3), GameUtil.getRandomInt(this.y - 3, this.y + 3));
				}
			}
		}
		
		if(this.has("follow"))
		{
			RoomAvatar follow = this.get("follow");
			int x = follow.x;
			int y = follow.y;
			if(follow.bodyRotation == 0)
			{
				y++;
			}
			else if(follow.bodyRotation == 1)
			{
				y++;
				x--;
			}
			else if(follow.bodyRotation == 2)
			{
				x--;
			}
			else if(follow.bodyRotation == 3)
			{
				x--;
				y--;
			}
			else if(follow.bodyRotation == 4)
			{
				y--;
			}
			else if(follow.bodyRotation == 5)
			{
				y--;
				x++;
			}
			else if(follow.bodyRotation == 6)
			{
				x++;
			}
			this.startMovingTo(x, y);
		}
	}

	@Override
	public void serialize(ServerMessage msg)
	{
		msg.appendInt(this.pet.id);
		msg.appendStr(this.pet.name);
		msg.appendStr("");
		msg.appendStr(this.pet.getFigure());
		msg.appendInt(this.avatarId);
		msg.appendInt(this.x);
		msg.appendInt(this.y);
		msg.appendStr(this.z);
		msg.appendInt(4);
		msg.appendInt(2);
		msg.appendInt(0);
	}

	@Override
	public String getAvatarKey()
	{
		return "p" + this.pet.id;
	}
}
