package mod_pets.msgs;


import mod_pets.Pet;
import mod_pets.PetInventory;
import mod_pets.RoomPet;

import org.openhabbo.communication.ClientMessage;
import org.openhabbo.communication.RequestHandler;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.Session;

public class PlacePetHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Read info
		int petId = request.readInt();
		int tileX = request.readInt();
		int tileY = request.readInt();

		// Allowed to place pets?
		RoomInstance room = session.getRoomInstance();
		if(!session.getRoomUser().hasRoomOwner && !room.getInfo().allowPets)
		{
			return;
		}

		// Can place pet here?
		if(!room.getMap().tileIsOpen(tileX, tileY))
		{
			return;
		}
		
		// Get pet inventory
		PetInventory inventory = session.get("pets");
		if(inventory == null)
		{
			return;
		}
		
		// Pet in inventory?
		Pet pet = inventory.removePet(petId);
		if(pet == null)
		{
			return;
		}
		
		// Spawn roomavatar
		RoomPet avatar = new RoomPet(pet);
		avatar.x = tileX;
		avatar.y = tileY;
		avatar.z = room.getMap().height[tileX][tileY];
		room.spawnAvatar(avatar);
		
		// Sync location
		avatar.syncAndSaveLocationInfo();
		
		// Look to owner!
		RoomUser me = session.getRoomUser();
		avatar.lookTo(me.x, me.y, false);
	}
}
