package mod_wired;

import java.util.List;

import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomUser;

public abstract class WiredTrigger extends WiredItem
{
	@Override
	protected void onTimeout()
	{
		this.execute(null);
	}
	
	public final void resetTimer()
	{
		this.clearTimeout();
	}
	
	public final void execute(RoomUser user)
	{
		// Get all items in the stack
		List<Item> items = this.getRoom().getFloorItemsOnTile(this.x, this.y);

		// Search for Conditions
		for(Item item : items)
		{
			// Is this a Condition?
			if(item instanceof WiredCondition)
			{
				// Evaluate the Condition
				WiredCondition condition = (WiredCondition)item;
				if(condition.evaluate(user) == false)
				{
					// Atleast one Condition wasn't met, don't execute any Actions
					return;
				}
			}
		}
		
		// Search for Actions
		for(Item item : items)
		{
			// Is this an Action?
			if(item instanceof WiredAction)
			{
				// Flash the action!
				WiredAction action = (WiredAction)item;
				action.setTempState("1");
				
				// Execute immediately or delay?
				if(action.delay == 0)
				{
					action.execute(user);
				}
				else
				{
					action.execute(user, action.delay);
				}
			}
		}
	}
}
