package mod_wired.actions;

import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.util.GameUtil;

import mod_wired.WiredAction;

public class WiredMoveRotateItemsAction extends WiredAction
{
	public final static int MOVE = 0;
	public final static int ROTATE = 1;
	public final static Byte[] ROTATIONS = new Byte[] { 0, 2, 4, 6 };
	
	@Override
	protected int getInterface()
	{
		return 4;
	}
	
	@Override
	protected int getAmountOfParams()
	{
		return 2;
	}
	
	@Override
	public boolean requiresUser()
	{
		return false;
	}
	
	@Override
	public void execute(RoomUser user)
	{
		int move = this.getParam(MOVE);
		int rotate = this.getParam(ROTATE);
		for(Item item : this.getSelectedItems())
		{
			// Determine new position
			int x = item.x, y = item.y;
			if(move == 1)
			{
				if(GameUtil.getRandomInt(0, 1) == 0)
				{
					x += GameUtil.getRandomDelta(-1, 1);
				}
				else
				{
					y += GameUtil.getRandomDelta(-1, 1);
				}
			}
			else if(move == 2)
			{
				x += GameUtil.getRandomDelta(-1, 1);
			}
			else if(move == 3)
			{
				y += GameUtil.getRandomDelta(-1, 1);
			}
			else if(move == 4)
			{
				y--;
			}
			else if(move == 5)
			{
				x++;
			}
			else if(move == 6)
			{
				y++;
			}
			else if(move == 7)
			{
				x--;
			}
			
			// Determine new rotation
			byte rotation = item.rotation;
			if(rotate == 1)
			{
				rotation += 2;
			}
			else if(rotate == 2)
			{
				rotation -= 2;
			}
			else if(rotate == 3)
			{
				rotation = GameUtil.getRandomElement(ROTATIONS);
			}
			
			// Validate new rotation
			if(rotation > 6)
			{
				rotation = 0;
			}
			else if(rotation < 0)
			{
				rotation = 6;
			}
			
			// Issue the move
			int oldX = item.x;
			int oldY = item.y;
			float oldZ = item.z;
			if(this.getRoom().moveFloorItem(item, x, y, rotation, null))
			{
				// Changed position?
				if(item.x != oldX || item.y != oldY)
				{
					// Slide over there!
					ServerMessage msg = new ServerMessage(EServerMessage.SlideObjectBundle);
					msg.appendInt(oldX);
					msg.appendInt(oldY);
					msg.appendInt(item.x);
					msg.appendInt(item.y);
					msg.appendInt(1);
					msg.appendInt(item.id);
					msg.appendStr(oldZ);
					msg.appendStr(item.z);
					msg.appendInt(-1);
					this.getRoom().broadcast(msg);
				}
			}
		}
	}
}
