package mod_wired.actions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONObject;
import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomUser;

import mod_wired.WiredAction;

public class WiredRestoreItemsToSnapshotsAction extends WiredAction
{
	public final static int PARAM_STATE = 0;
	public final static int PARAM_ROTATION = 1;
	public final static int PARAM_POSITION = 2;
	
	private List<ItemSnapshot> snapshots;
	
	@Override
	protected int getInterface()
	{
		return 3;
	}
	
	@Override
	protected int getAmountOfParams()
	{
		return 3;
	}
	
	@Override
	public boolean requiresUser()
	{
		return false;
	}
	
	@Override
	public void execute(RoomUser user)
	{
		// Restore all selected items to their saved snapshots
		List<Item> selected = this.getSelectedItems();
		for(ItemSnapshot snapshot : this.snapshots)
		{
			Item item = this.getRoom().getItem(snapshot.itemId);
			if(selected.contains(item))
			{
				// Check what needs restoring
				if(this.getParam(PARAM_STATE) == 1)
				{
					item.state = snapshot.state;
				}
				if(this.getParam(PARAM_ROTATION) == 1)
				{
					item.rotation = snapshot.rotation;
				}
				if(this.getParam(PARAM_POSITION) == 1)
				{
					item.x = snapshot.x;
					item.y = snapshot.y;
					item.z = snapshot.z;
				}
				
				// Refresh
				ServerMessage msg = new ServerMessage(EServerMessage.ObjectUpdate);
				msg.appendObject(item);
				this.getRoom().broadcast(msg);
			}
		}
		
		// Rebuild the floormap
		this.getRoom().rebuildFloorMap();
	}

	public void updateSnapshots()
	{
		this.snapshots.clear();
		for(Item item : this.getSelectedItems())
		{
			ItemSnapshot snapshot = new ItemSnapshot();
			snapshot.itemId = item.id;
			if(this.getParam(PARAM_STATE) == 1)
			{
				snapshot.state = item.state;
			}
			if(this.getParam(PARAM_ROTATION) == 1)
			{
				snapshot.rotation = item.rotation;
			}
			if(this.getParam(PARAM_POSITION) == 1)
			{
				snapshot.x = item.x;
				snapshot.y = item.y;
				snapshot.z = item.z;
			}
			this.snapshots.add(snapshot);
		}
		this.saveSettings();
	}
	
	@Override
	protected void onWrap(JSONObject data) throws Exception
	{
		super.onWrap(data);
		
		// Wrap all snapshots
		List<Map<String, Object>> all = new ArrayList<Map<String, Object>>(5);
		for(ItemSnapshot snapshot : this.snapshots)
		{
			Map<String, Object> ss = new HashMap<String, Object>(6);
			ss.put("id", snapshot.itemId);
			ss.put("x", snapshot.x);
			ss.put("y", snapshot.y);
			ss.put("z", snapshot.z);
			ss.put("r", snapshot.rotation);
			ss.put("s", snapshot.state);
			all.add(ss);
		}
		data.put("mem", all);
	}
	
	@Override
	protected void onUnwrap(JSONObject data) throws Exception
	{
		super.onUnwrap(data);
		
		// Unwrap all snapshots
		this.snapshots = new ArrayList<ItemSnapshot>(5);
		JSONArray array = data.optJSONArray("mem");
		if(array != null)
		{
			for(int x = 0; x < array.length(); x++)
			{
				ItemSnapshot snapshot = new ItemSnapshot();
				JSONObject obj = array.optJSONObject(x);
				snapshot.itemId = obj.optInt("id");
				snapshot.x = obj.optInt("x");
				snapshot.y = obj.optInt("y");
				snapshot.z = (float) obj.optDouble("z");
				snapshot.rotation = (byte) obj.optInt("r");
				snapshot.state = obj.optString("s");
				this.snapshots.add(snapshot);
			}
		}
	}
	
	private class ItemSnapshot
	{
		public int itemId;
		public int x;
		public int y;
		public float z;
		public byte rotation;
		public String state;
	}
}