package mod_wired.actions;

import java.util.List;
import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONObject;
import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomUser;

import mod_wired.WiredAction;

public class WiredRestoreItemsToSnapshotsAction extends WiredAction
{
	public final static int PARAM_STATE = 0;
	public final static int PARAM_ROTATION = 1;
	public final static int PARAM_POSITION = 2;
	
	private List<ItemSnapshot> snapshots;

	@Override
	protected int getInterface()
	{
		return 3;
	}
	
	@Override
	protected int getAmountOfParams()
	{
		return 3;
	}
	
	@Override
	public boolean requiresUser()
	{
		return false;
	}
	
	@Override
	public void execute(RoomUser user)
	{
		// Restore all selected items to their saved snapshots
		List<Item> selected = this.getSelectedItems();
		for(ItemSnapshot snapshot : this.snapshots)
		{
			Item item = this.getRoom().getItem(snapshot.itemId);
			if(selected.contains(item))
			{
				// Check what needs restoring
				int oldX = item.x;
				int oldY = item.y;
				float oldZ = item.z;
				if(this.getParam(PARAM_STATE) == 1)
				{
					item.state = snapshot.state;
				}
				if(this.getParam(PARAM_ROTATION) == 1)
				{
					item.rotation = snapshot.rotation;
				}
				if(this.getParam(PARAM_POSITION) == 1)
				{
					item.x = snapshot.x;
					item.y = snapshot.y;
					item.z = snapshot.z;
				}
				
				// Refresh
				ServerMessage msg = new ServerMessage(EServerMessage.ObjectUpdate);
				msg.appendObject(item);
				this.getRoom().broadcast(msg);
				
				// Slide over there!
				msg.init(EServerMessage.SlideObjectBundle);
				msg.appendInt(oldX);
				msg.appendInt(oldY);
				msg.appendInt(item.x);
				msg.appendInt(item.y);
				msg.appendInt(1);
				msg.appendInt(item.id);
				msg.appendStr(oldZ);
				msg.appendStr(item.z);
				msg.appendInt(-1);
				this.getRoom().broadcast(msg);
			}
		}
		
		// Rebuild the floormap
		this.getRoom().rebuildFloorMap();
	}

	public void updateSnapshots()
	{
		this.snapshots.clear();
		for(Item item : this.getSelectedItems())
		{
			ItemSnapshot snapshot = new ItemSnapshot();
			snapshot.itemId = item.id;
			if(this.getParam(PARAM_STATE) == 1)
			{
				snapshot.state = item.state;
			}
			if(this.getParam(PARAM_ROTATION) == 1)
			{
				snapshot.rotation = item.rotation;
			}
			if(this.getParam(PARAM_POSITION) == 1)
			{
				snapshot.x = item.x;
				snapshot.y = item.y;
				snapshot.z = item.z;
			}
			this.snapshots.add(snapshot);
		}
		this.saveSettings();
	}
	
	@Override
	protected void onWrap(JSONObject data) throws Exception
	{
		super.onWrap(data);
		
		// Wrap all snapshots
		List<JSONArray> arrays = new ArrayList<JSONArray>(5);
		for(ItemSnapshot snapshot : this.snapshots)
		{
			JSONArray array = new JSONArray();
			array.put(snapshot.itemId);
			array.put(snapshot.x);
			array.put(snapshot.y);
			array.put(snapshot.z);
			array.put(snapshot.rotation);
			array.put(snapshot.state);
			arrays.add(array);
		}
		data.put("mem", arrays);
	}
	
	@Override
	protected void onUnwrap(JSONObject data) throws Exception
	{
		super.onUnwrap(data);
		
		// Unwrap all snapshots
		this.snapshots = new ArrayList<ItemSnapshot>(5);
		JSONArray arrays = data.optJSONArray("mem");
		if(arrays != null)
		{
			for(int x = 0; x < arrays.length(); x++)
			{
				ItemSnapshot snapshot = new ItemSnapshot();
				JSONArray array = arrays.optJSONArray(x);
				if(array != null)
				{
					snapshot.itemId = array.optInt(0);
					snapshot.x = array.optInt(1);
					snapshot.y = array.optInt(2);
					snapshot.z = (float) array.optDouble(3);
					snapshot.rotation = (byte) array.optInt(4);
					snapshot.state = array.optString(5);
					this.snapshots.add(snapshot);
				}
			}
		}
	}
	
	private final class ItemSnapshot
	{
		public int itemId;
		public int x;
		public int y;
		public float z;
		public byte rotation;
		public String state;
	}
}