package mod_wired.triggers;

import org.openhabbo.communication.ServerMessage;
import org.openhabbo.events.EventListener;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.Session;

import mod_wired.WiredItem;
import mod_wired.WiredTrigger;

public class WiredUserSaysPhraseTrigger extends WiredTrigger
{
	public String msg;
	public boolean ownerOnly;
	
	@Override
	public void onToggle(RoomUser user, int operation)
	{
		// Require room rights
		if(!user.hasRoomRights)
		{
			return;
		}
		
		// Get settings
		Boolean ownerOnly = this.getSetting("ownerOnly");
        if(ownerOnly == null)
        {
        	ownerOnly = false;
        }
        
		// Send settings
		ServerMessage msg = new ServerMessage(650);
        msg.appendInt(0);
        msg.appendInt(0);
        msg.appendInt(0);
        msg.appendInt(this.definition.spriteId);
        msg.appendInt(this.id);
        msg.appendStr((String)this.getSetting("msg"));
        msg.appendBool(ownerOnly);
        msg.appendBool(true);
        user.getSession().send(msg);
	}
	
	public static class ChatHook implements EventListener
	{
		@Override
		public boolean handle(Object subject, Object args)
		{
			Session session = (Session)subject;
			String text = (String)args;
			
			// Matches the secret word?
			for(Item item : session.getRoomInstance().getAllItems())
			{
				// Phrase trigger?
				if(item instanceof WiredUserSaysPhraseTrigger)
				{
					// Phrase matches?
					String msg = ((WiredItem) item).getSetting("msg");
					if(text.equals(msg))
					{
						// Owner-only check (if enabled)
						boolean ownerOnly = ((WiredItem) item).getSetting("ownerOnly");
						if(!ownerOnly || session.getRoomUser().hasRoomOwner)
						{
							// Let the user know he got the phrase correct
							session.sendWhisperNotif(text);
							
							// Fire the trigger
							((WiredTrigger)item).execute(session.getRoomUser());
							
							// Don't chat the secret to room ;)
							return false;
						}
					}
				}
			}
			
			// Nothing happened!
			return true;
		}
	}
}
