package com.fjordev.eventgraph.actor

import scala.collection.mutable

import akka.actor.{Actor, ActorLogging}

import com.fjordev.eventgraph.model.{AttendingUser, Event}

import EventDatabase._

class EventDatabase extends Actor with ActorLogging
{
  val events = mutable.Map[Long, Event]()
  val attending = mutable.Map[Long, Seq[AttendingUser]]()

  log.info("Events: {}", events.size)

  def receive =
  {
    case UpdateEvent(e) => events(e.id) = e; log.info("updated '{}'", e.name)

    case UpdateAttending(id, a) => attending(id) = a; log.debug("{} users attending #{}", a.size, id)

    case SearchEvents(query) => sender ! (events.values filter(_.name matches query))
  }
}

object EventDatabase
{
  case class UpdateEvent(event: Event)

  case class UpdateAttending(eventId: Long, attending: Seq[AttendingUser])

  case class SearchEvents(query: String)
}
