package com.fjordev.eventgraph.actor

import scala.collection.JavaConversions._

import akka.actor.{ActorRef, ActorLogging, Actor}

import facebook4j.Reading

import com.fjordev.eventgraph.util.FacebookAPI

import EventFetcher.FetchEvent, AttendingFetcher.FetchAttending

import EventFinder._

class EventFinder(eventFetcherRouter: ActorRef, attendingFetcherRouter: ActorRef) extends Actor with ActorLogging
{
  val apiClient = FacebookAPI.get()
  val idsOnly = new Reading().fields("id")

  def receive =
  {
    case FindEvents(query: String) => searchEventIds(query) foreach fetchFullEvent
  }

  def searchEventIds(query: String) = apiClient.searchEvents(query, idsOnly).iterator map(_.getId.toLong) toStream

  def fetchFullEvent(id: Long)
  {
    eventFetcherRouter forward FetchEvent(id)

    attendingFetcherRouter forward FetchAttending(id)
  }
}

object EventFinder
{
  case class FindEvents(query: String)
}
