package com.fjordev.eventgraph.actor

import akka.actor.{Actor, Props, ActorLogging}
import akka.routing.RoundRobinPool

import EventMaster._

class EventMaster extends Actor with ActorLogging
{
  // Backing store for events
  val db = context.actorOf(Props[EventDatabase], "db")

  // Fetcher router (fetchers know the database)
  val fetchers = context.actorOf(RoundRobinPool(5).props(Props(classOf[EventFetcher], db)), "fetchers")

  // Finder router (finders know the fetcher router)
  val finders = context.actorOf(RoundRobinPool(5).props(Props(classOf[EventFinder], fetchers)), "finders")

  // Yay!
  log.info("Initialized database, finders and fetchers")

  def receive =
  {
    case AddEventsToDatabase(query) => finders ! EventFinder.FindEvents(query)
  }
}

object EventMaster
{
  case class AddEventsToDatabase(query: String)
}