package com.fjordev.eventgraph.actor

import akka.actor.{Actor, Props, ActorLogging}
import akka.routing.RoundRobinPool

import EventMaster._

class EventMaster extends Actor with ActorLogging
{
  // Backing store for events
  val db = context.actorOf(Props[EventDatabase], "db")

  // For fetching event data
  val eventFetcherRouter = context.actorOf(RoundRobinPool(5).props(Props(classOf[EventFetcher], db)), "efetchers")

  // For fetching attendees
  val attendingFetcherRouter = context.actorOf(RoundRobinPool(5).props(Props(classOf[AttendingFetcher], db)), "afetchers")

  // Finder router (finders know the fetchers)
  val finderRouter = context.actorOf(RoundRobinPool(5).props(Props(classOf[EventFinder], eventFetcherRouter, attendingFetcherRouter)), "finders")

  // Yay!
  log.info("Initialized database, finders and fetchers")

  def receive =
  {
    case AddEventsToDatabase(query) => finderRouter forward EventFinder.FindEvents(query)
  }
}

object EventMaster
{
  case class AddEventsToDatabase(query: String)
}