package com.fjordev.eventgraph.actor

import scala.collection.mutable
import scala.collection.JavaConversions._
import scala.concurrent.duration.FiniteDuration

import akka.actor.{ActorLogging, Actor, Cancellable}

import facebook4j.internal.org.json.JSONArray

import com.fjordev.eventgraph.util.FacebookAPI

abstract class FqlBatcher(batchSize: Int, maxWait: FiniteDuration) extends Actor with ActorLogging
{
  private val queries = mutable.Map[String, String]()

  private val apiClient = FacebookAPI.get()

  private var timeout: Cancellable = null

  log.info("batch={} maxWait={} key={}", batchSize, maxWait, apiClient.getOAuthAccessToken.getToken.substring(0, 32) + "XXX")

  def addQuery(id: Long, fql: String)
  {
    if(queries.size == 0)
    {
      import context._
      timeout = context.system.scheduler.scheduleOnce(maxWait) { processBatch() }
    }
    else if(queries.size == batchSize)
    {
      processBatch()
    }

    queries(id.toString) = fql
  }

  def processBatch()
  {
    if(timeout != null)
    {
      timeout.cancel()
    }

    log.debug("executing {} queries", queries.size)

    val results = apiClient.executeMultiFQL(queries)
    results.foreach(kv => processQueryResults(kv._1.toLong, kv._2))

    queries.clear()
  }

  def processQueryResults(id: Long, result: JSONArray)
}