package com.fjordev.eventgraph.util

import scala.collection.mutable

import facebook4j.{Facebook, FacebookException}

import org.joda.time.DateTime

import com.fjordev.eventgraph.model.{Gender, AttendingUser, Event}

trait EventFetching
{
  // Allows operator syntax
  def fetchEvent(eventId: Long): Option[Event] = fetchEvent(eventId)

  def fetchEvent(eventId: Long, facebook: Facebook = FacebookAPI.get()): Option[Event] =
  {
    try
    {
      // Retrieve event data
      val event = facebook.getEvent(eventId.toString)

      // Query and parse attending users
      val users = facebook.executeFQL("SELECT uid,first_name,last_name,sex FROM user WHERE uid IN(SELECT uid FROM event_member WHERE eid=" + eventId + " AND rsvp_status='attending' LIMIT 999999) LIMIT 999999")
      val attending = mutable.Buffer[AttendingUser]()
      for (i <- (0 until users.length))
      {
        val user = users.getJSONObject(i)
        attending += AttendingUser(
          user.getString("uid").toLong,
          user.getString("first_name"),
          user.getString("last_name"),
          Gender.parse(user.getString("sex")))
      }

      // Return immutable model
      Some(Event(
        event.getId.toLong,
        event.getName,
        event.getDescription,
        new DateTime(event.getStartTime),
        if (event.getEndTime != null) new DateTime(event.getEndTime) else null,
        event.getLocation,
        null,
        0,
        attending))
    }
    catch
      {
        case e: FacebookException => println(s"Error fetching/parsing event data for $eventId, exception = $e")

          None
      }
  }
}
