package com.fjordev.eventgraph.view

import scala.collection.mutable.StringBuilder

import com.fjordev.eventgraph.model.{Gender, Event}

trait EventPresenting
{
  // Allows operator syntax
  def presentEvent(event: Event): Unit = presentEvent(event, false)

  def presentEvent(event: Event, listAttendees: Boolean = false)
  {
    // Basic info
    val view = new StringBuilder()
    view ++= s"'${event.name}' @ ${event.location} from ${event.startTime} - ${event.endTime}\n"

    // Do some math on the attendees
    val (males, females) = event.attending.partition(_.gender == Gender.Male)
    val maleRatio = (males.length.toFloat / (males.length + females.length).toFloat)
    val femaleRatio = 1 - maleRatio
    view ++= s"Attending: ${event.attending.size} [males: ${males.size} (${maleRatio * 100}%) | females: ${females.size} (${femaleRatio * 100}%)]\n"

    // List the attendees?
    if(listAttendees)
    {
      for (user <- event.attending)
      {
        view ++= s"#${user.id} - ${user.firstName} ${user.lastName} (${user.gender})\n"
      }
    }

    //val nameFrequencies = event.attending.groupBy(_.firstName).mapValues(_.size)

    // Done!
    println(view)
  }
}
