package com.fjordev.popbird.internal

import akka.actor.{PoisonPill, Actor, ActorLogging, Props}

import ApplicationActor._
import com.fjordev.popbird.{Account, Application}

class ApplicationActor(app: Application) extends Actor with ActorLogging
{
  val clientFactory = new TwitterClientFactory(app)

  log.info(s"Hello world, I am '${app.name}' with ${app.accounts.size} accounts and my API key is ${app.key}")

  def receive =
  {
    case GetAccount(token, secret) =>
    {
      // Get/create account data
      val userId = token.substring(0, token.indexOf('-')).toLong
      val account = app.accounts.getOrElseUpdate(userId,
      {
        log.info(s"Creating new account for Twitter user #$userId...")

        new Account(app, token, secret)
      })

      // Get/create actor
      val props = Props(classOf[AccountActor], account, clientFactory)
      context.actorOf(props, name = account.userId.toString)

      // Return IMMUTABLE account data
      sender ! account
    }

    case RemoveAccount(userId) =>
    {
      app.accounts.get(userId) match
      {
        case Some(account) =>
        {
          app.accounts.remove(userId)

          log.info(s"Successfully removed $account!")
        }
      }
      app.accounts.remove(userId).map(account => context.actorSelection(account.userId.toString) ! PoisonPill)
    }
  }
}

object ApplicationActor
{

  /**
   * Retrieve a copy of the data of a given account.
   */
  case class GetAccount(token: String, secret: String)

  /**
   * Flag the specified account as 'deleted' and halt all related business logic.
   * @param userId The Twitter user ID of the account to stop.
   */
  case class RemoveAccount(userId: Long)

}