package com.fjordev.popbird.internal

import twitter4j.{Twitter, TwitterFactory}
import twitter4j.conf.ConfigurationBuilder
import twitter4j.auth.AccessToken

import com.fjordev.popbird.{Account, Application}

/**
 * A {@link Application} specific factory that creates {@link Twitter} clients for {@link Account}s.
 * @param app The Twitter third party application to create clients for.
 */
class TwitterClientFactory(app: Application)
{
  private val factory = new TwitterFactory(new ConfigurationBuilder().setOAuthConsumerKey(app.key).setOAuthConsumerSecret(app.secret).build())

  def newClient(account: Account): Twitter =
  {


    val client = factory.getInstance(new AccessToken(account.token, account.secret))

    // PARANOID: remove traces of Twitter4J in the headers (X-Twitter-Client, etc)
    Twitter4JUtil.neuterHttpHeaders(client)

    client
  }
}
