package net.nillus.habbo.communication.messages.incoming.catalog;

import net.nillus.habbo.communication.ClientMessage;
import net.nillus.habbo.communication.EServerMessage;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.core.Logger;
import net.nillus.habbo.game.catalog.Catalog;
import net.nillus.habbo.game.catalog.CatalogProduct;
import net.nillus.habbo.game.items.EItemFlags;
import net.nillus.habbo.game.users.UserManager;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.util.SecurityUtil;

public class PurchaseGiftHandler implements RequestHandler
{
	@Override
	@SuppressWarnings("unused")
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Parse purchase info
		int pageId = request.readInt();
		int productId = request.readInt();
		String purchaseData = SecurityUtil.filterStr(request.readStr(), false); // Preserve linebreaks
		String giftUsername = SecurityUtil.filterStr(request.readStr());
		String giftNote = SecurityUtil.filterStr(request.readStr(), false); // Preserve linebreaks

		// Valid product?
		CatalogProduct product = Catalog.getInstance().getProduct(productId);
		if(product == null || !product.isPurchasableBy(session.getUser()) || !product.isPhysicalItem() || product.itemDefinition.is(EItemFlags.NOT_GIFTABLE))
		{
			Logger.getInstance().warning(this, String.format("User #%d tried to gift-purchase product #%d from the Catalog, which is not existent/available/giftable by this user.", session.getUser().id, productId));
			return;
		}
		
		// Receiver exists?
		int receiverId = UserManager.getInstance().getUserId(giftUsername);
		if(receiverId == 0)
		{
			response.init(EServerMessage.PurchaseGiftError);
			response.appendInt(1); // "User does not exist"
			response.appendStr(giftUsername);
			session.send(response);
			return;
		}

		// Issue purchase
		if(!Catalog.getInstance().purchaseAsGift(product, session.getUser(), purchaseData, receiverId, giftNote))
		{
			session.sendNotif(String.format("Purchase of catalog product #%d failed, you will not be billed. Please try again in a few minutes. If this keeps happening, please contact Support.", product.id));
			return;
		}
		
		// Bill the purchasing user for gifted product
		Catalog.getInstance().billUser(session, product);
		
		// Purchase okay!
		response.init(EServerMessage.PurchaseOK);
		response.appendInt(product.itemDefinition.id);
		response.appendStr(product.code);
		response.appendInt(product.costCredits);
		response.appendInt(product.costPixels);
		response.appendInt(product.costQuestPoints);
		response.appendInt(1);
		response.appendStr(product.getType());
		response.appendInt(product.itemDefinition.displayId);
		response.appendStr("");
		response.appendInt(1);
		response.appendInt(-1);
		response.appendStr("");
		session.send(response);
	}
}
