package net.nillus.habbo.communication.messages.incoming.inventory.items;

import net.nillus.habbo.communication.ClientMessage;
import net.nillus.habbo.communication.EServerMessage;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.game.items.Item;
import net.nillus.habbo.game.items.impl.DecorationItem;
import net.nillus.habbo.game.rooms.instances.RoomInstance;
import net.nillus.habbo.sessions.Session;

public class ApplyRoomDecorationHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to apply decoration?
		if(session.getRoomUser() == null || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Verify decoration item
		Item item = session.getItemInventory().getItem(request.readInt());
		if(item == null || !(item instanceof DecorationItem))
		{
			return;
		}

		// Remove from inventory and delete
		session.getItemInventory().removeItem(item.id);
		item.delete();
		
		// Apply new props
		RoomInstance room = session.getRoomInstance();
		if(item.definition.data.equals("wallpaper"))
		{
			room.getInfo().wallpaper = item.state;
		}
		else if(item.definition.data.equals("floor"))
		{
			room.getInfo().floor = item.state;
		}
		else if(item.definition.data.equals("landscape"))
		{
			room.getInfo().landscape = item.state;
		}
		room.getInfo().saveDecoration();

		// Notify room
		response.init(EServerMessage.RoomProperty);
		response.appendStr(item.definition.sprite);
		response.appendStr(item.state);
		room.broadcast(response);
	}
}
