package net.nillus.habbo.communication.messages.incoming.room.engine;

import net.nillus.habbo.communication.ClientMessage;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.game.items.Item;
import net.nillus.habbo.game.items.impl.PostItItem;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.sessions.SessionManager;
import net.nillus.habbo.util.SecurityUtil;

public class SavePostItHandler implements RequestHandler
{
	private final static String[] ALLOWED_COLORS = new String[] { "FFFF33", "FF9CFF", "9CCEFF", "9CFF9C" };
	
	public boolean colorIsValid(String color)
	{
		for(int x = 0; x < ALLOWED_COLORS.length; x++)
		{
			if(ALLOWED_COLORS[x].equals(color))
			{
				return true;
			}
		}
		return false;
	}
	
	public String getDefaultColor()
	{
		return ALLOWED_COLORS[0];
	}
	
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Get item
		Item item = session.getRoomInstance().getItem(request.readInt());
		if(item != null && item instanceof PostItItem)
		{
			// Parse new data
			String[] info = request.readStr().split(" ", 2);
			String color = info[0];
			String text = SecurityUtil.filterStr(info[1], false);
			
			// Validate new color
			if(!this.colorIsValid(color))
			{
				// Blacklist IP permanently and disconnect, waar rook is, is vuur
				SessionManager.getInstance().endSession(session.id, String.format("post.it scripting - user #%d (%s)", session.getUser().id, session.getUser().username), true);
				return;
			}
			
			// If user hasn't got room rights...
			if(!session.getRoomUser().hasRoomRights)
			{
				// Then only allow extending of the text (previously this check was clientside)
				if(!item.data.startsWith(text))
				{
					return;
				}
			}
			
			// Save item
			item.data = text;
			item.setState(color);
		}
	}
}
