package habbod.communication;

import habbod.net.codec.Base64Encoding;
import habbod.net.codec.WireEncoding;

public final class ClientMessage
{
	public final static int HEADER_LENGTH = 3;
	public final static int TYPE_LENGTH = 2;
	
	public int type;
	private byte[] payload;
	private int pointer;

	public ClientMessage()
	{
		
	}
	
	public ClientMessage(byte[] bytes, int length) {
		// TODO Auto-generated constructor stub
	}

	public int getType()
	{
		return this.type;
	}
	
	public void setType(int type)
	{
		this.type = type;
	}
	
	public byte[] getPayload()
	{
		return this.payload;
	}
	
	public void setPayload(byte[] payload)
	{
		this.payload = payload;
	}
	
	public final String getBody()
	{
		return new String(this.payload);
	}

	public final int length()
	{
		return this.payload.length;
	}

	public final int remaining()
	{
		return this.payload.length - this.pointer;
	}
	
	public final void reset()
	{
		this.pointer = 0;
	}

	public final void advance(int n)
	{
		this.pointer += n;
	}

	public final boolean contains(byte b)
	{
		for(int x = 0; x < this.payload.length; x++)
		{
			if(this.payload[x] == b)
			{
				return true;
			}
		}
		return false;
	}

	private final byte[] readBytes(int n)
	{
		byte[] data = new byte[n];
		for(int x = 0; x < n; x++, this.pointer++)
		{
			data[x] = this.payload[this.pointer];
		}
		return data;
	}

	public final boolean readFixedBool()
	{
		return(this.payload[this.pointer++] == 'A');
	}

	public final int readFixedInt()
	{
		return Base64Encoding.decodeInt(new byte[] { this.payload[this.pointer++], this.payload[this.pointer++] });
	}

	public final String readStr()
	{
		return new String(this.readBytes(this.readFixedInt()));
	}
	
	public final String[] readStrArray()
	{
		String[] arr = new String[this.readInt()];
		for(int x = 0; x < arr.length; x++)
		{
			arr[x] = this.readStr();
		}
		return arr;
	}

	public final int readParseFixedInt()
	{
		try
		{
			return Integer.parseInt(this.readStr());
		}
		catch(NumberFormatException ex)
		{
			return 0;
		}
	}

	public final Boolean readBool()
	{
		return(this.remaining() > 0 && this.payload[this.pointer++] == WireEncoding.POSITIVE);
	}

	public final int readInt()
	{
		byte[] bytes = new byte[this.remaining() < WireEncoding.MAX_INTEGER_BYTE_AMOUNT ? this.remaining() : WireEncoding.MAX_INTEGER_BYTE_AMOUNT];
		for(int x = 0, y = this.pointer; x < bytes.length; x++, y++)
		{
			bytes[x] = this.payload[y];
		}
		int[] ret = WireEncoding.decodeInt(bytes);
		this.pointer += ret[1];
		return ret[0];
	}

	public final String header()
	{
		return new String(Base64Encoding.encodeInt(this.type, TYPE_LENGTH));
	}

	@Override
	public final String toString()
	{
		return this.header() + this.getBody();
	}
}
