package habbod.core;

import habbod.ServerInstance;

public final class StatsMonitor extends Thread
{
	public final static int UPDATE_INTERVAL_SECONDS = 30;
	
	private ServerInstance instance;
	private long updateIntervalSeconds;
	private int peakOnlineUsers;
	
	public StatsMonitor(ServerInstance instance)
	{
		this.setName("StatsMonitor");
		this.setPriority(Thread.MIN_PRIORITY);
		this.instance = instance;
		this.updateIntervalSeconds = UPDATE_INTERVAL_SECONDS;
	}
	
	public void setUpdateInterval(int seconds)
	{
		this.updateIntervalSeconds = seconds;
	}
	
	public void reloadPeakOnlineUsers()
	{
		this.peakOnlineUsers = ServerInstance.get().getDb().queryInt("SELECT onlineusers_peak FROM system_monitor");
	}
	
	public void resetStatistics()
	{
		this.instance.getDb().execute("UPDATE users SET online=0");
		this.instance.getDb().execute("UPDATE rooms SET users_now=0");
	}
	
	public void updateFromSnapshot()
	{
		// Gather system stats
		long memUsage = this.instance.getMemoryUsage();
		int dbConns = this.instance.getDb().getConnMgr().count();

		// Gather game stats
		int onlineUsers = this.instance.getSessionMgr().authedSessions();
		int roomInstances = this.instance.getGameMgr().getRoomMgr().getAllRoomInstances().size();
		
		// Update peak?
		if(onlineUsers > this.peakOnlineUsers)
		{
			this.peakOnlineUsers = onlineUsers;
		}
		
		// Update stats
		ServerInstance.get().getDb().execute("UPDATE system_monitor SET onlineusers=" + onlineUsers + ",onlineusers_peak=" + this.peakOnlineUsers + ",roominstances=" + roomInstances + ",memusage=" + memUsage + ",dbconnections=" + dbConns);
	}
	
	@Override
	public void run()
	{
		while(!Thread.interrupted())
		{
			// Sleep...
			try
			{
				Thread.sleep(this.updateIntervalSeconds * 1000);
			}
			catch (InterruptedException e)
			{
				
			}
			
			// Update!
			this.updateFromSnapshot();
		}
	}
}
