package net.nillus.habbo.communication.messages.incoming.inventory.badges;

import java.util.Map.Entry;

import mod_badges.BadgeInventory;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.messages.incoming.ClientMessage;
import net.nillus.habbo.communication.messages.outgoing.EServerMessage;
import net.nillus.habbo.communication.messages.outgoing.ServerMessage;
import net.nillus.habbo.sessions.Session;

public class SetActivatedBadgesHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Get badge inventory
		BadgeInventory badges = session.get("badges");
		if(badges == null)
		{
			return;
		}

		// Clear current slots
		badges.clearEquipped();

		// Parse new slot content from the request
		while(request.remaining() > 0)
		{
			// Parse this slot
			byte slot = (byte) request.readInt();
			String badge = request.readStr();

			// Attempt to equip this badge
			badges.equip(slot, badge);
		}

		// Force update on logout
		badges.requireSave();

		// Build update msg
		response.init(EServerMessage.UserBadges);
		response.appendInt(session.getUser().id);
		response.appendInt(badges.getEquipped().size());
		for(Entry<Byte, String> badge : badges.getEquipped().entrySet())
		{
			response.appendInt(badge.getKey());
			response.appendStr(badge.getValue());
		}

		// Send update msg to client(s)
		if(session.inRoom())
		{
			session.getRoomInstance().broadcast(response);
		}
		else
		{
			session.send(response);
		}
	}
}
