package net.nillus.habbo.communication.messages.incoming.room.engine;

import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.messages.incoming.ClientMessage;
import net.nillus.habbo.communication.messages.outgoing.ServerMessage;
import net.nillus.habbo.items.EItemFlags;
import net.nillus.habbo.items.Item;
import net.nillus.habbo.sessions.Session;

public class PlaceItemHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to place items in this room?
		if(!session.inRoom() || !session.getRoomUser().hasRoomRights)
		{
			return;
		}

		// All data is in one string... weird
		String[] data = request.readStr().split(" ");
		int itemId = Integer.parseInt(data[0]);

		// Validate item from inventory
		Item item = session.getItemInventory().getItem(itemId);
		if(item == null)
		{
			return;
		}

		// What kind of item is being placed?
		boolean placementOK = false;
		if(item.is(EItemFlags.FLOORITEM))
		{
			int tileX = Integer.parseInt(data[1]);
			int tileY = Integer.parseInt(data[2]);
			byte rotation = Byte.parseByte(data[3]);
			placementOK = session.getRoomInstance().placeFloorItem(item, tileX, tileY, rotation, session);
		}
		else if(item.is(EItemFlags.WALLITEM))
		{
			String position = data[1] + " " + data[2] + " " + data[3];
			placementOK = session.getRoomInstance().placeWallItem(item, position, session);
		}

		// Placement okay?
		if(placementOK)
		{
			session.getItemInventory().removeItem(item.id);
		}
	}
}
