package net.nillus.habbo.core;

import java.io.FileInputStream;
import java.util.Properties;

public final class PropertiesBox
{
	private Properties props;

	public PropertiesBox()
	{
		this.props = new Properties();
	}

	public boolean load(String filePath)
	{
		try
		{
			this.props.load(new FileInputStream(filePath));
			return true;
		}
		catch(Exception ex)
		{
			return false;
		}
	}

	public String get(String propName)
	{
		return this.get(propName, null);
	}

	public String get(String propName, String valDefault)
	{
		return this.props.getProperty(propName, valDefault);
	}

	public int getInt(String propName)
	{
		return this.getInt(propName, 0);
	}

	public int getInt(String propName, int valDefault)
	{
		if(this.props.containsKey(propName))
		{
			try
			{
				return Integer.parseInt(this.props.getProperty(propName, ""));
			}
			catch(NumberFormatException ex)
			{

			}
		}
		return valDefault;
	}

	public int size()
	{
		return this.props.size();
	}
	
	/* Singleton */
	private final static PropertiesBox instance = new PropertiesBox();

	public final static PropertiesBox getInstance()
	{
		return instance;
	}
}
