package net.nillus.habbo.core;

import net.nillus.habbo.habbod;
import net.nillus.habbo.game.rooms.RoomManager;
import net.nillus.habbo.sessions.SessionManager;

public final class StatsMonitor extends Thread
{
	public final static int UPDATE_INTERVAL_SECONDS = 30;
	
	private long updateIntervalSeconds;
	private int peakOnlineUsers;
	
	public StatsMonitor()
	{
		this.setName("StatsMonitor");
		this.setPriority(Thread.MIN_PRIORITY);
		this.updateIntervalSeconds = UPDATE_INTERVAL_SECONDS;
	}
	
	public void setUpdateInterval(int seconds)
	{
		this.updateIntervalSeconds = seconds;
	}
	
	public void reloadPeakOnlineUsers()
	{
		this.peakOnlineUsers = habbod.getInstance().getDb().queryInt("SELECT onlineusers_peak FROM system_monitor");
	}
	
	public void resetStatistics()
	{
		habbod.getInstance().getDb().execute("UPDATE users SET online=0");
		habbod.getInstance().getDb().execute("UPDATE rooms SET users_now=0");
	}
	
	public void updateFromSnapshot()
	{
		// Gather system stats
		long memUsage = habbod.getInstance().getMemoryUsage() / 1024;
		int dbConns = habbod.getInstance().getDb().getConnMgr().count();

		// Gather game stats
		int onlineUsers = SessionManager.getInstance().authedSessions();
		int roomInstances = RoomManager.getInstance().getAllRoomInstances().size();
		
		// Update peak?
		if(onlineUsers > this.peakOnlineUsers)
		{
			this.peakOnlineUsers = onlineUsers;
		}
		
		// Update stats
		habbod.getInstance().getDb().execute("UPDATE system_monitor SET onlineusers=" + onlineUsers + ",onlineusers_peak=" + this.peakOnlineUsers + ",roominstances=" + roomInstances + ",memusage=" + memUsage + ",dbconnections=" + dbConns);
	}
	
	@Override
	public void run()
	{
		while(!Thread.interrupted())
		{
			// Sleep...
			try
			{
				Thread.sleep(this.updateIntervalSeconds * 1000);
			}
			catch (InterruptedException e)
			{
				
			}
			
			// Update!
			this.updateFromSnapshot();
		}
	}
}
