package habbod.game;

import habbod.ServerInstance;
import habbod.game.access.AccessControl;
import habbod.game.catalog.Catalog;
import habbod.game.catalog.VoucherManager;
import habbod.game.help.HelpTool;
import habbod.game.items.ItemManager;
import habbod.game.items.pets.PetManager;
import habbod.game.messenger.MessengerService;
import habbod.game.moderation.ModerationCenter;
import habbod.game.moderation.chatlog.ChatlogService;
import habbod.game.rooms.RoomManager;
import habbod.game.rooms.navigator.RoomEventManager;
import habbod.game.rooms.navigator.RoomNavigator;
import habbod.game.users.UserManager;
import habbod.game.users.achievements.AchievementManager;
import habbod.game.users.subscriptions.SubscriptionManager;

public class GameManager
{
	private ServerInstance instance;
	
	private AccessControl accessCtrl;
	private UserManager userMgr;
	private MessengerService messengerSvc;
	private AchievementManager achievementMgr;
	private SubscriptionManager subscriptionMgr;
	
	private ModerationCenter moderationCenter;
	private ChatlogService chatlogSvc;
	private HelpTool helptool;
	
	private ItemManager itemMgr;
	private PetManager petMgr;
	private Catalog catalog;
	private VoucherManager voucherMgr;
	
	private RoomManager roomMgr;
	private RoomNavigator navigator;
	private RoomEventManager roomEventMgr;
	
	public GameManager(ServerInstance instance)
	{
		this.instance = instance;
	}
	
	public void start()
	{
		// Load accesscontrol
		this.accessCtrl = new AccessControl();
		this.accessCtrl.reloadUserRoles();
		
		// Load usermanager
		this.userMgr = new UserManager();
		
		// Load messenger service
		this.messengerSvc = new MessengerService();
		
		// Load achievement manager
		this.achievementMgr = new AchievementManager();
		this.achievementMgr.reload();
		
		// Load subscription manager
		this.subscriptionMgr = new SubscriptionManager();
		this.subscriptionMgr.reload();
		
		// Load moderation center
		this.moderationCenter = new ModerationCenter();
		this.moderationCenter.loadPendingTickets();
		
		// Load chatlog service
		this.chatlogSvc = new ChatlogService();
		this.chatlogSvc.start();
		
		// Load helptool
		this.helptool = new HelpTool();
		this.helptool.reload();
		
		// Load item manager
		this.itemMgr = new ItemManager();
		this.itemMgr.reloadDefinitions();
		
		// Load pet manager
		this.petMgr = new PetManager();
		
		// Load catalog
		this.catalog = new Catalog();
		this.catalog.reload();
		
		// Load voucher manager
		this.voucherMgr = new VoucherManager();

		// Create RoomManager, spawn initial workers and reload model data
		int numRoomWorkers = this.instance.getProperties().getInt("rooms.workers", Runtime.getRuntime().availableProcessors());
		this.roomMgr = new RoomManager(numRoomWorkers);
		this.roomMgr.reloadModels();

		// Load Navigator, cache stuff, etc
		int navigatorRefreshTime = this.instance.getProperties().getInt("rooms.navigator.refreshtime", 5);
		this.navigator = new RoomNavigator(navigatorRefreshTime);
		this.navigator.reload();
		
		// Load room event manager
		this.roomEventMgr = new RoomEventManager();
	}
	
	public void stop()
	{
		this.roomMgr.shutdown();
	}
	
	public AccessControl getAccessControl()
	{
		return this.accessCtrl;
	}
	
	public UserManager getUserMgr()
	{
		return this.userMgr;
	}
	
	public MessengerService getMessengerSvc()
	{
		return this.messengerSvc;
	}
	
	public AchievementManager getAchievementMgr()
	{
		return this.achievementMgr;
	}
	
	public SubscriptionManager getSubscriptionMgr()
	{
		return this.subscriptionMgr;
	}
	
	public ModerationCenter getModerationCenter()
	{
		return this.moderationCenter;
	}
	
	public ChatlogService getChatlogSvc()
	{
		return this.chatlogSvc;
	}
	
	public HelpTool getHelpTool()
	{
		return this.helptool;
	}
	
	public ItemManager getItemMgr()
	{
		return this.itemMgr;
	}
	
	public PetManager getPetManager()
	{
		return this.petMgr;
	}
	
	public Catalog getCatalog()
	{
		return this.catalog;
	}
	
	public VoucherManager getVoucherMgr()
	{
		return this.voucherMgr;
	}
	
	public RoomManager getRoomMgr()
	{
		return this.roomMgr;
	}
	
	public RoomNavigator getNavigator()
	{
		return this.navigator;
	}
	
	public RoomEventManager getRoomEventMgr()
	{
		return this.roomEventMgr;
	}
}
