package habbod.game.items;

import habbod.game.rooms.instances.RoomAvatar;
import habbod.game.rooms.instances.RoomInstance;
import habbod.game.rooms.instances.RoomTile;
import habbod.game.rooms.instances.RoomUser;
import habbod.game.rooms.items.RoomItem;
import habbod.sessions.Session;

public abstract class ItemInteractor<I extends RoomItem>
{
	private I item;
	private int pulseTimer;
	
	public final I getItem()
	{
		return this.item;
	}
	
	public final void setItem(I item)
	{
		this.item = item;
	}
	
	public final RoomInstance getRoom()
	{
		return this.item.getRoom();
	}
	
	public final boolean requiresPulse()
	{
		return(this.pulseTimer > 0);
	}

	protected final boolean hasTimeout()
	{
		return this.requiresPulse();
	}

	protected final void setTimeout(int pulses)
	{
		this.pulseTimer = pulses;
	}

	protected final void clearTimeout()
	{
		this.pulseTimer = 0;
	}
	
	public final void pulse()
	{
		// Pulse!
		this.pulseTimer--;
		this.onPulse();

		// End of timeout?
		if(this.pulseTimer <= 0)
		{
			this.clearTimeout();
			this.onTimeout();
		}
	}
	
	public final void clear()
	{
		this.onClear();
		this.clearTimeout();
		this.setItem(null);
	}
	
	/** Triggers */
	protected void onClear()
	{
		
	}
	
	protected void onPulse()
	{
		
	}
	
	protected void onTimeout()
	{
		
	}
	
	protected void onRoomAdd(RoomInstance room)
	{
		
	}

	protected void onRoomRemove(RoomInstance room)
	{
		
	}

	public boolean onPlace(Session session, RoomInstance room)
	{
		return true;
	}

	public boolean onPickup(Session session)
	{
		return true;
	}
	
	public void onGenerateMapTile(RoomTile tile)
	{
		
	}
	
	public void onTrigger(Session session, int instruction)
	{
		
	}
	
	public void onAvatarStepsOn(RoomAvatar avatar)
	{
		
	}
	
	public void onAvatarStepsOff(RoomAvatar avatar)
	{
		
	}

	public boolean onToggle(RoomUser user, int instruction) {
		// TODO Auto-generated method stub
		return false;
	}
}
