package habbod.game.items;

import habbod.ServerInstance;
import habbod.communication.EServerMessage;
import habbod.communication.ServerMessage;
import habbod.sessions.Session;
import habbod.storage.DatabaseResult;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;


public final class ItemInventory
{
	private final Session session;
	private final Map<Integer, InventoryItem> items;

	public ItemInventory(Session session)
	{
		this.session = session;
		this.items = new LinkedHashMap<Integer, InventoryItem>(); // Use LinkedHashMap to preserve order (and don't use natural sorting on item IDs)
	}

	public void clear()
	{
		this.items.clear();
	}

	public void reload()
	{
		// Clear old items
		this.clear();

		// Reload items
		DatabaseResult result = ServerInstance.get().getDb().query("SELECT * FROM items WHERE owner_id=" + this.session.getUser().id + " AND room_id=0");
		while(result.row())
		{
			InventoryItem item = new InventoryItem();
			item.id = result.getInt("id");
			item.definition = ItemManager.getInstance().getDefinition(result.getInt("definition_id"));
			item.state = result.getString("state");
			item.data = result.getString("data");
			if(item != null)
			{
				this.items.put(item.id, item);
			}
		}
		result.release();
	}

	public boolean containsItem(int itemId)
	{
		return this.items.containsKey(itemId);
	}

	public InventoryItem getItem(int itemId)
	{
		return this.items.get(itemId);
	}

	public void addItem(InventoryItem item, boolean save)
	{
		item.update(this.session.getUser().id);
		this.items.put(item.id, item);
	}

	public void removeItem(int itemId)
	{
		this.items.remove(itemId);
		
		// Notify user
		ServerMessage msg = new ServerMessage(EServerMessage.InventoryRemove);
		msg.appendInt(itemId);
		this.session.send(msg);
	}

	public void refresh()
	{
		this.session.send(new ServerMessage(EServerMessage.InventoryUpdate));
	}

	public final Collection<InventoryItem> getItems()
	{
		return this.items.values();
	}
}
