package habbod.game.rooms.items;

import habbod.ServerInstance;
import habbod.communication.ServerMessage;
import habbod.game.items.EItemFlags;
import habbod.game.rooms.Room;
import habbod.game.rooms.instances.RoomAvatar;
import habbod.game.rooms.instances.RoomTile;
import habbod.game.rooms.instances.RoomUser;
import habbod.game.rooms.items.wired.WiredTrigger;
import habbod.game.rooms.items.wired.triggers.WiredUserStepsOffItemTrigger;
import habbod.game.rooms.items.wired.triggers.WiredUserStepsOnItemTrigger;
import habbod.storage.SQLHelper;

import java.util.List;
import java.util.ArrayList;


public abstract class RoomFloorItem extends RoomItem
{
	public int x;
	public int y;
	public float z;
	public byte rotation;
	
	private List<WiredTrigger> triggers;
	
	public float getTopHeight()
	{
		return (this.z + this.definition.height);
	}
	
	@Override
	public void refreshState()
	{
		//this.getRoom().broadcast(ObjectDataUpdateMessageComposer.compose(this));
	}
	
	public void addTrigger(WiredTrigger trigger)
	{
		if(this.triggers != null)
		{
			this.triggers = new ArrayList<WiredTrigger>(1);
		}
		this.triggers.add(trigger);
	}
	
	public void removeTrigger(WiredTrigger trigger)
	{
		if(this.triggers != null)
		{
			this.triggers.remove(trigger);
		}
	}
	
	public void executeTriggers(RoomUser user, Class<? extends WiredTrigger> type)
	{
		if(this.triggers != null)
		{
			for(WiredTrigger trigger : this.triggers)
			{
				if(trigger.getClass() == type)
				{
					trigger.execute(user);
				}
			}
		}
	}
	
	public RoomTile getTile()
	{
		return this.getRoom().getMap().getTile(this.x, this.y);
	}
	
	public List<RoomTile> getTiles()
	{
		return this.getRoom().getMap().getAffectedTiles(this.x, this.y, this.definition.length, this.definition.width, this.rotation);
	}
	
	public RoomTile getTileInFront()
	{
		int x = this.x;
		int y = this.y;
		if(this.rotation == 0)
		{
			y--;
		}
		else if(this.rotation == 2)
		{
			x++;
		}
		else if(this.rotation == 4)
		{
			y++;
		}
		else if(this.rotation == 6)
		{
			x--;
		}
		return this.getRoom().getMap().getTile(x, y);
	}

	public RoomTile getTileBehind()
	{
		int x = this.x;
		int y = this.y;
		if(this.rotation == 0)
		{
			y++;
		}
		else if(this.rotation == 2)
		{
			x--;
		}
		else if(this.rotation == 4)
		{
			y--;
		}
		else if(this.rotation == 6)
		{
			x++;
		}
		return this.getRoom().getMap().getTile(x, y);
	}
	
	public void onAvatarStepOn(RoomAvatar avatar)
	{
		// Default item interaction types
		if(this.is(EItemFlags.CAN_SIT_ON_TOP))
		{
			avatar.headRotation = avatar.bodyRotation = this.rotation;
			avatar.addStatus("sit", Float.toString(avatar instanceof RoomUser ? this.definition.height : this.definition.height / 2));
		}
		else if(this.is(EItemFlags.CAN_LAY_ON_TOP))
		{
			avatar.headRotation = avatar.bodyRotation = this.rotation;
			avatar.addStatus("lay", Float.toString(this.definition.height) + " null");
		}
		
		// Raise triggers
		if(avatar instanceof RoomUser)
		{
			this.getRoom().executeTriggers(this.id, (RoomUser)avatar, WiredUserStepsOnItemTrigger.class);
		}
	}

	public void onAvatarStepOff(RoomAvatar avatar)
	{
		// Raise triggers
		if(avatar instanceof RoomUser)
		{
			this.getRoom().executeTriggers(this.id, (RoomUser)avatar, WiredUserStepsOffItemTrigger.class);
		}
	}
	
	@Override
	public void serialize(ServerMessage msg)
	{
		msg.appendInt(this.id);
		msg.appendInt(this.definition.displayId);
		msg.appendInt(this.x);
		msg.appendInt(this.y);
		msg.appendInt(this.rotation);
		msg.appendStr(this.z);
		msg.appendInt(0);
		msg.appendStr(this.state);
		msg.appendInt(-1);
		msg.appendBool(false);
	}
	
	@Override
	public void update()
	{
		Room room = this.getRoom().getInfo();
		ServerInstance.get().getDb().execute("UPDATE items SET owner_id=" + room.ownerId + ",room_id=" + room.id + ",x=" + this.x + ",y=" + this.y + ",z=" + this.z + ",rotation=" + this.rotation + ",state='" + SQLHelper.escape(this.state) + "',data='" + SQLHelper.escape(this.data) + "' WHERE id=" + this.id);
 	}
}
