package habbod.game.rooms.items.wired.actions;

import habbod.communication.EServerMessage;
import habbod.communication.ServerMessage;
import habbod.game.rooms.instances.RoomUser;
import habbod.game.rooms.items.RoomFloorItem;
import habbod.game.rooms.items.wired.WiredAction;

import java.util.List;
import java.util.ArrayList;


import org.json.JSONArray;
import org.json.JSONObject;


public class WiredRestoreItemsToSnapshotsAction extends WiredAction
{
	public final static int PARAM_STATE = 0;
	public final static int PARAM_ROTATION = 1;
	public final static int PARAM_POSITION = 2;
	
	private List<ItemSnapshot> snapshots;

	@Override
	protected int getInterface()
	{
		return 3;
	}
	
	@Override
	protected int getAmountOfParams()
	{
		return 3;
	}
	
	@Override
	public boolean requiresUser()
	{
		return false;
	}
	
	@Override
	public void execute(RoomUser user)
	{
		// Restore all selected items to their saved snapshots
		List<RoomFloorItem> selected = this.getSelectedItems();
		for(ItemSnapshot snapshot : this.snapshots)
		{
			RoomFloorItem item = this.getRoom().getFloorItem(snapshot.itemId);
			if(selected.contains(item))
			{
				// Backup old values
				int oldX = item.x;
				int oldY = item.y;
				float oldZ = item.z;
				
				// Placeholders for new values
				byte newRotation = item.rotation;
				int newX = oldX;
				int newY = oldY;
				
				// Restore what needs restoring
				if(this.getParam(PARAM_STATE) == 1)
				{
					item.setState(snapshot.state);
				}
				if(this.getParam(PARAM_ROTATION) == 1)
				{
					newRotation = snapshot.rotation;
				}
				if(this.getParam(PARAM_POSITION) == 1)
				{
					newX = snapshot.x;
					newY = snapshot.y;
				}
				
				// Move item
				if(this.getRoom().moveFloorItem(item, newX, newY, newRotation, null))
				{
					// Slide over there!
					ServerMessage msg = new ServerMessage(EServerMessage.SlideObjectBundle);
					msg.appendInt(oldX);
					msg.appendInt(oldY);
					msg.appendInt(item.x);
					msg.appendInt(item.y);
					msg.appendInt(1);
					msg.appendInt(item.id);
					msg.appendStr(oldZ);
					msg.appendStr(item.z);
					msg.appendInt(-1);
					this.getRoom().broadcast(msg);
				}
			}
		}
	}

	public void updateSnapshots()
	{
		this.snapshots.clear();
		for(RoomFloorItem item : this.getSelectedItems())
		{
			ItemSnapshot snapshot = new ItemSnapshot();
			snapshot.itemId = item.id;
			if(this.getParam(PARAM_STATE) == 1)
			{
				snapshot.state = item.state;
			}
			if(this.getParam(PARAM_ROTATION) == 1)
			{
				snapshot.rotation = item.rotation;
			}
			if(this.getParam(PARAM_POSITION) == 1)
			{
				snapshot.x = item.x;
				snapshot.y = item.y;
				snapshot.z = item.z;
			}
			this.snapshots.add(snapshot);
		}
		this.saveSettings();
	}
	
	@Override
	protected void onWrap(JSONObject data) throws Exception
	{
		super.onWrap(data);
		
		// Wrap all snapshots
		List<JSONArray> arrays = new ArrayList<JSONArray>(5);
		for(ItemSnapshot snapshot : this.snapshots)
		{
			JSONArray array = new JSONArray();
			array.put(snapshot.itemId);
			array.put(snapshot.x);
			array.put(snapshot.y);
			array.put(snapshot.z);
			array.put(snapshot.rotation);
			array.put(snapshot.state);
			arrays.add(array);
		}
		data.put("mem", arrays);
	}
	
	@Override
	protected void onUnwrap(JSONObject data) throws Exception
	{
		super.onUnwrap(data);
		
		// Unwrap all snapshots
		this.snapshots = new ArrayList<ItemSnapshot>(5);
		JSONArray arrays = data.optJSONArray("mem");
		if(arrays != null)
		{
			for(int x = 0; x < arrays.length(); x++)
			{
				ItemSnapshot snapshot = new ItemSnapshot();
				JSONArray array = arrays.optJSONArray(x);
				if(array != null)
				{
					snapshot.itemId = array.optInt(0);
					snapshot.x = array.optInt(1);
					snapshot.y = array.optInt(2);
					snapshot.z = (float) array.optDouble(3);
					snapshot.rotation = (byte) array.optInt(4);
					snapshot.state = array.optString(5);
					this.snapshots.add(snapshot);
				}
			}
		}
	}
	
	private final class ItemSnapshot
	{
		public int itemId;
		public int x;
		public int y;
		public float z;
		public byte rotation;
		public String state;
	}
}