package habbod.game.rooms.navigator;

import habbod.game.rooms.RoomManager;
import habbod.game.rooms.instances.RoomInstance;
import habbod.sessions.Session;
import habbod.sessions.SessionManager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;


public final class RoomEventManager
{
	public final static int ALL_EVENTS = -1;
	private final static int CATEGORY_AMOUNT = 12;
	
	private final Map<Integer, RoomEvent> events;
	private final Map<Integer, List<RoomEvent>> categories;
	
	public RoomEventManager()
	{
		this.events = new ConcurrentHashMap<Integer, RoomEvent>();
		this.categories = new HashMap<Integer, List<RoomEvent>>(CATEGORY_AMOUNT);
		for(int categoryId = -1; categoryId < CATEGORY_AMOUNT; categoryId++)
		{
			this.categories.put(categoryId, new CopyOnWriteArrayList<RoomEvent>());
		}
	}
	
	public void addEvent(RoomEvent event)
	{
		// Add to listings
		this.events.put(event.room.id, event);
		this.categories.get(ALL_EVENTS).add(event);
		this.categories.get(event.categoryId).add(event);
		
		// Update session
		Session session = SessionManager.getInstance().getSession(event.room.ownerId);
		if(session != null)
		{
			session.setRoomEvent(event);
		}
		
		// Update room instance
		RoomInstance room = RoomManager.getInstance().getRoomInstance(event.room.id, false);
		if(room != null)
		{
			room.setEvent(event);
		}
	}
	
	public void removeEvent(RoomEvent event)
	{
		// Remove from listings
		this.events.remove(event.room.id);
		this.categories.get(ALL_EVENTS).remove(event);
		this.categories.get(event.categoryId).remove(event);
		
		// Update session
		Session session = SessionManager.getInstance().getSession(event.room.ownerId);
		if(session != null)
		{
			session.setRoomEvent(null);
		}
		
		// Update room instance
		RoomInstance room = RoomManager.getInstance().getRoomInstance(event.room.id, false);
		if(room != null)
		{
			room.setEvent(null);
		}
	}
	
	public void updateEventCategory(RoomEvent event, int newCategoryId)
	{
		this.categories.get(event.categoryId).remove(event);
		event.categoryId = newCategoryId;
		this.categories.get(event.categoryId).add(event);
	}
	
	public List<RoomEvent> getEventsForCategory(int categoryId)
	{
		List<RoomEvent> events = this.categories.get(categoryId);
		if(events == null)
		{
			events = new ArrayList<RoomEvent>(0);
		}
		return events;
	}
	
	public List<RoomEvent> getAllEvents()
	{
		return this.getEventsForCategory(ALL_EVENTS);
	}
	
	public RoomEvent getEventForRoom(int roomId)
	{
		return this.events.get(roomId);
	}
	
	public int getCategoryAmount()
	{
		return this.categories.size() - 1;
	}
	
	private final static RoomEventManager instance = new RoomEventManager();
	
	public final static RoomEventManager getInstance()
	{
		return instance;
	}
}
