package habbod.game.util;

/**
 * Validates Habbo figures. This class has been taken from uberEmu by Meth0d. (www.meth0d.org)
 * 
 * @author Roy 'Meth0d' de Jong
 */
public final class FigureValidator
{
	public static boolean figureIsValid(String figure)
	{
		// Actually a figure string to validate?
		if(figure == null || figure.length() == 0)
		{
			return false;
		}

		boolean hasHead = false;
		try
		{
			// Parse the sets from the figure and ensure there are atleast 4 sets
			String[] sets = figure.split("\\.");
			if(sets.length < 4)
			{
				return false;
			}

			// Process all the sets
			for(String set : sets)
			{
				// Parse the parts in this set and ensure there are atleast 3 parts
				String[] parts = set.split("-");
				if(parts.length < 3)
				{
					return false;
				}

				// Validate this part
				String partId = parts[0];
				int type = Integer.parseInt(parts[1]);
				int color = Integer.parseInt(parts[1]);
				if(partId.length() != 2 || type <= 0 || color < 0)
				{
					return false;
				}

				// Is this the head part?
				if(!hasHead && partId.equals("hd"))
				{
					hasHead = true;
				}
			}
		}
		catch(Exception ex)
		{
			// Something is wrong with the figure
			return false;
		}

		// If the check runs to there, then the figure is valid, but only if it has a 'hd' part!
		return hasHead;
	}
}
