package net.nillus.habbo.game.users.subscriptions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.nillus.habbo.habbod;
import net.nillus.habbo.core.Logger;
import net.nillus.habbo.storage.DatabaseResult;
import net.nillus.habbo.storage.SQLHelper;

public final class SubscriptionManager
{
	private Map<String, Subscription> types;
	
	public SubscriptionManager()
	{
		this.types = Collections.emptyMap();
	}
	
	public void reload()
	{
		// Create new collection
		Map<String, Subscription> tmpTypes = new HashMap<String, Subscription>(0);
		
		// Load all subscriptions from database
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT * FROM subscriptions");
		while(result.row())
		{
			// Parse subscription info
			Subscription sub = new Subscription();
			sub.id = result.getString("id");
			sub.name = result.getString("name");
			sub.pricePerMonth = result.getInt("price");
			sub.specialRightsLevel = result.getByte("specialrightslevel");
			
			// Get subscription rights
			DatabaseResult result2 = habbod.getInstance().getDb().query("SELECT `right` FROM subscription_rights WHERE subscription_id='" + SQLHelper.escape(sub.id) + "'");
			while(result2.row())
			{
				sub.rights.add(result2.getString("right"));
			}
			result2.release();
			
			// Get subscription badges
			result2 = habbod.getInstance().getDb().query("SELECT badge FROM subscription_badges WHERE subscription_id='" + SQLHelper.escape(sub.id) + "'");
			while(result2.row())
			{
				sub.badges.add(result2.getString("badge"));
			}
			result2.release();
			
			// Add it!
			tmpTypes.put(sub.id, sub);
			Logger.getInstance().info("SubscriptionMgr", String.format("Loaded '%s': %d right(s) and %d badge(s)", sub.id, sub.rights.size(), sub.badges.size()));
		}
		result.release();
		
		// Set instance field
		this.types = Collections.unmodifiableMap(tmpTypes);
	}
	
	public List<UserSubscription> getSubscriptionsForUser(int userId)
	{
		List<UserSubscription> subs = new ArrayList<UserSubscription>(0);
		
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT subscription_id,activated_at,expires_at FROM user_subscriptions WHERE user_id=" + userId);
		while(result.row())
		{
			UserSubscription sub = new UserSubscription();
			sub.type = this.getSubscriptionType(result.getString("subscription_id"));
			if(sub.type != null)
			{
				sub.activatedAt = result.getTime("activated_at");
				sub.expiresAt = result.getTime("expires_at");
				subs.add(sub);
			}
		}
		result.release();
		
		return subs;
	}
	
	public Subscription getSubscriptionType(String subscriptionId)
	{
		return this.types.get(subscriptionId);
	}
	
	private final static SubscriptionManager instance = new SubscriptionManager();
	
	public final static SubscriptionManager getInstance()
	{
		return instance;
	}
}
