package net.nillus.habbo.game.util;

import java.security.SecureRandom;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Random;

import net.nillus.habbo.game.rooms.instances.RoomInstance;

public final class GameUtil
{
	private final static Random random = new SecureRandom();
	private final static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("dd-MM-yyy");
	private final static SimpleDateFormat DATETIME_FORMATTER = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
	public final static String DATA_DELIMITER = "/";

	public static <T> T getRandomElement(T[] elements)
	{
		return elements[random.nextInt(elements.length)];
	}
	

	public static <T> T getRandomElement(List<T> elements)
	{
		return elements.get(random.nextInt(elements.size()));
	}

	public static int getRandomInt(int min, int max)
	{
		return(min + random.nextInt(max - min + 1));
	}

	public static int getRandomDelta(int min, int max)
	{
		int delta = 0;
		while(delta == 0)
		{
			delta = GameUtil.getRandomInt(min, max);
		}
		return delta;
	}
	
	public final static String formatDate(long millis)
	{
		return GameUtil.formatDate(new Date(millis));
	}

	public final static String formatDate(Date date)
	{
		return DATE_FORMAT.format(date);
	}

	public final static String getCurrentDate()
	{
		return GameUtil.formatDate(new Date());
	}

	public final static String formatDateTime(long millis)
	{
		return GameUtil.formatDateTime(new Date(millis));
	}

	public final static String getCurrentDateTime()
	{
		return GameUtil.formatDateTime(new Date());
	}

	public final static String formatDateTime(Date date)
	{
		return DATETIME_FORMATTER.format(date);
	}

	public final static int secondsToRoomPulses(final int seconds)
	{
		return((seconds * 1000) / RoomInstance.PULSE_INTERVAL);
	}

	public final static int secondsToDays(final int seconds)
	{
		return (int) Math.ceil(seconds / (60 * 60 * 24));
	}

	public final static int daysBetween(final long millis1, long millis2)
	{
		long diff = millis2 - millis1;
		return GameUtil.secondsToDays((int) (diff / 1000));
	}

	public final static int daysSince(final long millis)
	{
		return GameUtil.daysBetween(millis, System.currentTimeMillis());
	}

	public final static int parseInt(String s)
	{
		if(s != null && !s.isEmpty())
		{
			try
			{
				return Integer.parseInt(s);
			}
			catch(Exception ex)
			{

			}
		}
		return 0;
	}
}
