package habbod.communication.messages.incoming.catalog;

import habbod.communication.ClientMessage;
import habbod.communication.EServerMessage;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.game.catalog.Catalog;
import habbod.game.catalog.CatalogPage;
import habbod.game.users.User;
import habbod.sessions.Session;

import java.util.ArrayList;
import java.util.List;


public class GetCatalogIndexHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Request a list of all index pages ('trees') visible to the user
		List<CatalogPage> indexPages = Catalog.getInstance().getVisibleIndexPagesForUser(session.getUser());
		
		// Build the response
		response.init(EServerMessage.CatalogIndex);
		response.appendBool(false);
		response.appendInt(0);
		response.appendInt(0);
		response.appendInt(-1);
		response.appendStr("");
		response.appendInt(indexPages.size());
		for(CatalogPage page : indexPages)
		{
			this.serializePageMetaDataForUser(page, session.getUser(), response);
		}
		session.send(response);
	}

	private void serializePageMetaDataForUser(CatalogPage page, User user, ServerMessage msg)
	{
		// Determine the visible children
		List<CatalogPage> children = new ArrayList<CatalogPage>();
		for(CatalogPage child : page.pages)
		{
			if(child.isVisibleTo(user))
			{
				children.add(child);
			}
		}

		// Serialize metadata of this page + visible children
		msg.appendBool(true);
		msg.appendInt(page.iconColor);
		msg.appendInt(page.iconImage);
		msg.appendInt(page.id);
		msg.appendStr(page.name);
		msg.appendInt(children.size());
		for(CatalogPage child : children)
		{
			this.serializePageMetaDataForUser(child, user, msg);
		}
	}
}
