package habbod.communication.messages.incoming.catalog;

import habbod.communication.ClientMessage;
import habbod.communication.EServerMessage;
import habbod.communication.EServerMessageAlternate;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.core.Logger;
import habbod.game.catalog.Catalog;
import habbod.game.catalog.CatalogProduct;
import habbod.game.items.EItemFlags;
import habbod.game.items.InventoryItem;
import habbod.game.util.SecurityUtil;
import habbod.sessions.Session;

import java.util.List;


public class PurchaseFromCatalogHandler implements RequestHandler
{
	@Override
	@SuppressWarnings("unused")
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Parse purchase info
		int pageId = request.readInt();
		int productId = request.readInt();
		String purchaseData = SecurityUtil.filterStr(request.readStr(), false); // Preserve linebreaks

		// Valid product?
		CatalogProduct product = Catalog.getInstance().getProduct(productId);
		if(product == null || !product.isPurchasableBy(session.getUser()))
		{
			Logger.getInstance().warning(this, String.format("User #%d tried to purchase product #%d from the Catalog, which is not existent/available/purchasable to this user.", session.getUser().id, productId));
			return;
		}

		// Attempt item purchase
		List<InventoryItem> shipping = Catalog.getInstance().purchaseProduct(product, session.getUser(), purchaseData, true);
		if(shipping == null)
		{
			session.sendNotif(String.format("Purchase of catalog product #%d failed, you will not be billed. Please try again in a few minutes. If this keeps happening, please contact Support.", product.id));
			return;
		}

		// Any items?
		if(!shipping.isEmpty())
		{
			// Add them to inventory
			int tab = 0;
			for(InventoryItem item : shipping)
			{
				tab = item.is(EItemFlags.FLOORITEM) ? 1 : 2;
				session.getItemInventory().addItem(item, false);
			}
			session.getItemInventory().refresh();
			
			// Notify of changes
			response.init(EServerMessageAlternate.INVENTORY_NEW_ITEM);
			response.appendBool(true);
			response.appendInt(tab);
			response.appendInt(shipping.size());
			for(InventoryItem item : shipping)
			{
				response.appendInt(item.id);
			}
			session.send(response);
		}
		
		// Bill user for product
		Catalog.getInstance().billUser(session, product);

		// Purchase okay!
		response.init(EServerMessage.PurchaseOK);
		response.appendInt(product.itemDefinition.id);
		response.appendStr(product.code);
		response.appendInt(product.costCredits);
		response.appendInt(product.costPixels);
		response.appendInt(product.costQuestPoints);
		response.appendInt(1);
		response.appendStr(product.getType());
		response.appendInt(product.itemDefinition.displayId);
		response.appendStr("");
		response.appendInt(1);
		response.appendInt(-1);
		response.appendStr("");
		session.send(response);
	}
}
