package habbod.communication.messages.incoming.catalog;

import habbod.communication.ClientMessage;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.game.catalog.VoucherManager;
import habbod.game.users.UserManager;
import habbod.game.util.GameUtil;
import habbod.sessions.Session;

public class RedeemVoucherHandler implements RequestHandler
{
	public final static int VoucherRedeemOk = 212;
	public final static int VoucherRedeemError = 213;
	
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Voucher exists?
		ContentVoucher voucher = VoucherManager.getInstance().getVoucher(request.readStr());
		if(voucher == null)
		{
			response.init(RedeemVoucherHandler.VoucherRedeemError);
			response.appendInt(1);
		}
		else
		{
			// Delete voucher
			VoucherManager.getInstance().deleteVoucher(voucher.code);
			
			// Process payload
			if(voucher.type.equals("credits"))
			{
				// Update credits
				int credits = GameUtil.parseInt(voucher.data);
				session.getUser().credits += credits;
				session.getUser().saveCreditsAndPixels();
				session.sendCreditBalance();
				
				// Log transaction
				UserManager.getInstance().logCreditTransaction(session.getUser().id, credits, "Credit voucher (" + voucher.code + ")");
			}
			
			// Okay!
			response.init(RedeemVoucherHandler.VoucherRedeemOk);
		}
		
		// Send result
		session.send(response);
	}
}
