package habbod.communication.messages.incoming.friendlist;

import habbod.communication.ClientMessage;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.game.messenger.InstantMessenger;
import habbod.game.messenger.MessengerService;
import habbod.game.messenger.MessengerBuddy;
import habbod.sessions.Session;

import java.util.ArrayList;
import java.util.List;



public class SearchUserHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Requires initialized messenger
		InstantMessenger messenger = session.getMessenger();
		if(messenger == null)
		{
			return;
		}

		// Valid criteria?
		String criteria = request.readStr().trim();
		if(criteria.length() == 0) return;

		// Get results and group them
		List<MessengerBuddy> buddies = new ArrayList<MessengerBuddy>();
		List<MessengerBuddy> nonBuddies = new ArrayList<MessengerBuddy>();
		for(MessengerBuddy user : MessengerService.getInstance().getUserSearchResults(criteria))
		{
			// Buddy or non-buddy?
			if(messenger.hasBuddy(user.userId))
			{
				buddies.add(user);
			}
			else
			{
				nonBuddies.add(user);
			}
		}

		// Serialize the results
		response.init(EMessengerMessage.UserSearchResult);
		response.appendInt(buddies.size());
		for(MessengerBuddy user : buddies)
		{
			this.serializeSearchResultUser(user, response);
		}
		response.appendInt(nonBuddies.size());
		for(MessengerBuddy user : nonBuddies)
		{
			this.serializeSearchResultUser(user, response);
		}
		session.send(response);
	}

	private void serializeSearchResultUser(MessengerBuddy user, ServerMessage msg)
	{
		msg.appendInt(user.userId);
		msg.appendStr(user.username);
		msg.appendStr(user.motto);
		msg.appendBool(user.online());
		msg.appendBool(user.inRoom());
		msg.appendStr("");
		msg.appendBool(false);
		msg.appendStr(user.figure);
		msg.appendStr(user.getLastOnlineTime());
		msg.appendStr("");
	}
}
