package habbod.communication.messages.incoming.moderation;

import habbod.communication.ClientMessage;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.game.rooms.RoomManager;
import habbod.game.rooms.RoomVisit;
import habbod.game.users.UserManager;
import habbod.sessions.Session;

import java.util.Date;
import java.util.List;


public class ModGetRoomVisitsHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Has mod?
		if(!session.getUser().rights.contains("fuse_mod"))
		{
			return;
		}
		
		// Get user
		int userId = request.readInt();
		String username = UserManager.getInstance().getUsername(userId);
		if(username == null)
		{
			return;
		}

		// Get last 50 visits
		List<RoomVisit> visits = RoomManager.getInstance().getLastRoomVisitsForUser(userId, 50);
		
		// Build & send response
		response.init(537);
		response.appendInt(userId);
		response.appendStr(username);
		response.appendInt(visits.size());
		for(RoomVisit visit : visits)
		{
			Date entry = new Date(visit.entry);
			response.appendBool(visit.room.isPublicRoom());
			response.appendInt(visit.room.id);
			response.appendStr(visit.room.name);
			response.appendInt(entry.getHours());
			response.appendInt(entry.getMinutes());
		}
		session.send(response);
	}
}
