package habbod.communication.messages.incoming.navigator;

import habbod.communication.ClientMessage;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.game.rooms.instances.RoomInstance;
import habbod.sessions.Session;

import java.util.List;


public class RateGuestRoomHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Get room and 'already rated' list
		RoomInstance room = session.getRoomInstance();
		List<Integer> rated = session.get("ratedRooms");
		if(room == null || rated == null)
		{
			return;
		}

		// Ensure this user can rate this room (not already rated, not owner)
		if(rated.contains(room.getInfo().id) || (session.getUser().id == room.getInfo().ownerId))
		{
			return;
		}

		// Apply rating
		int rating = request.readInt();
		if(rating == -1)
		{
			room.getInfo().rating--;
		}
		else
		{
			room.getInfo().rating++;
		}
		room.getInfo().saveRating();

		// Add to 'rated' list
		rated.add(room.getInfo().id);

		// Notify room
		response.init(345);
		response.appendInt(room.getInfo().rating);
		room.broadcast(response);
	}
}
