package habbod.communication.messages.incoming.room.items;

import habbod.communication.ClientMessage;
import habbod.communication.EServerMessage;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.core.Logger;
import habbod.game.catalog.Catalog;
import habbod.game.catalog.CatalogProduct;
import habbod.game.items.InventoryItem;
import habbod.game.rooms.items.RoomFloorItem;
import habbod.game.util.GameUtil;
import habbod.sessions.Session;

import java.util.List;


public class OpenPresentHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to open presentboxes in this room?
		if(!session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Verify the item
		RoomFloorItem item = session.getRoomInstance().getFloorItem(request.readInt());
		if(item == null || !item.definition.sprite.startsWith("present_"))
		{
			return;
		}

		// Pickup + delete the item
		InventoryItem invItem = session.getRoomInstance().pickupWallItem(item.id, session);
		invItem.delete();
		
		// Get + delete giftinfo for the item
		String[] giftInfo = item.data.split(GameUtil.DATA_DELIMITER, 2);
		if(giftInfo.length != 2)
		{
			Logger.getInstance().warning(session, String.format("Opening presentbox #%d failed: gift info invalid!", item.id));
			return;
		}

		// 'Present OK'
		response.init(219);
		response.appendInt(item.id);
		session.send(response);

		// Redeem contents
		CatalogProduct product = Catalog.getInstance().getProduct(GameUtil.parseInt(giftInfo[0]));
		List<InventoryItem> contents = Catalog.getInstance().purchaseProduct(product, session.getUser(), giftInfo[1], false);
		if(product == null || contents == null)
		{
			session.sendNotif("Invalid presentbox contents, sorry!");
			return;
		}
		
		// Present contents
		response.init(EServerMessage.PresentOpened);
		response.appendStr(product.itemDefinition.getType());
		response.appendInt(product.itemDefinition.displayId);
		response.appendStr(product.itemDefinition.name);
		session.send(response);
		
		// Add to inventory
		for(InventoryItem tmpItem : contents)
		{
			session.getItemInventory().addItem(tmpItem, false);
		}
		session.getItemInventory().refresh();
	}
}
