package habbod.communication.messages.incoming.roomsettings;

import habbod.communication.ClientMessage;
import habbod.communication.EServerMessage;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.game.rooms.Room;
import habbod.net.codec.WireEncoding;
import habbod.sessions.Session;

public class UpdateRoomIconHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to edit the icon?
		if(!(session.inRoom() && session.getRoomUser().hasRoomOwner))
		{
			return;
		}
		Room room = session.getRoomInstance().getInfo();

		// Parse the icon
		request.readInt(); // Layer amount
		int background = request.readInt();
		int foreground = request.readInt();
		int itemAmount = request.readInt();
		int[] itemKeys = new int[itemAmount];
		int[] itemValues = new int[itemAmount];
		for(int x = 0; x < itemAmount; x++)
		{
			itemKeys[x] = request.readInt();
			itemValues[x] = request.readInt();
		}

		// Rebuild and save the data
		StringBuilder sb = new StringBuilder(10);
		sb.append(WireEncoding.encodeIntToStr(background));
		sb.append(WireEncoding.encodeIntToStr(foreground));
		sb.append(WireEncoding.encodeIntToStr(itemAmount));
		for(int x = 0; x < itemAmount; x++)
		{
			sb.append(WireEncoding.encodeIntToStr(itemKeys[x]));
			sb.append(WireEncoding.encodeIntToStr(itemValues[x]));
		}
		room.icon = sb.toString();

		response.init(EServerMessage.RoomIconUpdateResult);
		response.appendInt(room.id);
		response.appendBool(true);
		session.send(response);

		response.init(EServerMessage.RoomSettingsSaved);
		response.appendInt(room.id);
		session.send(response);
	}
}
