package net.nillus.habbo.communication.messages.incoming.moderation;

import net.nillus.habbo.communication.ClientMessage;
import net.nillus.habbo.communication.EServerMessage;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.game.moderation.ModerationActionError;
import net.nillus.habbo.game.moderation.ModerationCenter;
import net.nillus.habbo.game.rooms.Room;
import net.nillus.habbo.game.rooms.RoomManager;
import net.nillus.habbo.game.rooms.instances.RoomInstance;
import net.nillus.habbo.game.rooms.navigator.RoomEvent;
import net.nillus.habbo.game.rooms.navigator.RoomEventManager;
import net.nillus.habbo.sessions.Session;

public class ModPerformRoomActionHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Has mod?
		if(!session.getUser().rights.contains("fuse_mod"))
		{
			return;
		}
		
		// Read info
		int roomId = request.readInt();
		boolean lockRoom = request.readBool();
		boolean inappropriateRoom = request.readBool();
		boolean kickRoom = request.readBool();
		
		// Perform actions
		RoomInstance room = RoomManager.getInstance().getRoomInstance(roomId, false);
		if(room == null)
		{
			return;
		}
		
		// Kick users + stop event?
		if(kickRoom)
		{
			try
			{
				ModerationCenter.getInstance().issueRoomKick(session.getUser().id, room.getInfo().id, null);
			}
			catch(ModerationActionError error)
			{
				session.sendNotif(error.generateNotif());
			}
			
			// Event?
			RoomEvent event = room.getEvent();
			if(event != null)
			{
				RoomEventManager.getInstance().removeEvent(event);
			}
		}
		
		// Room changes?
		if(lockRoom || inappropriateRoom)
		{
			// Lock room?
			if(lockRoom)
			{
				room.getInfo().state = Room.STATE_DOORBELL;
			}
			
			// Change room to inappropriate?
			if(inappropriateRoom)
			{
				room.getInfo().name = "In strijd met de regels";
				room.getInfo().description = "Deze kamer is gesloten door een moderator";
				room.getInfo().tags = new String[0];
			}
			
			// Save room
			room.getInfo().saveSettings();
			
			// Inform all room users to request new room info
			response.init(EServerMessage.RoomInfoUpdated);
			response.appendInt(room.getInfo().id);
			session.getRoomInstance().broadcast(response);
		}
	}
}
