package mod_gift;

import java.util.List;

import net.nillus.habbo.catalog.CatalogProduct;
import net.nillus.habbo.communication.messages.incoming.room.items.OpenPresentHandler;
import net.nillus.habbo.core.Logger;
import net.nillus.habbo.items.Item;
import net.nillus.habbo.items.ItemDefinition;
import net.nillus.habbo.items.ItemManager;
import net.nillus.habbo.items.impl.PresentBoxItem;
import net.nillus.habbo.modules.Module;
import net.nillus.habbo.modules.ModuleInfo;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.sessions.SessionManager;
import net.nillus.habbo.users.User;
import net.nillus.habbo.util.GameUtil;

@ModuleInfo(name = "mod_gift", description = "Enables users to purchase certain catalog products as gifts, wrapping them in gift presentbox item containing the bought items. Presentboxes can be placed in a room and opened to redeem the content.", author = "Nils 'Nillus'", url = "http://modules.openhabbo.org/mod_gift/", version = "1.0.0", dependencies = {})
public final class mod_gift extends Module
{
	public final static char NOT_GIFTABLE = 'O';

	public mod_gift()
	{
		this.getAutoLoader().itemImpl("PresentBox", PresentBoxItem.class);
		
		this.getAutoLoader().msgHandler(78, new OpenPresentHandler());
		this.getAutoLoader().msgHandler(472, new PurchaseGiftHandler());
		this.getAutoLoader().msgHandler(3030, new CanGiftHandler());
	}

	public static boolean purchaseAsGift(CatalogProduct product, User user, String purchaseData, int receiverId, String giftNote)
	{
		// Run pre-purchase logic, filter & apply purchase data
		try
		{
			purchaseData = product.itemDefinition.newInstance().processPurchaseDataToState(user, purchaseData);
		}
		catch(Exception ex)
		{
			purchaseData = Item.ERROR_DATA;
		}
		if(purchaseData != null && purchaseData.equals(Item.ERROR_DATA))
		{
			Logger.getInstance().warning("mod_gift", String.format("Gift-purchase of product #%d by user #%d failed, bad purchase data.", product.id, user.id));
			return false;
		}

		// Get all presentbox definitions, shuffle them
		List<ItemDefinition> availableBoxes = ItemManager.getInstance().getDefinitionsByClass(PresentBoxItem.class);
		
		// Any definitions?
		if(availableBoxes.isEmpty())
		{
			Logger.getInstance().warning("gift", "No item definitions with class PresentBoxItem available! Gifts cannot be purchased.");
			return false;
		}
		
		// Create presentbox, state=giftnote, data=purchasedata
		Item item = GameUtil.getRandomElement(availableBoxes).newInstance();
		item.ownerId = receiverId;
		item.state = "!" + giftNote;
		item.data = (product.id + GameUtil.DATA_DELIMITER + purchaseData);
		ItemManager.getInstance().storeItem(item);

		// Is the receiver online?
		Session receiver = SessionManager.getInstance().getSessionByUserId(receiverId);
		if(receiver != null)
		{
			// Add the item to the inventory component
			receiver.getItemInventory().addItem(item, false);
			receiver.getItemInventory().refresh();

			// Send out a notification (TODO: localize this?)
			receiver.sendNotif("You have received a gift!");
		}

		// Purchase OK!
		return true;
	}
}
