package mod_monitor;

import net.nillus.habbo.habbod;
import net.nillus.habbo.modules.Module;
import net.nillus.habbo.modules.ModuleInfo;
import net.nillus.habbo.rooms.RoomManager;
import net.nillus.habbo.sessions.SessionManager;

@ModuleInfo
(
		name = "mod_monitor",
		description = "Startup cleanup and periodically updating of server runtime stats in the database.", 
		author = "Nils 'Nillus'",
		url = "http://modules.openhabbo.org/mod_monitor/",
		version = "1.0.0",
		dependencies = { }
)
public final class mod_monitor extends Module
{
	private int peakOnlineUsers = 0;
	
	public mod_monitor()
	{
		this.enablePulseTimer(30);
	}
	
	@Override
	protected boolean onLoad()
	{
		this.peakOnlineUsers = habbod.getInstance().getDb().queryInt("SELECT onlineusers_peak FROM system_monitor");
		
		return true;
	}
	
	@Override
	protected void onUnload()
	{
		// Force last update
		this.onPulse();
		
		// Do this too
		habbod.getInstance().getDb().execute("UPDATE users SET online=0");
		habbod.getInstance().getDb().execute("UPDATE rooms SET users_now=0");
	}

	@Override
	protected void onPulse()
	{
		// Gather stats
		int onlineUsers = SessionManager.getInstance().authedSessions();
		int roomInstances = RoomManager.getInstance().getAllRoomInstances().size();
		long memUsage = Runtime.getRuntime().totalMemory() / 1024;
		int dbConns = habbod.getInstance().getDb().getConnMgr().count();

		// Update peak?
		if(onlineUsers > this.peakOnlineUsers)
		{
			this.peakOnlineUsers = onlineUsers;
		}
		
		// Update stats
		habbod.getInstance().getDb().execute("UPDATE system_monitor SET onlineusers=" + onlineUsers + ",onlineusers_peak=" + this.peakOnlineUsers + ",roominstances=" + roomInstances + ",memusage=" + memUsage + ",dbconnections=" + dbConns);
	}
	
	@Override
	public final String getStatusString()
	{
		return "running";
	}
}
