package mod_roomvisits;

import java.util.ArrayList;
import java.util.List;

import net.nillus.habbo.habbod;
import net.nillus.habbo.modules.Module;
import net.nillus.habbo.modules.ModuleInfo;
import net.nillus.habbo.rooms.Room;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.storage.DatabaseResult;
import net.nillus.habbo.storage.SQLHelper;

@ModuleInfo
(
		name = "mod_roomvisits",
		description = "Logs entry and exit of real users in rooms, with entry/exit timestamps.", 
		author = "Nils 'Nillus'",
		url = "http://modules.openhabbo.org/mod_roomvisits/",
		version = "1.0.0",
		dependencies = { }
)
public final class mod_roomvisits extends Module
{
	private final static int MAX_LIST_SIZE = 50;
	
	public mod_roomvisits()
	{
		this.getAutoLoader().msgHandler(436, new GetRecentRoomsHandler());
		this.getAutoLoader().eventListener(Session.onRoomEnter, new RoomEnterHook());
		this.getAutoLoader().eventListener(Session.onRoomExit, new RoomExitHook());
	}
	
	public static void logRoomEntry(int userId, int roomId)
	{
		habbod.getInstance().getDb().execute("INSERT INTO user_roomvisits(user_id,room_id,entry) VALUES (" + userId + "," + roomId + ",'" + SQLHelper.getDateTimeStr() + "')");
	}
	
	public static void logRoomExit(int userId, int roomId)
	{
		habbod.getInstance().getDb().execute("UPDATE user_roomvisits SET `exit` = '" + SQLHelper.getDateTimeStr() + "' WHERE user_id=" + userId + " AND room_id=" + roomId + " AND `exit` IS NULL LIMIT 1");
	}
	
	public static List<Room> getRecentGuestRoomsForUser(int userId)
	{
		List<Room> rooms = new ArrayList<Room>();

		List<Integer> exclude = new ArrayList<Integer>();
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT r.*,u.username AS owner,rv.entry FROM user_roomvisits rv JOIN rooms r ON r.id = rv.room_id JOIN users u ON u.id=r.owner_id WHERE rv.user_id=" + userId + " ORDER BY rv.entry DESC LIMIT " + mod_roomvisits.MAX_LIST_SIZE);
		while(result.row())
		{
			int roomId = result.getInt("id");
			if(!exclude.contains(roomId))
			{
				rooms.add(Room.parse(result));
				exclude.add(roomId);
			}
		}
		result.release();

		return rooms;
	}
	
	public static List<RoomVisit> getLastRoomVisitsForUser(int userId, int maxAmount)
	{
		List<RoomVisit> visits = new ArrayList<RoomVisit>(maxAmount);
		
		DatabaseResult result =  habbod.getInstance().getDb().query("SELECT r.*,u.username AS owner,rv.entry,rv.exit FROM user_roomvisits rv JOIN rooms r ON r.id = rv.room_id LEFT JOIN users u ON u.id=r.owner_id WHERE rv.user_id=" + userId + " ORDER BY rv.entry DESC LIMIT " + maxAmount);
		while(result.row())
		{
			RoomVisit visit = new RoomVisit();
			visit.room = Room.parse(result);
			visit.entry = result.getTime("entry");
			visit.exit = result.getTime("exit");
			visits.add(visit);
		}
		result.release();
		
		return visits;
	}
}