package mod_pets;

import java.util.ArrayList;
import java.util.List;

import mod_pets.msgs.*;
import net.nillus.habbo.habbod;
import net.nillus.habbo.modules.Module;
import net.nillus.habbo.modules.ModuleInfo;
import net.nillus.habbo.rooms.instances.RoomInstance;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.sessions.SessionManager;
import net.nillus.habbo.storage.DatabaseResult;
import net.nillus.habbo.storage.SQLHelper;


@ModuleInfo(name = "mod_pets", description = "Enables support for pet 'furniture items', including advanced AI logic etc.", author = "Nils 'Nillus'", url = "http://habbo.nillus.net/modules/mod_pets/", version = "1.0.0", dependencies = {})
public final class mod_pets extends Module
{
	public mod_pets()
	{
		this.getAutoLoader().itemImpl("Pet", PetItem.class);
		
		this.getAutoLoader().chatCmd("createpet", new CreatePetCommandHandler());
		
		this.getAutoLoader().eventListener(Session.onLogin, new LoginHook());
		this.getAutoLoader().eventListener(RoomInstance.onLoad, new RoomInstanceLoadHook());
		this.getAutoLoader().eventListener(RoomInstance.onUnload, new RoomInstanceUnloadHook());
		
		this.getAutoLoader().msgHandler(42, new CheckPetNameHandler());
		this.getAutoLoader().msgHandler(3000, new GetPetsInventoryHandler());
		this.getAutoLoader().msgHandler(3001, new GetPetInfoHandler());
		this.getAutoLoader().msgHandler(3002, new PlacePetHandler());
		this.getAutoLoader().msgHandler(3003, new PickUpPetHandler());
		this.getAutoLoader().msgHandler(3004, new GetPetCommandsHandler());
		this.getAutoLoader().msgHandler(3005, new RespectPetHandler());
		this.getAutoLoader().msgHandler(3007, new GetPetRacesHandler());
	}
	
	public static Pet getPetById(int petId)
	{
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT p.*,u.username AS owner FROM pets p JOIN users u ON u.id=p.owner_id WHERE p.id=" + petId);
		Pet pet = result.row() ? Pet.parse(result) : null;
		result.release();
		
		return pet;
	}
	
	public static void createPet(int userId, String name, int type, int breed, String color)
	{
		// Additional default info
		int experience = 0;
		int energy = 100;
		int nutrition = 0;
		int respect = 0;
		
		// Store pet, get ID
		int petId = habbod.getInstance().getDb().executeReturnKey("INSERT INTO pets(name,type,breed,color,experience,energy,nutrition,respect,owner_id,created_at) VALUES ('" + SQLHelper.escape(name) + "'," + type + "," + breed + ",'" + SQLHelper.escape(color) + "'," + experience + "," + energy + "," + nutrition + "," + respect + "," + userId + ",'" + SQLHelper.getDateTimeStr() + "')");
		
		// Owner online?
		Session session = SessionManager.getInstance().getSessionByUserId(userId);
		if(session != null)
		{
			// Add to inventory
			PetInventory inventory = session.get("pets");
			if(inventory != null)
			{
				inventory.addPet(mod_pets.getPetById(petId));
			}
		}
	}

	public static List<Pet> getPetsForRoom(int roomId)
	{
		List<Pet> pets = new ArrayList<Pet>(0);
		
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT p.*,u.username AS owner FROM pets p JOIN users u ON u.id=p.owner_id WHERE p.room_id=" + roomId);
		while(result.row())
		{
			pets.add(Pet.parse(result));
		}
		result.release();
		
		return pets;
	}
}
