package mod_wired;


import mod_wired.actions.*;
import mod_wired.conditions.*;
import mod_wired.triggers.*;
import mod_wired.addons.*;
import net.nillus.habbo.communication.messages.incoming.userdefinedroomevents.*;
import net.nillus.habbo.modules.Module;
import net.nillus.habbo.modules.ModuleInfo;
import net.nillus.habbo.sessions.Session;


@ModuleInfo
(
		name = "mod_wired",
		description = "Adds support for 'wired furniture' in guestrooms. Wired furniture consists of Effects, Triggers and Conditions, and users can 'wire them together' to make cool stuff happen in rooms.",
		author = "Nils 'Nillus'",
		url = "http://habbo.nillus.net/modules/mod_wired/",
		version = "1.0.0",
		dependencies = { }
)
public final class mod_wired extends Module
{
	public mod_wired()
	{
		// Message handlers
		this.getAutoLoader().msgHandler(3050, new UpdateTriggerHandler());
		this.getAutoLoader().msgHandler(3051, new UpdateActionHandler());
		this.getAutoLoader().msgHandler(3052, new UpdateConditionHandler());
		this.getAutoLoader().msgHandler(3054, new UpdateSnapshotsHandler());
		
		// Events
		this.getAutoLoader().eventListener(Session.onRoomEnter, new WiredUserEntersRoomTrigger.RoomEnterHook());
		this.getAutoLoader().eventListener(Session.onChat, new WiredUserSaysPhraseTrigger.ChatHook());
		
		// Items
			this.getAutoLoader().itemImpl("WiredGiveGameScoreAction", WiredGiveGameScoreAction.class);
			this.getAutoLoader().itemImpl("WiredMoveRotateItemsAction", WiredMoveRotateItemsAction.class);
			this.getAutoLoader().itemImpl("WiredResetTimersAction", WiredResetTimersAction.class);
			this.getAutoLoader().itemImpl("WiredRestoreItemsToSnapshotsAction", WiredRestoreItemsToSnapshotsAction.class);
			this.getAutoLoader().itemImpl("WiredShowMsgAction", WiredShowMsgAction.class);
			this.getAutoLoader().itemImpl("WiredTeleportToItemsAction", WiredTeleportToItemsAction.class);
			this.getAutoLoader().itemImpl("WiredToggleItemsAction", WiredToggleItemsAction.class);
			
			this.getAutoLoader().itemImpl("WiredItemsHaveItemsOnTopCondition", WiredItemsHaveItemsOnTopCondition.class);
			this.getAutoLoader().itemImpl("WiredItemsHaveUserCondition", WiredItemsHaveUserCondition.class);
			this.getAutoLoader().itemImpl("WiredItemsMatchCondition", WiredItemsMatchCondition.class);
			this.getAutoLoader().itemImpl("WiredTimerElapsedLessThanCondition", WiredTimerElapsedLessThanCondition.class);
			this.getAutoLoader().itemImpl("WiredTimerElapsedMoreThanCondition", WiredTimerElapsedMoreThanCondition.class);
			this.getAutoLoader().itemImpl("WiredTriggeringUserOnItemCondition", WiredTriggeringUserOnItemCondition.class);
			
			this.getAutoLoader().itemImpl("WiredGameEndsTrigger", WiredGameEndsTrigger.class);
			this.getAutoLoader().itemImpl("WiredGameScoreAchievedTrigger", WiredGameScoreAchievedTrigger.class);
			this.getAutoLoader().itemImpl("WiredGameStartsTrigger", WiredGameStartsTrigger.class);
			this.getAutoLoader().itemImpl("WiredRepeatTrigger", WiredRepeatTrigger.class);
			this.getAutoLoader().itemImpl("WiredTimerTrigger", WiredTimerTrigger.class);
			this.getAutoLoader().itemImpl("WiredUserEntersRoomTrigger", WiredUserEntersRoomTrigger.class);
			this.getAutoLoader().itemImpl("WiredUserSaysPhraseTrigger", WiredUserSaysPhraseTrigger.class);
			this.getAutoLoader().itemImpl("WiredUserStepsOffItemTrigger", WiredUserStepsOffItemTrigger.class);
			this.getAutoLoader().itemImpl("WiredUserStepsOnItemTrigger", WiredUserStepsOnItemTrigger.class);
			this.getAutoLoader().itemImpl("WiredUserTogglesItemTrigger", WiredUserTogglesItemTrigger.class);
			
			this.getAutoLoader().itemImpl("WiredPressurePad", WiredPressurePad.class);
			this.getAutoLoader().itemImpl("WiredPuzzleBox", WiredPuzzleBox.class);
			this.getAutoLoader().itemImpl("WiredRandomActionAddon", WiredRandomActionAddon.class);
			this.getAutoLoader().itemImpl("WiredSwitch", WiredSwitch.class);
			this.getAutoLoader().itemImpl("WiredUnusedActionAddon", WiredResetTimersAction.class);
		}
	}