package net.nillus.habbo.communication;

import net.nillus.habbo.communication.messages.incoming.avatar.UpdateMottoHandler;
import net.nillus.habbo.communication.messages.incoming.avatar.UpdateOutfitHandler;
import net.nillus.habbo.communication.messages.incoming.catalog.GetCatalogIndexHandler;
import net.nillus.habbo.communication.messages.incoming.catalog.GetCatalogPageHandler;
import net.nillus.habbo.communication.messages.incoming.catalog.GetSpecialCataPages2Handler;
import net.nillus.habbo.communication.messages.incoming.catalog.GetSpecialCataPagesHandler;
import net.nillus.habbo.communication.messages.incoming.catalog.PurchaseFromCatalogHandler;
import net.nillus.habbo.communication.messages.incoming.handshake.GetSessionParametersHandler;
import net.nillus.habbo.communication.messages.incoming.handshake.GetUserInfoHandler;
import net.nillus.habbo.communication.messages.incoming.handshake.HeartbeatHandler;
import net.nillus.habbo.communication.messages.incoming.handshake.SSOTicketHandler;
import net.nillus.habbo.communication.messages.incoming.inventory.achievements.GetAchievementsHandler;
import net.nillus.habbo.communication.messages.incoming.inventory.items.GetInventoryItemsHandler;
import net.nillus.habbo.communication.messages.incoming.inventory.purse.GetBalanceInfoHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.CanCreateRoomHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.CreateGuestRoomHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.GetGuestRoomHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.GetHomeRoomHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.GetNavigatorFrontpageHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.GetOwnRoomsHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.GetPopularRoomTagsHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.GetPopularRoomsHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.GetPublicRoomFiles;
import net.nillus.habbo.communication.messages.incoming.navigator.GetTopRatedRoomsHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.GetUserFlatCatsHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.GuestRoomSearchHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.PerformRoomSearchByTagsHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.SetHomeRoomHandler;
import net.nillus.habbo.communication.messages.incoming.navigator.TakeAllRightsHandler;
import net.nillus.habbo.communication.messages.incoming.room.action.AssignRightsHandler;
import net.nillus.habbo.communication.messages.incoming.room.action.BanUserHandler;
import net.nillus.habbo.communication.messages.incoming.room.action.KickBotHandler;
import net.nillus.habbo.communication.messages.incoming.room.action.KickUserHandler;
import net.nillus.habbo.communication.messages.incoming.room.action.LetUserInHandler;
import net.nillus.habbo.communication.messages.incoming.room.action.RemoveRightsHandler;
import net.nillus.habbo.communication.messages.incoming.room.avatar.DanceHandler;
import net.nillus.habbo.communication.messages.incoming.room.avatar.LookToHandler;
import net.nillus.habbo.communication.messages.incoming.room.avatar.MoveHandler;
import net.nillus.habbo.communication.messages.incoming.room.avatar.WaveHandler;
import net.nillus.habbo.communication.messages.incoming.room.chat.ChatHandler;
import net.nillus.habbo.communication.messages.incoming.room.chat.ShoutHandler;
import net.nillus.habbo.communication.messages.incoming.room.chat.StartTypingHandler;
import net.nillus.habbo.communication.messages.incoming.room.chat.StopTypingHandler;
import net.nillus.habbo.communication.messages.incoming.room.chat.WhisperHandler;
import net.nillus.habbo.communication.messages.incoming.room.engine.GetHeightMapHandler;
import net.nillus.habbo.communication.messages.incoming.room.engine.GetRoomContentDataHandler;
import net.nillus.habbo.communication.messages.incoming.room.engine.MoveFloorItemHandler;
import net.nillus.habbo.communication.messages.incoming.room.engine.MoveWallItemHandler;
import net.nillus.habbo.communication.messages.incoming.room.engine.PickupItemHandler;
import net.nillus.habbo.communication.messages.incoming.room.engine.PlaceItemHandler;
import net.nillus.habbo.communication.messages.incoming.room.engine.UseObjectHandler;
import net.nillus.habbo.communication.messages.incoming.room.session.GoToGuestRoomHandler;
import net.nillus.habbo.communication.messages.incoming.room.session.LeaveRoomHandler;
import net.nillus.habbo.communication.messages.incoming.room.session.OpenGuestRoomHandler;
import net.nillus.habbo.communication.messages.incoming.room.session.OpenPublicRoomHandler;
import net.nillus.habbo.communication.messages.incoming.roomsettings.DeleteRoomHandler;
import net.nillus.habbo.communication.messages.incoming.roomsettings.GetRoomSettingsHandler;
import net.nillus.habbo.communication.messages.incoming.roomsettings.SaveRoomSettingsHandler;
import net.nillus.habbo.communication.messages.incoming.roomsettings.UpdateRoomIconHandler;
import net.nillus.habbo.communication.messages.incoming.users.IgnoreUserHandler;
import net.nillus.habbo.communication.messages.incoming.users.RespectUserHandler;
import net.nillus.habbo.communication.messages.incoming.users.UnignoreUserHandler;
import net.nillus.habbo.core.Registry;

public final class ClientRequestRegistry
{
	private final static Registry<Integer, RequestHandler> registry = new Registry<Integer, RequestHandler>();

	public final static Registry<Integer, RequestHandler> get()
	{
		return registry;
	}

	static
	{
		// Stock msg handlers
		registry.register(206, new GetSessionParametersHandler());
		registry.register(415, new SSOTicketHandler());
		registry.register(7, new GetUserInfoHandler());
		registry.register(8, new GetBalanceInfoHandler());
		registry.register(44, new UpdateOutfitHandler());
		registry.register(484, new UpdateMottoHandler());
		registry.register(404, new GetInventoryItemsHandler());
		registry.register(391, new OpenGuestRoomHandler());
		registry.register(388, new GetPublicRoomFiles());
		registry.register(2, new OpenPublicRoomHandler());
		registry.register(53, new LeaveRoomHandler());
		registry.register(151, new GetUserFlatCatsHandler());
		registry.register(233, new GetHomeRoomHandler());
		registry.register(380, new GetNavigatorFrontpageHandler());
		registry.register(385, new GetGuestRoomHandler());
		registry.register(430, new GetPopularRoomsHandler());
		registry.register(431, new GetTopRatedRoomsHandler());
		registry.register(434, new GetOwnRoomsHandler());
		registry.register(382, new GetPopularRoomTagsHandler());
		registry.register(437, new GuestRoomSearchHandler());
		registry.register(438, new PerformRoomSearchByTagsHandler());
		//registry.register(215, new GetCampaignItemsHandler());
		registry.register(390, new GetHeightMapHandler());
		registry.register(126, new GetRoomContentDataHandler());
		registry.register(52, new ChatHandler());
		registry.register(55, new ShoutHandler());
		registry.register(56, new WhisperHandler());
		registry.register(75, new MoveHandler());
		registry.register(387, new CanCreateRoomHandler());
		registry.register(29, new CreateGuestRoomHandler());
		registry.register(400, new GetRoomSettingsHandler());
		registry.register(386, new UpdateRoomIconHandler());
		registry.register(401, new SaveRoomSettingsHandler());
		registry.register(96, new AssignRightsHandler());
		registry.register(97, new RemoveRightsHandler());
		registry.register(155, new TakeAllRightsHandler());
		registry.register(95, new KickUserHandler());
		registry.register(320, new BanUserHandler());
		registry.register(384, new SetHomeRoomHandler());
		registry.register(23, new DeleteRoomHandler());
		registry.register(79, new LookToHandler());
		registry.register(317, new StartTypingHandler());
		registry.register(318, new StopTypingHandler());
		registry.register(319, new IgnoreUserHandler());
		registry.register(322, new UnignoreUserHandler());
		registry.register(94, new WaveHandler());
		registry.register(93, new DanceHandler());
		registry.register(98, new LetUserInHandler());
		registry.register(59, new GoToGuestRoomHandler());
		registry.register(67, new PickupItemHandler());
		registry.register(73, new MoveFloorItemHandler());
		registry.register(90, new PlaceItemHandler());
		registry.register(91, new MoveWallItemHandler());
		registry.register(392, new UseObjectHandler()); // Generic trigger item
		registry.register(393, new UseObjectHandler()); // Generic trigger item
		registry.register(371, new RespectUserHandler());
		registry.register(314, new UseObjectHandler()); // Love Shuffler
		registry.register(441, new KickBotHandler());
		registry.register(315, new HeartbeatHandler());
		registry.register(101, new GetCatalogIndexHandler());
		registry.register(102, new GetCatalogPageHandler());
		registry.register(100, new PurchaseFromCatalogHandler());
		registry.register(3011, new GetSpecialCataPagesHandler());
		registry.register(473, new GetSpecialCataPages2Handler());
		registry.register(370, new GetAchievementsHandler());
	}
}
