package net.nillus.habbo.core;

import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public final class Registry<K, V>
{
	private final Map<K, V> entries;

	public Registry()
	{
		this.entries = new ConcurrentHashMap<K, V>();
	}

	public final void register(K key, V value)
	{
		this.entries.put(key, value);
	}

	public final void unregister(K key)
	{
		this.entries.remove(key);
	}

	public final boolean isKnown(K key)
	{
		return this.entries.containsKey(key);
	}

	public final V lookup(K key)
	{
		return this.entries.get(key);
	}

	public final Set<Entry<K, V>> getEntries()
	{
		return this.entries.entrySet();
	}

	public final void clear()
	{
		this.entries.clear();
	}

	public final int count()
	{
		return this.entries.size();
	}

	public final RegistryAutoManageProxy<K, V> createProxy()
	{
		return new RegistryAutoManageProxy<K, V>(this);
	}
}