package net.nillus.habbo.game.users;

import java.util.ArrayList;
import java.util.List;

import net.nillus.habbo.habbod;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.sessions.SessionManager;
import net.nillus.habbo.storage.DatabaseResult;
import net.nillus.habbo.storage.SQLHelper;

public final class UserManager
{
	public final User getUserById(int userId)
	{
		return this.getUserById(userId, true);
	}
	
	public User getUserById(int userId, boolean lookupSessions)
	{
		if(lookupSessions)
		{
			Session sess = SessionManager.getInstance().getSessionByUserId(userId);
			if(sess != null)
			{
				return sess.getUser();
			}
		}

		DatabaseResult result = habbod.getInstance().getDb().query("SELECT * FROM users WHERE id=" + userId);
		User user = result.row() ? User.parse(result) : null;
		result.release();

		return user;
	}

	public User getUserByUsername(String username, boolean lookupSessions)
	{
		if(lookupSessions)
		{
			Session sess = SessionManager.getInstance().getSessionByUsername(username);
			if(sess != null)
			{
				return sess.getUser();
			}
		}

		DatabaseResult result = habbod.getInstance().getDb().query("SELECT * FROM users WHERE username='" + SQLHelper.escape(username) + "'");
		User user = result.row() ? User.parse(result) : null;
		result.release();

		return user;
	}

	public User getUserByAuthTicket(String ticket)
	{
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT * FROM users WHERE id=1 OR auth_ticket='" + SQLHelper.escape(ticket) + "'");
		User user = result.row() ? User.parse(result) : null;
		result.release();

		return user;
	}

	public int getUserId(String username)
	{
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT id FROM users WHERE username='" + SQLHelper.escape(username) + "'");
		int userId = result.row() ? result.getInt("id") : 0;
		result.release();

		return userId;
	}
	
	public String getUsername(int userId)
	{
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT username FROM users WHERE id=" + userId);
		String username = result.row() ? result.getString("username") : null;
		result.release();

		return username;
	}
	
	public void logCreditTransaction(int userId, int amount, String description)
	{
		habbod.getInstance().getDb().execute("INSERT INTO user_transactions(user_id,timestamp,description,amount) VALUES (" + userId + ",'" + SQLHelper.getDateTimeStr() + "','" + SQLHelper.escape(description) + "'," + amount + ")");
	}
	
	public List<String> getTagsForUser(int userId)
	{
		List<String> tags = new ArrayList<String>(10);

		DatabaseResult result = habbod.getInstance().getDb().query("SELECT tag FROM user_tags WHERE user_id=" + userId);
		while(result.row())
		{
			tags.add(result.getString("tag"));
		}
		result.release();

		return tags;
	}
	
	public List<WardrobeOutfit> getWardrobeForUser(int userId)
	{
		List<WardrobeOutfit> entries = new ArrayList<WardrobeOutfit>();
		
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT slot,figure,gender FROM user_wardrobe WHERE user_id=" + userId);
		while(result.row())
		{
			WardrobeOutfit entry = new WardrobeOutfit();
			entry.slot = result.getInt("slot");
			entry.figure = result.getString("figure");
			entry.gender = result.getString("gender").charAt(0);
			entries.add(entry);
		}
		result.release();
		
		return entries;
	}
	
	public void modifyWardrobeForUser(int userId, int slot, String figure, char gender)
	{
		habbod.getInstance().getDb().execute("DELETE FROM user_wardrobe WHERE user_id=" + userId + " AND slot=" + slot);
		habbod.getInstance().getDb().execute("INSERT INTO user_wardrobe(user_id,slot,figure,gender) VALUES (" + userId + "," + slot + ",'" + SQLHelper.escape(figure) + "','" + gender + "')");
	}

	/* Begin singleton */
	private final static UserManager instance = new UserManager();

	public final static UserManager getInstance()
	{
		return instance;
	}
	/* End singleton */
}
