package net.nillus.habbo.modules;

import net.nillus.habbo.communication.ClientRequestRegistry;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.core.RegistryAutoManageProxy;
import net.nillus.habbo.events.Event;
import net.nillus.habbo.events.EventAutoManageProxy;
import net.nillus.habbo.events.EventListener;
import net.nillus.habbo.game.chatcommands.ChatCmdRegistry;
import net.nillus.habbo.game.chatcommands.ChatCommandHandler;
import net.nillus.habbo.game.items.Item;
import net.nillus.habbo.game.items.ItemImplRegistry;
import net.nillus.habbo.remote.RemoteCmdRegistry;
import net.nillus.habbo.remote.RemoteCommandHandler;

public final class ModulePayload
{
	private final EventAutoManageProxy events = new EventAutoManageProxy();
	private final RegistryAutoManageProxy<String, Class<? extends Item>> itemImpls = ItemImplRegistry.get().createProxy();
	private final RegistryAutoManageProxy<Integer, RequestHandler> msgHandlers = ClientRequestRegistry.get().createProxy();
	private final RegistryAutoManageProxy<String, ChatCommandHandler> chatCommands = ChatCmdRegistry.get().createProxy();
	private final RegistryAutoManageProxy<String, RemoteCommandHandler> remoteCommands = RemoteCmdRegistry.get().createProxy();

	public void eventListener(Event event, EventListener listener)
	{
		this.events.add(listener, event);
	}

	public void msgHandler(int type, RequestHandler handler)
	{
		this.msgHandlers.add(type, handler);
	}

	public void itemImpl(String name, Class<? extends Item> impl)
	{
		this.itemImpls.add(name, impl);
	}

	public void chatCmd(String cmd, ChatCommandHandler handler)
	{
		this.chatCommands.add(cmd, handler);
	}

	public void remoteCmd(String cmd, RemoteCommandHandler handler)
	{
		this.remoteCommands.add(cmd, handler);
	}

	public void inject()
	{
		this.events.load();
		this.msgHandlers.load();
		this.itemImpls.load();
		this.chatCommands.load();
		this.remoteCommands.load();
	}

	public void eject()
	{
		this.events.unload();
		this.msgHandlers.unload();
		this.itemImpls.unload();
		this.chatCommands.unload();
		this.remoteCommands.unload();
	}
}
