package habbod2.game.chatcommands.core;

import habbod2.System;
import static habbod2.core.Locale._I;
import habbod2.game.chatcommands.ChatCommandHandler;
import habbod2.game.chatcommands.ChatCommandManager;
import habbod2.game.util.GameError;
import habbod2.game.sessions.Session;

public class HelpCommand extends ChatCommandHandler
{
	@Override
	public void handle(Session session, String... args) throws GameError
	{
		// Get the manager
		ChatCommandManager cmdMgr = System.get().getGameMgr().getChatCmdMgr();
		
		// Show commands list or help on specific command?
		if(args[0].isEmpty())
		{
			StringBuilder notif = new StringBuilder();
			notif.append(_I("The following chatcommands are available for you:"));
			notif.append("\r\r");
			for(ChatCommandHandler cmd : cmdMgr.getAllCommands())
			{
				if(cmd.isAccessibleTo(session.getUser()))
				{
					notif.append(ChatCommandHandler.COMMAND_HEAD);
					notif.append(cmd.getCommand());
					notif.append("\t\t");
					notif.append(cmd.getDescription());
					notif.append("\r\r");
				}
			}
			notif.append(_I("End of command list."));
			session.sendSystemNotif(notif.toString());
		}
		else
		{
			// Command exists & accessible?
			ChatCommandHandler cmd = cmdMgr.getCommand(args[0]);
			if(cmd == null || !cmd.isAccessibleTo(session.getUser()))
			{
				// Get list with all commands
				this.handle(session);
			}
			else
			{
				// Notify user with usage + description of the command
				StringBuilder notif = new StringBuilder();
				notif.append(cmd.getUsage());
				notif.append("\r\r");
				notif.append(cmd.getDescription());
				if(cmd.getRequiredRight() != null)
				{
					notif.append("\r\r");
					notif.append(_I("Required right: %s", cmd.getRequiredRight()));
				}
				session.sendNotif(notif.toString());
			}
		}
	}
}
