package habbod2.game.chatcommands.room;

import habbod2.game.chatcommands.ChatCommandHandler;
import habbod2.game.items.InventoryItem;
import habbod2.game.items.ItemInventory;
import habbod2.game.util.GameError;
import habbod2.game.rooms.instances.RoomInstance;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.RoomWallItem;
import habbod2.game.rooms.items.wall.PostItItem;
import habbod2.game.sessions.Session;

public class PickAllCommand extends ChatCommandHandler
{
	@Override
	public void handle(Session session, String... args) throws GameError
	{
		// Requires room owner
		if(!session.getRoomUser().hasRoomOwner)
		{
			return;
		}
		
		// Pickup all eligible items
		RoomInstance instance = session.getRoomInstance();
		ItemInventory inventory = session.getItemInventory();
		for(RoomFloorItem item : instance.getFloorItems())
		{
			InventoryItem tmp = instance.pickupFloorItem(item.id, session);
			if(tmp != null)
			{
				inventory.addItem(tmp, true);
			}
		}
		for(RoomWallItem item : instance.getWallItems())
		{
			// Skip post.its
			if(!(item instanceof PostItItem))
			{
				InventoryItem tmp = instance.pickupWallItem(item.id, session);
				if(tmp != null)
				{
					inventory.addItem(tmp, true);
				}
			}
		}
		
		// Refresh inventory
		inventory.refresh();
	}
}
