package habbod2.communication;

import java.util.Map;
import java.util.HashMap;

import habbod2.System;
import habbod2.storage.DatabaseResult;

public class RequestRegistry
{
	private static Map<Integer, String> names = new HashMap<Integer, String>();
	private static Map<Integer, RequestHandler> registry = new HashMap<Integer, RequestHandler>();
	
	
	public static String getRequestName(int msgId)
	{
		return names.get(msgId);
	}
	
	public static RequestHandler getRequestHandler(int msgId)
	{
		return registry.get(msgId);
	}
	
	public static void reload()
	{
		// Clear current entries
		names.clear();
		registry.clear();
		
		// Load enables entries from database
		DatabaseResult result = System.get().getDb().query("SELECT * FROM system_requesthandlers WHERE enabled=1");
		while(result.row())
		{
			int msgId = result.getInt("id");
			String handlerName = result.getString("handler");
			String className = String.format("habbod2.communication.messages.incoming.%sHandler", handlerName);
			names.put(msgId, handlerName);
			
			try
			{
				RequestHandler handler = (RequestHandler) Class.forName(className).newInstance();
				registry.put(msgId, handler);
			}
			catch(Exception ex)
			{
				System.get().getLogger().error("RequestRegistry", String.format("Error registering #%d ('%s').", msgId, handlerName));
			}
		}
		result.release();
		
		// Log result
		System.get().getLogger().info("RequestRegistry", String.format("Loaded %d entries.", registry.size()));
	}
}
