package habbod2.communication.messages.incoming.moderator;

import habbod2.System;
import static habbod2.core.Locale._I;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.navigator.RoomInfoUpdatedMessageComposer;
import habbod2.game.GameManager;
import habbod2.game.moderation.ModerationActionError;
import habbod2.game.rooms.Room;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.navigator.RoomEvent;
import habbod2.game.sessions.Session;

public class ModerateRoomHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Has mod?
		if(!session.getUser().rights.contains("fuse_mod"))
		{
			return;
		}
		
		// Read info
		int roomId = request.readInt();
		boolean lockRoom = request.readBool();
		boolean inappropriateRoom = request.readBool();
		boolean kickRoom = request.readBool();
		
		// Get game
		GameManager game = System.get().getGameMgr();
		
		// Perform actions
		RoomInstance room = game.getRoomMgr().getRoomInstance(roomId, false);
		if(room == null)
		{
			return;
		}
		
		// Kick users + stop event?
		if(kickRoom)
		{
			try
			{
				game.getModerationCenter().issueRoomKick(session.getUser().id, room.getInfo().id, null);
			}
			catch(ModerationActionError error)
			{
				session.sendNotif(error.generateNotif());
			}
			
			// Event?
			RoomEvent event = room.getEvent();
			if(event != null)
			{
				game.getRoomEventMgr().removeEvent(event);
			}
		}
		
		// Room changes?
		if(lockRoom || inappropriateRoom)
		{
			// Lock room?
			if(lockRoom)
			{
				room.getInfo().state = Room.STATE_DOORBELL;
			}
			
			// Change room to inappropriate?
			if(inappropriateRoom)
			{
				room.getInfo().name = _I("Unacceptable to Hotel Management");
				room.getInfo().description = _I("This room has been closed by a Moderator");
				room.getInfo().tags = new String[2];
			}
			
			// Save room
			room.getInfo().saveSettings();
			room.broadcast(RoomInfoUpdatedMessageComposer.compose(room.getInfo()));
		}
	}
}
