package habbod2.communication.messages.incoming.navigator;

import java.util.List;

import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.navigator.RoomRatingMessageComposer;
import habbod2.game.rooms.instances.RoomInstance;
import habbod2.game.sessions.Session;

public class RateFlatHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Get room and 'already rated' list
		RoomInstance room = session.getRoomInstance();
		List<Integer> rated = session.getRatedRooms();
		if(room == null)
		{
			return;
		}

		// Ensure this user can rate this room (not already rated, not owner)
		if(rated.contains(room.getInfo().id) || (session.getUser().id == room.getInfo().ownerId))
		{
			return;
		}

		// Apply rating
		int rating = request.readInt();
		if(rating == -1)
		{
			room.getInfo().rating--;
		}
		else
		{
			room.getInfo().rating++;
		}
		room.getInfo().saveRating();

		// Add to 'rated' list
		rated.add(room.getInfo().id);

		// Notify room
		room.broadcast(RoomRatingMessageComposer.compose(room.getInfo().rating));
	}
}
