package habbod2.communication.messages.incoming.navigator;

import habbod2.net.codec.WireCodec;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.navigator.RoomInfoUpdatedMessageComposer;
import habbod2.communication.messages.outgoing.navigator.RoomThumbnailUpdateResultMessageComposer;
import habbod2.communication.messages.outgoing.roomsettings.RoomSettingsSavedMessageComposer;
import habbod2.game.rooms.Room;
import habbod2.game.sessions.Session;

public class UpdateRoomThumbnailHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Allowed to edit the icon?
		if(!(session.inRoom() && session.getRoomUser().hasRoomOwner))
		{
			return;
		}
		Room room = session.getRoom().getInfo();

		// Parse the icon
		request.readInt(); // Layer amount
		int background = request.readInt();
		int foreground = request.readInt();
		int itemAmount = request.readInt();
		int[] itemKeys = new int[itemAmount];
		int[] itemValues = new int[itemAmount];
		for(int x = 0; x < itemAmount; x++)
		{
			itemKeys[x] = request.readInt();
			itemValues[x] = request.readInt();
		}

		// Rebuild and save the data
		StringBuilder sb = new StringBuilder(10);
		sb.append(WireCodec.encodeIntToStr(background));
		sb.append(WireCodec.encodeIntToStr(foreground));
		sb.append(WireCodec.encodeIntToStr(itemAmount));
		for(int x = 0; x < itemAmount; x++)
		{
			sb.append(WireCodec.encodeIntToStr(itemKeys[x]));
			sb.append(WireCodec.encodeIntToStr(itemValues[x]));
		}
		room.icon = sb.toString();

		// Notify this user the icon was accepted
		session.send(RoomThumbnailUpdateResultMessageComposer.compose(room, true));

		// Notify this user that saving succeeded
		session.send(RoomSettingsSavedMessageComposer.compose(room));
		
		// Notify the room that someething has been updated
		session.getRoom().broadcast(RoomInfoUpdatedMessageComposer.compose(room));
	}
}
