package habbod2.communication.messages.incoming.room.engine;

import habbod2.System;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.game.items.EItemFlags;
import habbod2.game.items.InventoryItem;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.items.floor.JukeboxItem;
import habbod2.game.rooms.items.floor.RollerItem;
import habbod2.game.rooms.items.minigame.TimerItem;
import habbod2.game.rooms.items.wall.RoomDimmerItem;
import habbod2.game.sessions.Session;

import habbod2.communication.messages.outgoing.room.engine.PlaceObjectErrorMessageComposer;
import static habbod2.communication.messages.outgoing.room.engine.PlaceObjectErrorMessageComposer.*;

public class PlaceObjectHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Allowed to place items in this room?
		if(!session.inRoom() || !session.getRoomUser().hasRoomRights)
		{
			return;
		}
		
		// All data is in one string... weird
		String[] data = request.readStr().split(" ");
		int itemId = Integer.parseInt(data[0]);

		// Validate item from inventory
		InventoryItem item = session.getItemInventory().getItem(itemId);
		if(item == null)
		{
			return;
		}
		
		// Get room
		RoomInstance room = session.getRoom();
		
		// Roller limit
		if(item.is(RollerItem.class))
		{
			int limit = System.get().getProperties().getInt("game.rooms.rollers.limit");
			if(limit > 0 && !session.getUser().rights.contains("override_maxrollers"))
			{
				if(room.getRollers() != null && room.getRollers().getCount() >= limit)
				{
					session.send(PlaceObjectErrorMessageComposer.compose(ERROR_LIMIT_ROLLERS));
					return;
				}
			}
		}
		
		// Only one roomdimmer per room!
		else if(item.is(RoomDimmerItem.class))
		{
			if(room.getRoomDimmer() != null)
			{
				session.send(PlaceObjectErrorMessageComposer.compose(ERROR_NOT_HERE));
				return;
			}
		}
		
		// Only one jukebox per room!
		else if(item.is(JukeboxItem.class))
		{
			if(room.getJukebox() != null)
			{
				session.send(PlaceObjectErrorMessageComposer.compose(ERROR_LIMIT_SOUNDMACHINE));
				return;
			}
		}
		
		// Only one gametimer per room!
		else if(item.is(TimerItem.class))
		{
			if(room.getMinigame() != null && room.getMinigame().getTimer() != null)
			{
				session.send(PlaceObjectErrorMessageComposer.compose(ERROR_NOT_HERE));
				return;
			}
		}

		// What kind of item is being placed?
		boolean placementOK = false;
		if(item.is(EItemFlags.FLOORITEM))
		{
			int tileX = Integer.parseInt(data[1]);
			int tileY = Integer.parseInt(data[2]);
			byte rotation = Byte.parseByte(data[3]);
			placementOK = session.getRoom().placeItem(item, tileX, tileY, rotation);
		}
		else if(item.is(EItemFlags.WALLITEM))
		{
			String position = data[1] + " " + data[2] + " " + data[3];
			placementOK = session.getRoom().placeItem(item, position);
		}

		// Placement okay?
		if(placementOK)
		{
			session.getItemInventory().removeItem(item.id);
		}
	}
}
