package habbod2.communication.messages.incoming.room.engine;

import static habbod2.core.Locale._I;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.game.items.EItemFlags;
import habbod2.game.items.InventoryItem;
import habbod2.game.rooms.items.RoomItem;
import habbod2.game.sessions.Session;

public class PlaceObjectHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Allowed to place items in this room?
		if(!session.inRoom() || !session.getRoomUser().hasRoomRights)
		{
			return;
		}

		// All data is in one string... weird
		String[] data = request.readStr().split(" ");
		int itemId = Integer.parseInt(data[0]);

		// Validate item from inventory
		InventoryItem item = session.getItemInventory().getItem(itemId);
		if(item == null)
		{
			return;
		}
		
		// Singleton check
		if(item.is(EItemFlags.ROOM_SINGLETON))
		{
			for(RoomItem other : session.getRoomInstance().getItems())
			{
				if(other.definition.getRoomClass() == item.definition.getRoomClass())
				{
					session.sendNotif(_I("You can only place one item of this type in a room!"));
					return;
				}
			}
		}

		// What kind of item is being placed?
		boolean placementOK = false;
		if(item.is(EItemFlags.FLOORITEM))
		{
			int tileX = Integer.parseInt(data[1]);
			int tileY = Integer.parseInt(data[2]);
			byte rotation = Byte.parseByte(data[3]);
			placementOK = session.getRoomInstance().placeItem(item, tileX, tileY, rotation);
		}
		else if(item.is(EItemFlags.WALLITEM))
		{
			String position = data[1] + " " + data[2] + " " + data[3];
			placementOK = session.getRoomInstance().placeItem(item, position);
		}

		// Placement okay?
		if(placementOK)
		{
			session.getItemInventory().removeItem(item.id);
		}
	}
}
