package habbod2.communication.messages.incoming.room.furniture;

import habbod2.System;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.room.engine.PlaceObjectErrorMessageComposer;
import static habbod2.communication.messages.outgoing.room.engine.PlaceObjectErrorMessageComposer.ERROR_LIMIT_POSTITS;
import habbod2.game.items.InventoryItem;
import habbod2.game.rooms.items.RoomWallItem;
import habbod2.game.rooms.items.wall.PostItItem;
import habbod2.game.sessions.Session;

public class PlacePostItHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Allowed to place items in this room?
		if(!session.inRoom() || !session.getRoomUser().hasRoomRights)
		{
			return;
		}

		// Read info
		int itemId = request.readInt();
		String position = request.readStr();
		
		// Validate item from inventory
		InventoryItem item = session.getItemInventory().getItem(itemId);
		if(item == null || !item.is(PostItItem.class))
		{
			return;
		}
		
		// Post.it limit
		int limit = System.get().getProperties().getInt("game.rooms.stickies.limit");
		if(limit > 0 && !session.getUser().rights.contains("override_maxstickies"))
		{
			int count = 0;
			for(RoomWallItem tmp : session.getRoom().getWallItems())
			{
				if(tmp instanceof PostItItem)
				{
					count++;
					if(count == limit)
					{
						session.send(PlaceObjectErrorMessageComposer.compose(ERROR_LIMIT_POSTITS));
						return;
					}
				}
			}
		}

		// Issue placement
		if(session.getRoom().placeItem(item, position))
		{
			session.getItemInventory().removeItem(item.id);
		}
	}
}
