package habbod2.communication.messages.incoming.room.furniture;

import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.game.rooms.items.wall.RoomDimmerItem;
import habbod2.game.sessions.Session;

public class RoomDimmerSavePresetHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Only room owners can deal with the room dimmer
		RoomDimmerItem dimmer = session.getRoomInstance().getRoomDimmer();
		if(dimmer == null || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}
		
		// Read settings for preset X
		int presetId = request.readInt();
		boolean backgroundOnly = (request.readInt() == 2);
		String color = request.readStr();
		int intensity = request.readInt();
		
		// Validate settings
		RoomDimmerItem.Preset preset = dimmer.getPreset(presetId);
		if(preset == null || !RoomDimmerItem.Preset.isValidColor(color) || !RoomDimmerItem.Preset.isValidIntensity(intensity))
		{
			return;
		}
		
		// Apply & save settings
		dimmer.enabled = true;
		dimmer.currentPreset = presetId;
		preset.backgroundOnly = backgroundOnly;
		preset.color = color;
		preset.intensity = intensity;
		dimmer.saveSettings();
	}
}
