package habbod2.communication.messages.incoming.roomsettings;

import habbod2.System;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.navigator.RoomInfoUpdatedMessageComposer;
import habbod2.communication.messages.outgoing.room.engine.RoomVisualizationSettingsMessageComposer;
import habbod2.communication.messages.outgoing.roomsettings.RoomSettingsSavedMessageComposer;
import habbod2.game.rooms.Room;
import habbod2.game.rooms.navigator.RoomCategory;
import habbod2.game.rooms.navigator.RoomNavigator;
import habbod2.game.util.SecurityUtil;
import habbod2.game.sessions.Session;

public class SaveRoomSettingsHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Grab affected room
		int roomId = request.readInt();
		if(!session.inRoom() || roomId != session.getRoom().getInfo().id || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}
		Room room = session.getRoom().getInfo();

		// Validate new name
		String name = SecurityUtil.filterStr(request.readStr());
		if(name.length() == 0)
		{
			return;
		}

		// Validate new description
		String description = SecurityUtil.filterStr(request.readStr());

		// Validate new state + password
		byte state = (byte) request.readInt();
		if(state < Room.STATE_OPEN || state > Room.STATE_PASSWORD)
		{
			state = 0;
		}
		String password = SecurityUtil.filterStr(request.readStr());

		// Validate max users
		int maxUsers = request.readInt();
		if(maxUsers != 10 && maxUsers != 15 && maxUsers != 20 && maxUsers != 25)
		{
			return;
		}

		// Validate category
		int categoryId = request.readInt();
		RoomNavigator navi = System.get().getGameMgr().getNavigator();
		RoomCategory category = navi.getCategory(categoryId);
		if(category == null || !category.isAccessibleTo(session.getUser()))
		{
			categoryId = room.categoryId;
		}

		// Validate tags
		String[] tags = SecurityUtil.filterStrArray(request.readStrArray());
		if(tags.length > 2)
		{
			return;
		}
		else if(tags.length < 2)
		{
			String[] tmp = new String[2];
			for(int i = 0; i < tags.length; i++)
			{
				tmp[i] = tags[i];
			}
			tags = tmp;
		}

		// Save data
		room.name = name;
		room.description = description;
		room.state = state;
		room.password = password;
		room.usersMax = maxUsers;
		room.categoryId = categoryId;
		room.tags = tags;
		room.allowPets = request.readFixedBool();
		room.allowPetsEat = request.readFixedBool();
		room.allowWalkthrough = request.readFixedBool();
		room.hideWalls = request.readFixedBool();
		room.wallThickness = request.readInt();
		room.floorThickness = request.readInt();
		room.saveSettings();

		// Notify users
		session.send(RoomSettingsSavedMessageComposer.compose(room));
		session.getRoom().broadcast(RoomVisualizationSettingsMessageComposer.compose(room));
		session.getRoom().broadcast(RoomInfoUpdatedMessageComposer.compose(room));
	}
}
