package habbod2.communication;

import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;

import habbod2.game.sessions.Session;
import habbod2.net.protocol.ClientMessage;
import habbod2.util.NamedThreadFactory;

/*
 * This class is inline'd and everything for maximum throughput.
 */
public final class DeferredRequestHandlerExecutor
{
	private static final ExecutorService executor = Executors.newSingleThreadExecutor(new NamedThreadFactory("DeferredRequestHandler #%d"));
	
	public static final void submit(final Session session, final ClientMessage request, final DeferredRequestHandler handler)
	{
		executor.submit(new ExecutorTask(session, request, handler));
	}
	
	private static final class ExecutorTask implements Runnable
	{
		private final Session session;
		private final ClientMessage request;
		private final DeferredRequestHandler handler;
		
		public ExecutorTask(final Session session, final ClientMessage request, final DeferredRequestHandler handler)
		{
			this.session = session;
			this.request = request;
			this.handler = handler;
		}
		
		@Override
		public final void run()
		{
			this.handler.handle(this.session, this.request);
		}	
	}
}
