package habbod2.core;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import habbod2.System;
import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;

public class Configuration extends System.Component
{
	private Map<String, String> entries;
	
	public Configuration(System system)
	{
		super(system);
		this.entries = Collections.emptyMap();
	}
	
	public void reload()
	{
		// Create new map
		Map<String, String> tmpEntries = new HashMap<String, String>();
		
		// Load new configuration
		try(DatabaseResult result = this.getDb().query("SELECT * FROM system_config"))
		{
			while(result.row())
			{
				tmpEntries.put(result.getString("key"), result.getString("value"));
			}
		}
		
		// Overwrite map (no need for concurrency control!)
		this.entries = tmpEntries;
		this.getLogger().info(this, String.format("Loaded %d entries.", this.size()));
	}
	
	public String get(String key)
	{
		return this.get(key, null);
	}

	public String get(String key, String defaultValue)
	{
		String value = this.entries.get(key);
		if(value != null)
		{
			return value;
		}
		else
		{
			return defaultValue;
		}
	}

	public int getInt(String key)
	{
		return this.getInt(key, 0);
	}

	public int getInt(String key, int defaultValue)
	{
		String value = this.get(key);
		if(value != null)
		{
			try
			{
				return Integer.parseInt(value);
			}
			catch(NumberFormatException ex)
			{
				
			}
		}
		
		return defaultValue;
	}
	
	public boolean getBool(String prop)
	{
		return (this.getInt(prop) == 1);
	}
	
	public void unset(String key)
	{
		this.set(key, null);
	}
	
	public void set(String key, String value)
	{
		// Value unchanged?
		if(value != null && value.equals(this.get(key)))
		{
			return;
		}
		
		// Update entries map
		if(value == null)
		{
			this.entries.remove(key);
		}
		else
		{
			this.entries.put(key, value.toString());
		}
		
		// Update database
		this.getDb().execute("DELETE FROM system_config WHERE `key`='" + SQLHelper.escape(key) + "'");
		if(value != null)
		{
			this.getDb().execute("INSERT INTO system_config(`key`,`value`) VALUES ('" + SQLHelper.escape(key) + "','" + SQLHelper.escape(value) + "')");
 		}
	}
	
	public void set(String key, int value)
	{
		this.set(key, Integer.toString(value));
	}
	
	public void setz(String key, boolean value)
	{
		this.set(key, Boolean.toString(value));
	}
	
	public int size()
	{
		return this.entries.size();
	}
}
