package habbod2.events;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import habbod2.System;

public class Event
{
	private String id;
	private List<EventListener> listeners;

	public Event(String id)
	{
		this.id = id;
		this.listeners = new CopyOnWriteArrayList<EventListener>();
	}

	public void addListener(EventListener listener)
	{
		this.listeners.add(listener);
	}

	public void removeListener(EventListener listener)
	{
		this.listeners.remove(listener);
	}

	public boolean fire(Object subject)
	{
		return this.fire(subject, null);
	}

	public boolean fire(Object subject, Object args)
	{
		boolean result = true;
		if(this.hasListeners())
		{
			for(EventListener listener : this.listeners)
			{
				try
				{
					result = (result && listener.handle(subject, args));
				}
				catch(Exception ex)
				{
					System.get().getLogger().error(this, String.format("Error executing listener %s.", listener.getClass().getName()), ex);
					return false;
				}
			}
		}

		// 'result' will be false if one or more handlers returned false
		return result;
	}

	public String getId()
	{
		return this.id;
	}

	public boolean hasListeners()
	{
		return(!this.listeners.isEmpty());
	}

	@Override
	public String toString()
	{
		return this.id;
	}
}
