package habbod2.events;

import java.util.Map;
import java.util.HashMap;

public class EventArgs
{
	private int poolIndex;
	private Map<String, Object> args;

	private EventArgs(int poolIndex)
	{
		this.poolIndex = poolIndex;
		this.args = new HashMap<String, Object>();
	}
	
	public <T> EventArgs set(String key, T value)
	{
		this.args.put(key, value);
		return this;
	}

	public void unset(String key)
	{
		this.args.remove(key);
	}

	public boolean has(String key)
	{
		return this.args.containsKey(key);
	}

	@SuppressWarnings("unchecked")
	public <T> T get(String key)
	{
		Object value = this.args.get(key);
		if(value != null)
		{
			return (T) value;
		}
		else
		{
			return null;
		}
	}

	public void clear()
	{
		this.args.clear();
	}

	public int getPoolIndex()
	{
		return this.poolIndex;
	}

	private static int POOL_SIZE = 1024;
	private static volatile EventArgs[] pool;
	private static volatile boolean[] inUse;

	static
	{
		pool = new EventArgs[POOL_SIZE];
		for(int x = 0; x < POOL_SIZE; x++)
		{
			pool[x] = new EventArgs(x);
		}
	}

	public static EventArgs create()
	{
		for(int x = 0; x < POOL_SIZE; x++)
		{
			if(!inUse[x])
			{
				inUse[x] = true;
				return pool[x];
			}
		}
		return new EventArgs(POOL_SIZE - 1);
	}

	public static void release(EventArgs args)
	{
		args.clear();
		inUse[args.getPoolIndex()] = false;
	}
}
