package habbod2.net;

import org.jboss.netty.buffer.ChannelBuffer;
import org.jboss.netty.buffer.ChannelBuffers;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelFutureListener;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.channel.ExceptionEvent;
import org.jboss.netty.handler.codec.frame.FrameDecoder;

import habbod2.game.sessions.SessionManager;
import habbod2.net.codec.ClientMessageDecoder;
import habbod2.net.codec.ServerMessageEncoder;

public class NetHandshakeHandler extends FrameDecoder
{
	private static final char MSG_MAGIC_GAME = '@';
	private static final char MSG_MAGIC_POLICYFILE = '<';
	
	private SessionManager sessionMgr;
	private ServerMessageEncoder encoder;
	
	public NetHandshakeHandler(SessionManager sessionMgr)
	{
		this.sessionMgr = sessionMgr;
		this.encoder = new ServerMessageEncoder();
	}
	
	@Override
	protected Object decode(ChannelHandlerContext ctx, Channel channel, ChannelBuffer buffer) throws Exception
	{
		// Switch to requested protocol
		byte magic = buffer.readByte();
		if(magic == MSG_MAGIC_GAME)
		{
			// Configure pipeline
			NetConnectionHandler conn = new NetConnectionHandler(channel);
			channel.getPipeline().addLast("decoder", new ClientMessageDecoder());
			channel.getPipeline().addLast("encoder", this.encoder);
			channel.getPipeline().addLast("handler", conn);
			channel.getPipeline().remove(this);
			
			// Create session
			this.sessionMgr.startSession(conn);
		}
		else if(magic == MSG_MAGIC_POLICYFILE)
		{
			// Get policy file bytes and fill buffer
			byte[] bytes = CrossdomainPolicy.getXml().getBytes();
			ChannelBuffer data = ChannelBuffers.buffer(bytes.length);
			data.writeBytes(bytes);

			// Write buffer and close connection
			channel.write(data).addListener(ChannelFutureListener.CLOSE);
		}
		else
		{
			// Unknown protocol!
			channel.close();
			return null;
		}
		
		// Reset and forward buffer to other upstream handlers
		buffer.resetReaderIndex();
		return buffer.readBytes(buffer.readableBytes());
	}
	
	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, ExceptionEvent e)
	{
		ctx.getChannel().close();
	}
}
