package habbod2.storage;

import java.sql.Connection;

public class DatabaseConnection implements AutoCloseable
{
	public final long id;
	private final Connection connection;
	private final DatabaseConnectionPool pool;
	
	private long lastAccessTime;
	private boolean available;
	
	public DatabaseConnection(long id, Connection connection, DatabaseConnectionPool pool)
	{
		this.id = id;
		this.connection = connection;
		this.pool = pool;
	}

	public void release()
	{
		this.getPool().releaseConnection(this);
	}
	
	public Connection getInnerConnection()
	{
		return this.connection;
	}

	public DatabaseConnectionPool getPool()
	{
		return this.pool;
	}

	public long getLastAccessTime()
	{
		return this.lastAccessTime;
	}

	public long getIdleSeconds()
	{
		return((System.currentTimeMillis() - this.lastAccessTime) / 1000);
	}

	public boolean isAvailable()
	{
		return this.available;
	}

	public void available(boolean isAvailable)
	{
		this.available = isAvailable;
		this.lastAccessTime = System.currentTimeMillis();
	}

	@Override
	public void close()
	{
		this.release();
	}
}
