package habbod2.util;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Date;
import java.text.SimpleDateFormat;

public class Logger
{
	public enum ELogLevel
	{
		Debug, Info, Warning, Error
	}
	
	private SimpleDateFormat timeformat;
	private ELogLevel minImportance;
	private boolean dumpStackTraces;
	private BufferedWriter writer;

	public Logger()
	{
		this.timeformat = new SimpleDateFormat("dd-MM-yyyy @ HH:mm:ss:SS");
		this.minImportance = ELogLevel.Info;
	}

	public void setMinImportance(ELogLevel level)
	{
		this.minImportance = level;
	}
	
	public void setDumpStackTraces(boolean dumpStackTraces)
	{
		this.dumpStackTraces = dumpStackTraces;
	}

	private void writeLog(ELogLevel level, String line)
	{
		if(level.ordinal() < this.minImportance.ordinal())
		{
			return;
		}

		// Log to console
		System.out.println(line);

		// Log to file?
		if(this.writer != null)
		{
			if(level != ELogLevel.Debug)
			{
				try
				{
					synchronized(this.writer)
					{
						this.writer.write(line);
						this.writer.write("\r\n");
						this.writer.flush();
					}
				}
				catch(IOException ex)
				{
					System.out.println("Error logging to file!");
				}
			}
		}
	}

	public void debug(Object system, String s)
	{
		this.writeLog(ELogLevel.Debug, this.timeformat.format(new Date()) + " - [" + system + "][#debug] " + s);
	}

	public void info(Object system, String s)
	{
		this.writeLog(ELogLevel.Info, this.timeformat.format(new Date()) + " - [" + system + "] " + s);
	}

	public void warning(Object system, String s)
	{
		this.writeLog(ELogLevel.Warning, this.timeformat.format(new Date()) + " - [" + system + "][#warning] " + s);
	}

	public void error(Object system, String s)
	{
		this.writeLog(ELogLevel.Error, this.timeformat.format(new Date()) + " - [" + system + "][#error] " + s);
	}
	
	public void error(Object system, String s, Exception ex)
	{
		if(this.dumpStackTraces)
		{
			this.writeLog(ELogLevel.Error, this.timeformat.format(new Date()) + " - [" + system + "][#error] " + s + "\r\n" + Logger.convertStackTraceToString(ex));
		}
		else
		{
			this.error(system, s + " " + ex.getMessage());
		}
	}
	
	public boolean setDatedOutputFile()
	{
		// Append date to filename
		String date = new SimpleDateFormat("dd-MM-yyyy").format(new Date());
		
		// Set output file as normal
		return this.setOutputFile(date);
	}
	
	public boolean setOutputFile(String fileName)
	{
		// Append .log extension to file if not already there
		if(!fileName.endsWith(".log"))
		{
			fileName = fileName + ".log";
		}

		// Attempt to create filewriter (creates or appends)
		try
		{
			this.writer = new BufferedWriter(new FileWriter(fileName, true));
			return true;
		}
		catch(IOException ex)
		{
			System.out.println("ERROR creating new FileWriter for filename \"" + fileName + "\"");
			return false;
		}
	}

	private static String convertStackTraceToString(Exception ex)
	{
		StringWriter sw = new StringWriter();
		if(ex != null)
		{
			ex.printStackTrace(new PrintWriter(sw));
		}
		return sw.toString();
	}
}
