package habbod2.game.access;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import habbod2.System;
import habbod2.game.GameComponent;
import habbod2.game.GameManager;
import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;

public class AccessControl extends GameComponent
{
	private Map<Integer, UserRole> userRoles;

	public AccessControl(GameManager gameMgr)
	{
		super(gameMgr);
		this.userRoles = Collections.emptyMap();
	}

	public void reload()
	{
		// Create new collection
		Map<Integer, UserRole> tmpRoles = new HashMap<Integer, UserRole>();

		// Reload all the roles
		DatabaseResult result = System.get().getDb().query("SELECT id,name,level FROM roles ORDER BY id ASC");
		while(result.row())
		{
			UserRole role = new UserRole();
			role.id = result.getInt("id");
			role.name = result.getString("name");
			role.level = result.getInt("level");
			tmpRoles.put(role.id, role);
		}

		// Load all the custom rights for every role
		for(UserRole role : tmpRoles.values())
		{
			result.query("SELECT `right` FROM role_rights WHERE role_id=" + role.id);
			while(result.row())
			{
				role.rights.add(result.getString("right"));
			}
		}
		result.release();

		// Set instance field
		this.userRoles = Collections.unmodifiableMap(tmpRoles);

		// Done!
		this.getLogger().info("AccessControl", String.format("Loaded %d user roles.", this.userRoles.size()));
	}

	public int logUserLogin(int userId, String ip)
	{
		return this.getDb().executeReturnKey("INSERT INTO user_accesslog(user_id,ip,login) VALUES (" + userId + ",'" + SQLHelper.escape(ip) + "','" + SQLHelper.getDateTime() + "')");
	}

	public void logUserLogout(int accessId)
	{
		this.getDb().execute("UPDATE user_accesslog SET logout='" + SQLHelper.getDateTime() + "' WHERE id=" + accessId);
	}

	public void logUserAction(int issuerId, int targetId, String action, String message)
	{
		this.getDb().execute("INSERT INTO system_actionlog(timestamp,issuer_id,action,target_id,message) VALUES('" + SQLHelper.getDateTime() + "'," + issuerId + ",'" + SQLHelper.escape(action) + "'," + targetId + ",'" + SQLHelper.escape(message) + "')");
	}

	public UserRole getUserRole(int roleId)
	{
		return this.userRoles.get(roleId);
	}

	public Collection<UserRole> getAllUserRoles()
	{
		return this.userRoles.values();
	}
}
