package habbod2.game.availability;

import java.util.List;
import java.util.ArrayList;

import habbod2.game.GameComponent;
import habbod2.game.GameManager;
import habbod2.game.users.User;

public class AvailabilityManager extends GameComponent
{
	private boolean isAvailable;
	private List<String> overrideRights;
	
	public AvailabilityManager(GameManager gameMgr)
	{
		super(gameMgr);
		this.overrideRights = new ArrayList<String>();
		this.reload();
	}
	
	public void reload()
	{
		// Reload the status
		this.isAvailable = true;
		
		// Reload the rights that give access when not available
		this.overrideRights.clear();
		this.overrideRights.add("lockdown");
	}
	
	public AvailableAgainStatus getStatusForUser(User user)
	{
		// Available normally?
		if(this.isAvailable)
		{
			return null;
		}
		else
		{
			// Not available normally, check for rights
			for(String right : this.overrideRights)
			{
				if(user.rights.contains(right))
				{
					return null;
				}
			}
			
			// Not available, return status
			AvailableAgainStatus status = new AvailableAgainStatus();
			status.hour = 14;
			status.minute = 48;
			return status;
		}
	}
}
