package habbod2.game.items;

import habbod2.System;
import habbod2.storage.DatabaseResult;
import habbod2.game.rooms.items.RoomItem;
import habbod2.game.rooms.items.floor.GenericFloorItem;
import habbod2.game.rooms.items.wall.GenericWallItem;

public class ItemDefinition
{
	public int id;
	public String sprite;
	public int displayId;
	public int inventoryType;
	public String color;
	public byte length;
	public byte width;
	public float height;
	private char[] flags;
	private Class<? extends RoomItem> roomClass;
	public String data;
	public String name;
	public String description;
	
	public char getType()
	{
		return Character.toLowerCase(this.flags[0]);
	}
	
	public boolean is(char flag)
	{
		for(int i = 0; i < this.flags.length; i++)
		{
			if(this.flags[i] == flag)
			{
				return true;
			}
		}
		return false;
	}
	
	public boolean is(Class<? extends RoomItem> roomClass)
	{
		return (this.roomClass == roomClass);
	}

	public static ItemDefinition parse(DatabaseResult result)
	{
		ItemDefinition def = new ItemDefinition();
		def.id = result.getInt("id");
		def.sprite = result.getString("sprite");
		def.displayId = result.getInt("sprite_id");
		def.inventoryType = result.getInt("inventorytype");
		def.color = result.getString("color");
		def.length = result.getByte("length");
		def.width = result.getByte("width");
		def.height = result.getFloat("height");
		def.flags = (result.getString("flags") + EItemFlags.ALLOW_TRADE).toUpperCase().toCharArray();
		def.name = result.getString("name");
		def.description = result.getString("description");
		def.data = result.getString("data");
		def.loadRoomClass(result.getString("class"));
		return def;
	}

	@SuppressWarnings("unchecked")
	private void loadRoomClass(String name)
	{
		// Set default class just in case
		if(this.is(EItemFlags.WALLITEM))
		{
			this.roomClass = GenericWallItem.class;
		}
		else if(this.is(EItemFlags.FLOORITEM))
		{
			this.roomClass = GenericFloorItem.class;
		}
		
		// Do a best effort to honor the room class
		if(name != null && !name.isEmpty())
		{
			// Generate the full class name
			String className = String.format("habbod2.game.rooms.items.%s", name);
			
			// Attempt to load the class
			try
			{
				this.roomClass = (Class<? extends RoomItem>) Class.forName(className);
			}
			catch (Exception ex)
			{
				System.get().getLogger().warning("ItemDefinition", String.format("Definition #%d ('%s') refers to missing roomclass '%s', class ignored.", this.id, this.sprite, className));
			}
		}
	}
	
	public InventoryItem newItem()
	{
		InventoryItem item = new InventoryItem();
		item.definition = this;
		
		return item;
	}
	
	public RoomItem createRoomItem()
	{
		RoomItem item;
		try
		{
			item = this.roomClass.newInstance();
			item.definition = this;
			return item;
		}
		catch(Exception ex)
		{
			return null;
		}
	}
	
	public Class<? extends RoomItem> getRoomClass()
	{
		return this.roomClass;
	}
}
