package habbod2.game.items.trading;

import habbod2.System;
import habbod2.game.sessions.Session;
import habbod2.net.protocol.ServerMessage;
import habbod2.communication.messages.outgoing.inventory.trading.*;

public class Trade
{
	private TradePartner p1;
	private TradePartner p2;

	private Trade()
	{
		this.p1 = null;
		this.p2 = null;
	}

	public void initialize(TradePartner p1, TradePartner p2)
	{
		this.p1 = p1;
		this.p2 = p2;
		this.send(TradingOpenMessageComposer.compose(p1, p2));
	}

	public void end(Session closedBy)
	{
		// Unhook etc
		this.p1.clear();
		this.p2.clear();

		// Notify partners of closed trade
		if(closedBy == null)
		{
			this.send(TradingCompletedMessageComposer.compose());
		}
		else
		{
			this.send(TradingCloseMessageComposer.compose(this.get(closedBy)));
		}
	}

	public TradePartner get(Session session)
	{
		if(this.p1.getSession() == session)
		{
			return this.p1;
		}
		else if(this.p2.getSession() == session)
		{
			return this.p2;
		}
		else
		{
			return null;
		}
	}

	public void complete()
	{
		// Ready for the actual trade of items?
		if(this.bothAccepted() && this.bothConfirmed())
		{
			// Attempt to swap offers
			if(this.p1.verifyItems() && this.p2.verifyItems())
			{
				this.p1.transferItems(this.p2);
				this.p2.transferItems(this.p1);
			}
			else
			{
				System.get().getLogger().warning("Trade", String.format("Unsafe trade between #%d and %d was detected and ended.", this.p1.getSession().getUser().id, this.p2.getSession().getUser().id));
			}

			// Ended! :)
			this.end(null);
		}
	}

	public void update()
	{
		// Something has changed, partners will have to accept the changes
		this.p1.accept(false);
		this.p2.accept(false);

		// Send updates
		this.send(TradingItemListMessageComposer.compose(p1, p2));
	}

	public boolean bothAccepted()
	{
		return(this.p1.accepted() && this.p2.accepted());
	}
	
	public boolean bothConfirmed()
	{
		return (this.p1.confirmed() && this.p2.confirmed());
	}

	public void send(ServerMessage msg)
	{
		this.p1.getSession().send(msg);
		this.p2.getSession().send(msg);
	}

	public static Trade create(Session sess1, Session sess2)
	{
		// Cross-room trade? (lol why not)
		if(sess1.getRoomInstance() != sess2.getRoomInstance())
		{
			return null;
		}

		// One of these partners is already trading?
		if(sess1.isTrading() || sess2.isTrading())
		{
			return null;
		}

		// Create & initialize trade!
		Trade trade = new Trade();
		TradePartner p1 = new TradePartner(trade, sess1);
		TradePartner p2 = new TradePartner(trade, sess2);
		trade.initialize(p1, p2);

		// Trade created successfully
		return trade;
	}
}