package habbod2.game.messenger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import habbod2.game.GameManager;
import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;
import habbod2.util.ScheduledTask;
import habbod2.util.StringUtil;

public class MessengerService extends GameManager.Component implements Runnable
{
	private boolean shutdown;
	private Map<Integer, MessengerBuddy> registry;
	private ScheduledTask cleanup;
	
	public MessengerService(GameManager gameMgr)
	{
		super(gameMgr);
		this.registry = new ConcurrentHashMap<Integer, MessengerBuddy>();
	}
	
	public void start()
	{
		long cleanupTime = 30 * 1000;
		this.cleanup = this.getSystem().scheduleTask(this, cleanupTime);
	}
	
	public void shutdown()
	{
		this.cleanup.stop();
		this.shutdown = true;
	}
	
	public MessengerBuddy getBuddy(int userId, boolean attemptDbLoad, boolean incrementRef)
	{
		// Lookup the registry
		MessengerBuddy buddy = this.registry.get(userId);

		// Not found? Attempt load from database, then add to registry
		if(buddy == null && attemptDbLoad)
		{
			DatabaseResult result = this.getDb().query("SELECT id,username,figure,motto,lastonline,im_blockrequests FROM users WHERE id=" + userId);
			if(result.row())
			{
				buddy = this.parseBuddyFromRow(result);
				this.registry.put(buddy.userId, buddy);
			}
			result.release();
		}
		
		// Increment refcount?
		if(incrementRef && buddy != null)
		{
			buddy.refCount++;
		}
		
		return buddy;
	}

	public Map<Integer, MessengerBuddy> getBuddyListForUser(int userId)
	{
		Map<Integer, MessengerBuddy> buddies = new HashMap<Integer, MessengerBuddy>();

		DatabaseResult result = this.getDb().query("SELECT u.id,u.username,u.figure,u.motto,u.lastonline,u.im_blockrequests FROM users u INNER JOIN user_imbuddies rel ON u.id=rel.user_id WHERE rel.user2_id=" + userId + " AND rel.accepted=1 UNION ALL SELECT u.id,u.username,u.figure,u.motto,u.lastonline,u.im_blockrequests FROM users u INNER JOIN user_imbuddies rel ON u.id=rel.user2_id WHERE rel.user_id=" + userId + " AND rel.accepted=1");
		while(result.row())
		{
			// First lookup the registry...
			int buddyUserId = result.getInt("id");
			MessengerBuddy buddy = this.getBuddy(buddyUserId, false, true);

			// Not in registry?
			if(buddy == null)
			{
				// Accept the database data and add to the registry
				buddy = this.parseBuddyFromRow(result);
				buddy.refCount++;
				this.registry.put(buddy.userId, buddy);
			}

			// Add to the buddylist
			buddies.put(buddy.userId, buddy);
		}
		result.release();

		return buddies;
	}

	public Map<Integer, String> getBuddyRequestsForUser(int userId)
	{
		Map<Integer, String> requests = new HashMap<Integer, String>();

		DatabaseResult result = this.getDb().query("SELECT u.id,u.username FROM users u JOIN user_imbuddies rel ON u.id=rel.user_id WHERE rel.user2_id=" + userId + " AND rel.accepted=0");
		while(result.row())
		{
			requests.put(result.getInt("id"), result.getString("username"));
		}
		result.release();

		return requests;
	}

	public List<MessengerBuddy> getUserSearchResults(String criteria)
	{
		List<MessengerBuddy> buddies = new ArrayList<MessengerBuddy>();

		DatabaseResult result = this.getDb().query("SELECT id,username,figure,motto,lastonline,im_blockrequests FROM users WHERE username LIKE '" + SQLHelper.escape(criteria) + "%' LIMIT 30");
		while(result.row())
		{
			// Lookup registry first, optionally accept database data, but don't add to registry
			MessengerBuddy buddy = this.getBuddy(result.getInt("id"), false, false);
			if(buddy == null)
			{
				buddy = this.parseBuddyFromRow(result);
			}
			buddies.add(buddy);
		}
		result.release();

		return buddies;
	}

	public MessengerBuddy getBuddyInfoByUsername(String username)
	{
		// Fetch user info by username, or null if user does not exist
		DatabaseResult result = this.getDb().query("SELECT id,username,figure,motto,lastonline,im_blockrequests FROM users WHERE username='" + SQLHelper.escape(username) + "'");
		MessengerBuddy buddy = result.row() ? this.parseBuddyFromRow(result) : null;
		result.release();
		
		return buddy;
	}

	public void createBuddyRequest(int fromUserId, int toUserId)
	{
		// Uhh...
		if(fromUserId == toUserId)
		{
			return;
		}
		
		// Delete old requests and create new unaccepted relation in database
		this.deleteRelation(fromUserId, toUserId);
		this.getDb().execute("INSERT INTO user_imbuddies(user_id,user2_id) VALUES (" + fromUserId + "," + toUserId + ")");

		// Notify buddy if online
		MessengerBuddy to = this.getBuddy(toUserId, false, false);
		if(to != null && to.isOnline())
		{
			MessengerBuddy from = this.getBuddy(fromUserId, true, false);
			if(from != null)
			{
				to.messenger.addRequest(from);
			}
		}
	}

	public void acceptBuddyRequest(int toUserId, int fromUserId)
	{
		// Uhh...
		if(fromUserId == toUserId)
		{
			return;
		}
				
		// Update the relation in the database
		this.getDb().execute("UPDATE user_imbuddies SET accepted=1 WHERE user_id=" + fromUserId + " AND user2_id=" + toUserId);

		MessengerBuddy buddy;

		// to-user online?
		if((buddy = this.getBuddy(toUserId, false, false)) != null)
		{
			// Add the from-user to it's buddy list
			if(buddy.isOnline())
			{
				buddy.messenger.addBuddy(this.getBuddy(fromUserId, true, false));
			}
		}

		// from-user online?
		if((buddy = this.getBuddy(fromUserId, false, false)) != null)
		{
			// Add the to-user to it's buddylist
			if(buddy.isOnline())
			{
				buddy.messenger.addBuddy(this.getBuddy(toUserId, true, false));
			}
		}
	}

	public void deleteRelation(int userId1, int userId2)
	{
		this.getDb().execute("DELETE FROM user_imbuddies WHERE (user_id=" + userId1 + " AND user2_id=" + userId2 + ") OR (user_id=" + userId2 + " AND user2_id=" + userId1 + ") LIMIT 1");
	}
	
	public void logImMessage(MessengerBuddy sender, MessengerBuddy receiver, String message)
	{
		this.getDb().execute("INSERT INTO user_imlog(user_id,receiver_id,timestamp,message) VALUES (" + sender.userId + "," + receiver.userId + ",'" + SQLHelper.getDateTime() + "','" + SQLHelper.escape(message) + "')");
	}
	
	public void cleanup()
	{
		// Remove buddies that are no longer online / in a buddy list
		int removed = 0;
		Iterator<MessengerBuddy> it = this.registry.values().iterator();
		while(it.hasNext())
		{
			MessengerBuddy buddy = it.next();
			if(buddy.refCount <= 0)
			{
				it.remove();
				removed++;
			}
		}

		// Anything removed?
		if(removed > 0)
		{
			this.getLogger().info(this, String.format("Cleaned %d buddies.", removed));
		}
	}
	
	private MessengerBuddy parseBuddyFromRow(DatabaseResult row)
	{
		MessengerBuddy buddy = new MessengerBuddy();
		buddy.userId = row.getInt("id");
		buddy.username = row.getString("username");
		buddy.figure = row.getString("figure");
		buddy.motto = row.getString("motto");
		buddy.lastOnline = StringUtil.formatDateTime(row.getTime("lastonline"));
		buddy.blockRequests = row.getBoolean("im_blockrequests");
		
		return buddy;
	}
	
	public boolean isShutdown()
	{
		return this.shutdown;
	}
	
	public int registrySize()
	{
		return this.registry.size();
	}

	@Override
	public void run()
	{
		this.cleanup();
	}
}
