package habbod2.game.pets;

import habbod2.game.rooms.instance.RoomAvatar;
import habbod2.game.rooms.instance.RoomHuman;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.util.GameUtil;

public class RoomPet extends RoomAvatar
{
	private Pet pet;
	private RoomHuman rider;
	
	public RoomPet(Pet pet)
	{
		this.pet = pet;
		this.pathfindingAllowedOn = RoomTile.OPEN;
		if(this.pet.type.canSwim)
		{
			this.pathfindingAllowedOn |= RoomTile.WATER;
		}
	}

	public Pet getInfo()
	{
		return this.pet;
	}

	public void syncAndSaveLocationInfo()
	{
		this.pet.roomId = this.getRoom().getInfo().id;
		this.pet.x = this.x;
		this.pet.y = this.y;
		this.pet.z = this.z;
		this.pet.save();
	}

	@Override
	protected void onChatFromAvatar(RoomAvatar from, String text, boolean isShout)
	{
		// Only respond to users
		if(!(from instanceof RoomUser))
		{
			return;
		}

		// Only respond to owner
		RoomUser user = (RoomUser) from;
		if(user.getInfo().id != this.getInfo().ownerId)
		{
			return;
		}

		// Find out if text starts with pet name (case insensitive)
		text = text.toLowerCase();
		boolean isCommand = text.startsWith(this.getInfo().name.toLowerCase());

		// Pay attention...
		if(!isCommand)
		{
			// Rotate head
			this.lookTo(from.getTile(), true);
		}
		else
		{
			// Rotate body too (if needed)
			this.lookTo(from.getTile(), false);

			// Try for matches
			//PetUtil.handlePetCommand(this, user, text);
			
			//  Ignore commands?
			if(this.hasRider())
			{
				return;
			}
			
			// La la la testing
			this.statuses.clear();
			if(text.contains("ga"))
			{
				int x = GameUtil.getRandomInt(0, this.getRoom().getModel().getMaxX());
				int y = GameUtil.getRandomInt(0, this.getRoom().getModel().getMaxY());
				this.setGoal(this.getRoom().getMap().getTile(x, y));
			}
			else if(text.contains("zit"))
			{
				this.addStatus("sit", Float.toString(0));
			}
			else if(text.contains("lig"))
			{
				this.addStatus("lay", Float.toString(0));
			}
			else if(text.contains("spring"))
			{
				this.addStatus("jmp");
			}
			else if(text.contains("rol"))
			{
				this.addStatus("rll");
			}
			else if(text.contains("zwaai"))
			{
				this.addStatus("wav");
			}
			else if(text.contains("staan"))
			{
				this.addStatus("stn");
			}
			else if(text.contains("hier"))
			{
				this.setGoal(from.getTileInFront());
			}
			else if(text.contains("smeek"))
			{
				
			}
			else if(text.contains("dood"))
			{
				this.addStatus("ded", Float.toString(pet.z));
			}
			else if(text.contains("blijf"))
			{
				
			}
			else if(text.contains("volg"))
			{
				if(text.contains("stop"))
				{
					this.follow = null;
				}
				else
				{
					this.follow = from;
				}
			}
			else if(text.contains("dans"))
			{
				this.addStatus("dnc");
			}
		}
	}
	
	@Override
	public void pulse()
	{
		super.pulse();
		if(this.path.isEmpty())
		{
			if(this.follow == null && !this.hasRider())
			{
				if(GameUtil.getRandomInt(0, 10) == 0)
				{
					RoomTile tile = this.getRoom().getMap().getTile(GameUtil.getRandomInt(this.x - 5, this.y + 5), GameUtil.getRandomInt(this.y - 5, this.y + 5));
					this.setGoal(tile);
				}
			}
		}
	}
	
	@Override
	public String getAvatarKey()
	{
		return "p" + this.pet.id;
	}
	
	@Override
	public void invalidate()
	{
		super.invalidate();
		if(this.hasRider())
		{
			this.getRider().setMount(null);
		}
	}
	
	public RoomHuman getRider()
	{
		return this.rider;
	}
	
	public void setRider(RoomHuman human)
	{
		this.rider = human;
		this.follow = null;
		this.clearMovement();
	}
	
	public boolean hasRider()
	{
		return (this.rider != null);
	}
}
