package habbod2.game.rooms.instances;

import habbod2.communication.messages.outgoing.room.action.*;

public abstract class RoomHuman extends RoomAvatar
{
	public boolean isAsleep;
	public int danceId;
	
	public int carryItemId;
	public int carryItemTimer;
	
	private int roomEffectId;
	private int ownEffectId;
	
	private int warpPhase = 0;
	private RoomTile warpDestination;
	
	public void wave()
	{
		this.unidle();
		this.getRoom().broadcast(WaveMessageComposer.compose(this));
	}
	
	public void dance(int danceId)
	{
		this.unidle();
		this.stopCarryItem();
		this.danceId = danceId;
		this.getRoom().broadcast(DanceMessageComposer.compose(this));
	}
	
	public boolean isCarryingItem()
	{
		return (this.carryItemId > 0 && this.carryItemTimer > 0);
	}
	
	public void carryItem(int itemId)
	{
		this.carryItemId = itemId;
		if(itemId > 0)
		{
			this.carryItemTimer = 240; // 240 seconds (4 minutes)
		}
		else
		{
			this.carryItemTimer = 0;
		}
		this.getRoom().broadcast(CarryObjectMessageComposer.compose(this));
	}
	
	public void stopCarryItem()
	{
		this.carryItem(0);
	}
	
	public void setEffect(int effectId, boolean isOwn)
	{
		if(isOwn)
		{
			this.ownEffectId = effectId;
		}
		else
		{
			this.roomEffectId = effectId;
		}
		this.getRoom().broadcast(AvatarEffectMessageComposer.compose(this, this.getCurrentEffect()));
	}
	
	public int getCurrentEffect()
	{
		if(this.roomEffectId > 0)
		{
			return this.roomEffectId;
		}
		else
		{
			return this.ownEffectId;
		}
	}
	
	public void setAsleep(boolean isAsleep)
	{
		if(this.isAsleep != isAsleep)
		{
			this.isAsleep = isAsleep;
			this.getRoom().broadcast(SleepMessageComposer.compose(this));
		}
	}
	
	public void warp(RoomTile dest)
	{
		this.warpPhase = 4;
		this.warpDestination = dest;
	}
	
	
	public void pulse()
	{
		// RoomAvatar pulse()
		super.pulse();
		
		// Deal with carry item
		if(this.carryItemId > 0)
		{
			this.carryItemTimer--;
			if(this.carryItemTimer <= 0)
			{
				this.stopCarryItem();
			}
		}
		
		// Warping (with sparkles-effect)
		if(this.warpPhase > 0)
		{
			if(this.warpPhase == 4)
			{
				this.setEffect(4, false);
			}
			else if(this.warpPhase == 3)
			{
				this.setTile(this.warpDestination, false);
				this.warpDestination = null;
			}
			else if(this.warpPhase == 2)
			{
				this.setEffect(0, false);
			}
			this.warpPhase--;
		}
	}
	
	@Override
	public void unidle()
	{
		super.unidle();
		this.setAsleep(false);
	}
}
