package habbod2.game.rooms.items.wired.triggers;

import habbod2.events.EventListener;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.sessions.Session;
import habbod2.game.rooms.items.wired.WiredTrigger;

public class UserSaysPhraseTrigger extends WiredTrigger
{
	private static final int PARAM_OWNERONLY = 0;
	
	@Override
	public int getInterface()
	{
		return 0;
	}
	
	@Override
	protected int getAmountOfParams()
	{
		return 1;
	}
	
	@Override
	protected boolean suppliesUser()
	{
		return true;
	}
	
	public static class ChatHook implements EventListener
	{
		@Override
		public boolean handle(Object subject, Object args)
		{
			Session session = (Session)subject;
			String text = (String)args;
			
			// Matches the secret word?
			for(RoomFloorItem item : session.getRoomInstance().getFloorItems())
			{
				// Phrase trigger?
				if(item instanceof UserSaysPhraseTrigger)
				{
					// Phrase matches?
					UserSaysPhraseTrigger wired = (UserSaysPhraseTrigger)item;
					if(text.equals(wired.getText()))
					{
						// Owner-only check (if enabled)
						if(wired.getParam(PARAM_OWNERONLY) == 0 || session.getRoomUser().hasRoomOwner)
						{
							// Let the user know he got the phrase correct
							session.sendWhisperNotif(text);
							
							// Fire the trigger
							wired.execute(session.getRoomUser());
							
							// Don't chat the secret to room ;)
							return false;
						}
					}
				}
			}
			
			// Nothing happened!
			return true;
		}
	}
}
