package habbod2.game.users.achievements;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import habbod2.communication.messages.outgoing.inventory.achievements.AchievementMessageComposer;
import habbod2.communication.messages.outgoing.inventory.achievements.AchievementsScoreMessageComposer;
import habbod2.communication.messages.outgoing.notifications.AchievementNotificationMessageComposer;
import habbod2.game.GameComponent;
import habbod2.game.GameManager;
import habbod2.game.sessions.Session;
import habbod2.storage.DatabaseResult;

public class AchievementManager extends GameComponent
{
	private Map<String, Achievement> achievements;
	
	public AchievementManager(GameManager gameMgr)
	{
		super(gameMgr);
		this.achievements = Collections.emptyMap();
	}
	
	public void reload()
	{
		// Create new collection
		Map<String, Achievement> tmpAchievements = new HashMap<String, Achievement>();
		
		// Load new data
		DatabaseResult result = this.getDb().query("SELECT * FROM achievements");
		while(result.row())
		{
			// Get group
			Achievement ach = tmpAchievements.get(result.getString("group"));
			if(ach == null)
			{
				ach = new Achievement();
				ach.id = result.getInt("id");
				ach.group = result.getString("group");
				ach.category = result.getString("category");
				tmpAchievements.put(ach.group, ach);
			}
			
			// Add this as a level
			Achievement.Level level = new Achievement.Level();
			level.num = result.getInt("level");
			level.requiredProgress = result.getInt("required_progress");
			level.rewardPixels = result.getInt("reward_pixels");
			level.rewardScore = result.getInt("reward_score");
			ach.levels.add(level);
		}
		result.release();
		
		// Done!
		this.achievements = tmpAchievements;
		this.getLogger().info(this, String.format("Loaded %d achievements.", this.achievements.size()));
	}
	
	public boolean updateProgressForUser(int userId, String group, int progress)
	{
		// Get achievement
		Achievement ach = this.achievements.get(group);
		if(ach == null)
		{
			return false;
		}
		
		// Get user status
		UserAchievement status = null;
		int targetLevel = (status != null) ? status.level + 1 : 1;
		if(status != null && status.level > targetLevel)
		{
			// Already completed...
			return false;
		}
		
		// Get target level
		Achievement.Level target = ach.levels.get(targetLevel - 1);
		int newProgress = (status != null) ? (status.progress + progress) : progress;
		int newLevel = (status != null) ? status.level : 0;
		int newTarget = newLevel + 1;
		if(newTarget > ach.levels.size())
		{
			newTarget = ach.levels.size();
		}
		
		// Get session
		Session session = this.getGameMgr().getSessionMgr().getSessionByUserId(userId);
		
		// Enough progress for this level?
		if(newProgress < target.requiredProgress)
		{
			// TODO: update data
			
			// User online? Send status
			if(session != null)
			{
				session.send(AchievementMessageComposer.compose(ach, status, target));
			}
			
			// Not leveled up!
			return false;
		}
		else
		{
			// Determine new levels and remaining progress
			newLevel++;
			newTarget++;
			int remainingProgress = newProgress - target.requiredProgress;
			
			// TODO: update data, give badges, reward pixels, score
			
			// User online?
			if(session != null)
			{
				// TODO: refresh badges
				
				// TODO: refresh pixels
				
				// Achievement unlocked!
				session.send(AchievementNotificationMessageComposer.compose(ach, target.num, target.rewardPixels, target.rewardScore));
				session.send(AchievementsScoreMessageComposer.compose(session.getUser().achievementScore));
				session.send(AchievementMessageComposer.compose(ach, status, ach.levels.get(newTarget - 1)));
			}
			
			// Any remaining progress?
			if(remainingProgress > 0)
			{
				this.updateProgressForUser(userId, group, remainingProgress);
			}
			
			// Leveled up!
			return true;
		}
	}
	
	public Achievement getAchievement(String group)
	{
		return this.achievements.get(group);
	}
	
	public Collection<Achievement> getAchievements()
	{
		return this.achievements.values();
	}
}
