package habbod2.game.users.badges;

import java.util.List;
import java.util.ArrayList;

import habbod2.System;
import habbod2.communication.messages.outgoing.inventory.badges.BadgesMessageComposer;
import habbod2.game.sessions.Session;
import habbod2.storage.Database;
import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;

public class BadgeInventory
{
	public static final int SLOTS = 5;
	
	private Session session;
	private List<Badge> badges;
	private boolean requiresSave;

	public BadgeInventory(Session session)
	{
		this.session = session;
		this.badges = new ArrayList<Badge>(3);
		this.requiresSave = false;
	}

	public void clear()
	{
		this.badges.clear();
	}

	public void reload()
	{
		// Clear badges + equipped maps
		this.clear();

		// Reload available badges from database
		try(DatabaseResult result = System.get().getDb().query("SELECT badge,slot FROM user_badges WHERE user_id=" + this.session.getUser().id))
		{
			while(result.row())
			{
				Badge badge = new Badge();
				badge.code = result.getString("badge");
				badge.slot = result.getByte("slot");
				this.badges.add(badge);
			}
		}
	}
	
	public boolean add(String badge)
	{
		// Can't add the same badge twice
		if(this.hasBadge(badge, false))
		{
			return false;
		}
		else
		{
			// Add in inventory
			Badge b = new Badge();
			b.code = badge;
			b.slot = 0;
			this.badges.add(b);
			
			// Add in database
			System.get().getDb().execute("INSERT INTO user_badges(user_id,badge,slot) VALUES (" + this.session.getUser().id + ",'" + SQLHelper.escape(badge) + "'," + b.slot + ")");
			
			// Okay!
			return true;
		}
	}


	public void clearEquipped()
	{
		for(Badge badge : this.badges)
		{
			badge.slot = 0;
		}
		this.requireSave();
	}
	
	public void unequip(int slot)
	{
		for(Badge b : this.badges)
		{
			if(b.slot == slot)
			{
				b.slot = 0;
				this.requireSave();
				return;
			}
		}
	}
	
	public void unequip(String badge)
	{
		for(Badge b : this.badges)
		{
			if(b.code.equals(badge))
			{
				b.slot = 0;
				this.requireSave();
				return;
			}
		}
	}
	
	public boolean equip(int slot, String badge)
	{
		for(Badge b : this.badges)
		{
			if(b.code.equals(badge))
			{
				b.slot = (byte)slot;
				this.requireSave();
				return true;
			}
		}
		
		return false;
	}
	
	public void equipInFreeSlot(String badge)
	{
		// Find free slot
		for(int slot = 1; slot <= BadgeInventory.SLOTS; slot++)
		{
			// This slot is free?
			boolean isFree = true;
			for(Badge b : this.badges)
			{
				if(b.slot == slot)
				{
					isFree = false;
					break;
				}
			}
			
			// Equip?
			if(isFree)
			{
				this.equip(slot, badge);
			}
			else
			{
				// Even last slot isn't free?
				if(slot == BadgeInventory.SLOTS)
				{
					this.unequip(slot);
					this.equip(slot, badge);
				}
			}
		}
	}

	public void requireSave()
	{
		this.requiresSave = true;
	}

	public boolean requiresSave()
	{
		return this.requiresSave;
	}

	public void save()
	{
		// Update all slots
		Database db = System.get().getDb();
		for(Badge badge : this.badges)
		{
			db.execute("UPDATE user_badges SET slot=" + badge.slot + " WHERE user_id=" + this.session.getUser().id + " AND badge='" + SQLHelper.escape(badge.code) + "'");
		}
		
		// Saved!
		this.requiresSave = false;
	}

	public List<Badge> getBadges()
	{
		return this.badges;
	}

	public List<Badge> getEquippedBadges()
	{
		List<Badge> equipped = new ArrayList<Badge>(BadgeInventory.SLOTS);
		for(Badge badge : this.badges)
		{
			if(badge.slot > 0)
			{
				equipped.add(badge);
			}
		}
		
		return equipped;
	}
	
	public boolean hasBadge(String badge, boolean isEquipped)
	{
		for(Badge b : this.badges)
		{
			if(b.code.equals(badge) && (!isEquipped || (isEquipped && b.isEquipped())))
			{
				return true;
			}
		}
		
		return false;
	}
	
	public boolean isSlotFree(int slot)
	{
		for(Badge badge : this.badges)
		{
			if(badge.slot == slot)
			{
				return false;
			}
		}
		
		return true;
	}
	
	public void refresh()
	{
		this.session.send(BadgesMessageComposer.compose(this));
	}
}
