package habbod2.game.users.effects;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import habbod2.System;
import habbod2.communication.messages.outgoing.inventory.avatareffect.AvatarEffectActivatedMessageComposer;
import habbod2.communication.messages.outgoing.inventory.avatareffect.AvatarEffectAddedMessageComposer;
import habbod2.communication.messages.outgoing.inventory.avatareffect.AvatarEffectExpiredMessageComposer;
import habbod2.game.sessions.Session;
import habbod2.storage.DatabaseResult;

public class EffectInventory
{
	private Session session;
	private int currentEffect;
	private List<AvatarEffect> effects;

	public EffectInventory(Session session)
	{
		this.session = session;
		this.effects = new ArrayList<AvatarEffect>();
	}

	public void clear()
	{
		this.effects.clear();
	}

	public void reload()
	{
		// Clear old effects
		this.clear();

		// Load new ones from database
		DatabaseResult result = System.get().getDb().query("SELECT * FROM user_effects WHERE user_id=" + this.session.getUser().id);
		while(result.row())
		{
			AvatarEffect effect = new AvatarEffect();
			effect.userId = result.getInt("user_id");
			effect.effectId = result.getInt("effect_id");
			effect.totalDuration = result.getInt("total_duration");
			effect.enabled = result.getBoolean("enabled");
			if(effect.enabled)
			{
				effect.enabledAt = result.getTime("enabled_at");
			}
			if(effect.hasExpired())
			{
				effect.delete();
			}
			else
			{
				this.effects.add(effect);
			}
		}
		result.release();
	}

	public void addEffect(int effectId, int totalDuration)
	{
		// Create effect
		AvatarEffect effect = new AvatarEffect();
		effect.userId = this.session.getUser().id;
		effect.effectId = effectId;
		effect.totalDuration = totalDuration;
		effect.insert();

		// Add to list
		this.effects.add(effect);
		
		// Notify user
		this.session.send(AvatarEffectAddedMessageComposer.compose(effect));
	}

	public void enableEffect(int effectId)
	{
		AvatarEffect effect = this.getEffect(effectId);
		if(effect != null && !effect.enabled)
		{
			effect.enable();
			this.session.send(AvatarEffectActivatedMessageComposer.compose(effect));
		}
	}
	
	public void selectEffect(int effectId)
	{
		if(effectId > 0)
		{
			AvatarEffect effect = this.getEffect(effectId);
			if(effect != null && effect.enabled)
			{
				this.setCurrentEffect(effectId);
			}
		}
		else
		{
			this.setCurrentEffect(0);
		}
	}
	
	public int getCurrentEffect()
	{
		return this.currentEffect;
	}

	public void setCurrentEffect(int effectId)
	{
		this.currentEffect = effectId;
		if(session.inRoom())
		{
			session.getRoomUser().setEffect(effectId, true);
		}
	}

	public void checkAndStopExpiredEffects()
	{
		// Has effects?
		if(!this.effects.isEmpty())
		{
			// Process all effects
			Iterator<AvatarEffect> effects = this.effects.iterator();
			while(effects.hasNext())
			{
				// Effect expired?
				AvatarEffect effect = effects.next();
				if(effect.hasExpired())
				{
					// Delete data
					effect.delete();
					
					// Remove from iterator
					effects.remove();

					// Notify the user that the effect expired
					this.session.send(AvatarEffectExpiredMessageComposer.compose(effect));

					// Current effect? Disable it!
					if(effect.effectId == this.currentEffect)
					{
						this.setCurrentEffect(-1);
					}
				}
			}
		}
	}

	public boolean hasEffect(int effectId)
	{
		return (this.getEffect(effectId) != null);
	}
	
	public AvatarEffect getEffect(int effectId)
	{
		for(AvatarEffect effect : this.effects)
		{
			if(effect.effectId == effectId && !effect.hasExpired())
			{
				return effect;
			}
		}
		return null;
	}

	public List<AvatarEffect> getEffects()
	{
		return this.effects;
	}
}
