package habbod2.game.users.updaters;

import java.util.TimerTask;

import static habbod2.core.Locale._I;
import habbod2.game.GameManager;
import habbod2.game.sessions.Session;
import habbod2.game.users.User;
import habbod2.util.ScheduledTask;

public class AutoPixelsUpdater extends TimerTask
{
	private GameManager gameMgr;
	private int minutes;
	private ScheduledTask task;
	
	public AutoPixelsUpdater(GameManager gameMgr)
	{
		this.gameMgr = gameMgr;
	}

	public void reschedule()
	{
		// Cancel task?
		if(this.task != null)
		{
			this.task.stop();
			this.task = null;
		}
		
		// Schedule new task
		this.minutes = this.gameMgr.getConfig().getInt("game.users.autopixels.minutes");
		if(this.minutes > 0)
		{
			this.task = this.gameMgr.getSystem().scheduleTask(this, this.minutes * 60 * 1000);
		}
	}
	
	@Override
	public void run()
	{
		// Get configuration
		int add = this.gameMgr.getConfig().getInt("game.users.autopixels.add");
		int max = this.gameMgr.getConfig().getInt("game.users.autopixels.max");
		boolean socialBoost = this.gameMgr.getConfig().getBool("game.users.autopixels.socialboost");
		boolean notify = this.gameMgr.getConfig().getBool("game.users.autopixels.notify");
		
		// Enabled?
		int userAmount = 0;
		int totalAmount = 0;
		if(add > 0)
		{
			// Go through all online users
			for(Session session : this.gameMgr.getSessionMgr().getAllSessions())
			{
				// Get user
				User user = session.getUser();
				
				// Amount to add
				int toAdd = add;
				
				// User is alone in room?
				boolean isWithOthers = (session.inRoom() && session.getRoom().getInfo().usersNow > 1);
				
				// Social boost enabled?
				if(socialBoost && isWithOthers)
				{
					toAdd = toAdd * 2;
				}
			
				// Impose limit?
				if(max > 0 && (user.pixels + toAdd) > max)
				{
					toAdd = max - user.pixels;
				}
				
				// Anything to add?
				if(toAdd > 0)
				{
					// Add it
					user.pixels += toAdd;
					user.saveCurrencies();
					session.sendPixelBalance();
					
					// Social boost hint?
					if(socialBoost && !isWithOthers)
					{
						session.sendNotif(_I("This time you have received the normal amount (%d) of pixels. Play in a room with other users to receive twice as much.", add));
					}
					
					// General notification?
					if(notify)
					{
						session.sendNotif(_I("You have received %d pixels!", toAdd));
					}
					
					// Processed a user!
					userAmount++;
					totalAmount += toAdd;
				}
			}
		}
		
		// Done!
		if(userAmount > 0)
		{
			this.gameMgr.getLogger().info("AutoPixelsUpdater", String.format("%d user(s) received a total of %d pixels.", userAmount, totalAmount));
		}
		
		// Configuration changed?
		int minutes = this.gameMgr.getConfig().getInt("game.users.autopixels.minutes");
		if(minutes != this.minutes)
		{
			this.reschedule();
		}
	}
}
