package habbod2.game.users.updaters;

import java.util.Calendar;

import habbod2.game.GameManager;
import habbod2.game.sessions.Session;
import habbod2.game.users.User;
import habbod2.util.StringUtil;
import habbod2.communication.messages.outgoing.handshake.UserObjectMessageComposer;

public class AutoRespectUpdater extends GameManager.Component implements Runnable
{
	public AutoRespectUpdater(GameManager gameMgr)
	{
		super(gameMgr);
	}
	
	private Calendar getNow()
	{
		Calendar c = Calendar.getInstance();
		c.setTimeInMillis(System.currentTimeMillis());
		return c;
	}

	public void scheduleDailyAtMidnight()
	{
		// First time: today, 0:00 (if past today, it will run immediately)
		Calendar firstTime = this.getNow();
		firstTime.set(Calendar.HOUR_OF_DAY, 0);
		firstTime.set(Calendar.MINUTE, 0);
		firstTime.set(Calendar.SECOND, 0);
		firstTime.set(Calendar.MILLISECOND, 0);
		this.getSystem().scheduleTaskWithFirstTime(this, 60 * 60 * 24 * 1000, firstTime.getTimeInMillis());
	}
	
	@Override
	public void run()
	{
		// Supposed to run now?
		Calendar now = this.getNow();
		Calendar lastUpdate = Calendar.getInstance();
		lastUpdate.setTimeInMillis(StringUtil.parseDateTime(this.getConfig().get("game.users.autorespect.lastupdate")));
		if(now.get(Calendar.DAY_OF_YEAR) == lastUpdate.get(Calendar.DAY_OF_YEAR))
		{
			this.getLogger().info(this, "Already updated today, will run again at midnight.");
			return;
		}
		
		// Get reset amounts
		int userRespectPoints = this.getConfig().getInt("game.users.autorespect.users.amount");
		int petRespectPoints = this.getConfig().getInt("game.users.autorespect.pets.amount");
		
		// Update it in the database
		this.getDb().execute("UPDATE users SET respect_points=" + userRespectPoints + ",petrespect_points=" + petRespectPoints);
		
		// Update + refresh online users
		for(Session session : this.getGameMgr().getSessionMgr().getAllSessions())
		{
			User user = session.getUser();
			user.respectPoints = userRespectPoints;
			user.petRespectPoints = petRespectPoints;
			session.send(UserObjectMessageComposer.compose(user));
		}
		
		// Done!
		this.getConfig().set("game.users.autorespect.lastupdate", StringUtil.getCurrentDateTime());
		this.getLogger().info(this, String.format("Updated respect points for all users. (users=%d, pets=%d)", userRespectPoints, petRespectPoints));
	}
}
