package habbod2.game.util;

import java.security.SecureRandom;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Random;

public class GameUtil
{
	private static final Random random = new SecureRandom();
	private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("dd-MM-yyy");
	private static final SimpleDateFormat DATETIME_FORMATTER = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
	public static final String DATA_DELIMITER = "/";

	public static void reseedRandom()
	{
		random.setSeed(System.nanoTime());
	}
	
	public static <T> T getRandomElement(T[] elements)
	{
		int length = elements.length;
		if(length > 0)
		{
			return elements[random.nextInt(length)];
		}
		else
		{
			return null;
		}
	}
	

	public static <T> T getRandomElement(List<T> elements)
	{
		if(elements.isEmpty())
		{
			return null;
		}
		else
		{
			return elements.get(random.nextInt(elements.size()));
		}
	}

	public static int getRandomInt(int min, int max)
	{
		return(min + random.nextInt(max - min + 1));
	}

	public static int getRandomDelta(int min, int max)
	{
		int delta = 0;
		while(delta == 0)
		{
			delta = GameUtil.getRandomInt(min, max);
		}
		return delta;
	}
	
	public static String formatDate(long millis)
	{
		return GameUtil.formatDate(new Date(millis));
	}

	public static String formatDate(Date date)
	{
		return DATE_FORMAT.format(date);
	}

	public static String getCurrentDate()
	{
		return GameUtil.formatDate(new Date());
	}

	public static String formatDateTime(long millis)
	{
		return GameUtil.formatDateTime(new Date(millis));
	}

	public static String getCurrentDateTime()
	{
		return GameUtil.formatDateTime(new Date());
	}

	public static String formatDateTime(Date date)
	{
		return DATETIME_FORMATTER.format(date);
	}

	public static int secondsToRoomPulses(final int seconds)
	{
		return((seconds * 1000) / 500 /*RoomInstance.PULSE_INTERVAL*/);
	}

	public static int secondsToDays(final int seconds)
	{
		return (int) Math.ceil(seconds / (60 * 60 * 24));
	}

	public static int daysBetween(final long millis1, long millis2)
	{
		long diff = millis2 - millis1;
		return GameUtil.secondsToDays((int) (diff / 1000));
	}

	public static int daysSince(final long millis)
	{
		return GameUtil.daysBetween(millis, System.currentTimeMillis());
	}

	public static int parseInt(String s)
	{
		if(s != null && !s.isEmpty())
		{
			try
			{
				return Integer.parseInt(s);
			}
			catch(Exception ex)
			{

			}
		}
		return 0;
	}
	
	public static float parseFloat(String s)
	{
		if(s != null && !s.isEmpty())
		{
			try
			{
				return Float.parseFloat(s);
			}
			catch(Exception ex)
			{

			}
		}
		return 0.0f;
	}
}
