package habbod2.game.util;

import habbod2.game.rooms.instance.RoomInstance;

import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;

public class GameUtil
{
	private static final Random random = new SecureRandom();
	public static final String DATA_DELIMITER = "/";

	public static void reseedRandom()
	{
		random.setSeed(System.nanoTime());
	}
	
	public static <T> T getRandomElement(T[] elements)
	{
		int length = elements.length;
		if(length > 0)
		{
			return elements[random.nextInt(length)];
		}
		else
		{
			return null;
		}
	}
	

	public static <T> T getRandomElement(List<T> elements)
	{
		if(elements.isEmpty())
		{
			return null;
		}
		else
		{
			return elements.get(random.nextInt(elements.size()));
		}
	}
	
	public static <T> T getRandomElement(Collection<T> elements)
	{
		List<T> list = new ArrayList<T>(elements);
		return GameUtil.getRandomElement(list);
	}

	public static int getRandomInt(int min, int max)
	{
		return Math.min(min, max) + random.nextInt(Math.abs(max - min) + 1);
	}
	
	

	public static int secondsToRoomPulses(final int seconds)
	{
		return((seconds * 1000) / RoomInstance.PULSE_INTERVAL);
	}
	
	public static int roomPulsesToSeconds(final int pulses)
	{
		return (pulses * RoomInstance.PULSE_INTERVAL);
	}

	public static int secondsToDays(final int seconds)
	{
		return (int) Math.ceil(seconds / (60 * 60 * 24));
	}

	public static int daysBetween(final long millis1, long millis2)
	{
		long diff = millis2 - millis1;
		return GameUtil.secondsToDays((int) (diff / 1000));
	}

	public static int daysSince(final long millis)
	{
		return GameUtil.daysBetween(millis, System.currentTimeMillis());
	}
}
