package habbod2.game.util;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

public class WordFilter
{
	private String censor;
	private boolean isEnabled;
	private List<String> words;

	public WordFilter()
	{
		this.words = new CopyOnWriteArrayList<String>();
	}
	
	public void clear()
	{
		this.words.clear();
	}
	
	public boolean addWord(String word)
	{
		word = normalize(word);
		if(this.words.contains(word))
		{
			return false;
		}
		else
		{
			this.words.add(word);
			return true;
		}
	}
	
	public boolean removeWord(String word)
	{
		return this.words.remove(normalize(word));
	}

	public String filter(String str)
	{
		if(this.isEnabled)
		{
			for(String word : this.words)
			{
				// Case-insensitive!
				str = str.replaceAll("(?i)" + word, this.censor);
			}
		}
		return str;
	}
	
	public String[] filter(String[] strs)
	{
		if(this.isEnabled)
		{
			for(int i = 0; i < strs.length; i++)
			{
				strs[i] = this.filter(strs[i]);
			}
		}
		return strs;
	}
	
	public String getCensor()
	{
		return this.censor;
	}
	
	public void setSensor(String censor)
	{
		if(censor.isEmpty())
		{
			censor = null;
		}
		this.censor = censor;
	}
	
	public boolean isEnabled()
	{
		return this.isEnabled;
	}
	
	public void setEnabled(boolean isEnabled)
	{
		this.isEnabled = isEnabled;
	}
	
	public int words()
	{
		return this.words.size();
	}
	
	private static String normalize(String word)
	{
		word = word.replace(".", "\\.");
		return word;
	}
	
	private static final WordFilter instance = new WordFilter();
	
	public static final WordFilter getInstance()
	{
		return instance;
	}
}
