package habbod2.game.rooms.items.floor;

import java.util.ArrayList;
import java.util.List;

import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.util.GameUtil;

public class TagPoleItem extends RoomFloorItem
{
	private int effectId;
	private int tileDefinitionId;
	private RoomUser tagger;
	
	@Override
	public void onRoomAdd(RoomInstance room)
	{
		String[] data = this.definition.data.split(" ");
		this.effectId = GameUtil.parseInt(data[0]);
		if(data.length >= 2)
		{
			this.tileDefinitionId = GameUtil.parseInt(data[1]);
		}
	}
	
	@Override
	public void onRoomRemove(RoomInstance room)
	{
		this.setTagger(null);
	}
	
	public void setTagger(RoomUser tagger)
	{	
		// Remove old tagger
		if(this.tagger != null)
		{
			this.tagger.setTagger(null);
			this.tagger = null;
		}
		
		// Set new tagger
		if(tagger != null)
		{
			this.tagger = tagger;
			this.tagger.setTagger(this);
		}
	}
	
	@Override
	public boolean requiresPulse()
	{
		return true;
	}
	
	@Override
	protected void onPulse()
	{
		// Tagger still valid?
		if(this.tagger != null)
		{
			if(this.tagger.invalidated() || !this.canBeTagger(this.tagger))
			{
				this.setTagger(null);
			}
		}
		
		// Game needs new tagger?
		if(this.tagger == null || this.tagger.invalidated())
		{
			// Find all users that are eligible
			List<RoomUser> users = new ArrayList<RoomUser>();
			for(RoomUser user : this.getRoom().getAllUsers())
			{
				if(!user.isTagger() && this.canBeTagger(user))
				{
					users.add(user);
				}
			}
			
			// Any players?
			if(!users.isEmpty())
			{
				this.setTagger(GameUtil.getRandomElement(users));
			}
		}
	}
	
	public boolean canBeTagger(RoomUser user)
	{
		// Tile check disabled?
		if(this.tileDefinitionId <= 0)
		{
			return true;
		}
		else
		{
			// Search through all the items where the user is currently on
			for(RoomFloorItem item : user.getTile().getItems())
			{
				if(item.definition.id == this.tileDefinitionId)
				{
					return true;
				}
			}
			
			// Not on game tile
			return false;
		}
	}
	
	public int getEffectId()
	{
		return this.effectId;
	}
}
