package habbod2.game.rooms.items.minigame;

import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.minigames.BanzaiGame;
import habbod2.game.rooms.minigames.Game;
import habbod2.game.util.GameUtil;

public class TimerItem extends RoomFloorItem
{
	@Override
	protected void onRoomAdd(RoomInstance room)
	{
		room.setMinigame(new BanzaiGame(room));
	}
	
	@Override
	protected void onRoomRemove(RoomInstance room)
	{
		room.getMinigame().destroy();
		room.setMinigame(null);
	}
	
	@Override
	protected boolean onToggle(RoomUser user, int instruction)
	{
		// Needs rights
		if(user != null && !user.hasRoomOwner)
		{
			return false;
		}
		
		// Get game
		Game game = this.getRoom().getMinigame();
		
		// Start/stop game
		if(instruction <= 1)
		{
			if(!game.isRunning())
			{
				game.start();
			}
			else
			{
				game.stop();
			}
		}
		
		// Add time to timer
		else if(instruction == 2)
		{
			// Determine new time left
			int timeLeft = this.getTimeLeft();
			if(timeLeft >= 0 && timeLeft < 30)
			{
				timeLeft = 30;
			}
			else if(timeLeft >= 30 && timeLeft < 60)
			{
				timeLeft = 60;
			}
			else if(timeLeft >= 60 && timeLeft < 120)
			{
				timeLeft = 120;
			}
			else if(timeLeft >= 120 && timeLeft < 180)
			{
				timeLeft = 180;
			}
			else if(timeLeft >= 180 && timeLeft < 240)
			{
				timeLeft = 240;
			}
			else if(timeLeft >= 240 && timeLeft < 300)
			{
				timeLeft = 300;
			}
			else if(timeLeft >= 300)
			{
				timeLeft = 600;
			}
			
			// Update time left
			game.setTimeLeft(timeLeft);
		}
		
		return true;
	}
	
	public void setTimeLeft(int seconds)
	{
		this.setState(seconds);
	}
	
	public int getTimeLeft()
	{
		return GameUtil.parseInt(this.state);
	}
}
