package habbod2.game.rooms.items.minigame.ball;

import habbod2.communication.messages.outgoing.room.engine.SlideObjectBundleMessageComposer;
import habbod2.game.rooms.geometry.Direction;
import habbod2.game.rooms.instance.RoomAvatar;
import habbod2.game.rooms.instance.RoomMap;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.minigame.banzai.PuckItem;

public class BallItem extends RoomFloorItem
{
	/*
	 * bal uit doel trekken: ervoor staan (facing), toggle
voor de bal staan, toggle: rol max 4 vooruit in richting van facing
over de bal heenrennen, einde van pad? schiet max 6 vooruit in richting van facing. Niet einde van pad? dribbelen, 1 vakje vooruit
Vooruit rollen: botsing? Reverse direction (opposite), tel verder vanaf daar
	 * @see habbod2.game.rooms.items.RoomFloorItem#onAvatarStepOn(habbod2.game.rooms.instance.RoomAvatar)
	 */
	
	private RoomAvatar lastAvatar;
	
	@Override
	public void onAvatarStepOn(RoomAvatar avatar)
	{
		if(avatar.getTile() == avatar.goal)
		{
			this.shoot(avatar, 3);
		}
		else
		{
			this.shoot(avatar, 1);
		}
	}
	
	@Override
	public boolean onToggle(RoomUser user, int instruction)
	{
		if(user.getTile().touches(this.getTile()))
		{
			return this.shoot(user, 1);
		}
		else
		{
			return false;
		}
	}
	
	public boolean shoot(RoomAvatar avatar, final int maxTiles)
	{
		// Record begin position
		RoomTile from = this.getTile();
		RoomTile to = from;
		RoomMap map = this.getRoom().getMap();
		
		// Shoot max x tiles in front of avatar, reverse and resume direction at obstacles
		int moved = 0;
		Direction dir = avatar.bodyDirection;
		while(moved < maxTiles)
		{
			RoomTile tile = map.getTile(to.x + dir.modX, to.y + dir.modY);
			if(tile != null && tile.isOpen())
			{
				to = tile;
			}
			else
			{
				dir = dir.invert();
			}
			moved++;
		}
		
		// Movable?
		if(to == from)
		{
			return false;
		}
		else
		{
			// Update item
			float fromZ = this.z;
			this.getRoom().moveItem(this.id, to.x, to.y, this.rotation, false);
			
			// Set state
			int state;
			if(this instanceof PuckItem)
			{
				state = 0;
				if(avatar instanceof RoomUser)
				{
					RoomUser user = (RoomUser)avatar;
					if(user.getPlayer() != null)
					{
						state = user.getPlayer().getTeam().getId().ordinal() + 1;
					}
				}
			}
			else
			{
				state = 1;
			}
			this.setTempState(state);
			
			// Record last avatar
			this.lastAvatar = avatar;
			
			// OK!
			this.getRoom().broadcast(SlideObjectBundleMessageComposer.compose(this, from, fromZ, null));
			return true;
		}
	}
	
	public RoomAvatar getLastAvatar()
	{
		return this.lastAvatar;
	}
	
	public void clearLastAvatar()
	{
		this.lastAvatar = null;
	}
}
