package habbod2.game.rooms.items.minigame.banzai;

import habbod2.game.rooms.instance.RoomAvatar;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.minigames.GameTeam.ETeam;
import habbod2.game.rooms.minigames.GameUserPlayer;
import habbod2.game.rooms.minigames.game.BanzaiGame;

public class GameTileItem extends RoomFloorItem
{
	public static final int LEVEL_EMPTY = 0;
	public static final int LEVEL_MEDIUM = 1;
	public static final int LEVEL_SEALED = 2;
	
	private ETeam teamId;
	private int level;
	
	private GameUserPlayer pendingPlayer;
	private boolean pendingReady;
	
	// 3-4-5 = red
	// 6-7-8 = green
	// 9-10-11 = blue
	// 12-13-14 = yellow
	
	// level: 0-1-2 (2 = locked)
	
	public void onAvatarStepOn(RoomAvatar avatar)
	{
		// Only RoomUsers can interact
		if(!(avatar instanceof RoomUser))
		{
			return;
		}
		
		// Only game players in a running game of Banzai
		GameUserPlayer player = ((RoomUser)avatar).getPlayer();
		if(player == null || !(player.getGame() instanceof BanzaiGame) || !player.getGame().isRunning())
		{
			return;
		}
		
		// Process the tile!
		this.pendingPlayer = player;
		((BanzaiGame)player.getGame()).tileUpdated(this);
	}
	
	public void update(boolean isEnabled, ETeam teamId, int level)
	{
		this.teamId = teamId;
		this.level = level;
		if(this.teamId == null)
		{
			this.setState(isEnabled ? 1 : 0);
		}
		else
		{
			this.setState(((teamId.ordinal() + 1) * 3) + level);
		}
	}
	
	public ETeam getTeamId()
	{
		return this.teamId;
	}
	
	public int getLevel()
	{
		return this.level;
	}
	
	public GameUserPlayer getPendingPlayer()
	{
		return this.pendingPlayer;
	}
	
	public void clearPendingPlayer()
	{
		this.pendingPlayer = null;
		this.pendingReady = false;
	}
	
	public boolean updatePendingTimer()
	{
		if(this.pendingReady)
		{
			return true;
		}
		else
		{
			this.pendingReady = true;
			return false;
		}
	}
	
	public static String generateState(boolean isEnabled, ETeam teamId, int level)
	{
		if(!isEnabled)
		{
			return "0";
		}
		else
		{
			return Integer.toString(((teamId.ordinal() + 1) * 3) + level);
		}
	}
}
