package habbod2.game.rooms.items.wired;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONObject;

import habbod2.net.protocol.ServerMessage;
import habbod2.communication.messages.outgoing.userdefinedroomevents.WiredFurniActionMessageComposer;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;

public abstract class WiredAction extends WiredItem
{
	private int delay;
	private RoomUser user;
	
	public abstract void execute(RoomUser user);
	protected abstract boolean requiresUser();
	
	@Override
	public void onRoomRemove(RoomInstance room)
	{
		super.onRoomRemove(room);
		this.user = null;
	}
	
	@Override
	protected void onWrap(JSONObject data) throws Exception
	{
		super.onWrap(data);
		if(this.delay > 0) data.put("del", this.delay);
	}
	
	@Override
	protected void onUnwrap(JSONObject data) throws Exception
	{
		super.onUnwrap(data);
		this.delay = data.optInt("del");
	}
	
	protected void execute(RoomUser user, int delay)
	{
		this.user = user;
		this.setTimeout(delay);
	}
	
	public int getDelay()
	{
		return this.delay;
	}
	
	public void setDelay(int delay)
	{
		this.delay = delay;
	}
	
	@Override
	protected void onTimeout()
	{
		// If a user is required, then validate it
		if(!this.requiresUser() || (user != null && !this.user.invalidated()))
		{
			this.flash();
			this.execute(this.user);
		}
		
		// Clear user
		this.user = null;
	}
	
	public List<WiredTrigger> getIncompatibleTriggers()
	{
		List<WiredTrigger> incompatible = new ArrayList<WiredTrigger>(0);
		
		// This item needs a user?
		if(this.requiresUser())
		{
			// Search all items in the stack
			for(RoomFloorItem item : this.getStackedItems())
			{
				// Item is trigger?
				if(item instanceof WiredTrigger)
				{
					// Trigger is incompatible?
					WiredTrigger trigger = (WiredTrigger)item;
					if(!trigger.suppliesUser())
					{
						incompatible.add(trigger);
					}
				}
			}
		}
		
		return incompatible;
	}
	
	@Override
	public ServerMessage getDialog()
	{
		return WiredFurniActionMessageComposer.compose(this);
	}
}
