package habbod2.game.rooms.items.wired;

import java.util.List;
import java.util.ArrayList;

import habbod2.net.protocol.ServerMessage;
import habbod2.communication.messages.outgoing.userdefinedroomevents.WiredFurniTriggerMessageComposer;
import habbod2.game.rooms.instances.RoomInstance;
import habbod2.game.rooms.instances.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.util.GameUtil;
import habbod2.game.rooms.items.wired.addons.*;

public abstract class WiredTrigger extends WiredItem
{
	private long lastTime;
	
	protected abstract boolean suppliesUser();
	
	public final void execute(RoomUser user)
	{
		// Infinite loop prevention
		if((System.currentTimeMillis() - lastTime) < RoomInstance.PULSE_INTERVAL)
		{
			return;
		}
		this.lastTime = System.currentTimeMillis();
		
		// Fire!
		this.flash();
		
		// Get all items in the stack
		List<RoomFloorItem> items = this.getStackedItems();

		// Search for Addons
		boolean useRandomAction = false;
		boolean useUnusedAction = false;
		float invertConditionsHeightTreshold = 999;
		for(RoomFloorItem item : items)
		{
			if(item instanceof WiredAddon)
			{
				if(item instanceof RandomActionAddon)
				{
					useRandomAction = true;
				}
				else if(item instanceof UnusedActionAddon)
				{
					useUnusedAction = true;
				}
				else if(item instanceof InvertTopConditionsAddon)
				{
					if(item.z < invertConditionsHeightTreshold)
					{
						invertConditionsHeightTreshold = item.z;
					}
				}
			}
		}
		
		// Search for Conditions
		for(RoomFloorItem item : items)
		{
			// Is this a Condition?
			if(item instanceof WiredCondition)
			{
				System.out.println(item.definition.getRoomClass().toString());
				
				// Determine the desired result
				boolean requiredResult = true;
				if(item.z > invertConditionsHeightTreshold)
				{
					requiredResult = false;
				}
			
				// Evaluate the Condition
				WiredCondition condition = (WiredCondition)item;
				if(condition.evaluate(user) == requiredResult)
				{
					condition.flash();
				}
				else
				{
					return;
				}
			}
		}
		
		// Execute the proper Action
		if(useRandomAction)
		{
			// Pick a random Action
			List<WiredAction> actions = new ArrayList<WiredAction>(2);
			for(RoomFloorItem item : items)
			{
				if(item instanceof WiredAction)
				{
					actions.add((WiredAction)item);
				}
			}
			
			/// ... and execute it!
			this.executeAction(GameUtil.getRandomElement(actions), user);
		}
		else
		{
			// Execute all Actions in the stack
			for(RoomFloorItem item : items)
			{
				if(item instanceof WiredAction)
				{
					this.executeAction((WiredAction)item, user);
				}
			}
		}
	}
	
	private void executeAction(WiredAction action, RoomUser user)
	{
		if(action != null && (user != null || !action.requiresUser()))
		{
			action.flash();
			if(action.getDelay() == 0)
			{
				action.execute(user);
			}
			else
			{
				action.execute(user, action.getDelay());
			}
		}
	}
	
	public List<WiredAction> getIncompatibleActions()
	{
		List<WiredAction> incompatible = new ArrayList<WiredAction>(0);
		
		// This trigger doesn't supply a user?
		if(!this.suppliesUser())
		{
			// Search all items in the stack
			for(RoomFloorItem item : this.getStackedItems())
			{
				// Item is action?
				if(item instanceof WiredAction)
				{
					// Action is incompatible?
					WiredAction action = (WiredAction)item;
					if(action.requiresUser())
					{
						incompatible.add(action);
					}
				}
			}
		}
		
		return incompatible;
	}
	
	@Override
	public ServerMessage getDialog()
	{
		return WiredFurniTriggerMessageComposer.compose(this);
	}
}
