package habbod2.communication.messages.incoming.friendlist;

import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.friendlist.FollowFriendFailedMessageComposer;
import static habbod2.communication.messages.outgoing.friendlist.FollowFriendFailedMessageComposer.*;
import habbod2.communication.messages.outgoing.room.session.RoomForwardMessageComposer;
import habbod2.game.messenger.MessengerBuddy;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.sessions.Session;

public class FollowFriendHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Determine buddy to follow / 'stalk'
		int buddyId = request.readInt();
		MessengerBuddy buddy = session.getMessenger().getBuddy(buddyId);
		
		// Determine room of buddy
		RoomInstance room = null;
		if(buddy != null && buddy.inRoom())
		{
			room = buddy.getSession().getRoom();
		}
		
		// Determine error
		int error = -1;
		if(buddy == null)
		{
			error = ERROR_NOFRIEND;
		}
		else if(!buddy.isOnline())
		{
			error = ERROR_OFFLINE;
		}
		else if(!buddy.inRoom())
		{
			error = ERROR_NOTINROOM;
		}
		else if(room == session.getRoom())
		{
			return;
		}
		
		// Failed?
		if(error != -1)
		{
			session.send(FollowFriendFailedMessageComposer.compose(error));
		}
		else
		{
			session.send(RoomForwardMessageComposer.compose(room.getInfo()));
		}
	}
}
