package habbod2.communication.messages.incoming.room.avatar;

import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.game.pets.RoomPet;
import habbod2.game.rooms.instance.RoomActor;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.floor.TagPoleItem;
import habbod2.game.sessions.Session;

public class LookToHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// User in room?
		RoomUser self = session.getRoomUser();
		if(self != null)
		{
			// Get the tile that was clicked
			RoomTile tile = self.getRoom().getMap().getTile(request.readInt(), request.readInt());
			if(tile == null)
			{
				return;
			}
			
			// Rotate actor
			self.lookTo(tile, false);
			
			// Any actor on tile in front?
			tile = self.getTileInFront();
			if(tile == null || !tile.hasActor())
			{
				return;
			}
			
			// Handle tagging game
			if(self.isTagger())
			{
				// Find the user
				TagPoleItem tagPole = self.getTagPole();
				for(RoomUser user : self.getRoom().getAllUsers())
				{
					if(user.at(tile) && !user.isTagger() && tagPole.canBeTagger(user))
					{
						tagPole.setTagger(user);
						return;
					}
				}
			}
			
			// Handle mounting of pets
			if(!self.isMounted() && session.getUser().hasRight("mount_pet"))
			{
				for(RoomActor actor : self.getRoom().getAllActors())
				{
					if(actor.at(tile) && actor instanceof RoomPet)
					{
						RoomPet pet = (RoomPet)actor;
						if(!pet.hasRider() && pet.getInfo().type.isMountable())
						{
							self.setMount(pet);
							break;
						}
					}
				}
			}
		}
	}
}
