package habbod2.communication.messages.incoming.room.engine;

import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.game.pets.Pet;
import habbod2.game.pets.RoomPet;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.sessions.Session;

public class PlacePetHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Allowed to place pets?
		RoomInstance room = session.getRoom();
		if(!room.getInfo().allowOtherPets && !session.getRoomUser().hasRoomRights)
		{
			return;
		}
		
		// Read info
		int petId = request.readInt();
		int tileX = request.readInt();
		int tileY = request.readInt();
		boolean specificTile = session.getRoomUser().hasRoomOwner;
		
		// Pet in inventory?
		Pet pet = session.getPetInventory().removePet(petId);
		if(pet == null)
		{
			return;
		}
		
		// Work out tile
		RoomTile tile = specificTile ? room.getMap().getTile(tileX, tileY) : session.getRoomUser().getTileInFront();
		if(tile == null || !tile.isOpen() || (tile.is(RoomTile.WATER) && !pet.type.canSwim))
		{
			if(specificTile)
			{
				return;
			}
			else
			{
				tile = room.getMap().getDoor();
			}
		}
		
		// Spawn roomactor
		RoomPet actor = new RoomPet(pet);
		room.spawnActor(actor, tile);
		
		// Save to room or inventory?
		if(room.getInfo().ownerId == session.getUser().id)
		{
			actor.syncAndSaveLocationInfo(room.getInfo());
		}
		else
		{
			actor.syncAndSaveLocationInfo(null);
		}
		
		// Greet owner
		actor.greetOwner();
		
		// Look to owner!
		RoomUser me = session.getRoomUser();
		actor.lookTo(me.getTile(), false);
	}
}
