package habbod2.game.chatcommands;

import habbod2.game.users.User;
import habbod2.game.util.GameError;
import habbod2.game.sessions.Session;

public abstract class ChatCommandHandler
{
	public static final char COMMAND_HEAD = ':';

	private String command;
	private String description;
	private String requiredRight;
	private String[] syntax;
	
	public String getCommand()
	{
		return this.command;
	}
	
	public String getDescription()
	{
		return this.description;
	}
	
	public String getRequiredRight()
	{
		return this.requiredRight;
	}
	
	public String[] getSyntax()
	{
		return this.syntax;
	}
	
	public String getUsage()
	{
		StringBuilder sb = new StringBuilder();
		sb.append(ChatCommandHandler.COMMAND_HEAD);
		sb.append(this.command);
		if(this.syntax != null)
		{
			for(String arg : syntax)
			{
				sb.append(' ');
				sb.append('[');
				sb.append(arg);
				sb.append(']');
			}
		}
		
		return sb.toString();
	}
	
	public void initialize(String command, String description, String requiredRight, String syntax)
	{
		this.command = command;
		this.description = description;
		this.requiredRight = requiredRight;
		if(syntax != null && !syntax.isEmpty())
		{
			this.syntax = syntax.split(" ");
		}
	}
	
	public boolean isAccessibleTo(User user)
	{
		return this.requiredRight == null || user.hasRight(this.requiredRight);
	}
	
	public abstract void handle(Session session, String... args) throws GameError;
}
