package habbod2.game.misc;

import habbod2.game.GameManager;
import habbod2.game.util.WordFilter;
import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;

public class WordFilterManager extends GameManager.Component
{
	private WordFilter filter;
	
	public WordFilterManager(GameManager gameMgr)
	{
		super(gameMgr);
		this.filter = WordFilter.getInstance();
	}
	
	public void reload()
	{
		// Load censor
		this.filter.setEnabled(this.getConfig().getBool("wordfilter.enabled"));
		this.filter.setSensor(this.getConfig().get("wordfilter.censor", "$#!*@"));
		
		// Clear old words
		this.filter.clear();
		
		// Load new words
		try(DatabaseResult result = this.getDb().query("SELECT word FROM system_wordfilter"))
		{
			while(result.row())
			{
				this.filter.addWord(result.getString("word"));
			}
		}
		
		// Done!
		this.getLogger().info(this, String.format("Loaded %d words, censor='%s', enabled=%s.", this.filter.words(), this.filter.getCensor(), this.filter.isEnabled()));
	}
	
	public boolean addWord(String word)
	{
		boolean success = this.filter.addWord(word);
		if(success)
		{
			this.getDb().execute("INSERT INTO system_wordfilter(word) VALUES ('" + SQLHelper.escape(word) + "')");
		}
		return success;
	}
	
	public boolean removeWord(String word)
	{
		boolean success = this.filter.removeWord(word);
		if(success)
		{
			this.getDb().execute("DELETE FROM system_wordfilter WHERE word='" + SQLHelper.escape(word) + "'");
		}
		return success;
	}
	
	public int words()
	{
		return this.filter.words();
	}
}
