package habbod2.game.pets;

import java.util.ArrayList;
import java.util.List;

import habbod2.Instance;
import habbod2.game.util.GameUtil;

public class Pet
{
	private static final int[] LEVELS = new int[] { 100, 200, 400, 600, 1000, 1300, 1800, 2400, 3200, 4300, 7200, 8500, 10100, 13300, 17500, 23000, 51900 };

	// Base obey chance
	private static final double BASE_OBEY_CHANCE = 0.75;
	
	// Min/max seconds between 'hunger'
	private static final int TIMER_EAT_MIN = 2 * 60;
	private static final int TIMER_EAT_MAX = 10 * 60;
	
	// Min/max seconds between 'thirst'
	private static final int TIMER_DRINK_MIN = 2 * 60;
	private static final int TIMER_DRINK_MAX = 5 * 60;
	
	// Min/max seconds between 'sleepy'
	private static final int TIMER_SLEEP_MIN = 5 * 60;
	private static final int TIMER_SLEEP_MAX = 15 * 60;
	//private static final float TIMER_SLEEP_AGE_MULTIPLIER = 1.0f;
	
	public int id;
	public String name;

	public PetType type;
	public int breed;
	public String color;

	public int ownerId;
	public String ownerName;
	public long createdTime;

	public int level;
	public int experience;
	public int energy;
	public int happiness;
	public int respect;
	
	public int eatTimer;
	public int drinkTimer;
	public int sleepTimer;
	
	public int roomId;
	public int x;
	public int y;
	public float z;

	public String getFigure()
	{
		return this.type.id + " " + this.breed + " " + this.color;
	}

	public int getAgeInDays()
	{
		return GameUtil.daysSince(this.createdTime);
	}
	
	public final int getMaxLevel()
	{
		return 20;
	}
	
	public int getMaxEnergy()
	{
		return 100 + ((this.level - 1) * 20);
	}
	
	public int getEnergyPercentage()
	{
		return (this.energy / this.getMaxEnergy()) * 100;
	}
	
	public void modifyEnergy(int energy)
	{
		this.energy += energy;
		if(this.energy < 0)
		{
			this.energy = 0;
		}
		else
		{
			int max = this.getMaxEnergy();
			if(this.energy > max)
			{
				this.energy = max;
			}
		}
	}
	
	public final int getMaxHappiness()
	{
		return 100;
	}
	
	public void modifyHappiness(int happiness)
	{
		this.happiness += happiness;
		if(this.happiness < 0)
		{
			this.happiness = 0;
		}
		else
		{
			int max = this.getMaxHappiness();
			if(this.happiness > max)
			{
				this.happiness = max;
			}
		}
	}

	public int getExperienceGoal()
	{
		int index = this.level + 1;
		if(index >= Pet.LEVELS.length - 1)
		{
			index = Pet.LEVELS.length - 1;
		}
		return Pet.LEVELS[index];
	}
	
	public void recalculateLevel()
	{
		this.level = Pet.LEVELS.length + 1;
		for(int tmpLevel = 0; tmpLevel < Pet.LEVELS.length; ++tmpLevel)
		{
			if(this.experience < Pet.LEVELS[tmpLevel])
			{
				this.level = tmpLevel + 1;
				break;
			}
		}
	}
	
	public void resetEatTimer(boolean wasSuccess)
	{
		int seconds = GameUtil.getRandomInt(TIMER_EAT_MIN, TIMER_EAT_MAX);
		if(!wasSuccess)
		{
			seconds = seconds / 2;
		}
		this.eatTimer = GameUtil.secondsToRoomPulses(seconds);
	}
	
	public void resetDrinkTimer(boolean wasSuccess)
	{
		int seconds = GameUtil.getRandomInt(TIMER_DRINK_MIN, TIMER_DRINK_MAX);
		if(!wasSuccess)
		{
			seconds = seconds / 2;
		}
		this.drinkTimer = GameUtil.secondsToRoomPulses(seconds);
	}
	
	public void resetSleepTimer(boolean wasSuccess)
	{
		// TODO: account age
		int seconds = GameUtil.getRandomInt(TIMER_SLEEP_MIN, TIMER_SLEEP_MAX);
		if(!wasSuccess)
		{
			seconds = seconds / 2;
		}
		this.sleepTimer = GameUtil.secondsToRoomPulses(seconds);
	}
	
	public void save()
	{
		Instance.get().getDb().execute("UPDATE pets SET experience=" + this.experience + ",energy=" + this.energy + ",happiness=" + this.happiness + ",respect=" + this.respect + ",timer_eat=" + this.eatTimer + ",timer_drink=" + this.drinkTimer + ",timer_sleep=" + this.sleepTimer + ",room_id=" + this.roomId + ",x=" + this.x + ",y=" + this.y + ",z=" + this.z + " WHERE id=" + this.id);
	}
	
	public List<PetCommand> getAccessibleCommands()
	{
		List<PetCommand> accessible = new ArrayList<PetCommand>();
		for(PetCommand cmd : this.type.getCommands())
		{
			if(this.level >= cmd.level)
			{
				accessible.add(cmd);
			}
		}
		
		return accessible;
	}
	
	public double getObeyChance()
	{
		return BASE_OBEY_CHANCE + (this.level * ((BASE_OBEY_CHANCE - 0.001) / this.getMaxLevel()));
	}
}
