package habbod2.game.rooms;

import habbod2.Instance;
import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;

public class Room implements Comparable<Room>
{
	public static final byte STATE_OPEN = 0;
	public static final byte STATE_DOORBELL = 1;
	public static final byte STATE_PASSWORD = 2;

	public int id;
	public int ownerId;
	public String ownerName;
	public String name;
	public String description;
	public String modelName;
	public String files;
	public int categoryId;
	public byte state;
	public String password;
	public int usersNow;
	public int usersMax;
	public String wallpaper;
	public String floor;
	public String landscape;
	public boolean hideWalls;
	public int wallThickness;
	public int floorThickness;
	public int rating;
	public int visits;
	public String icon;
	public String[] tags;
	public boolean allowOtherPets;
	public boolean allowOtherPetsEat;
	public boolean allowWalkthrough;
	public boolean allKicking;
	public boolean isFrontpageItem;

	public boolean isPublicRoom()
	{
		return(this.ownerId == 0);
	}

	public boolean isGuestRoom()
	{
		return(this.ownerId > 0);
	}

	public void saveSettings()
	{
		Instance.get().getDb().execute("UPDATE rooms SET name='" + SQLHelper.escape(this.name) + "',description='" + SQLHelper.escape(this.description) + "',category_id=" + this.categoryId + ",state=" + this.state + ",password='" + SQLHelper.escape(this.password) + "',users_max=" + this.usersMax + ",icon='" + SQLHelper.escape(this.icon) + "',tag1='" + SQLHelper.escape(this.tags[0]) + "',tag2='" + SQLHelper.escape(this.tags[1]) + "',allow_pets=" + (this.allowOtherPets ? '1' : '0') + ",allow_pets_eat=" + (this.allowOtherPetsEat ? '1' : '0') + ",allow_walkthrough=" + (this.allowWalkthrough ? '1' : '0') + ",hide_walls=" + (this.hideWalls ? '1' : '0') + ",wall_thickness=" + this.wallThickness + ",floor_thickness=" + this.floorThickness + ",all_kicking=" + (this.allKicking ? '1' : '0') + " WHERE id=" + this.id);
	}

	public void saveDecoration()
	{
		Instance.get().getDb().execute("UPDATE rooms SET wallpaper='" + SQLHelper.escape(this.wallpaper) + "',floor='" + SQLHelper.escape(this.floor) + "',landscape='" + SQLHelper.escape(this.landscape) + "' WHERE id=" + this.id);
	}

	public void saveUserStats()
	{
		Instance.get().getDb().execute("UPDATE rooms SET users_now=" + this.usersNow + ",visits=" + this.visits + " WHERE id=" + this.id);
	}

	public void saveRating()
	{
		Instance.get().getDb().execute("UPDATE rooms SET rating=" + this.rating + " WHERE id=" + this.id);
	}

	public static Room parse(DatabaseResult result)
	{
		Room room = new Room();
		room.id = result.getInt("id");
		room.ownerId = result.getInt("owner_id");
		if(room.ownerId > 0)
		{
			room.ownerName = result.getString("owner");
		}
		else
		{
			room.files = result.getString("files");
		}
		room.name = result.getString("name");
		room.description = result.getString("description");
		room.modelName = result.getString("model_name");
		room.categoryId = result.getInt("category_id");
		room.state = result.getByte("state");
		room.password = result.getString("password");
		room.usersNow = result.getInt("users_now");
		room.usersMax = result.getInt("users_max");
		room.wallpaper = result.getString("wallpaper");
		room.floor = result.getString("floor");
		room.landscape = result.getString("landscape");
		room.hideWalls = result.getBoolean("hide_walls");
		room.wallThickness = result.getInt("wall_thickness");
		room.floorThickness = result.getInt("floor_thickness");
		room.rating = result.getInt("rating");
		room.visits = result.getInt("visits");
		room.icon = result.getString("icon");
		room.tags = new String[] { result.getString("tag1"), result.getString("tag2") };
		room.allowOtherPets = result.getBoolean("allow_pets");
		room.allowOtherPetsEat = result.getBoolean("allow_pets_eat");
		room.allowWalkthrough = result.getBoolean("allow_walkthrough");
		room.allKicking = result.getBoolean("all_kicking");
		return room;
	}
	
	@Override
	public boolean equals(Object other)
	{
		return(other instanceof Room && ((Room)other).id == this.id);
	}

	@Override
	public int compareTo(Room other)
	{
		if(this.usersNow > other.usersNow)
		{
			return 1;
		}
		else if(this.usersNow < other.usersNow)
		{
			return -1;
		}
		else
		{
			return 0;
		}
	}
}
