package habbod2.game.rooms.items;

import habbod2.Instance;
import habbod2.game.rooms.Room;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.items.InventoryItem;
import habbod2.game.items.Item;
import habbod2.storage.SQLHelper;
import habbod2.util.StringUtil;

public abstract class RoomItem extends Item
{
	private RoomInstance room;
	private int pulseTimer;
	
	public final RoomInstance getRoom()
	{
		return this.room;
	}
	
	public final void setRoom(RoomInstance newRoom)
	{
		if(this.room != null || newRoom != null)
		{
			if(newRoom != null)
			{
				this.room = newRoom;
				this.onRoomAdd();
			}
			else
			{
				this.onRoomRemove();
				this.room = null;
			}
		}
	}
	
	public final void clearRoom()
	{
		this.setRoom(null);
	}
	
	public final boolean invalidated()
	{
		return (this.room == null);
	}
	
	protected void onRoomAdd()
	{
		
	}
	
	protected void onRoomRemove()
	{
		
	}
	
	public final void setState(int state)
	{
		this.setState(Integer.toString(state));
	}
	
	public final void setState(String state)
	{
		if(!state.equals(this.state))
		{
			this.state = state;
			this.requiresDbUpdate = true;
			this.refreshState(false);
		}
	}
	
	public final void setTempState(int state)
	{
		this.setTempState(Integer.toString(state));
	}

	public final void setTempState(String state)
	{
		String backup = this.state;
		this.state = state;
		this.refreshState(true);
		this.state = backup;
	}

	public abstract void refreshState(boolean isTemporarily);
	
	public final void toggle(RoomUser user, int instruction)
	{
		// Toggle
		String previousState = this.state;
		this.onToggle(user, instruction);
		
		// State changed?
		if(this.state != null && !this.state.equals(previousState))
		{
			this.onStateChange(user);
		}
	}
	
	protected void onToggle(RoomUser user, int instruction)
	{
		// Multiple states?
		if(this.definition.states > 0)
		{
			if(user == null || user.hasRoomRights)
			{
				this.nextState();
			}
		}
	}
	
	public void onStateChange(RoomUser user)
	{
		
	}
	
	public void nextState()
	{
		if(this.definition.states > 0)
		{
			int nextState = StringUtil.parseInt(this.state) + 1;
			if(nextState >= this.definition.states)
			{
				nextState = 0;
			}
			this.setState(nextState);
		}
	}

	public boolean requiresPulse()
	{
		return this.hasTimeout();
	}

	protected final boolean hasTimeout()
	{
		return(this.pulseTimer > 0);
	}

	protected final void setTimeout(int pulses)
	{
		this.pulseTimer = pulses;
	}

	protected final void clearTimeout()
	{
		this.pulseTimer = 0;
	}

	public final void pulse()
	{		
		// Pulse!
		this.onPulse();
		if(this.pulseTimer > 0)
		{
			this.pulseTimer--;
		}
		
		// End of timeout?
		if(this.pulseTimer == 0)
		{
			this.clearTimeout();
			this.onTimeout();
		}
	}

	protected void onPulse()
	{
		// Optionally overridden by impl class
	}

	protected void onTimeout()
	{
		// Optionally overridden by impl class
	}
	
	public final void update()
	{
		Room room = this.getRoom().getInfo();
		Instance.get().getDb().execute("UPDATE items SET owner_id=" + room.ownerId + ",room_id=" + room.id + ",position='" + SQLHelper.escape(this.getDbPosition()) + "',state='" + SQLHelper.escape(this.state) + "',data='" + SQLHelper.escape(this.data) + "' WHERE id=" + this.id);
		this.requiresDbUpdate = false;
	}
	
	public abstract String getDbPosition();
	
	public abstract void setPositionFromDb(String position);
	
	public InventoryItem toInventoryItem()
	{
		InventoryItem item = new InventoryItem();
		item.id = this.id;
		item.definition = this.definition;
		item.state = this.state;
		item.data = this.data;
		
		return item;
	}
}
