package habbod2.game.rooms.minigames.game;

import java.util.List;
import java.util.ArrayList;

import habbod2.Instance;
import habbod2.storage.Database;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.minigame.TimerItem;
import habbod2.game.rooms.items.minigame.banzai.GameTileItem;
import habbod2.game.rooms.minigames.Game;
import habbod2.game.rooms.minigames.GameTeam;
import habbod2.game.rooms.minigames.GameUserPlayer;
import habbod2.game.rooms.minigames.GameUserPlayer.GameValues;

public class BanzaiGame extends Game
{
	private static final int EFFECT_OFFSET = 33;
	
	private List<GameTileItem> pendingTiles = new ArrayList<GameTileItem>();
	
	public BanzaiGame(TimerItem timer)
	{
		super(timer);
	}
	
	@Override
	protected void onBegin()
	{
		// Power on game tiles
		for(RoomFloorItem item : this.getItems())
		{
			if(item instanceof GameTileItem)
			{
				((GameTileItem)item).update(true, null, GameTileItem.LEVEL_EMPTY);
			}
		}
	}
	
	@Override
	protected void onEnd()
	{
		// Clear pending tiles
		this.pendingTiles.clear();
		
		// Power off game tiles
		for(RoomFloorItem item : this.getItems())
		{
			if(item instanceof GameTileItem)
			{
				((GameTileItem)item).update(false, null, GameTileItem.LEVEL_EMPTY);
			}
		}
		
		// Update stats
		Database db = Instance.get().getDb();
		GameTeam winner = this.getBestTeam();
		for(GameTeam team : this.getTeams())
		{
			for(GameUserPlayer player : team)
			{
				BanzaiValues values = player.getValues();
				if(values.lockedTileCount > 0)
				{
					db.execute("UPDATE users SET bb_tiles=bb_tiles+" + values.lockedTileCount + " WHERE id=" + player.getUser().getInfo().id);
				}
				if(team == winner)
				{
					db.execute("UPDATE users SET bb_wins=bb_wins+1 WHERE id=" + player.getUser().getInfo().id);
				}
			}
		}
	}

	@Override
	protected void onComplete(GameTeam winner)
	{
		
	}
	
	@Override
	protected void onPulse()
	{
		// Only process tiles when the game is actually running
		if(this.isRunning())
		{
			// Any pending tiles?
			if(!this.pendingTiles.isEmpty())
			{
				// Go through all the tiles
				for(GameTileItem tile : this.pendingTiles)
				{
					// Get player that moved over this tile
					GameUserPlayer player = tile.popPendingPlayer();
					if(player == null)
					{
						continue;
					}
					
					// Get player values
					BanzaiValues values = player.getValues();
					if(values == null)
					{
						continue;
					}
					
					// Tile is not locked yet?
					if(!tile.isLocked())
					{
						// Upgrade the tile
						if(tile.getTeamId() != player.getTeam().getId())
						{
							tile.update(true, player.getTeam().getId(), GameTileItem.LEVEL_EMPTY);
						}
						else
						{
							tile.update(true, player.getTeam().getId(), tile.getLevel() + 1);
						}
						
						// Tile is locked now?
						if(tile.isLocked())
						{
							// Scored a point!
							player.addScore(1);
							values.lockedTileCount++;
							
							// TODO: solve shape and determine extra tiles
							for(GameTileItem t : BanzaiSolver.solveShapesGetExtraTiles(tile, this.getRoom().getMap(), player.getTeam().getId()))
							{
								t.update(true, player.getTeam().getId(), GameTileItem.LEVEL_LOCKED);
								player.addScore(1);
								values.lockedTileCount++;
							}
						}
					}
				}
				
				// Count remaining tiles
				int lockableTiles = 0;
				for(RoomFloorItem item : this.getItems())
				{
					if(item instanceof GameTileItem)
					{
						GameTileItem tile = (GameTileItem)item;
						if(!tile.isLocked() && tile.getTile().topItem == tile)
						{
							lockableTiles++;
						}
					}
				}
				
				// Game ended?
				if(lockableTiles == 0)
				{
					this.complete();
				}
			}
		}
		else
		{
			// Get winner
			GameTeam winner = this.getBestTeam();
			
			// Winner? Flash winning tiles
			if(winner != null)
			{
				for(RoomFloorItem item : this.getItems())
				{
					if(item instanceof GameTileItem)
					{
						GameTileItem tile = (GameTileItem)item;
						if(tile.getTeamId() == winner.getId())
						{
							int displayLevel = (this.shutdownTimer % 2 == 0) ? tile.getLevel() : GameTileItem.LEVEL_EMPTY;
							tile.setState(GameTileItem.generateState(true, tile.getTeamId(), displayLevel));
						}
					}
				}
			}
		}
	}
	
	public void tileTouched(GameTileItem tile)
	{
		if(!this.pendingTiles.contains(tile))
		{
			this.pendingTiles.add(tile);
		}
	}
	
	@Override
	public int getEffectOffset()
	{
		return BanzaiGame.EFFECT_OFFSET;
	}
	
	@Override
	public GameValues createValuesForPlayer()
	{
		return new BanzaiValues();
	}
	
	public class BanzaiValues implements GameUserPlayer.GameValues
	{
		public GameTileItem pendingTile;
		public GameTileItem lastTile;
		public int lockedTileCount;
	}
}
