package habbod2.game.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.ThreadLocalRandom;

import habbod2.game.rooms.instance.RoomInstance;

public class GameUtil
{
	public static final String DATA_DELIMITER = "/";
	
	private static final ThreadLocalRandom getRng()
	{
		return ThreadLocalRandom.current();
	}
	
	public static <T> T getRandomElement(T[] elements)
	{
		int length = elements.length;
		if(length > 0)
		{
			return elements[getRng().nextInt(length)];
		}
		else
		{
			return null;
		}
	}

	public static <T> T getRandomElement(List<T> elements)
	{
		int size = elements.size();
		if(size > 0)
		{
			return elements.get(getRng().nextInt(size));
		}
		else
		{
			return null;
		}
	}
	
	public static <T> T getRandomElement(Collection<T> elements)
	{
		List<T> list = new ArrayList<T>(elements);
		return getRandomElement(list);
	}

	public static int getRandomInt(int min, int max)
	{
		return getRng().nextInt(min, max + 1);
	}
	
	public static boolean getRandomBool(double p)
	{
		return (getRng().nextDouble() < p);
	}
	
	public static int secondsToRoomPulses(final int seconds)
	{
		return seconds * (1000 / RoomInstance.PULSE_INTERVAL);
	}
	
	public static int roomPulsesToSeconds(final int pulses)
	{
		return (pulses * RoomInstance.PULSE_INTERVAL);
	}

	public static int secondsToDays(final int seconds)
	{
		return (int) Math.ceil(seconds / (60 * 60 * 24));
	}

	public static int daysBetween(final long millis1, long millis2)
	{
		long diff = millis2 - millis1;
		return secondsToDays((int) (diff / 1000));
	}

	public static int daysSince(final long millis)
	{
		return daysBetween(millis, System.currentTimeMillis());
	}
	
	private static final float HEIGHT_DEC_PLACES = 3;
	private static final float HEIGHT_ROUNDER = (float) Math.pow(10, HEIGHT_DEC_PLACES);
	
	public static float roundHeight(float height)
	{
		if(height > 0.0f && height < 0.001f)
		{
			return 0.001f;
		}
		else
		{
			return Math.round(height * HEIGHT_ROUNDER) / HEIGHT_ROUNDER;
		}
	}
}
