package habbod2.game.rooms.items.floor;

import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;

public class OneWayDoorItem extends UsableFloorItem
{
	@Override
	protected void onToggle(RoomUser user, int instruction)
	{
		// User operating?
		if(user == null)
		{
			return;
		}
		
		// Attempt to move infront of the gate
		RoomTile frontTile = this.getTileInFront();
		if(!user.at(frontTile) && user.canMove())
		{
			user.setGoal(frontTile);
			return;
		}

		// Is the other end of the gate blocked, not by an interactive item?
		RoomTile behind = this.getTileBehind();
		if(behind == null || !behind.isOpen() || this.inUse())
		{
			// Don't bother entering the gate
			return;
		}

		// Tadaa
		this.setUser(user);

		// Lock own movement, move into the gate
		user.lockMovement();
		user.overrideNextTile = true;
		user.setGoal(this.getTile());

		// Wait...
		this.setTimeout(2);
	}

	@Override
	protected void onTimeout()
	{
		// In use?
		if(this.inUse())
		{
			// User in the gate?
			RoomUser user = this.getUser();
			if(user.at(this.getTile()))
			{
				// Move to tile behind
				user.setGoal(this.getTileBehind());

				// Set a new timeout
				this.setTimeout(1);
			}

			// User exited the gate?
			else if(user.at(this.getTileBehind()))
			{
				user.unlockMovement();
				this.clearUser();
			}
		}
	}
}
