package habbod2.game.rooms.items.minigame.banzai;

import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.minigames.Game;
import habbod2.game.rooms.minigames.GameTeam;
import habbod2.game.rooms.minigames.GameTeam.ETeam;
import habbod2.game.util.GameUtil;

public class ApparatusItem extends RoomFloorItem
{
	// 0 = off, 1 = on, 2 = on, 3 = noise, 4 = Red, 5 = Green, 6 = Blue, 7 = Yellow
	
	// als rood voor ligt -> 4
	// als groen voor ligt -> 5
	// als blauw voor ligt -> 6
	// als geel voor ligt -> 7
	
	private boolean isEnabled;
	
	public void enable()
	{
		this.setState(1);
		this.setTimeout(GameUtil.secondsToRoomPulses(3));
	}
	
	public void disable()
	{
		this.setState(0);
		this.clearTimeout();
		this.isEnabled = false;
	}
	
	public void showLeader(ETeam teamId)
	{
		if(teamId != null)
		{
			this.setState(10 + teamId.ordinal());
		}
		else
		{
			this.setState(1);
		}
	}
	
	public void showWinner(ETeam teamId)
	{
		if(teamId != null)
		{
			this.setState(20 + teamId.ordinal());
		}
		else
		{
			this.setState(3);
		}
	}
	
	@Override
	protected void onRoomAdd()
	{
		this.disable();
	}
	
	@Override
	protected void onTimeout()
	{
		Game game = this.getRoom().getGame();
		if(game == null)
		{
			this.disable();
		}
		else
		{
			// 3... 2... 1... expired?
			if(!this.isEnabled)
			{
				game.getTimer().begin();
				this.isEnabled = true;
			}
			else if(game.isShuttingDown())
			{
				GameTeam winner = game.getBestTeam();
				this.showWinner((winner != null) ? winner.getId() : null);
			}
			
			// Show leading team
			else
			{
				// Get leading team
				GameTeam leader = null;
				if(this.getRoom().getGame() != null)
				{
					leader = this.getRoom().getGame().getBestTeam();
				}
				
				// Set leading team
				if(leader == null)
				{
					this.showLeader(null);
				}
				else
				{
					this.showLeader(leader.getId());
				}
			}
			
			// Update again later
			this.setTimeout(GameUtil.secondsToRoomPulses(1));
		}
	}
}
