package habbod2.game.rooms.items.wired.triggers;

import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.sessions.Session;
import habbod2.game.rooms.items.wired.WiredTrigger;
import habbod2.util.StringUtil;

public class UserSaysPhraseTrigger extends WiredTrigger
{
	private static final int PARAM_OWNERONLY = 0;
	
	private String[] cards;
	
	@Override
	public int getInterface()
	{
		return 0;
	}
	
	@Override
	protected int getAmountOfParams()
	{
		return 1;
	}
	
	@Override
	protected boolean suppliesUser()
	{
		return true;
	}
	
	@Override
	protected void onSettingsUpdate()
	{
		// Cache this for StringUtil.matchesWildCards usage
		String[] cards = StringUtil.getWildCards(this.getText());
		
		// Only set them when atleast one card is non-empty
		this.cards = null;
		for(String card : cards)
		{
			if(!card.isEmpty())
			{
				this.cards = cards;
				return;
			}
		}
	}
	
	public static boolean matchAndExecutesTrigger(Session session, String message)
	{
		// Go through all items
		for(RoomFloorItem item : session.getRoom().getFloorItems())
		{
			// UserSaysPhraseTrigger?
			if(item instanceof UserSaysPhraseTrigger)
			{
				// Phrase matches?
				UserSaysPhraseTrigger trigger = (UserSaysPhraseTrigger)item;
				if(trigger.cards != null && StringUtil.matchesWildCards(message, trigger.cards))
				{
					// Owner-only check (if enabled)
					if(trigger.getParam(PARAM_OWNERONLY) == 0 || session.getRoomUser().hasRoomOwner)
					{
						// Let the user know he got the phrase correct
						session.sendWhisperNotif(message);
						
						// Fire the trigger
						trigger.execute(session.getRoomUser());
						
						// Don't chat the secret to room ;)
						return true;
					}
				}
			}
		}
		
		// Nothing happened!
		return false;
	}
}
