package prinzip;

import java.io.IOException;
import java.util.List;

import org.apache.log4j.Logger;

import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.User;

import com.google.common.collect.Lists;

import asg.cliche.Command;
import asg.cliche.Shell;
import asg.cliche.ShellFactory;

public class InteractiveConsole
{
	private static final Logger logger = Logger.getLogger(InteractiveConsole.class);
	
	private final Server server;
	
	private InteractiveConsole(Server server)
	{
		this.server = server;
	}
	
    @Command(name="add")
    public void addAccount(String username, String password, String searchLanguage, String keywordsStr, String importUsers) throws TwitterException
    {
    	// Deal with @
    	username = sanitizeName(username);
    	
    	// Sanitize keywords
    	List<String> keywords = this.parseKeywords(keywordsStr);
    	
    	// Resolve import accounts
    	List<Long> importUserIds = this.resolveUserIds(importUsers);
    	
    	// Register account with the system
    	this.server.getAccountMgr().addAccount(username, password, searchLanguage, keywords, importUserIds);
    }
    
    private String sanitizeName(String name)
    {
    	return name.trim().replace("@", "");
    }
    
    private List<String> parseKeywords(String input)
    {
    	// Parse keywords
		List<String> keywords = Lists.newArrayList();
		for(String keyword : input.split(","))
		{
			keyword = keyword.trim();
			if(!keyword.isEmpty())
			{
				keywords.add(keyword);
			}
		}
		
		return keywords;
    }
    
    private List<Long> resolveUserIds(String input) throws TwitterException
    {
    	// Parse usernames
    	List<String> userNames = Lists.newArrayList();
    	for(String userName : input.split(","))
    	{
			userName = sanitizeName(userName);
			if(!userName.isEmpty())
			{
				userNames.add(userName);
			}
    	}
    	
    	// Resolve user IDs
    	List<Long> userIds = Lists.newArrayList();
    	if(!userNames.isEmpty())
    	{
	    	Twitter utility = this.server.getAccountMgr().getUtilityAccount();
	    	if(utility == null)
	    	{
	    		logger.warn("no utility account available, imports not resolved");
	    	}
	    	else
	    	{
		    	for(User user : utility.lookupUsers(userNames.toArray(new String[0])))
		    	{
		    		userIds.add(user.getId());
		    	}
	    	}
    	}
    	
    	return userIds;
    }
    
    public static void enterShell(Server server) throws IOException
    {
    	Shell shell = ShellFactory.createConsoleShell("prinzip", "", new InteractiveConsole(server));
    	shell.commandLoop();
    }
}
