package prinzip.apps;

import java.io.IOException;

import org.jsoup.Jsoup;
import org.jsoup.Connection.Response;
import org.jsoup.nodes.Document;

import prinzip.accounts.Account;
import prinzip.apps.AppAuthorizationException.AuthorizationError;
import prinzip.util.Twitter4JUtil;
import prinzip.util.UserAgentGenerator;

import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;
import twitter4j.auth.AccessToken;
import twitter4j.auth.RequestToken;
import twitter4j.conf.Configuration;
import twitter4j.conf.ConfigurationBuilder;

public class TwitterApp
{
	public int appId;
	public String name;
	public String key;
	public String secret;
	
	private TwitterFactory twitterFactory;
	
	public void initialize()
	{
		Configuration conf = new ConfigurationBuilder()
		.setOAuthConsumerKey(this.key)
		.setOAuthConsumerSecret(this.secret)
		.build();
		
		// Set a random user agent etc
		String userAgent = UserAgentGenerator.generateApiAgent();
		Twitter4JUtil.configureHttpFingerprints(conf, "Prinzip", "", userAgent);
		
		this.twitterFactory = new TwitterFactory(conf);
	}
	
	public Twitter getUnauthorizedTwitter()
	{
		return this.twitterFactory.getInstance();
	}
	
	public Twitter getTwitter(Account account)
	{
		if(account.appId != this.appId)
		{
			throw new IllegalStateException("account is not configured to use this app");
		}
		
		return this.twitterFactory.getInstance(new AccessToken(account.authToken, account.authSecret));
	}
	
	public AccessToken authorize(Account account) throws TwitterException
	{
		// Get a Twitter client
		Twitter twitter = this.twitterFactory.getInstance();
		
		try
		{
			// Get a request token from Twitter
			RequestToken requestToken = twitter.getOAuthRequestToken();
		
			// Use the same (browser!) agent for the GET and POSTs
			String userAgent = UserAgentGenerator.generateBrowserAgent();
			
			// GET the authorization page, parse the RoR authenticity_token
			Response getResponse = Jsoup.connect(requestToken.getAuthorizationURL()).userAgent(userAgent).execute();
			String authenticityToken = getResponse.parse().getElementById("oauth_form").getElementsByAttributeValue("name", "authenticity_token").val();
			
			// POST the authorization page with the credentials, request token, RoR token and cookies
			Document resultDoc = Jsoup.connect(requestToken.getAuthorizationURL())
			.data("oauth_token", requestToken.getToken())
			.data("authenticity_token", authenticityToken)
			.data("session[username_or_email]", account.username)
			.data("session[password]", account.password)
			.cookies(getResponse.cookies())
			.userAgent(userAgent)
			.referrer(requestToken.getAuthorizationURL())
			.post();
			
			// Parse the PIN
			String pin = resultDoc.getElementsByTag("code").text();
			
			// If there is no PIN, then something went horribly wrong
			if(pin.isEmpty())
			{
				if(!resultDoc.text().contains(this.name))
				{
					throw new AppAuthorizationException(AuthorizationError.BAD_APP, this.name, null);
				}
				else
				{
					throw new AppAuthorizationException(AuthorizationError.BAD_ACCOUNT, account.username, null);
				}
			}
			else
			{
				return twitter.getOAuthAccessToken(requestToken, pin);
			}
		}
		catch(IOException e)
		{
			throw new TwitterException("authorization network error", e);
		}
	}
}
