package prinzip.apps;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.apache.log4j.Logger;
import org.jongo.MongoCollection;

import prinzip.Server;

import twitter4j.conf.ConfigurationBuilder;

public class TwitterAppManager extends Server.Referent
{
	private static final Logger logger = Logger.getLogger(TwitterAppManager.class);
	
	private TwitterApp[] apps = new TwitterApp[0];
	private MongoCollection dbApps;
	
	public TwitterAppManager(Server server)
	{
		super(server);
		this.dbApps = this.getServer().getDB().getCollection("apps");
		dbApps.ensureIndex("{appId:1,name:1,key:1}", "{unique:true}");
		
		logger.info(String.format("API %s", new ConfigurationBuilder().build().getRestBaseURL()));
	}
	
	public boolean reloadApps()
	{
		List<TwitterApp> list = new ArrayList<TwitterApp>();
		for(TwitterApp app : this.dbApps.find().as(TwitterApp.class))
		{
			app.initialize();
			
			list.add(app);
			
			logger.info(String.format("loaded app %s (%s)", app.name, app.key));
		}
		
		
		this.apps = list.toArray(this.apps);
		
		return (list.size() > 0);
	}
	
	public TwitterApp selectApp()
	{
		if(this.apps.length > 0)
		{
			Random rand = new Random();
			return this.apps[rand.nextInt(this.apps.length)];
		}
		else
		{
			return null;
		}
	}
	
	public TwitterApp createApp(String name, String key, String secret)
	{
		TwitterApp app = new TwitterApp();
		app.appId = this.apps.length;
		app.name = name;
		app.key = key;
		app.secret = secret;
		
		if(this.dbApps.save(app).getError() == null)
		{
			this.reloadApps();
			return app;
		}
		else
		{
			return null;
		}
	}
	
	public TwitterApp getApp(int appId)
	{
		for(TwitterApp app : this.apps)
		{
			if(app.appId == appId)
			{
				return app;
			}
		}
		
		return null;
	}
}
