package prinzip.engine;

import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

public abstract class ClockworkAction implements Runnable
{	
	private static final Logger logger = Logger.getLogger(ClockworkAction.class);
	
	protected abstract void execute() throws Exception;
	
	protected abstract void onCompletion();
	
	@Override
	public final void run()
	{
		if(logger.isDebugEnabled()) logger.debug(String.format("executing %s in %s", this, Thread.currentThread().getName()));
		
		try
		{
			this.execute();
			this.onCompletion();
		}
		catch(Exception e)
		{
			this.onException(e);
		}
	}
	
	protected void onException(Exception e)
	{
		// Override this method for custom handling
		logger.error(String.format("caught exception while executing %s", this), e);
	}
	
	protected abstract ClockworkEngine getEngine();
	
	protected void rescheduleAfter(long delay, TimeUnit unit)
	{
		if(logger.isDebugEnabled()) logger.debug(String.format("reschedule %s after %d %s", this, delay, unit.name().toLowerCase()));
		this.getEngine().scheduleAfter(delay, unit, this);
	}
	
	protected abstract String getDescription();
	
	@Override
	public String toString()
	{
		String desc = this.getDescription();
		if(desc != null)
		{
			return desc;
		}
		else
		{
			return this.getClass().getSimpleName();
		}
	}
}
