package prinzip.engine;

import java.util.Date;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

public class ClockworkEngine
{
	private static final Logger logger = Logger.getLogger(ClockworkEngine.class);
	
	private final ScheduledExecutorService scheduler;
	
	public ClockworkEngine()
	{
		this(getAllowedCpus());
		this.scheduler.scheduleAtFixedRate(new MemoryWatchdog(), 0, 15, TimeUnit.MINUTES);
	}
	
	public ClockworkEngine(int threads)
	{
		this.scheduler = Executors.newScheduledThreadPool(threads, new ThreadFactory()
		{
			private int threadCounter;
			
			@Override
			public Thread newThread(Runnable r)
			{
				int workerNum = ++this.threadCounter;
				return new Thread(r, String.format("Clockworker #%d", workerNum));
			}	
		});
		
		logger.info(String.format("running on %d threads (@ %d CPU cores)", threads, getAllowedCpus()));
	}
	
	public void executeNow(ClockworkAction action)
	{
		this.scheduler.execute(action);
	}
	
	public void scheduleAfter(long delay, TimeUnit unit, ClockworkAction action) 
	{
		this.scheduler.schedule(action, delay, unit);
		
		Date time = new Date(System.currentTimeMillis() + unit.toMillis(delay));
		if(logger.isDebugEnabled()) logger.debug(String.format("scheduled %s at %s", action, time));
	}
	
	public void scheduleAt(Date time, ClockworkAction action)
	{
		long delay = time.getTime() - System.currentTimeMillis();
		this.scheduler.schedule(action, delay, TimeUnit.MILLISECONDS);
		
		if(logger.isDebugEnabled()) logger.debug(String.format("scheduled %s at %s", action, time));
	}
	
	private static int getAllowedCpus()
	{
		return Runtime.getRuntime().availableProcessors();
	}
}
