package prinzip.util;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Map;

import org.apache.log4j.Logger;

import twitter4j.conf.Configuration;

public class Twitter4JUtil
{
	private static final Logger logger = Logger.getLogger(Twitter4JUtil.class);
	
	private static Method setClientURL;
	private static Method setUserAgent;
	private static Field requestHeaders;
	
	private static boolean confHackAvailable;
	
	static
	{
		try
		{
			Class<?> ConfigurationBase = Class.forName("twitter4j.conf.ConfigurationBase");
			setClientURL = ConfigurationBase.getDeclaredMethod("setClientURL", String.class);
			setClientURL.setAccessible(true);
			setUserAgent = ConfigurationBase.getDeclaredMethod("setUserAgent", String.class);
			setUserAgent.setAccessible(true);
			requestHeaders = ConfigurationBase.getDeclaredField("requestHeaders");
			requestHeaders.setAccessible(true);
			
			confHackAvailable = true;
			if(logger.isDebugEnabled()) logger.debug("custom API fingerprinting available");
		}
		catch(Exception e)
		{
			logger.warn(String.format("could not acquire full access to ConfigurationBase, Twitter4J API must have changed"));
			logger.warn(String.format("custom API fingerprinting is not available"));
		}
	}
	
	@SuppressWarnings("unchecked")
	public static void configureHttpFingerprints(Configuration conf, String clientName, String clientUrl, String userAgent)
	{
		if(confHackAvailable)
		{
			try
			{
				setClientURL.invoke(conf, clientUrl);
				setUserAgent.invoke(conf, userAgent);
				
				Map<String, String> headers = (Map<String, String>) requestHeaders.get(conf);
				headers.put("X-Twitter-Client", clientName);
			}
			catch(Exception e)
			{
				logger.error("error while applying conf hack", e);
			}
		}
	}
	
	public static void silenceLoggers()
	{
		System.setProperty("twitter4j.loggerFactory", "twitter4j.internal.logging.NullLoggerFactory"); 		
	}
}
