create table stream (
  type                      varchar(1) not null,
  player_id                 integer not null,
  timestamp                 datetime,
  loser_id                  integer,
  param                     integer)
;

create table matches (
  id                        bigint auto_increment not null,
  server_id                 integer not null,
  start_time                datetime not null,
  end_time                  datetime not null,
  player_one_id             integer not null,
  player_two_id             integer not null,
  winner_id                 integer,
  constraint pk_matches primary key (id))
;

create table news (
  id                        integer auto_increment not null,
  headline                  varchar(255),
  body                      varchar(255),
  image_url                 varchar(255),
  timestamp                 datetime,
  constraint pk_news primary key (id))
;

create table players (
  id                        integer auto_increment not null,
  token                     varchar(255) not null,
  name                      varchar(32) not null,
  avatar                    varchar(255) not null,
  motto                     varchar(255) not null,
  country                   varchar(255) not null,
  locale                    varchar(255) not null,
  platform                  varchar(255) not null,
  server_id                 integer not null,
  opponent_id               bigint not null,
  xp                        integer not null,
  level                     integer not null,
  logins                    integer not null,
  matches                   integer not null,
  wins                      integer not null,
  losses                    integer not null,
  balance                   integer not null,
  items                     varchar(255) not null,
  fb_uid                    bigint,
  fb_token                  varchar(255),
  liked                     tinyint(1) default 0 not null,
  joined                    datetime not null,
  last_update               datetime not null,
  constraint uq_null_fb_uid unique (fb_uid),
  constraint pk_players primary key (id))
;

create table servers (
  id                        integer auto_increment not null,
  name                      varchar(255),
  host                      varchar(255),
  port                      integer,
  started                   datetime,
  memory_usage              float,
  online_players            integer,
  total_logins              bigint,
  active_matches            integer,
  total_matches             bigint,
  avg_waiting_time          integer,
  version                   varchar(255),
  last_update               datetime,
  heartbeat                 integer not null,
  constraint pk_servers primary key (id))
;

alter table stream add constraint fk_stream_player_1 foreign key (player_id) references players (id) on delete restrict on update restrict;
create index ix_stream_player_1 on stream (player_id);
alter table stream add constraint fk_stream_loser_2 foreign key (loser_id) references players (id) on delete restrict on update restrict;
create index ix_stream_loser_2 on stream (loser_id);
alter table matches add constraint fk_matches_server_3 foreign key (server_id) references servers (id) on delete restrict on update restrict;
create index ix_matches_server_3 on matches (server_id);
alter table matches add constraint fk_matches_playerOne_4 foreign key (player_one_id) references players (id) on delete restrict on update restrict;
create index ix_matches_playerOne_4 on matches (player_one_id);
alter table matches add constraint fk_matches_playerTwo_5 foreign key (player_two_id) references players (id) on delete restrict on update restrict;
create index ix_matches_playerTwo_5 on matches (player_two_id);
alter table matches add constraint fk_matches_winner_6 foreign key (winner_id) references players (id) on delete restrict on update restrict;
create index ix_matches_winner_6 on matches (winner_id);


